@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2008, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -47,7 +47,7 @@ public class ExitAction extends DelegateAction
     public static final String VALUE_NAME = "Exit";
     public static final String VALUE_SMALL_ICON = null;
     public static final String VALUE_LARGE_ICON = null;
-    public static final Integer VALUE_MNEMONIC = new Integer(88);
+    public static final Integer VALUE_MNEMONIC = 88;
     public static final KeyStroke VALUE_ACCELERATOR = null;
     public static final String VALUE_SHORT_DESCRIPTION = "Exits the application";
     public static final String VALUE_LONG_DESCRIPTION = "Exits the application";
