@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2021, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -249,7 +249,6 @@ public class GTKLookAndFeel extends SynthLookAndFeel {
                 region == Region.SPINNER ||
                 region == Region.TABLE ||
                 region == Region.TEXT_AREA ||
-                region == Region.TEXT_FIELD ||
                 region == Region.TEXT_PANE ||
                 region == Region.TREE);
     }
@@ -312,7 +311,7 @@ public class GTKLookAndFeel extends SynthLookAndFeel {
             "com.sun.java.swing.plaf.gtk.GTKPainter$ListTableFocusBorder",
             "getNoFocusCellBorder");
 
-        GTKStyleFactory factory = (GTKStyleFactory)getStyleFactory();
+        SynthStyleFactory factory = getStyleFactory();
         GTKStyle tableStyle = (GTKStyle)factory.getStyle(null, Region.TREE);
         Color tableBg = tableStyle.getGTKColor(SynthConstants.ENABLED,
                 GTKColorType.TEXT_BACKGROUND);
@@ -505,7 +504,7 @@ public class GTKLookAndFeel extends SynthLookAndFeel {
                 this.region = region;
             }
             public Object createValue(UIDefaults table) {
-                GTKStyleFactory factory = (GTKStyleFactory)getStyleFactory();
+                SynthStyleFactory factory = getStyleFactory();
                 GTKStyle style = (GTKStyle)factory.getStyle(null, region);
                 return style.getDefaultFont();
             }
@@ -578,6 +577,7 @@ public class GTKLookAndFeel extends SynthLookAndFeel {
                  }),
             "ComboBox.font", new FontLazyValue(Region.COMBO_BOX),
             "ComboBox.isEnterSelectablePopup", Boolean.TRUE,
+            "ComboBox.squareButton", Boolean.FALSE,
 
 
             "EditorPane.caretForeground", caretColor,
@@ -591,6 +591,7 @@ public class GTKLookAndFeel extends SynthLookAndFeel {
             "FileChooser.ancestorInputMap",
                new UIDefaults.LazyInputMap(new Object[] {
                      "ESCAPE", "cancelSelection",
+                 "BACK_SPACE", "Go Up",
                  "ctrl ENTER", "approveSelection"
                  }),
             "FileChooserUI", "com.sun.java.swing.plaf.gtk.GTKLookAndFeel",
@@ -803,7 +804,7 @@ public class GTKLookAndFeel extends SynthLookAndFeel {
             "PasswordField.font", new FontLazyValue(Region.PASSWORD_FIELD),
 
 
-            "PopupMenu.consumeEventOnClose", Boolean.TRUE,
+            "PopupMenu.consumeEventOnClose", Boolean.FALSE,
             "PopupMenu.selectedWindowInputMapBindings", new Object[] {
                   "ESCAPE", "cancel",
                     "DOWN", "selectNext",
@@ -1305,7 +1306,7 @@ public class GTKLookAndFeel extends SynthLookAndFeel {
     }
 
     protected void initSystemColorDefaults(UIDefaults table) {
-        GTKStyleFactory factory = (GTKStyleFactory)getStyleFactory();
+        SynthStyleFactory factory = getStyleFactory();
         GTKStyle windowStyle =
                 (GTKStyle)factory.getStyle(null, Region.INTERNAL_FRAME);
         table.put("window", windowStyle.getGTKColor(SynthConstants.ENABLED,
@@ -1357,7 +1358,7 @@ public class GTKLookAndFeel extends SynthLookAndFeel {
         // colors specific only for GTK
         // It is impossible to create a simple GtkWidget without specifying the
         // type. So for GtkWidget we can use any appropriate concrete type of
-        // wigdet. LABEL in this case.
+        // widget. LABEL in this case.
         GTKStyle widgetStyle = (GTKStyle)factory.getStyle(null, Region.LABEL);
         Color bg = widgetStyle.getGTKColor(SynthConstants.ENABLED,
                                            GTKColorType.BACKGROUND);
@@ -1422,6 +1423,7 @@ public class GTKLookAndFeel extends SynthLookAndFeel {
         }
 
         if (UNIXToolkit.getGtkVersion() == UNIXToolkit.GtkVersions.GTK2) {
+            @SuppressWarnings("removal")
             String version = AccessController.doPrivileged(
                     new GetPropertyAction("jdk.gtk.version"));
             if (version != null) {
