@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1999, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -27,12 +27,13 @@ package com.sun.jndi.ldap;
 
 import javax.naming.*;
 import javax.naming.directory.*;
-import javax.naming.spi.*;
 import javax.naming.ldap.*;
 
 import java.util.Hashtable;
 import java.util.StringTokenizer;
 import com.sun.jndi.toolkit.dir.SearchFilter;
+import com.sun.naming.internal.NamingManagerHelper;
+import com.sun.naming.internal.ObjectFactoriesFilter;
 
 /**
  * A context for handling referrals.
@@ -116,8 +117,8 @@ final class LdapReferralContext implements DirContext, LdapContext {
 
             Object obj;
             try {
-                obj = NamingManager.getObjectInstance(ref, null, null, env);
-
+                obj = NamingManagerHelper.getObjectInstance(ref, null, null,
+                        env, ObjectFactoriesFilter::checkLdapFilter);
             } catch (NamingException e) {
 
                 if (handleReferrals == LdapClient.LDAP_REF_THROW) {
@@ -182,7 +183,7 @@ final class LdapReferralContext implements DirContext, LdapContext {
         if (urlString == null) {
             urlName = null;
         } else {
-            urlName = urlString.equals("") ? new CompositeName() :
+            urlName = urlString.isEmpty() ? new CompositeName() :
                 new CompositeName().add(urlString);
         }
     }
@@ -198,8 +199,8 @@ final class LdapReferralContext implements DirContext, LdapContext {
 
     void setHopCount(int hopCount) {
         this.hopCount = hopCount;
-        if ((refCtx != null) && (refCtx instanceof LdapCtx)) {
-            ((LdapCtx)refCtx).setHopCount(hopCount);
+        if (refCtx instanceof LdapCtx ldapCtx) {
+            ldapCtx.setHopCount(hopCount);
         }
     }
 
@@ -888,7 +889,7 @@ final class LdapReferralContext implements DirContext, LdapContext {
 
     // ---------------------- Private methods  ---------------------
     private Name toName(String name) throws InvalidNameException {
-        return name.equals("") ? new CompositeName() :
+        return name.isEmpty() ? new CompositeName() :
             new CompositeName().add(name);
     }
 
