@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1999, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -27,6 +27,7 @@ package com.sun.jndi.toolkit.dir;
 import javax.naming.*;
 import javax.naming.directory.*;
 import java.util.Enumeration;
+import java.util.HexFormat;
 import java.util.StringTokenizer;
 import java.util.Vector;
 import java.util.Locale;
@@ -81,7 +82,7 @@ public class SearchFilter implements AttrFilter {
      * Utility routines used by member classes
      */
 
-    // does some pre-processing on the string to make it look exactly lik
+    // does some pre-processing on the string to make it look exactly like
     // what the parser expects. This only needs to be called once.
     protected void normalizeFilter() {
         skipWhiteSpace(); // get rid of any leading whitespaces
@@ -399,7 +400,7 @@ public class SearchFilter implements AttrFilter {
                     !value.toLowerCase(Locale.ENGLISH).startsWith(
                         subStrs.nextToken().toLowerCase(Locale.ENGLISH))) {
                 if(debug) {
-                    System.out.println("faild initial test");
+                    System.out.println("failed initial test");
                 }
                 return false;
             }
@@ -420,7 +421,7 @@ public class SearchFilter implements AttrFilter {
             // do we need to end with the last token?
             if(proto.charAt(proto.length() - 1) != WILDCARD_TOKEN &&
                currentPos != value.length() ) {
-                if(debug) {System.out.println("faild final test");}
+                if(debug) {System.out.println("failed final test");}
                 return false;
             }
 
@@ -466,26 +467,6 @@ public class SearchFilter implements AttrFilter {
         return answer;
     }
 
-    // Writes the hex representation of a byte to a StringBuffer.
-    private static void hexDigit(StringBuffer buf, byte x) {
-        char c;
-
-        c = (char) ((x >> 4) & 0xf);
-        if (c > 9)
-            c = (char) ((c-10) + 'A');
-        else
-            c = (char)(c + '0');
-
-        buf.append(c);
-        c = (char) (x & 0xf);
-        if (c > 9)
-            c = (char)((c-10) + 'A');
-        else
-            c = (char)(c + '0');
-        buf.append(c);
-    }
-
-
     /**
       * Returns the string representation of an object (such as an attr value).
       * If obj is a byte array, encode each item as \xx, where xx is hex encoding
@@ -506,13 +487,9 @@ public class SearchFilter implements AttrFilter {
 
         if (obj instanceof byte[]) {
             // binary data must be encoded as \hh where hh is a hex char
+            HexFormat hex = HexFormat.of().withUpperCase().withPrefix("\\");
             byte[] bytes = (byte[])obj;
-            StringBuffer b1 = new StringBuffer(bytes.length*3);
-            for (int i = 0; i < bytes.length; i++) {
-                b1.append('\\');
-                hexDigit(b1, bytes[i]);
-            }
-            return b1.toString();
+            return hex.formatHex(bytes);
         }
         if (!(obj instanceof String)) {
             str = obj.toString();
