@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2025, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  */
 /**
@@ -31,6 +31,8 @@ import java.security.SecureRandom;
 import java.security.Signature;
 import java.security.SignatureException;
 import java.security.spec.AlgorithmParameterSpec;
+import java.security.spec.MGF1ParameterSpec;
+import java.security.spec.PSSParameterSpec;
 
 import com.sun.org.apache.xml.internal.security.algorithms.JCEMapper;
 import com.sun.org.apache.xml.internal.security.algorithms.SignatureAlgorithmSpi;
@@ -43,8 +45,6 @@ import org.w3c.dom.Element;
 import org.w3c.dom.Text;
 
 import javax.xml.crypto.dsig.DigestMethod;
-import java.security.spec.MGF1ParameterSpec;
-import java.security.spec.PSSParameterSpec;
 
 public abstract class SignatureBaseRSA extends SignatureAlgorithmSpi {
 
@@ -65,20 +65,24 @@ public abstract class SignatureBaseRSA extends SignatureAlgorithmSpi {
 
     public SignatureBaseRSA(Provider provider) throws XMLSignatureException {
         String algorithmID = JCEMapper.translateURItoJCEID(this.engineGetURI());
-        LOG.debug("Created SignatureRSA using {}", algorithmID);
+        this.signatureAlgorithm = getSignature(provider, algorithmID);
+        LOG.debug("Created SignatureRSA using {0}", algorithmID);
+    }
 
+    Signature getSignature(Provider provider, String algorithmID)
+        throws XMLSignatureException {
         try {
             if (provider == null) {
                 String providerId = JCEMapper.getProviderId();
                 if (providerId == null) {
-                    this.signatureAlgorithm = Signature.getInstance(algorithmID);
+                    return Signature.getInstance(algorithmID);
 
                 } else {
-                    this.signatureAlgorithm = Signature.getInstance(algorithmID, providerId);
+                    return Signature.getInstance(algorithmID, providerId);
                 }
 
             } else {
-                this.signatureAlgorithm = Signature.getInstance(algorithmID, provider);
+                return Signature.getInstance(algorithmID, provider);
             }
 
         } catch (NoSuchAlgorithmException | NoSuchProviderException ex) {
@@ -88,6 +92,7 @@ public abstract class SignatureBaseRSA extends SignatureAlgorithmSpi {
     }
 
     /** {@inheritDoc} */
+    @Override
     protected void engineSetParameter(AlgorithmParameterSpec params)
         throws XMLSignatureException {
         try {
@@ -98,6 +103,7 @@ public abstract class SignatureBaseRSA extends SignatureAlgorithmSpi {
     }
 
     /** {@inheritDoc} */
+    @Override
     protected boolean engineVerify(byte[] signature) throws XMLSignatureException {
         try {
             return this.signatureAlgorithm.verify(signature);
@@ -107,11 +113,13 @@ public abstract class SignatureBaseRSA extends SignatureAlgorithmSpi {
     }
 
     /** {@inheritDoc} */
+    @Override
     protected void engineInitVerify(Key publicKey) throws XMLSignatureException {
         engineInitVerify(publicKey, this.signatureAlgorithm);
     }
 
     /** {@inheritDoc} */
+    @Override
     protected byte[] engineSign() throws XMLSignatureException {
         try {
             return this.signatureAlgorithm.sign();
@@ -121,17 +129,20 @@ public abstract class SignatureBaseRSA extends SignatureAlgorithmSpi {
     }
 
     /** {@inheritDoc} */
+    @Override
     protected void engineInitSign(Key privateKey, SecureRandom secureRandom)
         throws XMLSignatureException {
         engineInitSign(privateKey, secureRandom, this.signatureAlgorithm);
     }
 
     /** {@inheritDoc} */
+    @Override
     protected void engineInitSign(Key privateKey) throws XMLSignatureException {
         engineInitSign(privateKey, (SecureRandom)null);
     }
 
     /** {@inheritDoc} */
+    @Override
     protected void engineUpdate(byte[] input) throws XMLSignatureException {
         try {
             this.signatureAlgorithm.update(input);
@@ -141,6 +152,7 @@ public abstract class SignatureBaseRSA extends SignatureAlgorithmSpi {
     }
 
     /** {@inheritDoc} */
+    @Override
     protected void engineUpdate(byte input) throws XMLSignatureException {
         try {
             this.signatureAlgorithm.update(input);
@@ -150,6 +162,7 @@ public abstract class SignatureBaseRSA extends SignatureAlgorithmSpi {
     }
 
     /** {@inheritDoc} */
+    @Override
     protected void engineUpdate(byte[] buf, int offset, int len) throws XMLSignatureException {
         try {
             this.signatureAlgorithm.update(buf, offset, len);
@@ -159,22 +172,26 @@ public abstract class SignatureBaseRSA extends SignatureAlgorithmSpi {
     }
 
     /** {@inheritDoc} */
+    @Override
     protected String engineGetJCEAlgorithmString() {
         return this.signatureAlgorithm.getAlgorithm();
     }
 
     /** {@inheritDoc} */
+    @Override
     protected String engineGetJCEProviderName() {
         return this.signatureAlgorithm.getProvider().getName();
     }
 
     /** {@inheritDoc} */
+    @Override
     protected void engineSetHMACOutputLength(int HMACOutputLength)
         throws XMLSignatureException {
         throw new XMLSignatureException("algorithms.HMACOutputLengthOnlyForHMAC");
     }
 
     /** {@inheritDoc} */
+    @Override
     protected void engineInitSign(
         Key signingKey, AlgorithmParameterSpec algorithmParameterSpec
     ) throws XMLSignatureException {
@@ -356,10 +373,53 @@ public abstract class SignatureBaseRSA extends SignatureAlgorithmSpi {
         }
     }
 
+    public abstract static class SignatureBaseRSAPSS extends SignatureBaseRSA {
+
+        public SignatureBaseRSAPSS() throws XMLSignatureException {
+            super();
+        }
+
+        public SignatureBaseRSAPSS(Provider provider) throws XMLSignatureException {
+            super(provider);
+        }
+
+        @Override
+        Signature getSignature(Provider provider, String algorithmID)
+            throws XMLSignatureException {
+            try {
+                Signature sig;
+                if (provider == null) {
+                    String providerId = JCEMapper.getProviderId();
+                    if (providerId == null) {
+                        sig = Signature.getInstance("RSASSA-PSS");
+                    } else {
+                        sig = Signature.getInstance("RSASSA-PSS", providerId);
+                    }
+                } else {
+                    sig = Signature.getInstance("RSASSA-PSS", provider);
+                }
+                try {
+                    sig.setParameter(getPSSParameterSpec());
+                } catch (InvalidAlgorithmParameterException e) {
+                    throw new NoSuchAlgorithmException("Should not happen", e);
+                }
+                return sig;
+            } catch (NoSuchAlgorithmException | NoSuchProviderException e) {
+                return super.getSignature(provider, algorithmID);
+            }
+        }
+
+        abstract PSSParameterSpec getPSSParameterSpec();
+    }
+
     /**
      * Class SignatureRSASHA1MGF1
      */
-    public static class SignatureRSASHA1MGF1 extends SignatureBaseRSA {
+    public static class SignatureRSASHA1MGF1 extends SignatureBaseRSAPSS {
+
+        private static final PSSParameterSpec SHA1_MGF1_PARAMS
+                = new PSSParameterSpec("SHA-1", "MGF1", MGF1ParameterSpec.SHA1,
+                20, PSSParameterSpec.TRAILER_FIELD_BC);
 
         /**
          * Constructor SignatureRSASHA1MGF1
@@ -379,12 +439,21 @@ public abstract class SignatureBaseRSA extends SignatureAlgorithmSpi {
         public String engineGetURI() {
             return XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA1_MGF1;
         }
+
+        @Override
+        public PSSParameterSpec getPSSParameterSpec() {
+            return SHA1_MGF1_PARAMS;
+        }
     }
 
     /**
      * Class SignatureRSASHA224MGF1
      */
-    public static class SignatureRSASHA224MGF1 extends SignatureBaseRSA {
+    public static class SignatureRSASHA224MGF1 extends SignatureBaseRSAPSS {
+
+        private static final PSSParameterSpec SHA224_MGF1_PARAMS
+                = new PSSParameterSpec("SHA-224", "MGF1", MGF1ParameterSpec.SHA224,
+                28, PSSParameterSpec.TRAILER_FIELD_BC);
 
         /**
          * Constructor SignatureRSASHA224MGF1
@@ -404,12 +473,21 @@ public abstract class SignatureBaseRSA extends SignatureAlgorithmSpi {
         public String engineGetURI() {
             return XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA224_MGF1;
         }
+
+        @Override
+        public PSSParameterSpec getPSSParameterSpec() {
+            return SHA224_MGF1_PARAMS;
+        }
     }
 
     /**
      * Class SignatureRSASHA256MGF1
      */
-    public static class SignatureRSASHA256MGF1 extends SignatureBaseRSA {
+    public static class SignatureRSASHA256MGF1 extends SignatureBaseRSAPSS {
+
+        private static final PSSParameterSpec SHA256_MGF1_PARAMS
+                = new PSSParameterSpec("SHA-256", "MGF1", MGF1ParameterSpec.SHA256,
+                32, PSSParameterSpec.TRAILER_FIELD_BC);
 
         /**
          * Constructor SignatureRSASHA256MGF1
@@ -429,12 +507,21 @@ public abstract class SignatureBaseRSA extends SignatureAlgorithmSpi {
         public String engineGetURI() {
             return XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA256_MGF1;
         }
+
+        @Override
+        public PSSParameterSpec getPSSParameterSpec() {
+            return SHA256_MGF1_PARAMS;
+        }
     }
 
     /**
      * Class SignatureRSASHA384MGF1
      */
-    public static class SignatureRSASHA384MGF1 extends SignatureBaseRSA {
+    public static class SignatureRSASHA384MGF1 extends SignatureBaseRSAPSS {
+
+        private static final PSSParameterSpec SHA384_MGF1_PARAMS
+                = new PSSParameterSpec("SHA-384", "MGF1", MGF1ParameterSpec.SHA384,
+                48, PSSParameterSpec.TRAILER_FIELD_BC);
 
         /**
          * Constructor SignatureRSASHA384MGF1
@@ -454,12 +541,21 @@ public abstract class SignatureBaseRSA extends SignatureAlgorithmSpi {
         public String engineGetURI() {
             return XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA384_MGF1;
         }
+
+        @Override
+        public PSSParameterSpec getPSSParameterSpec() {
+            return SHA384_MGF1_PARAMS;
+        }
     }
 
     /**
      * Class SignatureRSASHA512MGF1
      */
-    public static class SignatureRSASHA512MGF1 extends SignatureBaseRSA {
+    public static class SignatureRSASHA512MGF1 extends SignatureBaseRSAPSS {
+
+        private static final PSSParameterSpec SHA512_MGF1_PARAMS
+                = new PSSParameterSpec("SHA-512", "MGF1", MGF1ParameterSpec.SHA512,
+                64, PSSParameterSpec.TRAILER_FIELD_BC);
 
         /**
          * Constructor SignatureRSASHA512MGF1
@@ -479,12 +575,22 @@ public abstract class SignatureBaseRSA extends SignatureAlgorithmSpi {
         public String engineGetURI() {
             return XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA512_MGF1;
         }
+
+        @Override
+        public PSSParameterSpec getPSSParameterSpec() {
+            return SHA512_MGF1_PARAMS;
+        }
     }
 
     /**
      * Class SignatureRSA3_SHA224MGF1
      */
-    public static class SignatureRSASHA3_224MGF1 extends SignatureBaseRSA {
+    public static class SignatureRSASHA3_224MGF1 extends SignatureBaseRSAPSS {
+
+        private static final PSSParameterSpec SHA3_224_MGF1_PARAMS
+                = new PSSParameterSpec("SHA3-224", "MGF1",
+                new MGF1ParameterSpec("SHA3-224"),
+                28, PSSParameterSpec.TRAILER_FIELD_BC);
 
         /**
          * Constructor SignatureRSASHA3_224MGF1
@@ -504,12 +610,22 @@ public abstract class SignatureBaseRSA extends SignatureAlgorithmSpi {
         public String engineGetURI() {
             return XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA3_224_MGF1;
         }
+
+        @Override
+        public PSSParameterSpec getPSSParameterSpec() {
+            return SHA3_224_MGF1_PARAMS;
+        }
     }
 
     /**
      * Class SignatureRSA3_SHA256MGF1
      */
-    public static class SignatureRSASHA3_256MGF1 extends SignatureBaseRSA {
+    public static class SignatureRSASHA3_256MGF1 extends SignatureBaseRSAPSS {
+
+        private static final PSSParameterSpec SHA3_256_MGF1_PARAMS
+                = new PSSParameterSpec("SHA3-256", "MGF1",
+                new MGF1ParameterSpec("SHA3-256"),
+                32, PSSParameterSpec.TRAILER_FIELD_BC);
 
         /**
          * Constructor SignatureRSASHA3_256MGF1
@@ -529,12 +645,22 @@ public abstract class SignatureBaseRSA extends SignatureAlgorithmSpi {
         public String engineGetURI() {
             return XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA3_256_MGF1;
         }
+
+        @Override
+        public PSSParameterSpec getPSSParameterSpec() {
+            return SHA3_256_MGF1_PARAMS;
+        }
     }
 
     /**
      * Class SignatureRSA3_SHA384MGF1
      */
-    public static class SignatureRSASHA3_384MGF1 extends SignatureBaseRSA {
+    public static class SignatureRSASHA3_384MGF1 extends SignatureBaseRSAPSS {
+
+        private static final PSSParameterSpec SHA3_384_MGF1_PARAMS
+                = new PSSParameterSpec("SHA3-384", "MGF1",
+                new MGF1ParameterSpec("SHA3-384"),
+                48, PSSParameterSpec.TRAILER_FIELD_BC);
 
         /**
          * Constructor SignatureRSASHA3_384MGF1
@@ -554,12 +680,22 @@ public abstract class SignatureBaseRSA extends SignatureAlgorithmSpi {
         public String engineGetURI() {
             return XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA3_384_MGF1;
         }
+
+        @Override
+        public PSSParameterSpec getPSSParameterSpec() {
+            return SHA3_384_MGF1_PARAMS;
+        }
     }
 
     /**
      * Class SignatureRSASHA3_512MGF1
      */
-    public static class SignatureRSASHA3_512MGF1 extends SignatureBaseRSA {
+    public static class SignatureRSASHA3_512MGF1 extends SignatureBaseRSAPSS {
+
+        private static final PSSParameterSpec SHA3_512_MGF1_PARAMS
+                = new PSSParameterSpec("SHA3-512", "MGF1",
+                new MGF1ParameterSpec("SHA3-512"),
+                64, PSSParameterSpec.TRAILER_FIELD_BC);
 
         /**
          * Constructor SignatureRSASHA3_512MGF1
@@ -579,10 +715,15 @@ public abstract class SignatureBaseRSA extends SignatureAlgorithmSpi {
         public String engineGetURI() {
             return XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA3_512_MGF1;
         }
+
+        @Override
+        public PSSParameterSpec getPSSParameterSpec() {
+            return SHA3_512_MGF1_PARAMS;
+        }
     }
 
     public static class SignatureRSASSAPSS extends SignatureBaseRSA {
-        PSSParameterSpec pssParameterSpec;
+        private PSSParameterSpec pssParameterSpec;
 
         public enum DigestAlgorithm {
             SHA224("SHA-224", DigestMethod.SHA224, 28),
