@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -27,7 +27,9 @@ package com.sun.source.util;
 
 import java.util.List;
 
+import javax.lang.model.element.Element;
 import javax.lang.model.element.Name;
+import javax.lang.model.util.Elements;
 import javax.tools.Diagnostic;
 import javax.tools.JavaFileObject;
 
@@ -43,6 +45,7 @@ import com.sun.source.doctree.DocTypeTree;
 import com.sun.source.doctree.EndElementTree;
 import com.sun.source.doctree.EntityTree;
 import com.sun.source.doctree.ErroneousTree;
+import com.sun.source.doctree.EscapeTree;
 import com.sun.source.doctree.HiddenTree;
 import com.sun.source.doctree.IdentifierTree;
 import com.sun.source.doctree.IndexTree;
@@ -58,6 +61,8 @@ import com.sun.source.doctree.SerialDataTree;
 import com.sun.source.doctree.SerialFieldTree;
 import com.sun.source.doctree.SerialTree;
 import com.sun.source.doctree.SinceTree;
+import com.sun.source.doctree.SnippetTree;
+import com.sun.source.doctree.SpecTree;
 import com.sun.source.doctree.StartElementTree;
 import com.sun.source.doctree.SummaryTree;
 import com.sun.source.doctree.SystemPropertyTree;
@@ -79,7 +84,7 @@ import com.sun.source.doctree.VersionTree;
  */
 public interface DocTreeFactory {
     /**
-     * Creates a new {@code AttributeTree} object, to represent an HTML attribute in an HTML tag.
+     * Creates a new {@code AttributeTree} object, to represent an attribute in an HTML element or tag.
      * @param name  the name of the attribute
      * @param vkind the kind of the attribute value
      * @param value the value, if any, of the attribute
@@ -173,6 +178,29 @@ public interface DocTreeFactory {
      */
     ErroneousTree newErroneousTree(String text, Diagnostic<JavaFileObject> diag);
 
+    /**
+     * Creates a new {@code EscapeTree} object, to represent an escaped character.
+     *
+     * @apiNote This method does not itself constrain the set of valid escape sequences,
+     * although the set may be effectively constrained to those defined in the
+     * <a href="{@docRoot}/../specs/javadoc/doc-comment-spec.html#escape-sequences">
+     * Documentation Comment Specification for the Standard Doclet</a>,
+     * including the following context-sensitive escape sequences:
+     *
+     * <ul>
+     * <li>{@code @@}, representing {@code @}, where it would otherwise be treated as introducing a block or inline tag,
+     * <li>{@code @/}, representing {@code /}, as part of {@code *@/} to represent <code>&ast;&sol;</code>, and
+     * <li>{@code @*}, representing {@code *}, where it would otherwise be {@linkplain Elements#getDocComment(Element) discarded},
+     *     after whitespace at the beginning of a line.
+     * </ul>
+     *
+     * @param ch the character
+     * @return an {@code EscapeTree} object
+     *
+     * @since 21
+     */
+    EscapeTree newEscapeTree(char ch);
+
     /**
      * Creates a new {@code ThrowsTree} object, to represent an {@code @exception} tag.
      * @param name the name of the exception
@@ -269,13 +297,17 @@ public interface DocTreeFactory {
      * Creates a new {@code ReturnTree} object, to represent a {@code @return} tag
      * or {@code {@return}} tag.
      *
+     * @param isInline    {@code true} if this instance is as an inline tag,
+     *                    and {@code false} otherwise
+     * @param description the description of the return value of a method
+     *
+     * @return a {@code ReturnTree} object
+     * @throws UnsupportedOperationException if inline {@code {@return}} tags are
+     *                                       not supported
+     *
      * @implSpec This implementation throws {@code UnsupportedOperationException} if
      * {@code isInline} is {@code true}, and calls {@link #newReturnTree(List)} otherwise.
      *
-     * @param description the description of the return value of a method
-     * @return a {@code ReturnTree} object
-     * @throws UnsupportedOperationException if inline {@code {@return}} tags are
-     *      not supported
      * @since 16
      */
     default ReturnTree newReturnTree(boolean isInline, List<? extends DocTree> description) {
@@ -322,6 +354,24 @@ public interface DocTreeFactory {
      */
     SinceTree newSinceTree(List<? extends DocTree> text);
 
+    /**
+     * Creates a new {@code SnippetTree} object, to represent a {@code {@snippet }} tag.
+     * @param attributes the attributes of the tag
+     * @param text the body of the tag, or {@code null} if the tag has no body (not to be confused with an empty body)
+     * @return a {@code SnippetTree} object
+     * @since 18
+     */
+    SnippetTree newSnippetTree(List<? extends DocTree> attributes, TextTree text);
+
+    /**
+     * Creates a new {@code SpecTree} object, to represent an {@code @spec} tag.
+     * @param url    the url
+     * @param title  the title
+     * @return a {@code SpecTree} object
+     * @since 20
+     */
+    SpecTree newSpecTree(TextTree url, List<? extends DocTree> title);
+
     /**
      * Creates a new {@code StartElementTree} object, to represent the start of an HTML element.
      * @param name the name of the HTML element
@@ -399,6 +449,19 @@ public interface DocTreeFactory {
      */
     ValueTree newValueTree(ReferenceTree ref);
 
+    /**
+     * Creates a new {@code ValueTree} object, to represent a {@code {@value }} tag.
+     * @param format a format string for the value
+     * @param ref a reference to the value
+     * @return a {@code ValueTree} object
+     *
+     * @implSpec This implementation calls {@link #newValueTree(ReferenceTree) newValueTree(ref)}.
+     * @since 20
+     */
+    default ValueTree newValueTree(TextTree format, ReferenceTree ref) {
+        return newValueTree(ref);
+    }
+
     /**
      * Creates a new {@code VersionTree} object, to represent a {@code {@version }} tag.
      * @param text the content of the tag
