@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -40,23 +40,50 @@ import com.sun.jdi.request.*;
 import com.sun.jdi.connect.*;
 
 import java.util.*;
+import java.util.concurrent.CopyOnWriteArrayList;
 import java.io.*;
 
 public class TTY implements EventNotifier {
+    /**
+     * Commands that are repeatable on empty input.
+     */
+    protected static final Set<String> REPEATABLE = Set.of(
+        "up", "down", "step", "stepi", "next", "cont", "list", "pop", "reenter"
+    );
+
+    /**
+     * Commands that reset the default source line to be displayed by {@code list}.
+     */
+    protected static final Set<String> LIST_RESET = Set.of(
+        "run", "suspend", "resume", "up", "down", "kill", "interrupt", "threadgroup", "step", "stepi", "next", "cont",
+        "pop", "reenter"
+    );
+
     EventHandler handler = null;
 
     /**
      * List of Strings to execute at each stop.
      */
-    private List<String> monitorCommands = new ArrayList<String>();
+    private List<String> monitorCommands = new CopyOnWriteArrayList<>();
     private int monitorCount = 0;
 
     /**
      * The name of this tool.
      */
     private static final String progname = "jdb";
+    private static boolean trackVthreads;
 
-    private volatile boolean shuttingDown = false;
+    private volatile boolean shuttingDown;
+
+    /**
+     * The number of the next source line to target for {@code list}, if any.
+     */
+    protected Integer nextListTarget = null;
+
+    /**
+     * Whether to repeat when the user enters an empty command.
+     */
+    protected boolean repeat = false;
 
     public void setShuttingDown(boolean s) {
        shuttingDown = s;
@@ -334,6 +361,7 @@ public class TTY implements EventNotifier {
         {"read",         "y",         "y"},
         {"redefine",     "n",         "n"},
         {"reenter",      "n",         "n"},
+        {"repeat",       "y",         "y"},
         {"resume",       "n",         "n"},
         {"run",          "y",         "n"},
         {"save",         "n",         "n"},
@@ -407,12 +435,15 @@ public class TTY implements EventNotifier {
     };
 
 
-    void executeCommand(StringTokenizer t) {
+    /**
+     * @return the name (first token) of the command processed
+     */
+    String executeCommand(StringTokenizer t) {
         String cmd = t.nextToken().toLowerCase();
+
         // Normally, prompt for the next command after this one is done
         boolean showPrompt = true;
 
-
         /*
          * Anything starting with # is discarded as a no-op or 'comment'.
          */
@@ -425,8 +456,8 @@ public class TTY implements EventNotifier {
                 try {
                     int repeat = Integer.parseInt(cmd);
                     String subcom = t.nextToken("");
-                    while (repeat-- > 0) {
-                        executeCommand(new StringTokenizer(subcom));
+                    for (int r = 0; r < repeat; r += 1) {
+                        cmd = executeCommand(new StringTokenizer(subcom));
                         showPrompt = false; // Bypass the printPrompt() below.
                     }
                 } catch (NumberFormatException exc) {
@@ -434,6 +465,7 @@ public class TTY implements EventNotifier {
                 }
             } else {
                 int commandNumber = isCommand(cmd);
+
                 /*
                  * Check for an unknown command
                  */
@@ -447,7 +479,6 @@ public class TTY implements EventNotifier {
                     MessageOutput.println("Command is not supported on a read-only VM connection",
                                           cmd);
                 } else {
-
                     Commands evaluator = new Commands();
                     try {
                         if (cmd.equals("print")) {
@@ -534,7 +565,7 @@ public class TTY implements EventNotifier {
                              * arg 2 is false).
                              */
                             if ((handler == null) && Env.connection().isOpen()) {
-                                handler = new EventHandler(this, false);
+                                handler = new EventHandler(this, false, trackVthreads);
                             }
                         } else if (cmd.equals("memory")) {
                             evaluator.commandMemory();
@@ -549,7 +580,7 @@ public class TTY implements EventNotifier {
                         } else if (cmd.equals("unwatch")) {
                             evaluator.commandUnwatch(t);
                         } else if (cmd.equals("list")) {
-                            evaluator.commandList(t);
+                            nextListTarget = evaluator.commandList(t, repeat ? nextListTarget : null);
                         } else if (cmd.equals("lines")) { // Undocumented command: useful for testing.
                             evaluator.commandLines(t);
                         } else if (cmd.equals("classpath")) {
@@ -595,6 +626,8 @@ public class TTY implements EventNotifier {
                         } else if (cmd.equals("version")) {
                             evaluator.commandVersion(progname,
                                                      Bootstrap.virtualMachineManager());
+                        } else if (cmd.equals("repeat")) {
+                            doRepeat(t);
                         } else if (cmd.equals("quit") || cmd.equals("exit")) {
                             if (handler != null) {
                                 handler.shutdown();
@@ -619,6 +652,12 @@ public class TTY implements EventNotifier {
         if (showPrompt) {
             MessageOutput.printPrompt();
         }
+
+        if (LIST_RESET.contains(cmd)) {
+            nextListTarget = null;
+        }
+
+        return cmd;
     }
 
     /*
@@ -660,7 +699,6 @@ public class TTY implements EventNotifier {
         }
     }
 
-
     void readCommand(StringTokenizer t) {
         if (t.hasMoreTokens()) {
             String cmdfname = t.nextToken();
@@ -672,6 +710,19 @@ public class TTY implements EventNotifier {
         }
     }
 
+    protected void doRepeat(StringTokenizer t) {
+        if (t.hasMoreTokens()) {
+            var choice = t.nextToken().toLowerCase();
+            if ((choice.equals("on") || choice.equals("off")) && !t.hasMoreTokens()) {
+                repeat = choice.equals("on");
+            } else {
+                MessageOutput.println("repeat usage");
+            }
+        } else {
+            MessageOutput.println(repeat ? "repeat is on" : "repeat is off");
+        }
+    }
+
     /**
      * Read and execute a command file.  Return true if the file was read
      * else false;
@@ -742,14 +793,12 @@ public class TTY implements EventNotifier {
              * immediately, telling it (through arg 2) to stop on the
              * VM start event.
              */
-            this.handler = new EventHandler(this, true);
+            this.handler = new EventHandler(this, true, trackVthreads);
         }
         try {
             BufferedReader in =
                     new BufferedReader(new InputStreamReader(System.in));
 
-            String lastLine = null;
-
             Thread.currentThread().setPriority(Thread.NORM_PRIORITY);
 
             /*
@@ -787,6 +836,9 @@ public class TTY implements EventNotifier {
 
             // Process interactive commands.
             MessageOutput.printPrompt();
+
+            String lastLine = null;
+            String lastCommandName = null;
             while (true) {
                 String ln = in.readLine();
                 if (ln == null) {
@@ -808,7 +860,11 @@ public class TTY implements EventNotifier {
                 StringTokenizer t = new StringTokenizer(ln);
                 if (t.hasMoreTokens()) {
                     lastLine = ln;
-                    executeCommand(t);
+                    lastCommandName = executeCommand(t);
+                } else if (repeat && lastLine != null && REPEATABLE.contains(lastCommandName)) {
+                    // We want list auto-advance even if the user started with a listing target.
+                    String newCommand = lastCommandName.equals("list") && nextListTarget != null ? "list" : lastLine;
+                    executeCommand(new StringTokenizer(newCommand));
                 } else {
                     MessageOutput.printPrompt();
                 }
@@ -917,6 +973,8 @@ public class TTY implements EventNotifier {
                         return;
                     }
                 }
+            } else if (token.equals("-trackallthreads")) {
+                trackVthreads = true;
             } else if (token.equals("-X")) {
                 usageError("Use java minus X to see");
                 return;
@@ -938,6 +996,8 @@ public class TTY implements EventNotifier {
                    token.startsWith("-ss") || token.startsWith("-oss") ) {
 
                 javaArgs = addArgument(javaArgs, token);
+            } else if (token.startsWith("-R")) {
+                javaArgs = addArgument(javaArgs, token.substring(2));
             } else if (token.equals("-tclassic")) {
                 usageError("Classic VM no longer supported.");
                 return;
@@ -1068,8 +1128,8 @@ public class TTY implements EventNotifier {
         /*
          * Here are examples of jdb command lines and how the options
          * are interpreted as arguments to the program being debugged.
-         * arg1       arg2
-         * ----       ----
+         *                     arg1       arg2
+         *                     ----       ----
          * jdb hello a b       a          b
          * jdb hello "a b"     a b
          * jdb hello a,b       a,b
@@ -1106,14 +1166,14 @@ public class TTY implements EventNotifier {
                            connectSpec);
                 return;
             }
-            connectSpec += "options=" + javaArgs + ",";
+        }
+
+        if (connectSpec.startsWith("com.sun.jdi.CommandLineLaunch:") && trackVthreads) {
+            connectSpec += "includevirtualthreads=y,";
         }
 
         try {
-            if (! connectSpec.endsWith(",")) {
-                connectSpec += ","; // (Bug ID 4285874)
-            }
-            Env.init(connectSpec, launchImmediately, traceFlags);
+            Env.init(connectSpec, launchImmediately, traceFlags, trackVthreads, javaArgs);
             new TTY();
         } catch(Exception e) {
             MessageOutput.printException("Internal exception:", e);
