@@ -389,6 +389,11 @@ public class Flags {
      */
     public static final long SEALED = 1L<<62; // ClassSymbols
 
+    /**
+     * Flag to indicate type annotations have been queued for field initializers.
+     */
+    public static final long FIELD_INIT_TYPE_ANNOTATIONS_QUEUED = 1L<<53; // VarSymbols
+
     /**
      * Flag to indicate that the class/interface was declared with the non-sealed modifier.
      */
