@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -54,6 +54,11 @@ public class UnicodeReader {
      */
     private final int length;
 
+    /**
+     * Virtual position offset in the original buffer.
+     */
+    private final int offset;
+
     /**
      * Character buffer index of character currently being observed.
      */
@@ -85,6 +90,11 @@ public class UnicodeReader {
      */
     private boolean wasBackslash;
 
+    /**
+     * true if the last character was derived from an unicode escape sequence.
+     */
+    private boolean wasUnicodeEscape;
+
     /**
      * Log for error reporting.
      */
@@ -97,19 +107,55 @@ public class UnicodeReader {
      * @param array   array containing contents of source.
      * @param length  length of meaningful content in buffer.
      */
+    @SuppressWarnings("this-escape")
     protected UnicodeReader(ScannerFactory sf, char[] array, int length) {
+        this(sf.log, array, length);
+    }
+
+    /**
+     * Constructor.
+     *
+     * @param log     Log for error reporting.
+     * @param array   array containing contents of source.
+     * @param length  length of meaningful content in buffer.
+     */
+    protected UnicodeReader(Log log, char[] array, int length) {
+        this(log, array, 0, 0, length);
+    }
+
+    /**
+      * Constructor.
+      *
+      * @param log     Log for error reporting.
+      * @param array   array containing contents of source.
+      * @param pos     start of meaningful content in buffer.
+      * @param endPos  end of meaningful content in buffer.
+      */
+    @SuppressWarnings("this-escape")
+    protected UnicodeReader(Log log, char[] array, int offset, int pos, int endPos) {
         this.buffer = array;
-        this.length = length;
-        this.position = 0;
+        this.length = endPos;
+        this.offset = offset;
+        this.position = pos;
         this.width = 0;
         this.character = '\0';
         this.codepoint = 0;
         this.wasBackslash = false;
-        this.log = sf.log;
+        this.wasUnicodeEscape = false;
+        this.log = log;
 
         nextCodePoint();
     }
 
+    /**
+     * Returns the character buffer.
+     *
+     * @return character buffer.
+     */
+    protected char[] buffer() {
+        return buffer;
+    }
+
     /**
      * Returns the length of the buffer. This is length of meaningful content in buffer and
      * not the length of the buffer array.
@@ -161,13 +207,22 @@ public class UnicodeReader {
         // Fetch next character.
         nextCodeUnit();
 
-        // If second backslash is detected.
-        if (wasBackslash) {
-            // Treat like a normal character (not part of unicode escape.)
+        if (character == '\\' && (!wasBackslash || wasUnicodeEscape)) {
+            // Is a backslash and may be an unicode escape.
+            switch (unicodeEscape()) {
+                case BACKSLASH -> {
+                    wasUnicodeEscape = false;
+                    wasBackslash = !wasBackslash;
+                }
+                case VALID_ESCAPE -> {
+                    wasUnicodeEscape = true;
+                    wasBackslash = character == '\\' && !wasBackslash;
+                }
+                case BROKEN_ESCAPE -> nextUnicodeInputCharacter(); //skip broken unicode escapes
+            }
+        } else {
             wasBackslash = false;
-        } else if (character == '\\') {
-            // May be an unicode escape.
-            wasBackslash = !unicodeEscape();
+            wasUnicodeEscape = false;
         }
 
         // Codepoint and character match if not surrogate.
@@ -218,7 +273,7 @@ public class UnicodeReader {
      *
      * @return true if was an unicode escape.
      */
-    private boolean unicodeEscape() {
+    private UnicodeEscapeResult unicodeEscape() {
         // Start of unicode escape (past backslash.)
         int start = position + width;
 
@@ -236,7 +291,7 @@ public class UnicodeReader {
 
         // Needs to have been at least one u.
         if (index == start) {
-            return false;
+            return UnicodeEscapeResult.BACKSLASH;
         }
 
         int code = 0;
@@ -261,33 +316,39 @@ public class UnicodeReader {
         // If all digits are good.
         if (code >= 0) {
             character = (char)code;
+            return UnicodeEscapeResult.VALID_ESCAPE;
         } else {
-            log.error(position, Errors.IllegalUnicodeEsc);
+            log.error(index, Errors.IllegalUnicodeEsc);
+            return UnicodeEscapeResult.BROKEN_ESCAPE;
         }
+    }
 
-        // Return true even if error so that the invalid unicode escape is skipped.
-        return true;
+    private enum UnicodeEscapeResult {
+        BACKSLASH,
+        VALID_ESCAPE,
+        BROKEN_ESCAPE;
     }
 
     /**
-     * Return the current position in the character buffer.
+     * Return the virtual position in the character buffer.
      *
-     * @return  current position in the character buffer.
+     * @return  virtual position in the character buffer.
      */
     protected int position() {
-        return position;
+        return offset + position;
     }
 
 
     /**
-     * Reset the reader to the specified position.
+     * Reset the reader to the specified virtual position.
      * Warning: Do not use when previous character was an ASCII or unicode backslash.
      * @param pos
      */
     protected void reset(int pos) {
-        position = pos;
+        position = pos - offset;
         width = 0;
         wasBackslash = false;
+        wasUnicodeEscape = false;
         nextCodePoint();
     }
 
@@ -358,6 +419,9 @@ public class UnicodeReader {
     protected boolean isOneOf(char ch1, char ch2, char ch3) {
         return is(ch1) || is(ch2) || is(ch3);
     }
+    protected boolean isOneOf(char ch1, char ch2, char ch3, char ch4) {
+        return is(ch1) || is(ch2) || is(ch3) || is(ch4);
+    }
     protected boolean isOneOf(char ch1, char ch2, char ch3, char ch4, char ch5, char ch6) {
         return is(ch1) || is(ch2) || is(ch3) || is(ch4) || is(ch5) || is(ch6);
     }
@@ -404,25 +468,68 @@ public class UnicodeReader {
         return false;
     }
 
+    /**
+     * Match one of the arguments and advance if a match. Returns true if a match.
+     */
     protected boolean acceptOneOf(char ch1, char ch2, char ch3) {
         if (isOneOf(ch1, ch2, ch3)) {
             next();
 
             return true;
         }
-
         return false;
     }
 
     /**
-     * Skip over all occurances of character.
+     * Return a reader which is bracketed by the currect position
+     * and the next line terminator.
+     *
+     * @return a new reader
+     */
+    protected UnicodeReader lineReader() {
+        int pos = position;
+        skipToEOLN();
+        int endPos = position;
+        accept('\r');
+        accept('\n');
+        return new UnicodeReader(log, buffer, offset, pos, endPos);
+    }
+
+    /**
+     * Return a reader which is bracketed by the {@code pos}
+     * and {@code endPos}.
+     *
+     * @param pos     initial position
+     * @param endPos  end position
+     *
+     * @return a new reader
+     */
+    protected UnicodeReader lineReader(int pos, int endPos) {
+        return new UnicodeReader(log, buffer, offset, pos - offset, endPos - offset);
+    }
+
+    /**
+     * Skip over all occurrences of character.
      *
      * @param ch character to accept.
+     *
+     * @return number of characters skipped
      */
-    protected void skip(char ch) {
+    protected int skip(char ch) {
+        int count = 0;
         while (accept(ch)) {
-            // next
+            count++;
         }
+        return count;
+    }
+
+    /**
+     * Is ASCII white space character.
+     *
+     * @return true if is ASCII white space character
+     */
+    protected boolean isWhitespace() {
+        return isOneOf(' ', '\t', '\f');
     }
 
     /**
@@ -434,18 +541,26 @@ public class UnicodeReader {
         }
     }
 
+    /**
+     * Is ASCII line terminator.
+     *
+     * @return true if is ASCII line terminator.
+     */
+    protected boolean isEOLN() {
+        return isOneOf('\r', '\n');
+    }
+
     /**
      * Skip to end of line.
      */
     protected void skipToEOLN() {
         while (isAvailable()) {
-            if (isOneOf('\r', '\n')) {
+            if (isEOLN()) {
                 break;
             }
 
             next();
         }
-
     }
 
     /**
@@ -464,7 +579,7 @@ public class UnicodeReader {
         }
 
         // Be prepared to retreat if not a match.
-        int savedPosition = position;
+        int savedPosition = position();
 
         nextCodePoint();
 
@@ -543,6 +658,41 @@ public class UnicodeReader {
         return Arrays.copyOfRange(buffer, beginIndex, endIndex);
     }
 
+    /**
+     * Returns a string subset of the input buffer.
+     * The returned string begins at the {@code beginIndex} and
+     * extends to the character at index {@code endIndex - 1}.
+     * Thus the length of the substring is {@code endIndex-beginIndex}.
+     * This behavior is like
+     * {@code String.substring(beginIndex, endIndex)}.
+     * Unicode escape sequences are not translated.
+     *
+     * @param  beginIndex the beginning index, inclusive.
+     * @param  endIndex the ending index, exclusive.
+     *
+     * @throws ArrayIndexOutOfBoundsException if either offset is outside of the
+     *         array bounds
+     */
+    public String getRawString(int beginIndex, int endIndex) {
+        return new String(buffer, beginIndex, endIndex - beginIndex);
+    }
+
+    /**
+     * Returns a string subset of the input buffer.
+     * The returned string begins at the {@code position} and
+     * extends to the character at index {@code length - 1}.
+     * Thus the length of the substring is {@code length-position}.
+     * This behavior is like
+     * {@code String.substring(position, length)}.
+     * Unicode escape sequences are not translated.
+     *
+     * @throws ArrayIndexOutOfBoundsException if either offset is outside of the
+     *         array bounds
+     */
+    public String getRawString() {
+        return getRawString(position, length);
+    }
+
     /**
      * This is a specialized version of UnicodeReader that keeps track of the
      * column position within a given character stream. Used for Javadoc
@@ -550,11 +700,6 @@ public class UnicodeReader {
      * to positions in the source file.
      */
     static class PositionTrackingReader extends UnicodeReader {
-        /**
-         * Offset from the beginning of the original reader buffer.
-         */
-        private final int offset;
-
         /**
          * Current column in the comment.
          */
@@ -563,13 +708,12 @@ public class UnicodeReader {
         /**
          * Constructor.
          *
-         * @param sf      Scan factory.
-         * @param array   Array containing contents of source.
-         * @param offset  Position offset in original source buffer.
+         * @param reader  existing reader
+         * @param pos     start of meaningful content in buffer.
+         * @param endPos  end of meaningful content in buffer.
          */
-        protected PositionTrackingReader(ScannerFactory sf, char[] array, int offset) {
-            super(sf, array, array.length);
-            this.offset = offset;
+        protected PositionTrackingReader(UnicodeReader reader, int pos, int endPos) {
+            super(reader.log, reader.getRawCharacters(pos, endPos), reader.offset + pos, 0, endPos - pos);
             this.column = 0;
         }
 
@@ -601,15 +745,6 @@ public class UnicodeReader {
         protected int column() {
             return column;
         }
-
-        /**
-         * Returns position relative to the original source buffer.
-         *
-         * @return
-         */
-        protected int offsetPosition() {
-            return position() + offset;
-        }
     }
 
 }
