@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1999, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -76,6 +76,12 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
         out.write(Convert.escapeUnicode(s.toString()));
     }
 
+    /** Print character. Should be only used internally for known ASCII characters.
+     */
+    private void print(char c) throws IOException {
+        out.write(c);
+    }
+
     /**
      * Print list.
      */
@@ -88,7 +94,7 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     /**
      * Print list with separators.
      */
-    protected void print(List<? extends DocTree> list, String sep) throws IOException {
+    private void print(List<? extends DocTree> list, char sep) throws IOException {
         if (list.isEmpty())
             return;
         boolean first = true;
@@ -107,7 +113,7 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     }
 
     protected void printTagName(DocTree node) throws IOException {
-        out.write("@");
+        out.write('@');
         out.write(node.getKind().tagName);
     }
 
@@ -121,25 +127,15 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     public Void visitAttribute(AttributeTree node, Void p) {
         try {
             print(node.getName());
-            String quote;
-            switch (node.getValueKind()) {
-                case EMPTY:
-                    quote = null;
-                    break;
-                case UNQUOTED:
-                    quote = "";
-                    break;
-                case SINGLE:
-                    quote = "'";
-                    break;
-                case DOUBLE:
-                    quote = "\"";
-                    break;
-                default:
-                    throw new AssertionError();
-            }
+            String quote = switch (node.getValueKind()) {
+                case EMPTY -> null;
+                case UNQUOTED -> "";
+                case SINGLE -> "'";
+                case DOUBLE -> "\"";
+            };
             if (quote != null) {
-                print("=" + quote);
+                print('=');
+                print(quote);
                 print(node.getValue());
                 print(quote);
             }
@@ -153,7 +149,7 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     public Void visitAuthor(AuthorTree node, Void p) {
         try {
             printTagName(node);
-            print(" ");
+            print(' ');
             print(node.getName());
         } catch (IOException e) {
             throw new UncheckedIOException(e);
@@ -176,7 +172,7 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
         try {
             printTagName(node);
             if (!node.getBody().isEmpty()) {
-                print(" ");
+                print(' ');
                 print(node.getBody());
             }
         } catch (IOException e) {
@@ -192,8 +188,8 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
             List<? extends DocTree> t = node.getBlockTags();
             print(b);
             if (!b.isEmpty() && !t.isEmpty())
-                print("\n");
-            print(t, "\n");
+                print('\n');
+            print(t, '\n');
         } catch (IOException e) {
             throw new UncheckedIOException(e);
         }
@@ -203,9 +199,9 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitDocRoot(DocRootTree node, Void p) {
         try {
-            print("{");
+            print('{');
             printTagName(node);
-            print("}");
+            print('}');
         } catch (IOException e) {
             throw new UncheckedIOException(e);
         }
@@ -227,7 +223,7 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
         try {
             print("</");
             print(node.getName());
-            print(">");
+            print('>');
         } catch (IOException e) {
             throw new UncheckedIOException(e);
         }
@@ -237,9 +233,9 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitEntity(EntityTree node, Void p) {
         try {
-            print("&");
+            print('&');
             print(node.getName());
-            print(";");
+            print(';');
         } catch (IOException e) {
             throw new UncheckedIOException(e);
         }
@@ -256,12 +252,23 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
         return null;
     }
 
+    @Override @DefinedBy(Api.COMPILER_TREE)
+    public Void visitEscape(EscapeTree node, Void p) {
+        try {
+            out.write('@');
+            print(node.getBody());
+        } catch (IOException e) {
+            throw new UncheckedIOException(e);
+        }
+        return null;
+    }
+
     @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitHidden(HiddenTree node, Void p) {
         try {
             printTagName(node);
             if (!node.getBody().isEmpty()) {
-                print(" ");
+                print(' ');
                 print(node.getBody());
             }
         } catch (IOException e) {
@@ -283,15 +290,15 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitIndex(IndexTree node, Void p) {
         try {
-            print("{");
+            print('{');
             printTagName(node);
-            print(" ");
+            print(' ');
             print(node.getSearchTerm());
             if (!node.getDescription().isEmpty()) {
-                print(" ");
+                print(' ');
                 print(node.getDescription());
             }
-            print("}");
+            print('}');
         } catch (IOException e) {
             throw new UncheckedIOException(e);
         }
@@ -301,9 +308,9 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitInheritDoc(InheritDocTree node, Void p) {
         try {
-            print("{");
+            print('{');
             printTagName(node);
-            print("}");
+            print('}');
         } catch (IOException e) {
             throw new UncheckedIOException(e);
         }
@@ -313,15 +320,15 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitLink(LinkTree node, Void p) {
         try {
-            print("{");
+            print('{');
             printTagName(node);
-            print(" ");
+            print(' ');
             print(node.getReference());
             if (!node.getLabel().isEmpty()) {
-                print(" ");
+                print(' ');
                 print(node.getLabel());
             }
-            print("}");
+            print('}');
         } catch (IOException e) {
             throw new UncheckedIOException(e);
         }
@@ -331,14 +338,14 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitLiteral(LiteralTree node, Void p) {
         try {
-            print("{");
+            print('{');
             printTagName(node);
             String body = node.getBody().getBody();
             if (!body.isEmpty() && !Character.isWhitespace(body.charAt(0))) {
-                print(" ");
+                print(' ');
             }
             print(node.getBody());
-            print("}");
+            print('}');
         } catch (IOException e) {
             throw new UncheckedIOException(e);
         }
@@ -349,12 +356,12 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     public Void visitParam(ParamTree node, Void p) {
         try {
             printTagName(node);
-            print(" ");
-            if (node.isTypeParameter()) print("<");
+            print(' ');
+            if (node.isTypeParameter()) print('<');
             print(node.getName());
-            if (node.isTypeParameter()) print(">");
+            if (node.isTypeParameter()) print('>');
             if (!node.getDescription().isEmpty()) {
-                print(" ");
+                print(' ');
                 print(node.getDescription());
             }
         } catch (IOException e) {
@@ -367,10 +374,10 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     public Void visitProvides(ProvidesTree node, Void p) {
         try {
             printTagName(node);
-            print(" ");
+            print(' ');
             print(node.getServiceType());
             if (!node.getDescription().isEmpty()) {
-                print(" ");
+                print(' ');
                 print(node.getDescription());
             }
         } catch (IOException e) {
@@ -393,13 +400,13 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     public Void visitReturn(ReturnTree node, Void p) {
         try {
             if (node.isInline()) {
-                print("{");
+                print('{');
             }
             printTagName(node);
-            print(" ");
+            print(' ');
             print(node.getDescription());
             if (node.isInline()) {
-                print("}");
+                print('}');
             }
         } catch (IOException e) {
             throw new UncheckedIOException(e);
@@ -414,7 +421,7 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
             boolean first = true;
             boolean needSep = true;
             for (DocTree t: node.getReference()) {
-                if (needSep) print(" ");
+                if (needSep) print(' ');
                 needSep = (first && (t instanceof ReferenceTree));
                 first = false;
                 print(t);
@@ -430,7 +437,7 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
         try {
             printTagName(node);
             if (!node.getDescription().isEmpty()) {
-                print(" ");
+                print(' ');
                 print(node.getDescription());
             }
         } catch (IOException e) {
@@ -444,7 +451,7 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
         try {
             printTagName(node);
             if (!node.getDescription().isEmpty()) {
-                print(" ");
+                print(' ');
                 print(node.getDescription());
             }
         } catch (IOException e) {
@@ -457,12 +464,12 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     public Void visitSerialField(SerialFieldTree node, Void p) {
         try {
             printTagName(node);
-            print(" ");
+            print(' ');
             print(node.getName());
-            print(" ");
+            print(' ');
             print(node.getType());
             if (!node.getDescription().isEmpty()) {
-                print(" ");
+                print(' ');
                 print(node.getDescription());
             }
         } catch (IOException e) {
@@ -475,7 +482,7 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     public Void visitSince(SinceTree node, Void p) {
         try {
             printTagName(node);
-            print(" ");
+            print(' ');
             print(node.getBody());
         } catch (IOException e) {
             throw new UncheckedIOException(e);
@@ -486,18 +493,18 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitSnippet(SnippetTree node, Void p) {
         try {
-            print("{");
+            print('{');
             printTagName(node);
             List<? extends DocTree> attrs = node.getAttributes();
             if (!attrs.isEmpty()) {
-                print(" ");
-                print(attrs, " ");
+                print(' ');
+                print(attrs, ' ');
             }
             if (node.getBody() != null) {
                 print(" :\n");
                 print(node.getBody());
             }
-            print("}");
+            print('}');
         } catch (IOException e) {
             throw new UncheckedIOException(e);
         }
@@ -508,9 +515,9 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     public Void visitSpec(SpecTree node, Void p) {
         try {
             printTagName(node);
-            print(" ");
+            print(' ');
             print(node.getURL());
-            print(" ");
+            print(' ');
             print(node.getTitle());
         } catch (IOException e) {
             throw new UncheckedIOException(e);
@@ -521,20 +528,20 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitStartElement(StartElementTree node, Void p) {
         try {
-            print("<");
+            print('<');
             print(node.getName());
             List<? extends DocTree> attrs = node.getAttributes();
             if (!attrs.isEmpty()) {
-                print(" ");
-                print(attrs, " ");
+                print(' ');
+                print(attrs, ' ');
                 DocTree last = node.getAttributes().get(attrs.size() - 1);
                 if (node.isSelfClosing() && last instanceof AttributeTree attributeTree
                         && attributeTree.getValueKind() == ValueKind.UNQUOTED)
-                    print(" ");
+                    print(' ');
             }
             if (node.isSelfClosing())
-                print("/");
-            print(">");
+                print('/');
+            print('>');
         } catch (IOException e) {
             throw new UncheckedIOException(e);
         }
@@ -544,13 +551,13 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitSummary(SummaryTree node, Void p) {
         try {
-            print("{");
+            print('{');
             printTagName(node);
             if (!node.getSummary().isEmpty()) {
-                print(" ");
+                print(' ');
                 print(node.getSummary());
             }
-            print("}");
+            print('}');
         } catch (IOException e) {
             throw new UncheckedIOException(e);
         }
@@ -560,11 +567,11 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitSystemProperty(SystemPropertyTree node, Void p) {
         try {
-            print("{");
+            print('{');
             printTagName(node);
-            print(" ");
+            print(' ');
             print(node.getPropertyName());
-            print("}");
+            print('}');
         } catch (IOException e) {
             throw new UncheckedIOException(e);
         }
@@ -585,10 +592,10 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     public Void visitThrows(ThrowsTree node, Void p) {
         try {
             printTagName(node);
-            print(" ");
+            print(' ');
             print(node.getExceptionName());
             if (!node.getDescription().isEmpty()) {
-                print(" ");
+                print(' ');
                 print(node.getDescription());
             }
         } catch (IOException e) {
@@ -600,9 +607,9 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitUnknownBlockTag(UnknownBlockTagTree node, Void p) {
         try {
-            print("@");
+            print('@');
             print(node.getTagName());
-            print(" ");
+            print(' ');
             print(node.getContent());
         } catch (IOException e) {
             throw new UncheckedIOException(e);
@@ -613,12 +620,12 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitUnknownInlineTag(UnknownInlineTagTree node, Void p) {
         try {
-            print("{");
-            print("@");
+            print('{');
+            print('@');
             print(node.getTagName());
-            print(" ");
+            print(' ');
             print(node.getContent());
-            print("}");
+            print('}');
         } catch (IOException e) {
             throw new UncheckedIOException(e);
         }
@@ -629,10 +636,10 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     public Void visitUses(UsesTree node, Void p) {
         try {
             printTagName(node);
-            print(" ");
+            print(' ');
             print(node.getServiceType());
             if (!node.getDescription().isEmpty()) {
-                print(" ");
+                print(' ');
                 print(node.getDescription());
             }
         } catch (IOException e) {
@@ -644,17 +651,17 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitValue(ValueTree node, Void p) {
         try {
-            print("{");
+            print('{');
             printTagName(node);
             if (node.getFormat() != null) {
-                print(" ");
+                print(' ');
                 print(node.getFormat());
             }
             if (node.getReference() != null) {
-                print(" ");
+                print(' ');
                 print(node.getReference());
             }
-            print("}");
+            print('}');
         } catch (IOException e) {
             throw new UncheckedIOException(e);
         }
@@ -665,7 +672,7 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     public Void visitVersion(VersionTree node, Void p) {
         try {
             printTagName(node);
-            print(" ");
+            print(' ');
             print(node.getBody());
         } catch (IOException e) {
             throw new UncheckedIOException(e);
@@ -676,7 +683,9 @@ public class DocPretty implements DocTreeVisitor<Void,Void> {
     @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitOther(DocTree node, Void p) {
         try {
-            print("(UNKNOWN: " + node + ")");
+            print("(UNKNOWN: ");
+            print(node);
+            print(')');
             println();
         } catch (IOException e) {
             throw new UncheckedIOException(e);
