@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -58,13 +58,6 @@ import java.util.zip.ZipFile;
  * a .class file, a directory, or a JAR file.
  */
 public class ClassFileReader implements Closeable {
-    /**
-     * Returns a ClassFileReader instance of a given path.
-     */
-    public static ClassFileReader newInstance(Path path) throws IOException {
-        return newInstance(path, null);
-    }
-
     /**
      * Returns a ClassFileReader instance of a given path.
      */
@@ -267,8 +260,8 @@ public class ClassFileReader implements Closeable {
             DirectoryIterator() throws IOException {
                 List<Path> paths = null;
                 try (Stream<Path> stream = Files.walk(path, Integer.MAX_VALUE)) {
-                    paths = stream.filter(ClassFileReader::isClass)
-                                  .collect(Collectors.toList());
+                    paths = stream.filter(ClassFileReader::isClass).toList();
+
                 }
                 this.entries = paths;
                 this.index = 0;
@@ -363,7 +356,8 @@ public class ClassFileReader implements Closeable {
         protected ClassFile readClassFile(JarFile jarfile, JarEntry e) throws IOException {
             try (InputStream is = jarfile.getInputStream(e)) {
                 ClassFile cf = ClassFile.read(is);
-                if (jarfile.isMultiRelease()) {
+                // exclude module-info.class since this jarFile is on classpath
+                if (jarfile.isMultiRelease() && !cf.getName().equals("module-info")) {
                     VersionHelper.add(jarfile, e, cf);
                 }
                 return cf;
@@ -444,5 +438,4 @@ public class ClassFileReader implements Closeable {
             throw new UnsupportedOperationException("Not supported yet.");
         }
     }
-    private static final String MODULE_INFO = "module-info.class";
 }
