@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -52,13 +52,14 @@ public class VirtualMachineManagerImpl implements VirtualMachineManagerService {
     private final ThreadGroup mainGroupForJDI;
     private ResourceBundle messages = null;
     private int vmSequenceNumber = 0;
-    private static final int majorVersion = 11;
+    private static final int majorVersion = Runtime.version().feature();
     private static final int minorVersion = 0;
 
     private static final Object lock = new Object();
     private static VirtualMachineManagerImpl vmm;
 
     public static VirtualMachineManager virtualMachineManager() {
+        @SuppressWarnings("removal")
         SecurityManager sm = System.getSecurityManager();
         if (sm != null) {
             JDIPermission vmmPermission =
@@ -98,12 +99,7 @@ public class VirtualMachineManagerImpl implements VirtualMachineManagerService {
 
             try {
                 connector = connectors.next();
-            } catch (ThreadDeath x) {
-                throw x;
-            } catch (Exception x) {
-                System.err.println(x);
-                continue;
-            } catch (Error x) {
+            } catch (Exception | Error x) {
                 System.err.println(x);
                 continue;
             }
@@ -127,12 +123,7 @@ public class VirtualMachineManagerImpl implements VirtualMachineManagerService {
 
             try {
                 transportService = transportServices.next();
-            } catch (ThreadDeath x) {
-                throw x;
-            } catch (Exception x) {
-                System.err.println(x);
-                continue;
-            } catch (Error x) {
+            } catch (Exception | Error x) {
                 System.err.println(x);
                 continue;
             }
