@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,6 +25,8 @@
 
 package java.awt.color;
 
+import java.io.Serial;
+
 /**
  * This exception is thrown when an error occurs in accessing or processing an
  * {@code ICC_Profile} object.
@@ -34,6 +36,7 @@ public class ProfileDataException extends RuntimeException {
     /**
      * Use serialVersionUID from JDK 1.2 for interoperability.
      */
+    @Serial
     private static final long serialVersionUID = 7286140888240322498L;
 
     /**
