@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -594,12 +594,12 @@ public class BufferedImage extends java.awt.Image
      *                  the raster has been premultiplied with alpha.
      * @param properties {@code Hashtable} of
      *                  {@code String}/{@code Object} pairs.
-     * @exception RasterFormatException if the number and
+     * @throws RasterFormatException if the number and
      * types of bands in the {@code SampleModel} of the
      * {@code Raster} do not match the number and types required by
      * the {@code ColorModel} to represent its color and alpha
      * components.
-     * @exception IllegalArgumentException if
+     * @throws IllegalArgumentException if
      *          {@code raster} is incompatible with {@code cm}
      * @see ColorModel
      * @see Raster
@@ -800,6 +800,7 @@ public class BufferedImage extends java.awt.Image
         }   // else if ((raster instanceof ByteComponentRaster) &&
     }
 
+    @SuppressWarnings("removal")
     private static boolean isStandard(ColorModel cm, WritableRaster wr) {
         final Class<? extends ColorModel> cmClass = cm.getClass();
         final Class<? extends WritableRaster> wrClass = wr.getClass();
@@ -1194,7 +1195,7 @@ public class BufferedImage extends java.awt.Image
      * @param h the height of the specified rectangular region
      * @return a {@code BufferedImage} that is the subimage of this
      *          {@code BufferedImage}.
-     * @exception RasterFormatException if the specified
+     * @throws RasterFormatException if the specified
      * area is not contained within this {@code BufferedImage}.
      */
     public BufferedImage getSubimage (int x, int y, int w, int h) {
@@ -1367,7 +1368,7 @@ public class BufferedImage extends java.awt.Image
      * @return the x offset of the tile grid.
      */
     public int getTileGridXOffset() {
-        return raster.getSampleModelTranslateX();
+        return raster.getMinX();
     }
 
     /**
@@ -1377,7 +1378,7 @@ public class BufferedImage extends java.awt.Image
      * @return the y offset of the tile grid.
      */
     public int getTileGridYOffset() {
-        return raster.getSampleModelTranslateY();
+        return raster.getMinY();
     }
 
     /**
@@ -1390,7 +1391,7 @@ public class BufferedImage extends java.awt.Image
      * @param tileY the y index of the requested tile in the tile array
      * @return a {@code Raster} that is the tile defined by the
      *          arguments {@code tileX} and {@code tileY}.
-     * @exception ArrayIndexOutOfBoundsException if both
+     * @throws ArrayIndexOutOfBoundsException if both
      *          {@code tileX} and {@code tileY} are not
      *          equal to 0
      */
@@ -1539,6 +1540,12 @@ public class BufferedImage extends java.awt.Image
   /**
    * Adds a tile observer.  If the observer is already present,
    * it receives multiple notifications.
+   * <p>
+   * This method ignores its parameters and does nothing,
+   * since {@code BufferedImage} is always checked out
+   * for writing and cannot be made read-only,
+   * so there can never be events to dispatch.
+   *
    * @param to the specified {@link TileObserver}
    */
     public void addTileObserver (TileObserver to) {
@@ -1548,6 +1555,10 @@ public class BufferedImage extends java.awt.Image
    * Removes a tile observer.  If the observer was not registered,
    * nothing happens.  If the observer was registered for multiple
    * notifications, it is now registered for one fewer notification.
+   * <p>
+   * This method ignores the given observer,
+   * since {@link #addTileObserver(TileObserver)} adds none.
+   *
    * @param to the specified {@code TileObserver}.
    */
     public void removeTileObserver (TileObserver to) {
@@ -1555,13 +1566,18 @@ public class BufferedImage extends java.awt.Image
 
     /**
      * Returns whether or not a tile is currently checked out for writing.
+     * The only tile in a {@code BufferedImage} is at (0,0) and it is always
+     * writable, so calling this method with (0,0) will always return
+     * {@code true}, and any other coordinate will cause an exception
+     * to be thrown.
+     *
      * @param tileX the x index of the tile.
      * @param tileY the y index of the tile.
      * @return {@code true} if the tile specified by the specified
      *          indices is checked out for writing; {@code false}
      *          otherwise.
-     * @exception ArrayIndexOutOfBoundsException if both
-     *          {@code tileX} and {@code tileY} are not equal
+     * @throws IllegalArgumentException if either
+     *          {@code tileX} or {@code tileY} is not equal
      *          to 0
      */
     public boolean isTileWritable (int tileX, int tileY) {
@@ -1575,13 +1591,23 @@ public class BufferedImage extends java.awt.Image
      * Returns an array of {@link Point} objects indicating which tiles
      * are checked out for writing.  Returns {@code null} if none are
      * checked out.
+     * <p>
+     * Since a {@code BufferedImage} consists of a single tile,
+     * and that tile is always checked out for writing,
+     * this method returns an array of one point.
+     * Further, the offset shall be consistent with
+     * {@link #getMinTileX()} and {@link #getMinTileY()},
+     * which are always (0,0) in {@code BufferedImage}.
+     * That will always be the coordinates of the single
+     * returned {@code Point}.
+     *
      * @return a {@code Point} array that indicates the tiles that
      *          are checked out for writing, or {@code null} if no
      *          tiles are checked out for writing.
      */
     public Point[] getWritableTileIndices() {
         Point[] p = new Point[1];
-        p[0] = new Point(0, 0);
+        p[0] = new Point();         // Default to (0,0).
 
         return p;
     }
@@ -1603,6 +1629,12 @@ public class BufferedImage extends java.awt.Image
    * Checks out a tile for writing.  All registered
    * {@code TileObservers} are notified when a tile goes from having
    * no writers to having one writer.
+   * <p>
+   * This method unconditionally returns the
+   * {@linkplain #getRaster() single tile} without checking
+   * the passed values. No listeners are notified since the
+   * returned tile is always checked out for writing.
+   *
    * @param tileX the x index of the tile
    * @param tileY the y index of the tile
    * @return a {@code WritableRaster} that is the tile, indicated by
@@ -1620,6 +1652,11 @@ public class BufferedImage extends java.awt.Image
    * to undefined results.  All registered {@code TileObservers}
    * are notified when a tile goes from having one writer to having no
    * writers.
+   * <p>
+   * This method immediately returns without checking the passed values.
+   * No listeners are notified since the {@linkplain #getRaster() single tile}
+   * is always checked out for writing.
+   *
    * @param tileX the x index of the tile
    * @param tileY the y index of the tile
    */
