@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,6 +25,7 @@
 
 package java.lang;
 
+import jdk.internal.util.OperatingSystem;
 import java.io.File;
 import java.io.FileDescriptor;
 import java.io.IOException;
@@ -34,9 +35,7 @@ import java.util.Arrays;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
-import java.util.StringJoiner;
 import jdk.internal.event.ProcessStartEvent;
-import sun.security.action.GetPropertyAction;
 
 /**
  * This class is used to create operating system processes.
@@ -191,6 +190,9 @@ import sun.security.action.GetPropertyAction;
 
 public final class ProcessBuilder
 {
+    // Lazily and racy initialize when needed, racy is ok, any logger is ok
+    private static System.Logger LOGGER;
+
     private List<String> command;
     private File directory;
     private Map<String,String> environment;
@@ -348,6 +350,7 @@ public final class ProcessBuilder
      * @see    System#getenv()
      */
     public Map<String,String> environment() {
+        @SuppressWarnings("removal")
         SecurityManager security = System.getSecurityManager();
         if (security != null)
             security.checkPermission(new RuntimePermission("getenv.*"));
@@ -469,9 +472,8 @@ public final class ProcessBuilder
      * @since 1.7
      */
     public abstract static class Redirect {
-        private static final File NULL_FILE = new File(
-                (GetPropertyAction.privilegedGetProperty("os.name")
-                        .startsWith("Windows") ? "NUL" : "/dev/null")
+        private static final File NULL_FILE =
+                new File((OperatingSystem.isWindows() ? "NUL" : "/dev/null")
         );
 
         /**
@@ -673,9 +675,8 @@ public final class ProcessBuilder
         public boolean equals(Object obj) {
             if (obj == this)
                 return true;
-            if (! (obj instanceof Redirect))
+            if (! (obj instanceof Redirect r))
                 return false;
-            Redirect r = (Redirect) obj;
             if (r.type() != this.type())
                 return false;
             assert this.file() != null;
@@ -1005,6 +1006,8 @@ public final class ProcessBuilder
      * be required to start a process on some operating systems.
      * As a result, the subprocess may inherit additional environment variable
      * settings beyond those in the process builder's {@link #environment()}.
+     * The minimal set of system dependent environment variables
+     * may override the values provided in the environment.
      *
      * <p>If there is a security manager, its
      * {@link SecurityManager#checkExec checkExec}
@@ -1067,6 +1070,19 @@ public final class ProcessBuilder
      *
      * @throws IOException if an I/O error occurs
      *
+     * @implNote
+     * In the reference implementation, logging of the command, arguments, directory,
+     * stack trace, and process id can be enabled.
+     * The logged information may contain sensitive security information and the potential exposure
+     * of the information should be carefully reviewed.
+     * Logging of the information is enabled when the logging level of the
+     * {@linkplain System#getLogger(String) system logger} named {@code java.lang.ProcessBuilder}
+     * is {@link System.Logger.Level#DEBUG Level.DEBUG} or {@link System.Logger.Level#TRACE Level.TRACE}.
+     * When enabled for {@code Level.DEBUG} only the process id, directory, command, and stack trace
+     * are logged.
+     * When enabled for {@code Level.TRACE} the arguments are included with the process id,
+     * directory, command, and stack trace.
+     *
      * @see Runtime#exec(String[], String[], java.io.File)
      */
     public Process start() throws IOException {
@@ -1093,14 +1109,15 @@ public final class ProcessBuilder
         // Throws IndexOutOfBoundsException if command is empty
         String prog = cmdarray[0];
 
+        @SuppressWarnings("removal")
         SecurityManager security = System.getSecurityManager();
         if (security != null)
             security.checkExec(prog);
 
         String dir = directory == null ? null : directory.toString();
 
-        for (int i = 1; i < cmdarray.length; i++) {
-            if (cmdarray[i].indexOf('\u0000') >= 0) {
+        for (String s : cmdarray) {
+            if (s.indexOf('\u0000') >= 0) {
                 throw new IOException("invalid null character in command");
             }
         }
@@ -1113,15 +1130,26 @@ public final class ProcessBuilder
                                      redirectErrorStream);
             ProcessStartEvent event = new ProcessStartEvent();
             if (event.isEnabled()) {
-                StringJoiner command = new StringJoiner(" ");
-                for (String s: cmdarray) {
-                    command.add(s);
-                }
                 event.directory = dir;
-                event.command = command.toString();
+                event.command = String.join(" ", cmdarray);
                 event.pid = process.pid();
                 event.commit();
             }
+            // Racy initialization for logging; errors in configuration may throw exceptions
+            System.Logger logger = LOGGER;
+            if (logger == null) {
+                LOGGER = logger = System.getLogger("java.lang.ProcessBuilder");
+            }
+            if (logger.isLoggable(System.Logger.Level.DEBUG)) {
+                boolean detail = logger.isLoggable(System.Logger.Level.TRACE);
+                var level = (detail) ? System.Logger.Level.TRACE : System.Logger.Level.DEBUG;
+                var cmdargs = (detail) ? String.join("\" \"", cmdarray) : cmdarray[0];
+                RuntimeException stackTraceEx = new RuntimeException("ProcessBuilder.start() debug");
+                LOGGER.log(level, "ProcessBuilder.start(): pid: " + process.pid() +
+                        ", dir: " + dir +
+                        ", cmd: \"" + cmdargs + "\"",
+                        stackTraceEx);
+            }
             return process;
         } catch (IOException | IllegalArgumentException e) {
             String exceptionInfo = ": " + e.getMessage();
@@ -1185,6 +1213,8 @@ public final class ProcessBuilder
      * be required to start a process on some operating systems.
      * As a result, the subprocess may inherit additional environment variable
      * settings beyond those in the process builder's {@link #environment()}.
+     * The minimal set of system dependent environment variables
+     * may override the values provided in the environment.
      * <p>
      * If there is a security manager, its
      * {@link SecurityManager#checkExec checkExec}
@@ -1264,6 +1294,10 @@ public final class ProcessBuilder
      * @throws  UnsupportedOperationException
      *          If the operating system does not support the creation of processes
      *
+     * @implNote
+     * In the reference implementation, logging of each process created can be enabled,
+     * see {@link ProcessBuilder#start()} for details.
+     *
      * @throws IOException if an I/O error occurs
      * @since 9
      */
@@ -1295,6 +1329,10 @@ public final class ProcessBuilder
                     redirects[1] = new RedirectPipeImpl();  // placeholder for new output
                 }
                 processes.add(builder.start(redirects));
+                if (prevOutput instanceof RedirectPipeImpl redir) {
+                    // Wrap the fd so it can be closed
+                    new Process.PipeInputStream(redir.getFd()).close();
+                }
                 prevOutput = redirects[1];
             }
         } catch (Exception ex) {
