@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -24,6 +24,7 @@
  */
 
 package java.lang.management;
+
 import java.io.FilePermission;
 import java.io.IOException;
 import javax.management.DynamicMBean;
@@ -37,10 +38,6 @@ import javax.management.InstanceNotFoundException;
 import javax.management.MalformedObjectNameException;
 import javax.management.StandardEmitterMBean;
 import javax.management.StandardMBean;
-import java.util.Collections;
-import java.util.List;
-import java.util.Set;
-import java.util.Map;
 import java.security.AccessController;
 import java.security.Permission;
 import java.security.PrivilegedAction;
@@ -48,11 +45,15 @@ import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
 import java.util.ArrayList;
 import java.util.Collection;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.List;
+import java.util.Map;
 import java.util.Optional;
 import java.util.ServiceLoader;
-import java.util.function.Function;
+import java.util.Set;
 import java.util.stream.Collectors;
-import static java.util.stream.Collectors.toMap;
 import java.util.stream.Stream;
 import javax.management.JMX;
 import sun.management.Util;
@@ -67,7 +68,7 @@ import sun.management.spi.PlatformMBeanProvider.PlatformComponent;
  * the management interface of a component of the Java virtual
  * machine.
  *
- * <h3><a id="MXBean">Platform MXBeans</a></h3>
+ * <h2><a id="MXBean">Platform MXBeans</a></h2>
  * <p>
  * A platform MXBean is a <i>managed bean</i> that
  * conforms to the <a href="../../../javax/management/package-summary.html">JMX</a>
@@ -93,7 +94,7 @@ import sun.management.spi.PlatformMBeanProvider.PlatformComponent;
  *
  * <p>
  * An application can access a platform MXBean in the following ways:
- * <h4>1. Direct access to an MXBean interface</h4>
+ * <h3>1. Direct access to an MXBean interface</h3>
  * <blockquote>
  * <ul>
  *     <li>Get an MXBean instance by calling the
@@ -113,7 +114,7 @@ import sun.management.spi.PlatformMBeanProvider.PlatformComponent;
  *         an MXBean of another running virtual machine.
  *         </li>
  * </ul>
- * <h4>2. Indirect access to an MXBean interface via MBeanServer</h4>
+ * <h3>2. Indirect access to an MXBean interface via MBeanServer</h3>
  * <ul>
  *     <li>Go through the platform {@code MBeanServer} to access MXBeans
  *         locally or a specific {@code MBeanServerConnection} to access
@@ -176,7 +177,7 @@ import sun.management.spi.PlatformMBeanProvider.PlatformComponent;
  * </tr>
  * <tr>
  * <th scope="row"> {@link PlatformLoggingMXBean} </th>
- * <td> {@link java.util.logging.LogManager#LOGGING_MXBEAN_NAME
+ * <td> {@link java.logging/java.util.logging.LogManager#LOGGING_MXBEAN_NAME
  *             java.util.logging:type=Logging}</td>
  * </tr>
  * </tbody>
@@ -246,6 +247,8 @@ import sun.management.spi.PlatformMBeanProvider.PlatformComponent;
  * @author  Mandy Chung
  * @since   1.5
  */
+@SuppressWarnings({"removal",
+                   "doclint:reference"}) // cross-module links
 public class ManagementFactory {
     // A class with only static fields and methods.
     private ManagementFactory() {};
@@ -254,42 +257,42 @@ public class ManagementFactory {
      * String representation of the
      * {@code ObjectName} for the {@link ClassLoadingMXBean}.
      */
-    public final static String CLASS_LOADING_MXBEAN_NAME =
+    public static final String CLASS_LOADING_MXBEAN_NAME =
         "java.lang:type=ClassLoading";
 
     /**
      * String representation of the
      * {@code ObjectName} for the {@link CompilationMXBean}.
      */
-    public final static String COMPILATION_MXBEAN_NAME =
+    public static final String COMPILATION_MXBEAN_NAME =
         "java.lang:type=Compilation";
 
     /**
      * String representation of the
      * {@code ObjectName} for the {@link MemoryMXBean}.
      */
-    public final static String MEMORY_MXBEAN_NAME =
+    public static final String MEMORY_MXBEAN_NAME =
         "java.lang:type=Memory";
 
     /**
      * String representation of the
      * {@code ObjectName} for the {@link OperatingSystemMXBean}.
      */
-    public final static String OPERATING_SYSTEM_MXBEAN_NAME =
+    public static final String OPERATING_SYSTEM_MXBEAN_NAME =
         "java.lang:type=OperatingSystem";
 
     /**
      * String representation of the
      * {@code ObjectName} for the {@link RuntimeMXBean}.
      */
-    public final static String RUNTIME_MXBEAN_NAME =
+    public static final String RUNTIME_MXBEAN_NAME =
         "java.lang:type=Runtime";
 
     /**
      * String representation of the
      * {@code ObjectName} for the {@link ThreadMXBean}.
      */
-    public final static String THREAD_MXBEAN_NAME =
+    public static final String THREAD_MXBEAN_NAME =
         "java.lang:type=Threading";
 
     /**
@@ -299,7 +302,7 @@ public class ManagementFactory {
      * can be formed by appending this string with
      * "{@code ,name=}<i>collector's name</i>".
      */
-    public final static String GARBAGE_COLLECTOR_MXBEAN_DOMAIN_TYPE =
+    public static final String GARBAGE_COLLECTOR_MXBEAN_DOMAIN_TYPE =
         "java.lang:type=GarbageCollector";
 
     /**
@@ -309,7 +312,7 @@ public class ManagementFactory {
      * can be formed by appending this string with
      * "{@code ,name=}<i>manager's name</i>".
      */
-    public final static String MEMORY_MANAGER_MXBEAN_DOMAIN_TYPE=
+    public static final String MEMORY_MANAGER_MXBEAN_DOMAIN_TYPE=
         "java.lang:type=MemoryManager";
 
     /**
@@ -319,7 +322,7 @@ public class ManagementFactory {
      * can be formed by appending this string with
      * {@code ,name=}<i>pool's name</i>.
      */
-    public final static String MEMORY_POOL_MXBEAN_DOMAIN_TYPE=
+    public static final String MEMORY_POOL_MXBEAN_DOMAIN_TYPE=
         "java.lang:type=MemoryPool";
 
     /**
@@ -465,7 +468,7 @@ public class ManagementFactory {
      *         MXBeans are registered into the platform {@code MBeanServer}
      *         at the first time this method is called.
      *
-     * @exception SecurityException if there is a security manager
+     * @throws SecurityException if there is a security manager
      * and the caller does not have the permission required by
      * {@link javax.management.MBeanServerFactory#createMBeanServer}.
      *
@@ -473,6 +476,7 @@ public class ManagementFactory {
      * @see javax.management.MBeanServerFactory#createMBeanServer
      */
     public static synchronized MBeanServer getPlatformMBeanServer() {
+        @SuppressWarnings("removal")
         SecurityManager sm = System.getSecurityManager();
         if (sm != null) {
             Permission perm = new MBeanServerPermission("createMBeanServer");
@@ -597,6 +601,7 @@ public class ManagementFactory {
         // Only allow MXBean interfaces from the platform modules loaded by the
         // bootstrap or platform class loader
         final Class<?> cls = mxbeanInterface;
+        @SuppressWarnings("removal")
         ClassLoader loader =
             AccessController.doPrivileged(
                 (PrivilegedAction<ClassLoader>) () -> cls.getClassLoader());
@@ -628,7 +633,7 @@ public class ManagementFactory {
     // using newPlatformMXBeanProxy(mbs, on, LoggingMXBean.class)
     // even though the underlying MXBean no longer implements
     // java.util.logging.LoggingMXBean.
-    // Altough java.util.logging.LoggingMXBean is deprecated, an application
+    // Although java.util.logging.LoggingMXBean is deprecated, an application
     // that uses newPlatformMXBeanProxy(mbs, on, LoggingMXBean.class) will
     // continue to work.
     //
@@ -872,17 +877,19 @@ public class ManagementFactory {
     public static Set<Class<? extends PlatformManagedObject>>
            getPlatformManagementInterfaces()
     {
-        return platformComponents()
+        // local variable required here; see JDK-8223553
+        Stream<Class<? extends PlatformManagedObject>> pmos = platformComponents()
                 .stream()
                 .flatMap(pc -> pc.mbeanInterfaces().stream())
                 .filter(clazz -> PlatformManagedObject.class.isAssignableFrom(clazz))
-                .map(clazz -> clazz.asSubclass(PlatformManagedObject.class))
-                .collect(Collectors.toSet());
+                .map(clazz -> clazz.asSubclass(PlatformManagedObject.class));
+        return pmos.collect(Collectors.toSet());
     }
 
     private static final String NOTIF_EMITTER =
         "javax.management.NotificationEmitter";
 
+    @SuppressWarnings("removal")
     private static void addMXBean(final MBeanServer mbs, String name, final Object pmo)
     {
         try {
@@ -913,55 +920,50 @@ public class ManagementFactory {
         return PlatformMBeanFinder.getMap().values();
     }
 
-    private static class PlatformMBeanFinder
-    {
+    private static class PlatformMBeanFinder {
         private static final Map<String, PlatformComponent<?>> componentMap;
+
         static {
             // get all providers
+            @SuppressWarnings("removal")
             List<PlatformMBeanProvider> providers = AccessController.doPrivileged(
-                (PrivilegedAction<List<PlatformMBeanProvider>>) () -> {
-                     List<PlatformMBeanProvider> all = new ArrayList<>();
-                     ServiceLoader.loadInstalled(PlatformMBeanProvider.class)
-                                  .forEach(all::add);
-                     all.add(new DefaultPlatformMBeanProvider());
-                     return all;
-                }, null, new FilePermission("<<ALL FILES>>", "read"),
-                         new RuntimePermission("sun.management.spi.PlatformMBeanProvider.subclass"));
+                new PrivilegedAction<>() {
+                    @Override
+                    public List<PlatformMBeanProvider> run() {
+                        List<PlatformMBeanProvider> all = new ArrayList<>();
+                        for (PlatformMBeanProvider provider : ServiceLoader.loadInstalled(PlatformMBeanProvider.class)) {
+                            all.add(provider);
+                        }
+                        all.add(new DefaultPlatformMBeanProvider());
+                        return all;
+                    }
+                }, null, new FilePermission("<<ALL FILES>>", "read"));
 
             // load all platform components into a map
-            componentMap = providers.stream()
-                .flatMap(p -> toPlatformComponentStream(p))
-                // The first one wins if multiple PlatformComponents
-                // with same ObjectName pattern,
-                .collect(toMap(PlatformComponent::getObjectNamePattern,
-                               Function.identity(),
-                              (p1, p2) -> p1));
+            var map = new HashMap<String, PlatformComponent<?>>();
+            for (PlatformMBeanProvider provider : providers) {
+                // For each provider, ensure that two different components are not declared
+                // with the same object name pattern.
+                var names = new HashSet<String>();
+                for (PlatformComponent<?> component : provider.getPlatformComponentList()) {
+                    String name = component.getObjectNamePattern();
+                    if (!names.add(name)) {
+                        throw new InternalError(name +
+                                " has been used as key by this provider" +
+                                ", it cannot be reused for " + component);
+                    }
+                    // The first one wins if multiple PlatformComponents defined by
+                    // different providers use the same ObjectName pattern
+                    map.putIfAbsent(name, component);
+                }
+            }
+            componentMap = map;
         }
 
         static Map<String, PlatformComponent<?>> getMap() {
             return componentMap;
         }
 
-        // Loads all platform components from a provider into a stream
-        // Ensures that two different components are not declared with the same
-        // object name pattern. Throws InternalError if the provider incorrectly
-        // declares two platform components with the same pattern.
-        private static Stream<PlatformComponent<?>>
-            toPlatformComponentStream(PlatformMBeanProvider provider)
-        {
-            return provider.getPlatformComponentList()
-                           .stream()
-                           .collect(toMap(PlatformComponent::getObjectNamePattern,
-                                          Function.identity(),
-                                          (p1, p2) -> {
-                                              throw new InternalError(
-                                                 p1.getObjectNamePattern() +
-                                                 " has been used as key for " + p1 +
-                                                 ", it cannot be reused for " + p2);
-                                          }))
-                           .values().stream();
-        }
-
         // Finds the first PlatformComponent whose mbeanInterfaceNames() list
         // contains the specified class name. An MBean interface can be implemented
         // by different MBeans, provided by different platform components.
@@ -1015,6 +1017,11 @@ public class ManagementFactory {
     }
 
     static {
+        loadNativeLib();
+    }
+
+    @SuppressWarnings("removal")
+    private static void loadNativeLib() {
         AccessController.doPrivileged((PrivilegedAction<Void>) () -> {
             System.loadLibrary("management");
             return null;
