@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -35,6 +35,7 @@ import java.util.Collection;
 import java.util.Deque;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.HexFormat;
 import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
@@ -475,9 +476,10 @@ final class Resolver {
                     if (actualHash == null)
                         findFail("Unable to compute the hash of module %s", dn);
                     if (!Arrays.equals(recordedHash, actualHash)) {
+                        HexFormat hex = HexFormat.of();
                         findFail("Hash of %s (%s) differs to expected hash (%s)" +
-                                 " recorded in %s", dn, toHexString(actualHash),
-                                 toHexString(recordedHash), descriptor.name());
+                                 " recorded in %s", dn, hex.formatHex(actualHash),
+                                hex.formatHex(recordedHash), descriptor.name());
                     }
                 }
             }
@@ -485,15 +487,6 @@ final class Resolver {
         }
     }
 
-    private static String toHexString(byte[] ba) {
-        StringBuilder sb = new StringBuilder(ba.length * 2);
-        for (byte b: ba) {
-            sb.append(String.format("%02x", b & 0xff));
-        }
-        return sb.toString();
-    }
-
-
     /**
      * Computes the readability graph for the modules in the given Configuration.
      *
@@ -505,12 +498,11 @@ final class Resolver {
      */
     private Map<ResolvedModule, Set<ResolvedModule>> makeGraph(Configuration cf) {
 
-        // initial capacity of maps to avoid resizing
-        int capacity = 1 + (4 * nameToReference.size())/ 3;
+        int moduleCount = nameToReference.size();
 
         // the "reads" graph starts as a module dependence graph and
         // is iteratively updated to be the readability graph
-        Map<ResolvedModule, Set<ResolvedModule>> g1 = new HashMap<>(capacity);
+        Map<ResolvedModule, Set<ResolvedModule>> g1 = HashMap.newHashMap(moduleCount);
 
         // the "requires transitive" graph, contains requires transitive edges only
         Map<ResolvedModule, Set<ResolvedModule>> g2;
@@ -519,7 +511,7 @@ final class Resolver {
         // as there may be selected modules that have a dependency on modules in
         // the parent configuration.
         if (ModuleLayer.boot() == null) {
-            g2 = new HashMap<>(capacity);
+            g2 = HashMap.newHashMap(moduleCount);
         } else {
             g2 = parents.stream()
                 .flatMap(Configuration::configurations)
@@ -546,7 +538,7 @@ final class Resolver {
 
         // populate g1 and g2 with the dependences from the selected modules
 
-        Map<String, ResolvedModule> nameToResolved = new HashMap<>(capacity);
+        Map<String, ResolvedModule> nameToResolved = HashMap.newHashMap(moduleCount);
 
         for (ModuleReference mref : nameToReference.values()) {
             ModuleDescriptor descriptor = mref.descriptor();
@@ -875,7 +867,7 @@ final class Resolver {
         Set<ModuleReference> result = new HashSet<>(beforeModules);
         for (ModuleReference mref : afterModules) {
             String name = mref.descriptor().name();
-            if (!beforeFinder.find(name).isPresent()
+            if (beforeFinder.find(name).isEmpty()
                     && findInParent(name) == null) {
                 result.add(mref);
             }
