@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1995, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -28,10 +28,9 @@ package java.net;
 import java.io.IOException;
 import java.io.UncheckedIOException;
 import java.nio.channels.DatagramChannel;
-import java.security.AccessController;
-import java.security.PrivilegedAction;
+import java.nio.channels.MulticastChannel;
+import java.util.Objects;
 import java.util.Set;
-import sun.net.NetProperties;
 import sun.nio.ch.DefaultSelectorProvider;
 
 /**
@@ -81,11 +80,11 @@ import sun.nio.ch.DefaultSelectorProvider;
  * <tbody>
  *   <tr>
  *     <th scope="row"> {@link java.net.StandardSocketOptions#SO_SNDBUF SO_SNDBUF} </th>
- *     <td> The size of the socket send buffer </td>
+ *     <td> The size of the socket send buffer in bytes </td>
  *   </tr>
  *   <tr>
  *     <th scope="row"> {@link java.net.StandardSocketOptions#SO_RCVBUF SO_RCVBUF} </th>
- *     <td> The size of the socket receive buffer </td>
+ *     <td> The size of the socket receive buffer in bytes </td>
  *   </tr>
  *   <tr>
  *     <th scope="row"> {@link java.net.StandardSocketOptions#SO_REUSEADDR SO_REUSEADDR} </th>
@@ -102,10 +101,142 @@ import sun.nio.ch.DefaultSelectorProvider;
  * </tbody>
  * </table>
  * </blockquote>
- * An implementation may also support additional options. In particular an implementation
- * may support <a href="MulticastSocket.html#MulticastOptions">multicast options</a> which
- * can be useful when using a plain {@code DatagramSocket} to send datagrams to a
- * multicast group.
+ * <p> In addition, the {@code DatagramSocket} class defines methods to {@linkplain
+ * #joinGroup(SocketAddress, NetworkInterface) join} and {@linkplain
+ * #leaveGroup(SocketAddress, NetworkInterface) leave} a multicast group, and
+ * supports <a href="DatagramSocket.html#MulticastOptions">multicast options</a> which
+ * are useful when {@linkplain #joinGroup(SocketAddress, NetworkInterface) joining},
+ * {@linkplain #leaveGroup(SocketAddress, NetworkInterface) leaving}, or sending datagrams
+ * to a multicast group.
+ * The following multicast options are supported:
+ * <blockquote>
+ * <a id="MulticastOptions"></a>
+ * <table class="striped">
+ * <caption style="display:none">Multicast options</caption>
+ * <thead>
+ *   <tr>
+ *     <th scope="col">Option Name</th>
+ *     <th scope="col">Description</th>
+ *   </tr>
+ * </thead>
+ * <tbody>
+ *   <tr>
+ *     <th scope="row"> {@link java.net.StandardSocketOptions#IP_MULTICAST_IF IP_MULTICAST_IF} </th>
+ *     <td> The network interface for Internet Protocol (IP) multicast datagrams </td>
+ *   </tr>
+ *   <tr>
+ *     <th scope="row"> {@link java.net.StandardSocketOptions#IP_MULTICAST_TTL
+ *       IP_MULTICAST_TTL} </th>
+ *     <td> The <em>time-to-live</em> for Internet Protocol (IP) multicast
+ *       datagrams </td>
+ *   </tr>
+ *   <tr>
+ *     <th scope="row"> {@link java.net.StandardSocketOptions#IP_MULTICAST_LOOP
+ *       IP_MULTICAST_LOOP} </th>
+ *     <td> Loopback for Internet Protocol (IP) multicast datagrams </td>
+ *   </tr>
+ * </tbody>
+ * </table>
+ * </blockquote>
+ * An implementation may also support additional options.
+ *
+ * @apiNote  <a id="Multicasting"></a><b>Multicasting with DatagramSocket</b>
+ *
+ * <p> {@link DatagramChannel} implements the {@link MulticastChannel} interface
+ * and provides an alternative API for sending and receiving multicast datagrams.
+ * The {@link MulticastChannel} API supports both {@linkplain
+ * MulticastChannel#join(InetAddress, NetworkInterface) any-source} and
+ * {@linkplain MulticastChannel#join(InetAddress, NetworkInterface, InetAddress)
+ * source-specific} multicast. Consider using {@code DatagramChannel} for
+ * multicasting.
+ *
+ * <p> {@code DatagramSocket} can be used directly for multicasting. However,
+ * contrarily to {@link MulticastSocket}, {@code DatagramSocket} doesn't call the
+ * {@link DatagramSocket#setReuseAddress(boolean)} method to enable the SO_REUSEADDR
+ * socket option by default. If creating a {@code DatagramSocket} intended to
+ * later join a multicast group, the caller should consider explicitly enabling
+ * the SO_REUSEADDR option.
+ *
+ * <p> An instance of {@code DatagramSocket} can be used to send or
+ * receive multicast datagram packets. It is not necessary to join a multicast
+ * group in order to send multicast datagrams. Before sending out multicast
+ * datagram packets however, the default outgoing interface for sending
+ * multicast datagram should first be configured using
+ * {@link #setOption(SocketOption, Object) setOption} and
+ * {@link StandardSocketOptions#IP_MULTICAST_IF}:
+ *
+ * <pre>{@code
+ *    DatagramSocket sender = new DatagramSocket(new InetSocketAddress(0));
+ *    NetworkInterface outgoingIf = NetworkInterface.getByName("en0");
+ *    sender.setOption(StandardSocketOptions.IP_MULTICAST_IF, outgoingIf);
+ *
+ *    // optionally configure multicast TTL; the TTL defines the scope of a
+ *    // multicast datagram, for example, confining it to host local (0) or
+ *    // link local (1) etc...
+ *    int ttl = ...; // a number between 0 and 255
+ *    sender.setOption(StandardSocketOptions.IP_MULTICAST_TTL, ttl);
+ *
+ *    // send a packet to a multicast group
+ *    byte[] msgBytes = ...;
+ *    InetAddress mcastaddr = InetAddress.getByName("228.5.6.7");
+ *    int port = 6789;
+ *    InetSocketAddress dest = new InetSocketAddress(mcastaddr, port);
+ *    DatagramPacket hi = new DatagramPacket(msgBytes, msgBytes.length, dest);
+ *    sender.send(hi);
+ * }</pre>
+ *
+ * <p> An instance of {@code DatagramSocket} can also be used to receive
+ * multicast datagram packets. A {@code DatagramSocket} that is created
+ * with the intent of receiving multicast datagrams should be created
+ * <i>unbound</i>. Before binding the socket, {@link #setReuseAddress(boolean)
+ * setReuseAddress(true)} should be configured:
+ *
+ * <pre>{@code
+ *    DatagramSocket socket = new DatagramSocket(null); // unbound
+ *    socket.setReuseAddress(true); // set reuse address before binding
+ *    socket.bind(new InetSocketAddress(6789)); // bind
+ *
+ *    // joinGroup 228.5.6.7
+ *    InetAddress mcastaddr = InetAddress.getByName("228.5.6.7");
+ *    InetSocketAddress group = new InetSocketAddress(mcastaddr, 0);
+ *    NetworkInterface netIf = NetworkInterface.getByName("en0");
+ *    socket.joinGroup(group, netIf);
+ *    byte[] msgBytes = new byte[1024]; // up to 1024 bytes
+ *    DatagramPacket packet = new DatagramPacket(msgBytes, msgBytes.length);
+ *    socket.receive(packet);
+ *    ....
+ *    // eventually leave group
+ *    socket.leaveGroup(group, netIf);
+ * }</pre>
+ *
+ * <p><a id="PlatformDependencies"></a><b>Platform dependencies</b>
+ * <p>The multicast implementation is intended to map directly to the native
+ * multicasting facility. Consequently, the following items should be considered
+ * when developing an application that receives IP multicast datagrams:
+ * <ol>
+ *    <li> Contrarily to {@link DatagramChannel}, the constructors of {@code DatagramSocket}
+ *        do not allow to specify the {@link ProtocolFamily} of the underlying socket.
+ *        Consequently, the protocol family of the underlying socket may not
+ *        correspond to the protocol family of the multicast groups that
+ *        the {@code DatagramSocket} will attempt to join.
+ *        <br>
+ *        There is no guarantee that a {@code DatagramSocket} with an underlying
+ *        socket created in one protocol family can join and receive multicast
+ *        datagrams when the address of the multicast group corresponds to
+ *        another protocol family. For example, it is implementation specific if a
+ *        {@code DatagramSocket} to an IPv6 socket can join an IPv4 multicast group
+ *        and receive multicast datagrams sent to the group.
+ *    </li>
+ *    <li> Before joining a multicast group, the {@code DatagramSocket} should be
+ *        bound to the wildcard address.
+ *        If the socket is bound to a specific address, rather than the wildcard address
+ *        then it is implementation specific if multicast datagrams are received
+ *        by the socket.
+ *    </li>
+ *    <li> The SO_REUSEADDR option should be enabled prior to binding the socket.
+ *        This is required to allow multiple members of the group to bind to the same address.
+ *    </li>
+ * </ol>
  *
  * @author  Pavani Diwanji
  * @see     java.net.DatagramPacket
@@ -526,24 +657,39 @@ public class DatagramSocket implements java.io.Closeable {
      * @see        SecurityManager#checkMulticast(InetAddress)
      * @see        SecurityManager#checkConnect
      * @revised 1.4
-     * @spec JSR-51
      */
     public void send(DatagramPacket p) throws IOException  {
         delegate().send(p);
     }
 
     /**
-     * Receives a datagram packet from this socket. When this method
-     * returns, the {@code DatagramPacket}'s buffer is filled with
-     * the data received. The datagram packet also contains the sender's
+     * Receives a datagram packet from this socket. This method blocks until a
+     * datagram is received.
+     *
+     * When this method returns, the {@code DatagramPacket}'s buffer is filled
+     * with the data received. The datagram packet also contains the sender's
      * IP address, and the port number on the sender's machine.
-     * <p>
-     * This method blocks until a datagram is received. The
-     * {@code length} field of the datagram packet object contains
+     * The {@code length} field of the datagram packet object contains
      * the length of the received message. If the message is longer than
      * the packet's length, the message is truncated.
-     * <p>
-     * If there is a security manager, and the socket is not currently
+     *
+     * <p> This method is {@linkplain Thread#interrupt() interruptible} in the
+     * following circumstances:
+     * <ol>
+     *   <li> The datagram socket is {@linkplain DatagramChannel#socket() associated}
+     *        with a {@link DatagramChannel DatagramChannel}. In that case,
+     *        interrupting a thread receiving a datagram packet will close the
+     *        underlying channel and cause this method to throw {@link
+     *        java.nio.channels.ClosedByInterruptException} with the interrupt
+     *        status set.
+     *   <li> The datagram socket uses the system-default socket implementation and
+     *        a {@linkplain Thread#isVirtual() virtual thread} is receiving a
+     *        datagram packet. In that case, interrupting the virtual thread will
+     *        cause it to wakeup and close the socket. This method will then throw
+     *        {@code SocketException} with the interrupt status set.
+     * </ol>
+     *
+     * <p> If there is a security manager, and the socket is not currently
      * connected to a remote address, a packet cannot be received if the
      * security manager's {@code checkAccept} method does not allow it.
      * Datagrams that are not permitted by the security manager are silently
@@ -563,7 +709,6 @@ public class DatagramSocket implements java.io.Closeable {
      * @see        java.net.DatagramPacket
      * @see        java.net.DatagramSocket
      * @revised 1.4
-     * @spec JSR-51
      */
     public void receive(DatagramPacket p) throws IOException {
         delegate().receive(p);
@@ -657,14 +802,20 @@ public class DatagramSocket implements java.io.Closeable {
      * of SO_SNDBUF then it is implementation specific if the
      * packet is sent or discarded.
      *
+     * @apiNote
+     * If {@code size > 0}, this method is equivalent to calling
+     * {@link #setOption(SocketOption, Object)
+     * setOption(StandardSocketOptions.SO_SNDBUF, size)}.
+     *
      * @param size the size to which to set the send buffer
-     * size. This value must be greater than 0.
+     * size, in bytes. This value must be greater than 0.
      *
      * @throws    SocketException if there is an error
      * in the underlying protocol, such as an UDP error.
      * @throws    IllegalArgumentException if the value is 0 or is
      * negative.
      * @see #getSendBufferSize()
+     * @see StandardSocketOptions#SO_SNDBUF
      * @since 1.2
      */
     public void setSendBufferSize(int size) throws SocketException {
@@ -673,12 +824,17 @@ public class DatagramSocket implements java.io.Closeable {
 
     /**
      * Get value of the SO_SNDBUF option for this {@code DatagramSocket}, that is the
-     * buffer size used by the platform for output on this {@code DatagramSocket}.
+     * buffer size, in bytes, used by the platform for output on this {@code DatagramSocket}.
+     *
+     * @apiNote
+     * This method is equivalent to calling {@link #getOption(SocketOption)
+     * getOption(StandardSocketOptions.SO_SNDBUF)}.
      *
      * @return the value of the SO_SNDBUF option for this {@code DatagramSocket}
      * @throws    SocketException if there is an error in
      * the underlying protocol, such as an UDP error.
      * @see #setSendBufferSize
+     * @see StandardSocketOptions#SO_SNDBUF
      * @since 1.2
      */
     public int getSendBufferSize() throws SocketException {
@@ -704,14 +860,20 @@ public class DatagramSocket implements java.io.Closeable {
      * Note: It is implementation specific if a packet larger
      * than SO_RCVBUF can be received.
      *
+     * @apiNote
+     * If {@code size > 0}, this method is equivalent to calling
+     * {@link #setOption(SocketOption, Object)
+     * setOption(StandardSocketOptions.SO_RCVBUF, size)}.
+     *
      * @param size the size to which to set the receive buffer
-     * size. This value must be greater than 0.
+     * size, in bytes. This value must be greater than 0.
      *
      * @throws    SocketException if there is an error in
      * the underlying protocol, such as an UDP error.
      * @throws    IllegalArgumentException if the value is 0 or is
      * negative.
      * @see #getReceiveBufferSize()
+     * @see StandardSocketOptions#SO_RCVBUF
      * @since 1.2
      */
     public void setReceiveBufferSize(int size) throws SocketException {
@@ -720,11 +882,16 @@ public class DatagramSocket implements java.io.Closeable {
 
     /**
      * Get value of the SO_RCVBUF option for this {@code DatagramSocket}, that is the
-     * buffer size used by the platform for input on this {@code DatagramSocket}.
+     * buffer size, in bytes, used by the platform for input on this {@code DatagramSocket}.
+     *
+     * @apiNote
+     * This method is equivalent to calling {@link #getOption(SocketOption)
+     * getOption(StandardSocketOptions.SO_RCVBUF)}.
      *
      * @return the value of the SO_RCVBUF option for this {@code DatagramSocket}
      * @throws    SocketException if there is an error in the underlying protocol, such as an UDP error.
      * @see #setReceiveBufferSize(int)
+     * @see StandardSocketOptions#SO_RCVBUF
      * @since 1.2
      */
     public int getReceiveBufferSize() throws SocketException {
@@ -755,6 +922,10 @@ public class DatagramSocket implements java.io.Closeable {
      * disabled after a socket is bound (See {@link #isBound()})
      * is not defined.
      *
+     * @apiNote
+     * This method is equivalent to calling {@link #setOption(SocketOption, Object)
+     * setOption(StandardSocketOptions.SO_REUSEADDR, on)}.
+     *
      * @param on  whether to enable or disable the
      * @throws    SocketException if an error occurs enabling or
      *            disabling the {@code SO_REUSEADDR} socket option,
@@ -764,6 +935,7 @@ public class DatagramSocket implements java.io.Closeable {
      * @see #bind(SocketAddress)
      * @see #isBound()
      * @see #isClosed()
+     * @see StandardSocketOptions#SO_REUSEADDR
      */
     public void setReuseAddress(boolean on) throws SocketException {
         delegate().setReuseAddress(on);
@@ -772,11 +944,16 @@ public class DatagramSocket implements java.io.Closeable {
     /**
      * Tests if SO_REUSEADDR is enabled.
      *
+     * @apiNote
+     * This method is equivalent to calling {@link #getOption(SocketOption)
+     * getOption(StandardSocketOptions.SO_REUSEADDR)}.
+     *
      * @return a {@code boolean} indicating whether or not SO_REUSEADDR is enabled.
      * @throws    SocketException if there is an error
      * in the underlying protocol, such as an UDP error.
      * @since   1.4
      * @see #setReuseAddress(boolean)
+     * @see StandardSocketOptions#SO_REUSEADDR
      */
     public boolean getReuseAddress() throws SocketException {
         return delegate().getReuseAddress();
@@ -789,6 +966,10 @@ public class DatagramSocket implements java.io.Closeable {
      * started with implementation specific privileges to enable this option or
      * send broadcast datagrams.
      *
+     * @apiNote
+     * This method is equivalent to calling {@link #setOption(SocketOption, Object)
+     * setOption(StandardSocketOptions.SO_BROADCAST, on)}.
+     *
      * @param  on
      *         whether or not to have broadcast turned on.
      *
@@ -798,6 +979,7 @@ public class DatagramSocket implements java.io.Closeable {
      *
      * @since 1.4
      * @see #getBroadcast()
+     * @see StandardSocketOptions#SO_BROADCAST
      */
     public void setBroadcast(boolean on) throws SocketException {
         delegate().setBroadcast(on);
@@ -805,11 +987,17 @@ public class DatagramSocket implements java.io.Closeable {
 
     /**
      * Tests if SO_BROADCAST is enabled.
+     *
+     * @apiNote
+     * This method is equivalent to calling {@link #getOption(SocketOption)
+     * getOption(StandardSocketOptions.SO_BROADCAST)}.
+     *
      * @return a {@code boolean} indicating whether or not SO_BROADCAST is enabled.
      * @throws    SocketException if there is an error
      * in the underlying protocol, such as an UDP error.
      * @since 1.4
      * @see #setBroadcast(boolean)
+     * @see StandardSocketOptions#SO_BROADCAST
      */
     public boolean getBroadcast() throws SocketException {
         return delegate().getBroadcast();
@@ -846,11 +1034,16 @@ public class DatagramSocket implements java.io.Closeable {
      * for Internet Protocol v6 {@code tc} is the value that
      * would be placed into the sin6_flowinfo field of the IP header.
      *
+     * @apiNote
+     * This method is equivalent to calling {@link #setOption(SocketOption, Object)
+     * setOption(StandardSocketOptions.IP_TOS, tc)}.
+     *
      * @param tc        an {@code int} value for the bitset.
      * @throws SocketException if there is an error setting the
      * traffic class or type-of-service
      * @since 1.4
      * @see #getTrafficClass
+     * @see StandardSocketOptions#IP_TOS
      */
     public void setTrafficClass(int tc) throws SocketException {
         delegate().setTrafficClass(tc);
@@ -866,11 +1059,16 @@ public class DatagramSocket implements java.io.Closeable {
      * set using the {@link #setTrafficClass(int)} method on this
      * DatagramSocket.
      *
+     * @apiNote
+     * This method is equivalent to calling {@link #getOption(SocketOption)
+     * getOption(StandardSocketOptions.IP_TOS)}.
+     *
      * @return the traffic class or type-of-service already set
      * @throws SocketException if there is an error obtaining the
      * traffic class or type-of-service value.
      * @since 1.4
      * @see #setTrafficClass(int)
+     * @see StandardSocketOptions#IP_TOS
      */
     public int getTrafficClass() throws SocketException {
         return delegate().getTrafficClass();
@@ -886,7 +1084,6 @@ public class DatagramSocket implements java.io.Closeable {
      * as well.
      *
      * @revised 1.4
-     * @spec JSR-51
      */
     public void close() {
         delegate().close();
@@ -914,7 +1111,6 @@ public class DatagramSocket implements java.io.Closeable {
      *          or {@code null} if this socket was not created for a channel
      *
      * @since 1.4
-     * @spec JSR-51
      */
     public DatagramChannel getChannel() {
         return null;
@@ -950,7 +1146,18 @@ public class DatagramSocket implements java.io.Closeable {
      * @see       java.net.DatagramSocketImplFactory#createDatagramSocketImpl()
      * @see       SecurityManager#checkSetFactory
      * @since 1.3
+     *
+     * @deprecated Use {@link DatagramChannel}, or subclass {@code DatagramSocket}
+     *    directly.
+     *    <br> This method provided a way in early JDK releases to replace the
+     *    system wide implementation of {@code DatagramSocket}. It has been mostly
+     *    obsolete since Java 1.4. If required, a {@code DatagramSocket} can be
+     *    created to use a custom implementation by extending {@code DatagramSocket}
+     *    and using the {@linkplain #DatagramSocket(DatagramSocketImpl) protected
+     *    constructor} that takes an {@linkplain DatagramSocketImpl implementation}
+     *    as a parameter.
      */
+    @Deprecated(since = "17")
     public static synchronized void
     setDatagramSocketImplFactory(DatagramSocketImplFactory fac)
             throws IOException
@@ -958,6 +1165,7 @@ public class DatagramSocket implements java.io.Closeable {
         if (factory != null) {
             throw new SocketException("factory already defined");
         }
+        @SuppressWarnings("removal")
         SecurityManager security = System.getSecurityManager();
         if (security != null) {
             security.checkSetFactory();
@@ -1042,16 +1250,108 @@ public class DatagramSocket implements java.io.Closeable {
         return delegate().supportedOptions();
     }
 
-    // Temporary solution until JDK-8237352 is addressed
-    private static final SocketAddress NO_DELEGATE = new SocketAddress() {};
-    private static final boolean USE_PLAINDATAGRAMSOCKET = usePlainDatagramSocketImpl();
-
-    private static boolean usePlainDatagramSocketImpl() {
-        PrivilegedAction<String> pa = () -> NetProperties.get("jdk.net.usePlainDatagramSocketImpl");
-        String s = AccessController.doPrivileged(pa);
-        return (s != null) && (s.isEmpty() || s.equalsIgnoreCase("true"));
+    /**
+     * Joins a multicast group.
+     *
+     * <p> In order to join a multicast group, the caller should specify
+     * the IP address of the multicast group to join, and the local
+     * {@linkplain NetworkInterface network interface} to receive multicast
+     * packets from.
+     * <ul>
+     *  <li> The {@code mcastaddr} argument indicates the IP address
+     *   of the multicast group to join. For historical reasons this is
+     *   specified as a {@code SocketAddress}.
+     *   The default implementation only supports {@link InetSocketAddress} and
+     *   the {@link InetSocketAddress#getPort() port} information is ignored.
+     *  </li>
+     *  <li> The {@code netIf} argument specifies the local interface to receive
+     *       multicast datagram packets, or {@code null} to defer to the interface
+     *       set for outgoing multicast datagrams.
+     *       If {@code null}, and no interface has been set, the behaviour is
+     *       unspecified: any interface may be selected or the operation may fail
+     *       with a {@code SocketException}.
+     *  </li>
+     * </ul>
+     *
+     * <p> It is possible to call this method several times to join
+     * several different multicast groups, or join the same group
+     * in several different networks. However, if the socket is already a
+     * member of the group, an {@link IOException} will be thrown.
+     *
+     * <p>If there is a security manager, this method first
+     * calls its {@code checkMulticast} method with the {@code mcastaddr}
+     * argument as its argument.
+     *
+     * @apiNote The default interface for sending outgoing multicast datagrams
+     * can be configured with {@link #setOption(SocketOption, Object)}
+     * with {@link StandardSocketOptions#IP_MULTICAST_IF}.
+     *
+     * @param  mcastaddr indicates the multicast address to join.
+     * @param  netIf specifies the local interface to receive multicast
+     *         datagram packets, or {@code null}.
+     * @throws IOException if there is an error joining, or when the address
+     *         is not a multicast address, or the platform does not support
+     *         multicasting
+     * @throws SecurityException if a security manager exists and its
+     *         {@code checkMulticast} method doesn't allow the join.
+     * @throws IllegalArgumentException if mcastaddr is {@code null} or is a
+     *         SocketAddress subclass not supported by this socket
+     * @see    SecurityManager#checkMulticast(InetAddress)
+     * @see    DatagramChannel#join(InetAddress, NetworkInterface)
+     * @see    StandardSocketOptions#IP_MULTICAST_IF
+     * @since  17
+     */
+    public void joinGroup(SocketAddress mcastaddr, NetworkInterface netIf)
+            throws IOException {
+        delegate().joinGroup(mcastaddr, netIf);
     }
 
+    /**
+     * Leave a multicast group on a specified local interface.
+     *
+     * <p>If there is a security manager, this method first
+     * calls its {@code checkMulticast} method with the
+     * {@code mcastaddr} argument as its argument.
+     *
+     * @apiNote
+     * The {@code mcastaddr} and {@code netIf} arguments should identify
+     * a multicast group that was previously {@linkplain
+     * #joinGroup(SocketAddress, NetworkInterface) joined} by
+     * this {@code DatagramSocket}.
+     * <p> It is possible to call this method several times to leave
+     * multiple different multicast groups previously joined, or leave
+     * the same group previously joined in multiple different networks.
+     * However, if the socket is not a member of the specified group
+     * in the specified network, an {@link IOException} will be
+     * thrown.
+     *
+     * @param  mcastaddr is the multicast address to leave. This should
+     *         contain the same IP address than that used for {@linkplain
+     *         #joinGroup(SocketAddress, NetworkInterface) joining}
+     *         the group.
+     * @param  netIf specifies the local interface or {@code null} to defer
+     *         to the interface set for outgoing multicast datagrams.
+     *         If {@code null}, and no interface has been set, the behaviour
+     *         is unspecified: any interface may be selected or the operation
+     *         may fail with a {@code SocketException}.
+     * @throws IOException if there is an error leaving or when the address
+     *         is not a multicast address.
+     * @throws SecurityException if a security manager exists and its
+     *         {@code checkMulticast} method doesn't allow the operation.
+     * @throws IllegalArgumentException if mcastaddr is {@code null} or is a
+     *         SocketAddress subclass not supported by this socket.
+     * @see    SecurityManager#checkMulticast(InetAddress)
+     * @see    #joinGroup(SocketAddress, NetworkInterface)
+     * @see    StandardSocketOptions#IP_MULTICAST_IF
+     * @since  17
+     */
+    public void leaveGroup(SocketAddress mcastaddr, NetworkInterface netIf)
+            throws IOException {
+        delegate().leaveGroup(mcastaddr, netIf);
+    }
+
+    private static final SocketAddress NO_DELEGATE = new SocketAddress() {};
+
     /**
      * Best effort to convert an {@link IOException}
      * into a {@link SocketException}.
@@ -1065,9 +1365,7 @@ public class DatagramSocket implements java.io.Closeable {
         Throwable cause = e.getCause();
         if (cause instanceof SocketException)
             return (SocketException) cause;
-        SocketException se = new SocketException(e.getMessage());
-        se.initCause(e);
-        return se;
+        return new SocketException(e.getMessage(), e);
     }
 
     /**
@@ -1088,13 +1386,12 @@ public class DatagramSocket implements java.io.Closeable {
      *                 {@code java.net.MulticastSocket}.
      * @return {@code null} if {@code bindaddr == NO_DELEGATE}, otherwise returns a
      * delegate for the requested {@code type}.
-     * @throws SocketException if an exception occurs while creating or binding the
+     * @throws SocketException if an exception occurs while creating or binding
      *                         the delegate.
      */
     static <T extends DatagramSocket> T createDelegate(SocketAddress bindaddr, Class<T> type)
             throws SocketException {
 
-        // Temporary solution until JDK-8237352 is addressed
         if (bindaddr == NO_DELEGATE) return null;
 
         assert type == DatagramSocket.class || type == MulticastSocket.class;
@@ -1103,14 +1400,11 @@ public class DatagramSocket implements java.io.Closeable {
         boolean initialized = false;
         try {
             DatagramSocketImplFactory factory = DatagramSocket.factory;
-            if (USE_PLAINDATAGRAMSOCKET || factory != null) {
+            if (factory != null) {
                 // create legacy DatagramSocket delegate
-                DatagramSocketImpl impl;
-                if (factory != null) {
-                    impl = factory.createDatagramSocketImpl();
-                } else {
-                    impl = DefaultDatagramSocketImplFactory.createDatagramSocketImpl(multicast);
-                }
+                DatagramSocketImpl impl = factory.createDatagramSocketImpl();
+                Objects.requireNonNull(impl,
+                        "Implementation returned by installed DatagramSocketImplFactory is null");
                 delegate = new NetMulticastSocket(impl);
                 ((NetMulticastSocket) delegate).getImpl(); // ensure impl.create() is called.
             } else {
