@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -30,6 +30,7 @@ import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
 import java.io.ObjectStreamException;
 import java.io.ObjectStreamField;
+import java.util.Locale;
 
 /**
  *
@@ -53,13 +54,13 @@ public class InetSocketAddress
     extends SocketAddress
 {
     // Private implementation class pointed to by all public methods.
-    private static class InetSocketAddressHolder {
+    private static final class InetSocketAddressHolder {
         // The hostname of the Socket Address
-        private String hostname;
+        private final String hostname;
         // The IP address of the Socket Address
-        private InetAddress addr;
+        private final InetAddress addr;
         // The port number of the Socket Address
-        private int port;
+        private final int port;
 
         private InetSocketAddressHolder(String hostname, InetAddress addr, int port) {
             this.hostname = hostname;
@@ -119,9 +120,8 @@ public class InetSocketAddress
 
         @Override
         public final boolean equals(Object obj) {
-            if (obj == null || !(obj instanceof InetSocketAddressHolder))
+            if (!(obj instanceof InetSocketAddressHolder that))
                 return false;
-            InetSocketAddressHolder that = (InetSocketAddressHolder)obj;
             boolean sameIP;
             if (addr != null)
                 sameIP = addr.equals(that.addr);
@@ -138,7 +138,7 @@ public class InetSocketAddress
             if (addr != null)
                 return addr.hashCode() + port;
             if (hostname != null)
-                return hostname.toLowerCase().hashCode() + port;
+                return hostname.toLowerCase(Locale.ROOT).hashCode() + port;
             return port;
         }
     }
@@ -265,9 +265,9 @@ public class InetSocketAddress
     }
 
     /**
-     * @serialField hostname String
-     * @serialField addr InetAddress
-     * @serialField port int
+     * @serialField hostname String the hostname of the Socket Address
+     * @serialField addr InetAddress the IP address of the Socket Address
+     * @serialField port int the port number of the Socket Address
      */
     @java.io.Serial
     private static final ObjectStreamField[] serialPersistentFields = {
@@ -275,6 +275,12 @@ public class InetSocketAddress
          new ObjectStreamField("addr", InetAddress.class),
          new ObjectStreamField("port", int.class)};
 
+    /**
+     * Writes the state of this object to the stream.
+     *
+     * @param  out the {@code ObjectOutputStream} to which data is written
+     * @throws IOException if an I/O error occurs
+     */
     @java.io.Serial
     private void writeObject(ObjectOutputStream out)
         throws IOException
@@ -287,6 +293,13 @@ public class InetSocketAddress
          out.writeFields();
      }
 
+    /**
+     * Restores the state of this object from the stream.
+     *
+     * @param  in the {@code ObjectInputStream} from which data is read
+     * @throws IOException if an I/O error occurs
+     * @throws ClassNotFoundException if a serialized class cannot be loaded
+     */
     @java.io.Serial
     private void readObject(ObjectInputStream in)
         throws IOException, ClassNotFoundException
@@ -309,6 +322,10 @@ public class InetSocketAddress
         UNSAFE.putReference(this, FIELDS_OFFSET, h);
     }
 
+    /**
+     * Throws {@code InvalidObjectException}, always.
+     * @throws ObjectStreamException always
+     */
     @java.io.Serial
     private void readObjectNoData()
         throws ObjectStreamException
@@ -374,11 +391,18 @@ public class InetSocketAddress
 
     /**
      * Constructs a string representation of this InetSocketAddress.
-     * This String is constructed by calling toString() on the InetAddress
-     * and concatenating the port number (with a colon). If the address
-     * is an IPv6 address, the IPv6 literal is enclosed in square brackets.
+     * This string is constructed by calling {@link InetAddress#toString()}
+     * on the InetAddress and concatenating the port number (with a colon).
+     * <p>
+     * If the address is an IPv6 address, the IPv6 literal is enclosed in
+     * square brackets, for example: {@code "localhost/[0:0:0:0:0:0:0:1]:80"}.
      * If the address is {@linkplain #isUnresolved() unresolved},
-     * {@code <unresolved>} is displayed in place of the address literal.
+     * {@code <unresolved>} is displayed in place of the address literal, for
+     * example {@code "foo/<unresolved>:80"}.
+     * <p>
+     * To retrieve a string representation of the hostname or the address, use
+     * {@link #getHostString()}, rather than parsing the string returned by this
+     * {@link #toString()} method.
      *
      * @return  a string representation of this object.
      */
@@ -409,9 +433,10 @@ public class InetSocketAddress
      */
     @Override
     public final boolean equals(Object obj) {
-        if (obj == null || !(obj instanceof InetSocketAddress))
-            return false;
-        return holder.equals(((InetSocketAddress) obj).holder);
+        if (obj instanceof InetSocketAddress addr) {
+            return holder.equals(addr.holder);
+        }
+        return false;
     }
 
     /**
