@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -172,14 +172,17 @@ public abstract class JarURLConnection extends URLConnection {
             throw new MalformedURLException("no !/ found in url spec:" + spec);
         }
 
-        jarFileURL = new URL(spec.substring(0, separator++));
+        @SuppressWarnings("deprecation")
+        var _unused = jarFileURL = new URL(spec.substring(0, separator++));
+
         /*
          * The url argument may have had a runtime fragment appended, so
          * we need to add a runtime fragment to the jarFileURL to enable
          * runtime versioning when the underlying jar file is opened.
          */
         if ("runtime".equals(url.getRef())) {
-            jarFileURL = new URL(jarFileURL, "#runtime");
+            @SuppressWarnings("deprecation")
+            var _unused2 = jarFileURL = new URL(jarFileURL, "#runtime");
         }
         entryName = null;
 
@@ -217,7 +220,7 @@ public abstract class JarURLConnection extends URLConnection {
      * a connection to an entry of a JAR file, the JAR file object is
      * returned
      *
-     * @exception IOException if an IOException occurs while trying to
+     * @throws    IOException if an IOException occurs while trying to
      * connect to the JAR file for this connection.
      *
      * @see #connect
@@ -230,7 +233,7 @@ public abstract class JarURLConnection extends URLConnection {
      * @return the manifest object corresponding to the JAR file object
      * for this connection.
      *
-     * @exception IOException if getting the JAR file for this
+     * @throws    IOException if getting the JAR file for this
      * connection causes an IOException to be thrown.
      *
      * @see #getJarFile
@@ -247,7 +250,7 @@ public abstract class JarURLConnection extends URLConnection {
      * @return the JAR entry object for this connection, or null if
      * the JAR URL for this connection points to a JAR file.
      *
-     * @exception IOException if getting the JAR file for this
+     * @throws    IOException if getting the JAR file for this
      * connection causes an IOException to be thrown.
      *
      * @see #getJarFile
@@ -264,7 +267,7 @@ public abstract class JarURLConnection extends URLConnection {
      * @return the Attributes object for this connection if the URL
      * for it points to a JAR file entry, null otherwise.
      *
-     * @exception IOException if getting the JAR entry causes an
+     * @throws    IOException if getting the JAR entry causes an
      * IOException to be thrown.
      *
      * @see #getJarEntry
@@ -281,7 +284,7 @@ public abstract class JarURLConnection extends URLConnection {
      * @return the main Attributes for the JAR file for this
      * connection.
      *
-     * @exception IOException if getting the manifest causes an
+     * @throws    IOException if getting the manifest causes an
      * IOException to be thrown.
      *
      * @see #getJarFile
@@ -303,7 +306,7 @@ public abstract class JarURLConnection extends URLConnection {
      * @return the Certificate object for this connection if the URL
      * for it points to a JAR file entry, null otherwise.
      *
-     * @exception IOException if getting the JAR entry causes an
+     * @throws    IOException if getting the JAR entry causes an
      * IOException to be thrown.
      *
      * @see #getJarEntry
