@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,17 +25,25 @@
 
 package java.nio;
 
-import jdk.internal.HotSpotIntrinsicCandidate;
 import jdk.internal.access.JavaNioAccess;
 import jdk.internal.access.SharedSecrets;
-import jdk.internal.access.foreign.MemorySegmentProxy;
 import jdk.internal.access.foreign.UnmapperProxy;
+import jdk.internal.foreign.AbstractMemorySegmentImpl;
+import jdk.internal.foreign.MemorySessionImpl;
+import jdk.internal.misc.ScopedMemoryAccess;
 import jdk.internal.misc.Unsafe;
 import jdk.internal.misc.VM.BufferPool;
+import jdk.internal.util.Preconditions;
 import jdk.internal.vm.annotation.ForceInline;
 
 import java.io.FileDescriptor;
+import java.lang.foreign.MemorySegment;
+import java.lang.ref.Reference;
+import java.util.List;
+import java.util.Objects;
 import java.util.Spliterator;
+import java.util.function.BiFunction;
+import java.util.function.Function;
 
 /**
  * A container for data of a specific primitive type.
@@ -173,26 +181,33 @@ import java.util.Spliterator;
  * specified to return the buffer upon which they are invoked.  This allows
  * method invocations to be chained; for example, the sequence of statements
  *
- * <blockquote><pre>
- * b.flip();
- * b.position(23);
- * b.limit(42);</pre></blockquote>
+ * {@snippet lang=java :
+ *     b.flip();
+ *     b.position(23);
+ *     b.limit(42);
+ * }
  *
  * can be replaced by the single, more compact statement
  *
- * <blockquote><pre>
- * b.flip().position(23).limit(42);</pre></blockquote>
+ * {@snippet lang=java :
+ *     b.flip().position(23).limit(42);
+ * }
  *
  *
  * @author Mark Reinhold
  * @author JSR-51 Expert Group
  * @since 1.4
+ * @sealedGraph
  */
 
-public abstract class Buffer {
+public abstract sealed class Buffer
+    permits ByteBuffer, CharBuffer, DoubleBuffer, FloatBuffer, IntBuffer,
+        LongBuffer, ShortBuffer {
     // Cached unsafe-access object
     static final Unsafe UNSAFE = Unsafe.getUnsafe();
 
+    static final ScopedMemoryAccess SCOPED_MEMORY_ACCESS = ScopedMemoryAccess.getScopedMemoryAccess();
+
     /**
      * The characteristics of Spliterators that traverse and split elements
      * maintained in Buffers.
@@ -204,7 +219,7 @@ public abstract class Buffer {
     private int mark = -1;
     private int position = 0;
     private int limit;
-    private int capacity;
+    private final int capacity;
 
     // Used by heap byte buffers or direct buffers with Unsafe access
     // For heap byte buffers this field will be the address relative to the
@@ -219,12 +234,12 @@ public abstract class Buffer {
     long address;
 
     // Used by buffers generated by the memory access API (JEP-370)
-    final MemorySegmentProxy segment;
+    final MemorySegment segment;
 
 
     // Creates a new buffer with given address and capacity.
     //
-    Buffer(long addr, int cap, MemorySegmentProxy segment) {
+    Buffer(long addr, int cap, MemorySegment segment) {
         this.address = addr;
         this.capacity = cap;
         this.segment = segment;
@@ -233,7 +248,7 @@ public abstract class Buffer {
     // Creates a new buffer with the given mark, position, limit, and capacity,
     // after checking invariants.
     //
-    Buffer(int mark, int pos, int lim, int cap, MemorySegmentProxy segment) {       // package-private
+    Buffer(int mark, int pos, int lim, int cap, MemorySegment segment) {       // package-private
         if (cap < 0)
             throw createCapacityException(cap);
         this.capacity = cap;
@@ -427,9 +442,10 @@ public abstract class Buffer {
      * <p> Invoke this method before using a sequence of channel-read or
      * <i>put</i> operations to fill this buffer.  For example:
      *
-     * <blockquote><pre>
-     * buf.clear();     // Prepare buffer for reading
-     * in.read(buf);    // Read data</pre></blockquote>
+     * {@snippet lang=java :
+     *     buf.clear();     // Prepare buffer for reading
+     *     in.read(buf);    // Read data
+     * }
      *
      * <p> This method does not actually erase the data in the buffer, but it
      * is named as if it did because it will most often be used in situations
@@ -453,11 +469,12 @@ public abstract class Buffer {
      * this method to prepare for a sequence of channel-write or relative
      * <i>get</i> operations.  For example:
      *
-     * <blockquote><pre>
-     * buf.put(magic);    // Prepend header
-     * in.read(buf);      // Read data into rest of buffer
-     * buf.flip();        // Flip buffer
-     * out.write(buf);    // Write header + data to channel</pre></blockquote>
+     * {@snippet lang=java :
+     *     buf.put(magic);    // Prepend header
+     *     in.read(buf);      // Read data into rest of buffer
+     *     buf.flip();        // Flip buffer
+     *     out.write(buf);    // Write header + data to channel
+     * }
      *
      * <p> This method is often used in conjunction with the {@link
      * java.nio.ByteBuffer#compact compact} method when transferring data from
@@ -480,10 +497,11 @@ public abstract class Buffer {
      * operations, assuming that the limit has already been set
      * appropriately.  For example:
      *
-     * <blockquote><pre>
-     * out.write(buf);    // Write remaining data
-     * buf.rewind();      // Rewind buffer
-     * buf.get(array);    // Copy data into array</pre></blockquote>
+     * {@snippet lang=java :
+     *     out.write(buf);    // Write remaining data
+     *     buf.rewind();      // Rewind buffer
+     *     buf.get(array);    // Copy data into array
+     * }
      *
      * @return  This buffer
      */
@@ -729,21 +747,36 @@ public abstract class Buffer {
     }
 
     /**
-     * Checks the given index against the limit, throwing an {@link
-     * IndexOutOfBoundsException} if it is not smaller than the limit
-     * or is smaller than zero.
+     * Exception formatter that returns an {@link IndexOutOfBoundsException}
+     * with no detail message.
      */
-    @HotSpotIntrinsicCandidate
+    private static final BiFunction<String,List<Number>,IndexOutOfBoundsException>
+        IOOBE_FORMATTER = Preconditions.outOfBoundsExceptionFormatter(new Function<>() {
+        @Override
+        public IndexOutOfBoundsException apply(String s) {
+            return new IndexOutOfBoundsException();
+        }
+    });
+
+    /**
+     * Checks the given index against the limit, throwing an {@link
+     * IndexOutOfBoundsException} if it is greater than the limit
+     * or is negative.
+     */
+    @ForceInline
     final int checkIndex(int i) {                       // package-private
-        if ((i < 0) || (i >= limit))
-            throw new IndexOutOfBoundsException();
-        return i;
+        return Preconditions.checkIndex(i, limit, IOOBE_FORMATTER);
     }
 
+    /**
+     * Checks the given index and number of bytes against the range
+     * {@code [0, limit]}, throwing an {@link
+     * IndexOutOfBoundsException} if the index is negative or the index
+     * plus the number of bytes is greater than the limit.
+     */
+    @ForceInline
     final int checkIndex(int i, int nb) {               // package-private
-        if ((i < 0) || (nb > limit - i))
-            throw new IndexOutOfBoundsException();
-        return i;
+        return Preconditions.checkIndex(i, limit - nb + 1, IOOBE_FORMATTER);
     }
 
     final int markValue() {                             // package-private
@@ -755,9 +788,18 @@ public abstract class Buffer {
     }
 
     @ForceInline
-    final void checkSegment() {
+    final MemorySessionImpl session() {
         if (segment != null) {
-            segment.checkValidState();
+            return ((AbstractMemorySegmentImpl)segment).sessionImpl();
+        } else {
+            return null;
+        }
+    }
+
+    final void checkSession() {
+        MemorySessionImpl session = session();
+        if (session != null) {
+            session.checkValidState();
         }
     }
 
@@ -765,50 +807,84 @@ public abstract class Buffer {
         // setup access to this package in SharedSecrets
         SharedSecrets.setJavaNioAccess(
             new JavaNioAccess() {
+
                 @Override
                 public BufferPool getDirectBufferPool() {
                     return Bits.BUFFER_POOL;
                 }
 
                 @Override
-                public ByteBuffer newDirectByteBuffer(long addr, int cap, Object obj, MemorySegmentProxy segment) {
+                public ByteBuffer newDirectByteBuffer(long addr, int cap, Object obj, MemorySegment segment) {
                     return new DirectByteBuffer(addr, cap, obj, segment);
                 }
 
                 @Override
-                public ByteBuffer newMappedByteBuffer(UnmapperProxy unmapperProxy, long address, int cap, Object obj, MemorySegmentProxy segment) {
-                    return new DirectByteBuffer(address, cap, obj, unmapperProxy.fileDescriptor(), unmapperProxy.isSync(), segment);
+                public ByteBuffer newMappedByteBuffer(UnmapperProxy unmapperProxy, long address, int cap, Object obj, MemorySegment segment) {
+                    return unmapperProxy == null
+                            ? new DirectByteBuffer(address, cap, obj, segment)
+                            : new DirectByteBuffer(address, cap, obj, unmapperProxy.fileDescriptor(), unmapperProxy.isSync(), segment);
                 }
 
                 @Override
-                public ByteBuffer newHeapByteBuffer(byte[] hb, int offset, int capacity, MemorySegmentProxy segment) {
-                    return new HeapByteBuffer(hb, offset, capacity, segment);
+                public ByteBuffer newHeapByteBuffer(byte[] hb, int offset, int capacity, MemorySegment segment) {
+                    return new HeapByteBuffer(hb, -1, 0, capacity, capacity, offset, segment);
                 }
 
                 @Override
-                public Object getBufferBase(ByteBuffer bb) {
-                    return bb.base();
+                public Object getBufferBase(Buffer buffer) {
+                    return buffer.base();
                 }
 
                 @Override
-                public long getBufferAddress(ByteBuffer bb) {
-                    return bb.address;
+                public long getBufferAddress(Buffer buffer) {
+                    return buffer.address;
                 }
 
                 @Override
-                public UnmapperProxy unmapper(ByteBuffer bb) {
-                    if (bb instanceof MappedByteBuffer) {
-                        return ((MappedByteBuffer)bb).unmapper();
+                public UnmapperProxy unmapper(Buffer buffer) {
+                    if (buffer instanceof MappedByteBuffer mbb) {
+                        return mbb.unmapper();
                     } else {
                         return null;
                     }
                 }
 
                 @Override
-                public MemorySegmentProxy bufferSegment(Buffer buffer) {
+                public MemorySegment bufferSegment(Buffer buffer) {
                     return buffer.segment;
                 }
 
+                @Override
+                public void acquireSession(Buffer buffer) {
+                    var scope = buffer.session();
+                    if (scope != null) {
+                        scope.acquire0();
+                    }
+                }
+
+                @Override
+                public void releaseSession(Buffer buffer) {
+                    try {
+                        var scope = buffer.session();
+                        if (scope != null) {
+                            scope.release0();
+                        }
+                    } finally {
+                        Reference.reachabilityFence(buffer);
+                    }
+                }
+
+                @Override
+                public boolean isThreadConfined(Buffer buffer) {
+                    var scope = buffer.session();
+                    return scope != null && scope.ownerThread() != null;
+                }
+
+                @Override
+                public boolean hasSession(Buffer buffer) {
+                    return buffer.session() != null;
+                }
+
                 @Override
                 public void force(FileDescriptor fd, long address, boolean isSync, long offset, long size) {
                     MappedMemoryUtils.force(fd, address, isSync, offset, size);
@@ -828,6 +904,21 @@ public abstract class Buffer {
                 public boolean isLoaded(long address, boolean isSync, long size) {
                     return MappedMemoryUtils.isLoaded(address, isSync, size);
                 }
+
+                @Override
+                public void reserveMemory(long size, long cap) {
+                    Bits.reserveMemory(size, cap);
+                }
+
+                @Override
+                public void unreserveMemory(long size, long cap) {
+                    Bits.unreserveMemory(size, cap);
+                }
+
+                @Override
+                public int pageSize() {
+                    return Bits.pageSize();
+                }
             });
     }
 
