@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -28,8 +28,10 @@ package java.nio.file;
 import java.io.File;
 import java.io.IOException;
 import java.net.URI;
+import java.nio.file.spi.FileSystemProvider;
 import java.util.Iterator;
 import java.util.NoSuchElementException;
+import java.util.Objects;
 
 /**
  * An object that may be used to locate a file in a file system. It will
@@ -71,10 +73,10 @@ import java.util.NoSuchElementException;
  * java.io.BufferedReader} to read text from a file "{@code access.log}". The
  * file is located in a directory "{@code logs}" relative to the current working
  * directory and is UTF-8 encoded.
- * <pre>
+ * {@snippet lang=java :
  *     Path path = FileSystems.getDefault().getPath("logs", "access.log");
  *     BufferedReader reader = Files.newBufferedReader(path, StandardCharsets.UTF_8);
- * </pre>
+ * }
  *
  * <a id="interop"></a><h2>Interoperability</h2>
  * <p> Paths associated with the default {@link
@@ -93,12 +95,124 @@ import java.util.NoSuchElementException;
  * multiple concurrent threads.
  *
  * @since 1.7
- * @see Paths
  */
 
 public interface Path
     extends Comparable<Path>, Iterable<Path>, Watchable
 {
+    /**
+     * Returns a {@code Path} by converting a path string, or a sequence of
+     * strings that when joined form a path string. If {@code more} does not
+     * specify any elements then the value of the {@code first} parameter is
+     * the path string to convert. If {@code more} specifies one or more
+     * elements then each non-empty string, including {@code first}, is
+     * considered to be a sequence of name elements and is joined to form a
+     * path string. The details as to how the Strings are joined is provider
+     * specific but typically they will be joined using the
+     * {@link FileSystem#getSeparator name-separator} as the separator.
+     * For example, if the name separator is "{@code /}" and
+     * {@code getPath("/foo","bar","gus")} is invoked, then the path string
+     * {@code "/foo/bar/gus"} is converted to a {@code Path}. A {@code Path}
+     * representing an empty path is returned if {@code first} is the empty
+     * string and {@code more} does not contain any non-empty strings.
+     *
+     * <p> The {@code Path} is obtained by invoking the {@link FileSystem#getPath
+     * getPath} method of the {@link FileSystems#getDefault default} {@link
+     * FileSystem}.
+     *
+     * <p> Note that while this method is very convenient, using it will imply
+     * an assumed reference to the default {@code FileSystem} and limit the
+     * utility of the calling code. Hence it should not be used in library code
+     * intended for flexible reuse. A more flexible alternative is to use an
+     * existing {@code Path} instance as an anchor, such as:
+     * {@snippet lang=java :
+     *     Path dir = ...
+     *     Path path = dir.resolve("file");
+     * }
+     *
+     * @param   first
+     *          the path string or initial part of the path string
+     * @param   more
+     *          additional strings to be joined to form the path string
+     *
+     * @return  the resulting {@code Path}
+     *
+     * @throws  InvalidPathException
+     *          if the path string cannot be converted to a {@code Path}
+     *
+     * @see FileSystem#getPath
+     *
+     * @since 11
+     */
+    public static Path of(String first, String... more) {
+        return FileSystems.getDefault().getPath(first, more);
+    }
+
+    /**
+     * Returns a {@code Path} by converting a URI.
+     *
+     * <p> This method iterates over the {@link FileSystemProvider#installedProviders()
+     * installed} providers to locate the provider that is identified by the
+     * URI {@link URI#getScheme scheme} of the given URI. URI schemes are
+     * compared without regard to case. If the provider is found then its {@link
+     * FileSystemProvider#getPath getPath} method is invoked to convert the
+     * URI.
+     *
+     * <p> In the case of the default provider, identified by the URI scheme
+     * "file", the given URI has a non-empty path component, and undefined query
+     * and fragment components. Whether the authority component may be present
+     * is platform specific. The returned {@code Path} is associated with the
+     * {@link FileSystems#getDefault default} file system.
+     *
+     * <p> The default provider provides a similar <em>round-trip</em> guarantee
+     * to the {@link java.io.File} class. For a given {@code Path} <i>p</i> it
+     * is guaranteed that
+     * <blockquote>{@code
+     * Path.of(}<i>p</i>{@code .}{@link Path#toUri() toUri}{@code ()).equals(}
+     * <i>p</i>{@code .}{@link Path#toAbsolutePath() toAbsolutePath}{@code ())}
+     * </blockquote>
+     * so long as the original {@code Path}, the {@code URI}, and the new {@code
+     * Path} are all created in (possibly different invocations of) the same
+     * Java virtual machine. Whether other providers make any guarantees is
+     * provider specific and therefore unspecified.
+     *
+     * @param   uri
+     *          the URI to convert
+     *
+     * @return  the resulting {@code Path}
+     *
+     * @throws  IllegalArgumentException
+     *          if preconditions on the {@code uri} parameter do not hold. The
+     *          format of the URI is provider specific.
+     * @throws  FileSystemNotFoundException
+     *          The file system, identified by the URI, does not exist and
+     *          cannot be created automatically, or the provider identified by
+     *          the URI's scheme component is not installed
+     * @throws  SecurityException
+     *          if a security manager is installed and it denies an unspecified
+     *          permission to access the file system
+     *
+     * @since 11
+     */
+    public static Path of(URI uri) {
+        String scheme =  uri.getScheme();
+        if (scheme == null)
+            throw new IllegalArgumentException("Missing scheme");
+
+        // check for default provider to avoid loading of installed providers
+        if (scheme.equalsIgnoreCase("file"))
+            return FileSystems.getDefault().provider().getPath(uri);
+
+        // try to find provider
+        for (FileSystemProvider provider: FileSystemProvider.installedProviders()) {
+            if (provider.getScheme().equalsIgnoreCase(scheme)) {
+                return provider.getPath(uri);
+            }
+        }
+
+        throw new FileSystemNotFoundException("Provider \"" + scheme + "\" not installed");
+    }
+
     /**
      * Returns the file system that created this object.
      *
@@ -152,9 +266,9 @@ public interface Path
      *
      * <p> If this path has more than one element, and no root component, then
      * this method is equivalent to evaluating the expression:
-     * <blockquote><pre>
-     * subpath(0,&nbsp;getNameCount()-1);
-     * </pre></blockquote>
+     * {@snippet lang=java :
+     *     subpath(0, getNameCount()-1);
+     * }
      *
      * @return  a path representing the path's parent
      */
@@ -249,9 +363,9 @@ public interface Path
      *
      * @implSpec
      * The default implementation is equivalent for this path to:
-     * <pre>{@code
+     * {@snippet lang=java :
      *     startsWith(getFileSystem().getPath(other));
-     * }</pre>
+     * }
      *
      * @param   other
      *          the given path string
@@ -300,14 +414,14 @@ public interface Path
      * "{@code foo/bar}" ends with "{@code foo/bar}" and "{@code bar}". It does
      * not end with "{@code r}" or "{@code /bar}". Note that trailing separators
      * are not taken into account, and so invoking this method on the {@code
-     * Path}"{@code foo/bar}" with the {@code String} "{@code bar/}" returns
+     * Path} "{@code foo/bar}" with the {@code String} "{@code bar/}" returns
      * {@code true}.
      *
      * @implSpec
      * The default implementation is equivalent for this path to:
-     * <pre>{@code
+     * {@snippet lang=java :
      *     endsWith(getFileSystem().getPath(other));
-     * }</pre>
+     * }
      *
      * @param   other
      *          the given path string
@@ -384,9 +498,9 @@ public interface Path
      *
      * @implSpec
      * The default implementation is equivalent for this path to:
-     * <pre>{@code
+     * {@snippet lang=java :
      *     resolve(getFileSystem().getPath(other));
-     * }</pre>
+     * }
      *
      * @param   other
      *          the path string to resolve against this path
@@ -416,9 +530,9 @@ public interface Path
      *
      * @implSpec
      * The default implementation is equivalent for this path to:
-     * <pre>{@code
+     * {@snippet lang=java :
      *     (getParent() == null) ? other : getParent().resolve(other);
-     * }</pre>
+     * }
      * unless {@code other == null}, in which case a
      * {@code NullPointerException} is thrown.
      *
@@ -443,9 +557,9 @@ public interface Path
      *
      * @implSpec
      * The default implementation is equivalent for this path to:
-     * <pre>{@code
+     * {@snippet lang=java :
      *     resolveSibling(getFileSystem().getPath(other));
-     * }</pre>
+     * }
      *
      * @param   other
      *          the path string to resolve against this path's parent
@@ -527,7 +641,7 @@ public interface Path
      * to the {@link java.io.File} class. For a given {@code Path} <i>p</i> it
      * is guaranteed that
      * <blockquote>
-     * {@link Paths#get(URI) Paths.get}{@code (}<i>p</i>{@code .toUri()).equals(}<i>p</i>
+     * {@link Path#of(URI) Path.of}{@code (}<i>p</i>{@code .toUri()).equals(}<i>p</i>
      * {@code .}{@link #toAbsolutePath() toAbsolutePath}{@code ())}
      * </blockquote>
      * so long as the original {@code Path}, the {@code URI}, and the new {@code
@@ -639,9 +753,9 @@ public interface Path
      *
      * @implSpec
      * The default implementation is equivalent for this path to:
-     * <pre>{@code
+     * {@snippet lang=java :
      *     new File(toString());
-     * }</pre>
+     * }
      * if the {@code FileSystem} which created this {@code Path} is the default
      * file system; otherwise an {@code UnsupportedOperationException} is
      * thrown.
@@ -731,25 +845,26 @@ public interface Path
      *
      * <p> An invocation of this method behaves in exactly the same way as the
      * invocation
-     * <pre>
-     *     watchable.{@link #register(WatchService,WatchEvent.Kind[],WatchEvent.Modifier[]) register}(watcher, events, new WatchEvent.Modifier[0]);
-     * </pre>
+     * {@snippet lang=java :
+     *     // @link substring="register" target="Watchable#register" :
+     *     register(watcher, events, new WatchEvent.Modifier[0]);
+     * }
      *
      * <p> <b>Usage Example:</b>
      * Suppose we wish to register a directory for entry create, delete, and modify
      * events:
-     * <pre>
+     * {@snippet lang=java :
      *     Path dir = ...
      *     WatchService watcher = ...
      *
      *     WatchKey key = dir.register(watcher, ENTRY_CREATE, ENTRY_DELETE, ENTRY_MODIFY);
-     * </pre>
+     * }
      *
      * @implSpec
      * The default implementation is equivalent for this path to:
-     * <pre>{@code
+     * {@snippet lang=java :
      *     register(watcher, events, new WatchEvent.Modifier[0]);
-     * }</pre>
+     * }
      *
      * @param   watcher
      *          The watch service to which this object is to be registered
@@ -798,7 +913,7 @@ public interface Path
      * {@code getName(index)}, where {@code index} ranges from zero to
      * {@code getNameCount() - 1}, inclusive.
      *
-     * @return  an iterator over the name elements of this path.
+     * @return  an iterator over the name elements of this path
      */
     @Override
     default Iterator<Path> iterator() {
