@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -34,6 +34,8 @@ import java.util.Hashtable;
 import java.io.ByteArrayInputStream;
 import java.io.IOException;
 import java.security.cert.*;
+import java.util.Objects;
+
 import sun.net.util.URLUtil;
 import sun.security.util.IOUtils;
 
@@ -50,6 +52,7 @@ import sun.security.util.IOUtils;
 
 public class CodeSource implements java.io.Serializable {
 
+    @java.io.Serial
     private static final long serialVersionUID = 4977541819976013951L;
 
     /**
@@ -76,16 +79,16 @@ public class CodeSource implements java.io.Serializable {
     private transient CertificateFactory factory = null;
 
     /**
-     * A String form of the URL for use as a key in HashMaps/Sets. The String
-     * form should be behave in the same manner as the URL when compared for
-     * equality in a HashMap/Set, except that no nameservice lookup is done
-     * on the hostname (only string comparison), and the fragment is not
-     * considered.
+     * A {@code String} form of the URL for use as a key in HashMaps/Sets.
+     * The {@code String} form should behave in the same manner as the URL
+     * when compared for equality in a HashMap/Set, except that no nameservice
+     * lookup is done on the hostname (only string comparison), and the
+     * fragment is not considered.
      */
     private transient String locationNoFragString;
 
     /**
-     * Constructs a CodeSource and associates it with the specified
+     * Constructs a {@code CodeSource} and associates it with the specified
      * location and set of certificates.
      *
      * @param url the location (URL).  It may be {@code null}.
@@ -105,7 +108,7 @@ public class CodeSource implements java.io.Serializable {
     }
 
     /**
-     * Constructs a CodeSource and associates it with the specified
+     * Constructs a {@code CodeSource} and associates it with the specified
      * location and set of code signers.
      *
      * @param url the location (URL).  It may be {@code null}.
@@ -141,14 +144,15 @@ public class CodeSource implements java.io.Serializable {
 
     /**
      * Tests for equality between the specified object and this
-     * object. Two CodeSource objects are considered equal if their
+     * object. Two {@code CodeSource} objects are considered equal if their
      * locations are of identical value and if their signer certificate
      * chains are of identical value. It is not required that
      * the certificate chains be in the same order.
      *
      * @param obj the object to test for equality with this object.
      *
-     * @return true if the objects are considered equal, false otherwise.
+     * @return {@code true} if the objects are considered equal,
+     * {@code false} otherwise.
      */
     @Override
     public boolean equals(Object obj) {
@@ -156,26 +160,13 @@ public class CodeSource implements java.io.Serializable {
             return true;
 
         // objects types must be equal
-        if (!(obj instanceof CodeSource))
-            return false;
-
-        CodeSource cs = (CodeSource) obj;
-
-        // URLs must match
-        if (location == null) {
-            // if location is null, then cs.location must be null as well
-            if (cs.location != null) return false;
-        } else {
-            // if location is not null, then it must equal cs.location
-            if (!location.equals(cs.location)) return false;
-        }
-
-        // certs must match
-        return matchCerts(cs, true);
+        return (obj instanceof CodeSource other)
+                && Objects.equals(location, other.location)
+                && matchCerts(other, true);
     }
 
     /**
-     * Returns the location associated with this CodeSource.
+     * Returns the location associated with this {@code CodeSource}.
      *
      * @return the location (URL), or {@code null} if no URL was supplied
      * during construction.
@@ -187,20 +178,21 @@ public class CodeSource implements java.io.Serializable {
     }
 
     /**
-     * Returns a String form of the URL for use as a key in HashMaps/Sets.
+     * Returns a {@code String} form of the URL for use as a key in
+     * HashMaps/Sets.
      */
     String getLocationNoFragString() {
         return locationNoFragString;
     }
 
     /**
-     * Returns the certificates associated with this CodeSource.
+     * Returns the certificates associated with this {@code CodeSource}.
      * <p>
-     * If this CodeSource object was created using the
+     * If this {@code CodeSource} object was created using the
      * {@link #CodeSource(URL url, CodeSigner[] signers)}
      * constructor then its certificate chains are extracted and used to
-     * create an array of Certificate objects. Each signer certificate is
-     * followed by its supporting certificate chain (which may be empty).
+     * create an array of {@code Certificate} objects. Each signer certificate
+     * is followed by its supporting certificate chain (which may be empty).
      * Each signer certificate and its supporting certificate chain is ordered
      * bottom-to-top (i.e., with the signer certificate first and the (root)
      * certificate authority last).
@@ -221,7 +213,7 @@ public class CodeSource implements java.io.Serializable {
                     signers[i].getSignerCertPath().getCertificates());
             }
             certs = certChains.toArray(
-                        new java.security.cert.Certificate[certChains.size()]);
+                    new java.security.cert.Certificate[0]);
             return certs.clone();
 
         } else {
@@ -230,13 +222,13 @@ public class CodeSource implements java.io.Serializable {
     }
 
     /**
-     * Returns the code signers associated with this CodeSource.
+     * Returns the code signers associated with this {@code CodeSource}.
      * <p>
-     * If this CodeSource object was created using the
+     * If this {@code CodeSource} object was created using the
      * {@link #CodeSource(URL url, java.security.cert.Certificate[] certs)}
      * constructor then its certificate chains are extracted and used to
-     * create an array of CodeSigner objects. Note that only X.509 certificates
-     * are examined - all other certificate types are ignored.
+     * create an array of {@code CodeSigner} objects. Note that only X.509
+     * certificates are examined - all other certificate types are ignored.
      *
      * @return a copy of the code signer array, or {@code null} if there
      * is none.
@@ -258,10 +250,12 @@ public class CodeSource implements java.io.Serializable {
     }
 
     /**
-     * Returns true if this CodeSource object "implies" the specified CodeSource.
+     * Returns true if this {@code CodeSource} object "implies" the specified
+     * {@code CodeSource}.
      * <p>
      * More specifically, this method makes the following checks.
-     * If any fail, it returns false. If they all succeed, it returns true.
+     * If any fail, it returns {@code false}. If they all succeed, it returns
+     * {@code true}.
      * <ul>
      * <li> <i>codesource</i> must not be null.
      * <li> If this object's certificates are not null, then all
@@ -308,9 +302,9 @@ public class CodeSource implements java.io.Serializable {
      * </ul>
      * <p>
      * For example, the codesource objects with the following locations
-     * and null certificates all imply
-     * the codesource with the location "http://www.example.com/classes/foo.jar"
-     * and null certificates:
+     * and {@code null} certificates all imply the codesource with the location
+     * {@code http://www.example.com/classes/foo.jar}
+     * and {@code null} certificates:
      * <pre>
      *     http:
      *     http://*.example.com/classes/*
@@ -318,13 +312,14 @@ public class CodeSource implements java.io.Serializable {
      *     http://www.example.com/classes/foo.jar
      * </pre>
      *
-     * Note that if this CodeSource has a null location and a null
-     * certificate chain, then it implies every other CodeSource.
+     * Note that if this {@code CodeSource} has a {@code null} location and a
+     * {@code null} certificate chain, then it implies every other
+     * {@code CodeSource}.
      *
-     * @param codesource CodeSource to compare against.
+     * @param codesource {@code CodeSource} to compare against.
      *
-     * @return true if the specified codesource is implied by this codesource,
-     * false if not.
+     * @return {@code true} if the specified codesource is implied by this
+     * codesource, {@code false} if not.
      */
     public boolean implies(CodeSource codesource)
     {
@@ -335,12 +330,12 @@ public class CodeSource implements java.io.Serializable {
     }
 
     /**
-     * Returns true if all the certs in this
-     * CodeSource are also in <i>that</i>.
+     * Returns {@code true} if all the certs in this
+     * {@code CodeSource} are also in <i>that</i>.
      *
-     * @param that the CodeSource to check against.
-     * @param strict if true then a strict equality match is performed.
-     *               Otherwise a subset match is performed.
+     * @param that the {@code CodeSource} to check against.
+     * @param strict if {@code true} then a strict equality match is performed.
+     *               Otherwise, a subset match is performed.
      */
     boolean matchCerts(CodeSource that, boolean strict)
     {
@@ -393,9 +388,9 @@ public class CodeSource implements java.io.Serializable {
 
 
     /**
-     * Returns true if two CodeSource's have the "same" location.
+     * Returns {@code true} if two CodeSource's have the "same" location.
      *
-     * @param that CodeSource to compare against
+     * @param that {@code CodeSource} to compare against
      */
     private boolean matchLocation(CodeSource that) {
         if (location == null)
@@ -471,9 +466,7 @@ public class CodeSource implements java.io.Serializable {
                 if (that.sp == null) {
                     that.sp = new SocketPermission(thatHost, "resolve");
                 }
-                if (!this.sp.implies(that.sp)) {
-                    return false;
-                }
+                return this.sp.implies(that.sp);
             }
         }
         // everything matches
@@ -481,10 +474,10 @@ public class CodeSource implements java.io.Serializable {
     }
 
     /**
-     * Returns a string describing this CodeSource, telling its
+     * Returns a string describing this {@code CodeSource}, telling its
      * URL and certificates.
      *
-     * @return information about this CodeSource.
+     * @return information about this {@code CodeSource}.
      */
     @Override
     public String toString() {
@@ -521,7 +514,11 @@ public class CodeSource implements java.io.Serializable {
      * followed by the certificate encoding itself which is written out as an
      * array of bytes. Finally, if any code signers are present then the array
      * of code signers is serialized and written out too.
+     *
+     * @param  oos the {@code ObjectOutputStream} to which data is written
+     * @throws IOException if an I/O error occurs
      */
+    @java.io.Serial
     private void writeObject(java.io.ObjectOutputStream oos)
         throws IOException
     {
@@ -555,7 +552,12 @@ public class CodeSource implements java.io.Serializable {
 
     /**
      * Restores this object from a stream (i.e., deserializes it).
+     *
+     * @param  ois the {@code ObjectInputStream} from which data is read
+     * @throws IOException if an I/O error occurs
+     * @throws ClassNotFoundException if a serialized class cannot be loaded
      */
+    @java.io.Serial
     private void readObject(java.io.ObjectInputStream ois)
         throws IOException, ClassNotFoundException
     {
@@ -571,7 +573,7 @@ public class CodeSource implements java.io.Serializable {
             // we know of 3 different cert types: X.509, PGP, SDSI, which
             // could all be present in the stream at the same time
             cfs = new Hashtable<>(3);
-            certList = new ArrayList<>(size > 20 ? 20 : size);
+            certList = new ArrayList<>(Math.min(size, 20));
         } else if (size < 0) {
             throw new IOException("size cannot be negative");
         }
@@ -595,7 +597,7 @@ public class CodeSource implements java.io.Serializable {
                 cfs.put(certType, cf);
             }
             // parse the certificate
-            byte[] encoded = IOUtils.readNBytes(ois, ois.readInt());
+            byte[] encoded = IOUtils.readExactlyNBytes(ois, ois.readInt());
             ByteArrayInputStream bais = new ByteArrayInputStream(encoded);
             try {
                 certList.add(cf.generateCertificate(bais));
@@ -626,7 +628,7 @@ public class CodeSource implements java.io.Serializable {
      * The array of certificates is a concatenation of certificate chains
      * where the initial certificate in each chain is the end-entity cert.
      *
-     * @return an array of code signers or null if none are generated.
+     * @return an array of code signers or {@code null} if none are generated.
      */
     private CodeSigner[] convertCertArrayToSignerArray(
         java.security.cert.Certificate[] certs) {
@@ -666,7 +668,7 @@ public class CodeSource implements java.io.Serializable {
             if (signers.isEmpty()) {
                 return null;
             } else {
-                return signers.toArray(new CodeSigner[signers.size()]);
+                return signers.toArray(new CodeSigner[0]);
             }
 
         } catch (CertificateException e) {
