@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -29,9 +29,6 @@ import java.io.InputStream;
 import java.util.Collection;
 import java.util.Iterator;
 import java.util.List;
-import java.security.Provider;
-import java.security.NoSuchAlgorithmException;
-import java.security.NoSuchProviderException;
 
 /**
  * This class defines the <i>Service Provider Interface</i> (<b>SPI</b>)
@@ -65,6 +62,11 @@ import java.security.NoSuchProviderException;
 
 public abstract class CertificateFactorySpi {
 
+    /**
+     * Constructor for subclasses to call.
+     */
+    public CertificateFactorySpi() {}
+
     /**
      * Generates a certificate object and initializes it with
      * the data read from the input stream {@code inStream}.
