@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,18 +25,15 @@
 
 package java.security.cert;
 
-import java.security.*;
-import java.security.spec.*;
+import sun.security.util.SignatureUtil;
+import sun.security.x509.X509CRLImpl;
 
 import javax.security.auth.x500.X500Principal;
-
 import java.math.BigInteger;
+import java.security.*;
+import java.util.Arrays;
 import java.util.Date;
 import java.util.Set;
-import java.util.Arrays;
-
-import sun.security.x509.X509CRLImpl;
-import sun.security.util.SignatureUtil;
 
 /**
  * <p>
@@ -66,7 +63,7 @@ import sun.security.util.SignatureUtil;
  * </pre>
  * <p>
  * More information can be found in
- * <a href="http://tools.ietf.org/html/rfc5280">RFC 5280: Internet X.509
+ * <a href="https://tools.ietf.org/html/rfc5280">RFC 5280: Internet X.509
  * Public Key Infrastructure Certificate and CRL Profile</a>.
  * <p>
  * The ASN.1 definition of {@code tbsCertList} is:
@@ -257,7 +254,7 @@ public abstract class X509CRL extends CRL implements X509Extension {
         byte[] tbsCRL = getTBSCertList();
         sig.update(tbsCRL, 0, tbsCRL.length);
 
-        if (sig.verify(getSignature()) == false) {
+        if (!sig.verify(getSignature())) {
             throw new SignatureException("Signature does not match.");
         }
     }
@@ -279,12 +276,6 @@ public abstract class X509CRL extends CRL implements X509Extension {
     public abstract int getVersion();
 
     /**
-     * <strong>Denigrated</strong>, replaced by {@linkplain
-     * #getIssuerX500Principal()}. This method returns the {@code issuer}
-     * as an implementation specific Principal object, which should not be
-     * relied upon by portable code.
-     *
-     * <p>
      * Gets the {@code issuer} (issuer distinguished name) value from
      * the CRL. The issuer name identifies the entity that signed (and
      * issued) the CRL.
@@ -316,7 +307,13 @@ public abstract class X509CRL extends CRL implements X509Extension {
      * {@code TeletexString} or {@code UniversalString}.
      *
      * @return a Principal whose name is the issuer distinguished name.
+     *
+     * @deprecated Use {@link #getIssuerX500Principal} instead. This method
+     * returns the {@code issuer} as an implementation specific
+     * {@code Principal} object, which should not be relied upon by portable
+     * code.
      */
+    @Deprecated(since="16")
     public abstract Principal getIssuerDN();
 
     /**
@@ -390,7 +387,7 @@ public abstract class X509CRL extends CRL implements X509Extension {
     public X509CRLEntry getRevokedCertificate(X509Certificate certificate) {
         X500Principal certIssuer = certificate.getIssuerX500Principal();
         X500Principal crlIssuer = getIssuerX500Principal();
-        if (certIssuer.equals(crlIssuer) == false) {
+        if (!certIssuer.equals(crlIssuer)) {
             return null;
         }
         return getRevokedCertificate(certificate.getSerialNumber());
