@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -53,12 +53,13 @@ public interface DSAPrivateKey extends DSAKey, java.security.PrivateKey {
     */
     @Deprecated
     @SuppressWarnings("serial")
-    static final long serialVersionUID = 7776497482533790279L;
+    @java.io.Serial
+   long serialVersionUID = 7776497482533790279L;
 
     /**
      * Returns the value of the private key, {@code x}.
      *
      * @return the value of the private key, {@code x}.
      */
-    public BigInteger getX();
+    BigInteger getX();
 }
