@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1996, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -37,6 +37,7 @@ import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.stream.Stream;
 
 import jdk.internal.reflect.CallerSensitive;
+import jdk.internal.reflect.CallerSensitiveAdapter;
 import jdk.internal.reflect.Reflection;
 
 
@@ -51,7 +52,7 @@ import jdk.internal.reflect.Reflection;
  * As part of its initialization, the {@code DriverManager} class will
  * attempt to load available JDBC drivers by using:
  * <ul>
- * <li>The {@code jdbc.drivers} system property which contains a
+ * <li>The {@systemProperty jdbc.drivers} system property which contains a
  * colon separated list of fully qualified class names of JDBC drivers. Each
  * driver is loaded using the {@linkplain ClassLoader#getSystemClassLoader
  * system class loader}:
@@ -647,6 +648,7 @@ public class DriverManager {
 
 
     //  Worker method called by the public getConnection() methods.
+    @CallerSensitiveAdapter
     private static Connection getConnection(
         String url, java.util.Properties info, Class<?> caller) throws SQLException {
         /*
