@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2021, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -146,8 +146,7 @@ public final class ZoneRules implements Serializable {
     /**
      * The map of recent transitions.
      */
-    private final transient ConcurrentMap<Integer, ZoneOffsetTransition[]> lastRulesCache =
-                new ConcurrentHashMap<Integer, ZoneOffsetTransition[]>();
+    private final transient ConcurrentMap<Integer, ZoneOffsetTransition[]> lastRulesCache;
     /**
      * The zero-length long array.
      */
@@ -259,10 +258,18 @@ public final class ZoneRules implements Serializable {
         }
 
         // last rules
-        if (lastRules.size() > 16) {
-            throw new IllegalArgumentException("Too many transition rules");
+        if (lastRules.size() > 0) {
+            Object[] temp = lastRules.toArray();
+            ZoneOffsetTransitionRule[] rulesArray = Arrays.copyOf(temp, temp.length, ZoneOffsetTransitionRule[].class);
+            if (rulesArray.length > 16) {
+                throw new IllegalArgumentException("Too many transition rules");
+            }
+            this.lastRules = rulesArray;
+            this.lastRulesCache = new ConcurrentHashMap<>();
+        } else {
+            this.lastRules = EMPTY_LASTRULES;
+            this.lastRulesCache = null;
         }
-        this.lastRules = lastRules.toArray(new ZoneOffsetTransitionRule[lastRules.size()]);
     }
 
     /**
@@ -286,6 +293,7 @@ public final class ZoneRules implements Serializable {
         this.savingsInstantTransitions = savingsInstantTransitions;
         this.wallOffsets = wallOffsets;
         this.lastRules = lastRules;
+        this.lastRulesCache = (lastRules.length > 0) ? new ConcurrentHashMap<>() : null;
 
         if (savingsInstantTransitions.length == 0) {
             this.savingsLocalTransitions = EMPTY_LDT_ARRAY;
@@ -322,6 +330,7 @@ public final class ZoneRules implements Serializable {
         this.savingsLocalTransitions = EMPTY_LDT_ARRAY;
         this.wallOffsets = standardOffsets;
         this.lastRules = EMPTY_LASTRULES;
+        this.lastRulesCache = null;
     }
 
     /**
@@ -965,11 +974,11 @@ public final class ZoneRules implements Serializable {
             doyEst = zeroDay - (365 * yearEst + yearEst / 4 - yearEst / 100 + yearEst / 400);
         }
         yearEst += adjust;  // reset any negative year
-        int marchDoy0 = (int) doyEst;
 
-        // convert march-based values back to january-based
-        int marchMonth0 = (marchDoy0 * 5 + 2) / 153;
-        yearEst += marchMonth0 / 10;
+        // convert march-based values back to january-based, adjust year
+        if (doyEst >= 306) {
+            yearEst++;
+        }
 
         // Cap to the max value
         return (int)Math.min(yearEst, Year.MAX_VALUE);
@@ -1037,15 +1046,12 @@ public final class ZoneRules implements Serializable {
         if (this == otherRules) {
            return true;
         }
-        if (otherRules instanceof ZoneRules) {
-            ZoneRules other = (ZoneRules) otherRules;
-            return Arrays.equals(standardTransitions, other.standardTransitions) &&
-                    Arrays.equals(standardOffsets, other.standardOffsets) &&
-                    Arrays.equals(savingsInstantTransitions, other.savingsInstantTransitions) &&
-                    Arrays.equals(wallOffsets, other.wallOffsets) &&
-                    Arrays.equals(lastRules, other.lastRules);
-        }
-        return false;
+        return (otherRules instanceof ZoneRules other)
+                && Arrays.equals(standardTransitions, other.standardTransitions)
+                && Arrays.equals(standardOffsets, other.standardOffsets)
+                && Arrays.equals(savingsInstantTransitions, other.savingsInstantTransitions)
+                && Arrays.equals(wallOffsets, other.wallOffsets)
+                && Arrays.equals(lastRules, other.lastRules);
     }
 
     /**
