@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2005, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -36,8 +36,9 @@ package java.util;
  *
  * @since 1.5
  */
-public class IllegalFormatCodePointException extends IllegalFormatException {
+public non-sealed class IllegalFormatCodePointException extends IllegalFormatException {
 
+    @java.io.Serial
     private static final long serialVersionUID = 19080630L;
 
     private int c;
