@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -187,15 +187,11 @@ public final class PropertyPermission extends BasicPermission {
      */
     @Override
     public boolean implies(Permission p) {
-        if (!(p instanceof PropertyPermission))
-            return false;
-
-        PropertyPermission that = (PropertyPermission) p;
-
         // we get the effective mask. i.e., the "and" of this and that.
         // They must be equal to that.mask for implies to return true.
-
-        return ((this.mask & that.mask) == that.mask) && super.implies(that);
+        return p instanceof PropertyPermission that
+                && ((this.mask & that.mask) == that.mask)
+                && super.implies(that);
     }
 
     /**
@@ -211,13 +207,9 @@ public final class PropertyPermission extends BasicPermission {
         if (obj == this)
             return true;
 
-        if (! (obj instanceof PropertyPermission))
-            return false;
-
-        PropertyPermission that = (PropertyPermission) obj;
-
-        return (this.mask == that.mask) &&
-            (this.getName().equals(that.getName()));
+        return obj instanceof PropertyPermission that
+                && this.mask == that.mask
+                && this.getName().equals(that.getName());
     }
 
     /**
@@ -334,16 +326,12 @@ public final class PropertyPermission extends BasicPermission {
      * @return the canonical string representation of the actions.
      */
     static String getActions(int mask) {
-        switch (mask & (READ|WRITE)) {
-            case READ:
-                return SecurityConstants.PROPERTY_READ_ACTION;
-            case WRITE:
-                return SecurityConstants.PROPERTY_WRITE_ACTION;
-            case READ|WRITE:
-                return SecurityConstants.PROPERTY_RW_ACTION;
-            default:
-                return "";
-        }
+        return switch (mask & (READ | WRITE)) {
+            case READ         -> SecurityConstants.PROPERTY_READ_ACTION;
+            case WRITE        -> SecurityConstants.PROPERTY_WRITE_ACTION;
+            case READ | WRITE -> SecurityConstants.PROPERTY_RW_ACTION;
+            default           -> "";
+        };
     }
 
     /**
@@ -471,38 +459,30 @@ final class PropertyPermissionCollection extends PermissionCollection
      */
     @Override
     public void add(Permission permission) {
-        if (! (permission instanceof PropertyPermission))
+        if (! (permission instanceof PropertyPermission pp))
             throw new IllegalArgumentException("invalid permission: "+
                                                permission);
         if (isReadOnly())
             throw new SecurityException(
                 "attempt to add a Permission to a readonly PermissionCollection");
 
-        PropertyPermission pp = (PropertyPermission) permission;
         String propName = pp.getName();
 
         // Add permission to map if it is absent, or replace with new
-        // permission if applicable. NOTE: cannot use lambda for
-        // remappingFunction parameter until JDK-8076596 is fixed.
-        perms.merge(propName, pp,
-            new java.util.function.BiFunction<>() {
-                @Override
-                public PropertyPermission apply(PropertyPermission existingVal,
-                                                PropertyPermission newVal) {
-
-                    int oldMask = existingVal.getMask();
-                    int newMask = newVal.getMask();
-                    if (oldMask != newMask) {
-                        int effective = oldMask | newMask;
-                        if (effective == newMask) {
-                            return newVal;
-                        }
-                        if (effective != oldMask) {
-                            return new PropertyPermission(propName, effective);
-                        }
+        // permission if applicable.
+        perms.merge(propName, pp, (existingVal, newVal) -> {
+                int oldMask = existingVal.getMask();
+                int newMask = newVal.getMask();
+                if (oldMask != newMask) {
+                    int effective = oldMask | newMask;
+                    if (effective == newMask) {
+                        return newVal;
+                    }
+                    if (effective != oldMask) {
+                        return new PropertyPermission(propName, effective);
                     }
-                    return existingVal;
                 }
+                return existingVal;
             }
         );
 
@@ -523,10 +503,9 @@ final class PropertyPermissionCollection extends PermissionCollection
      */
     @Override
     public boolean implies(Permission permission) {
-        if (! (permission instanceof PropertyPermission))
+        if (! (permission instanceof PropertyPermission pp))
             return false;
 
-        PropertyPermission pp = (PropertyPermission) permission;
         PropertyPermission x;
 
         int desired = pp.getMask();
