@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2005, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -86,8 +86,8 @@ import java.util.Locale;
  * methods of installed providers for the appropriate interface to find one that
  * supports the requested locale. If such a provider is found, its other
  * methods are called to obtain the requested object or name.  When checking
- * whether a locale is supported, the <a href="../Locale.html#def_extensions">
- * locale's extensions</a> are ignored by default. (If locale's extensions should
+ * whether a locale is supported, the {@linkplain Locale##def_extensions
+ * locale's extensions} are ignored by default. (If locale's extensions should
  * also be checked, the {@code isSupportedLocale} method must be overridden.)
  * If neither the Java runtime environment itself nor an installed provider
  * supports the requested locale, the methods go through a list of candidate
@@ -115,21 +115,24 @@ import java.util.Locale;
  * The search order of locale sensitive services can
  * be configured by using the {@systemProperty java.locale.providers} system property.
  * This system property declares the user's preferred order for looking up
- * the locale sensitive services separated by a comma. It is only read at
- * the Java runtime startup, so the later call to System.setProperty() won't
- * affect the order.
- * <p>
- * Java Runtime Environment provides the following four locale providers:
+ * the locale sensitive services separated by a comma. As this property value is
+ * read and cached only at the initialization of this class, users should specify the
+ * property on the java launcher command line. Setting it at runtime with
+ * {@link System#setProperty(String, String)} is discouraged and it may not affect
+ * the order.
+ * JDK Reference Implementation provides the following four
+ * locale providers:
  * <ul>
  * <li> "CLDR": A provider based on Unicode Consortium's
  * <a href="http://cldr.unicode.org/">CLDR Project</a>.
  * <li> "COMPAT": represents the locale sensitive services that is compatible
- * with the prior JDK releases up to JDK8 (same as JDK8's "JRE").
+ * with the prior JDK releases up to JDK 8 (same as JDK 8's "JRE"). This
+ * provider is deprecated and will be removed in the future release of JDK.
  * <li> "SPI": represents the locale sensitive services implementing the subclasses of
  * this {@code LocaleServiceProvider} class.
  * <li> "HOST": A provider that reflects the user's custom settings in the
  * underlying operating system. This provider may not be available, depending
- * on the Java Runtime Environment implementation.
+ * on the JDK Reference Implementation.
  * <li> "JRE": represents a synonym to "COMPAT". This name
  * is deprecated and will be removed in the future release of JDK.
  * </ul>
@@ -152,6 +155,7 @@ import java.util.Locale;
 public abstract class LocaleServiceProvider {
 
     private static Void checkPermission() {
+        @SuppressWarnings("removal")
         SecurityManager sm = System.getSecurityManager();
         if (sm != null) {
             sm.checkPermission(new RuntimePermission("localeServiceProvider"));
@@ -172,23 +176,22 @@ public abstract class LocaleServiceProvider {
     }
 
     /**
-     * Returns an array of all locales for which this locale service provider
-     * can provide localized objects or names. This information is used to
-     * compose {@code getAvailableLocales()} values of the locale-dependent
-     * services, such as {@code DateFormat.getAvailableLocales()}.
+     * {@return an array of all locales for which this locale service provider
+     * can provide localized objects or names}
+     *
+     * This information is used to compose {@code getAvailableLocales()}
+     * values of the locale-dependent services, such as
+     * {@code DateFormat.getAvailableLocales()}.
      *
      * <p>The array returned by this method should not include two or more
      * {@code Locale} objects only differing in their extensions.
-     *
-     * @return An array of all locales for which this locale service provider
-     * can provide localized objects or names.
      */
     public abstract Locale[] getAvailableLocales();
 
     /**
      * Returns {@code true} if the given {@code locale} is supported by
      * this locale service provider. The given {@code locale} may contain
-     * <a href="../Locale.html#def_extensions">extensions</a> that should be
+     * {@linkplain Locale##def_extensions extensions} that should be
      * taken into account for the support determination.
      *
      * <p>The default implementation returns {@code true} if the given {@code locale}
