@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2005, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -39,10 +39,8 @@ import java.util.HashSet;
  * <p>Note that additional source version constants will be added to
  * model future releases of the language.
  *
- * @author Joseph D. Darcy
- * @author Scott Seligman
- * @author Peter von der Ah&eacute;
  * @since 1.6
+ * @see java.lang.reflect.ClassFileFormatVersion
  */
 public enum SourceVersion {
     /*
@@ -59,10 +57,23 @@ public enum SourceVersion {
      *  10: local-variable type inference (var)
      *  11: local-variable syntax for lambda parameters
      *  12: no changes (switch expressions in preview)
-     *  13: no changes (switch expressions and text blocks in preview)
+     *  13: no changes (text blocks in preview; switch expressions in
+     *      second preview)
      *  14: switch expressions (pattern matching and records in
-     *      preview, text blocks in preview again)
-     *  15: text blocks (records and pattern matching in preview again)
+     *      preview; text blocks in second preview)
+     *  15: text blocks (sealed classes in preview; records and pattern
+     *      matching in second preview)
+     *  16: records and pattern matching (sealed classes in second preview)
+     *  17: sealed classes, floating-point always strict (pattern
+     *      matching for switch in preview)
+     *  18: no changes (pattern matching for switch in second preview)
+     *  19: no changes (pattern matching for switch in third preview,
+     *      record patterns in preview)
+     *  20: no changes (pattern matching for switch in fourth preview,
+     *      record patterns in second preview)
+     *  21: pattern matching for switch and record patterns (string
+     *      templates in preview, unnamed patterns and variables in
+     *      preview, unnamed classes and instance main methods in preview)
      */
 
     /**
@@ -74,15 +85,16 @@ public enum SourceVersion {
     RELEASE_0,
 
     /**
-     * The version recognized by the Java Platform 1.1.
+     * The version introduced by the Java Platform 1.1.
      *
-     * The language is {@code RELEASE_0} augmented with nested classes as described in the 1.1 update to
-     * <cite>The Java Language Specification, First Edition</cite>.
+     * The language is {@code RELEASE_0} augmented with nested classes
+     * as described in the 1.1 update to <cite>The Java Language
+     * Specification, First Edition</cite>.
      */
     RELEASE_1,
 
     /**
-     * The version recognized by the Java 2 Platform, Standard Edition,
+     * The version introduced by the Java 2 Platform, Standard Edition,
      * v 1.2.
      *
      * The language described in
@@ -93,7 +105,7 @@ public enum SourceVersion {
     RELEASE_2,
 
     /**
-     * The version recognized by the Java 2 Platform, Standard Edition,
+     * The version introduced by the Java 2 Platform, Standard Edition,
      * v 1.3.
      *
      * No major changes from {@code RELEASE_2}.
@@ -101,15 +113,18 @@ public enum SourceVersion {
     RELEASE_3,
 
     /**
-     * The version recognized by the Java 2 Platform, Standard Edition,
+     * The version introduced by the Java 2 Platform, Standard Edition,
      * v 1.4.
      *
      * Added a simple assertion facility.
+     *
+     * @see <a href="https://jcp.org/en/jsr/detail?id=41">
+     * JSR 41: A Simple Assertion Facility</a>
      */
     RELEASE_4,
 
     /**
-     * The version recognized by the Java 2 Platform, Standard
+     * The version introduced by the Java 2 Platform, Standard
      * Edition 5.0.
      *
      * The language described in
@@ -117,113 +132,270 @@ public enum SourceVersion {
      * Third Edition</cite>.  First release to support
      * generics, annotations, autoboxing, var-args, enhanced {@code
      * for} loop, and hexadecimal floating-point literals.
+     *
+     * @see <a href="https://jcp.org/en/jsr/detail?id=14">
+     * JSR 14: Add Generic Types To The Java&trade; Programming Language</a>
+     * @see <a href="https://jcp.org/en/jsr/detail?id=175">
+     * JSR 175: A Metadata Facility for the Java&trade; Programming Language</a>
+     * @see <a href="https://jcp.org/en/jsr/detail?id=201">
+     * JSR 201: Extending the Java&trade; Programming Language with Enumerations,
+     * Autoboxing, Enhanced for loops and Static Import</a>
      */
     RELEASE_5,
 
     /**
-     * The version recognized by the Java Platform, Standard Edition
+     * The version introduced by the Java Platform, Standard Edition
      * 6.
      *
      * No major changes from {@code RELEASE_5}.
+     *
+     * @see <a
+     * href="https://docs.oracle.com/javase/specs/jls/se6/html/j3TOC.html">
+     * <cite>The Java Language Specification, Third Edition</cite></a>
      */
     RELEASE_6,
 
     /**
-     * The version recognized by the Java Platform, Standard Edition
+     * The version introduced by the Java Platform, Standard Edition
      * 7.
      *
-     * Additions in this release include, diamond syntax for
+     * Additions in this release include diamond syntax for
      * constructors, {@code try}-with-resources, strings in switch,
      * binary literals, and multi-catch.
      * @since 1.7
+     *
+     * @see <a
+     * href="https://docs.oracle.com/javase/specs/jls/se7/html/index.html">
+     * <cite>The Java Language Specification, Java SE 7 Edition</cite></a>
+     * @see <a href="https://jcp.org/en/jsr/detail?id=334">
+     * JSR 334: Small Enhancements to the Java&trade; Programming Language</a>
      */
     RELEASE_7,
 
     /**
-     * The version recognized by the Java Platform, Standard Edition
+     * The version introduced by the Java Platform, Standard Edition
      * 8.
      *
      * Additions in this release include lambda expressions and default methods.
      * @since 1.8
+     *
+     * @see <a
+     * href="https://docs.oracle.com/javase/specs/jls/se8/html/index.html">
+     * <cite>The Java Language Specification, Java SE 8 Edition</cite></a>
+     * @see <a href="https://jcp.org/en/jsr/detail?id=335">
+     * JSR 335: Lambda Expressions for the Java&trade; Programming Language</a>
      */
     RELEASE_8,
 
     /**
-     * The version recognized by the Java Platform, Standard Edition
+     * The version introduced by the Java Platform, Standard Edition
      * 9.
      *
      * Additions in this release include modules and removal of a
      * single underscore from the set of legal identifier names.
      *
      * @since 9
+     *
+     * @see <a
+     * href="https://docs.oracle.com/javase/specs/jls/se9/html/index.html">
+     * <cite>The Java Language Specification, Java SE 9 Edition</cite></a>
+     * @see <a href="https://jcp.org/en/jsr/detail?id=376">
+     * JSR 376: Java&trade; Platform Module System</a>
+     * @see <a href="https://openjdk.org/jeps/213">
+     * JEP 213: Milling Project Coin</a>
      */
      RELEASE_9,
 
     /**
-     * The version recognized by the Java Platform, Standard Edition
+     * The version introduced by the Java Platform, Standard Edition
      * 10.
      *
      * Additions in this release include local-variable type inference
      * ({@code var}).
      *
      * @since 10
+     *
+     * @see <a
+     * href="https://docs.oracle.com/javase/specs/jls/se10/html/index.html">
+     * <cite>The Java Language Specification, Java SE 10 Edition</cite></a>
+     * @see <a href="https://openjdk.org/jeps/286">
+     * JEP 286: Local-Variable Type Inference</a>
      */
     RELEASE_10,
 
     /**
-     * The version recognized by the Java Platform, Standard Edition
+     * The version introduced by the Java Platform, Standard Edition
      * 11.
      *
      * Additions in this release include local-variable syntax for
      * lambda parameters.
      *
      * @since 11
+     *
+     * @see <a
+     * href="https://docs.oracle.com/javase/specs/jls/se11/html/index.html">
+     * <cite>The Java Language Specification, Java SE 11 Edition</cite></a>
+     * @see <a href="https://openjdk.org/jeps/323">
+     * JEP 323: Local-Variable Syntax for Lambda Parameters</a>
      */
     RELEASE_11,
 
     /**
-     * The version recognized by the Java Platform, Standard Edition
+     * The version introduced by the Java Platform, Standard Edition
      * 12.
+     * No major changes from the prior release.
      *
      * @since 12
+     *
+     * @see <a
+     * href="https://docs.oracle.com/javase/specs/jls/se12/html/index.html">
+     * <cite>The Java Language Specification, Java SE 12 Edition</cite></a>
      */
     RELEASE_12,
 
     /**
-     * The version recognized by the Java Platform, Standard Edition
+     * The version introduced by the Java Platform, Standard Edition
      * 13.
+     * No major changes from the prior release.
      *
      * @since 13
+     *
+     * @see <a
+     * href="https://docs.oracle.com/javase/specs/jls/se13/html/index.html">
+     * <cite>The Java Language Specification, Java SE 13 Edition</cite></a>
      */
     RELEASE_13,
 
     /**
-     * The version recognized by the Java Platform, Standard Edition
+     * The version introduced by the Java Platform, Standard Edition
      * 14.
      *
      * Additions in this release include switch expressions.
      *
      * @since 14
+     *
+     * @see <a
+     * href="https://docs.oracle.com/javase/specs/jls/se14/html/index.html">
+     * <cite>The Java Language Specification, Java SE 14 Edition</cite></a>
+     * @see <a href="https://openjdk.org/jeps/361">
+     * JEP 361: Switch Expressions</a>
      */
     RELEASE_14,
 
     /**
-     * The version recognized by the Java Platform, Standard Edition
+     * The version introduced by the Java Platform, Standard Edition
      * 15.
      *
      * Additions in this release include text blocks.
      *
      * @since 15
+     *
+     * @see <a
+     * href="https://docs.oracle.com/javase/specs/jls/se15/html/index.html">
+     * <cite>The Java Language Specification, Java SE 15 Edition</cite></a>
+     * @see <a href="https://openjdk.org/jeps/378">
+     * JEP 378: Text Blocks</a>
      */
     RELEASE_15,
 
     /**
-     * The version recognized by the Java Platform, Standard Edition
+     * The version introduced by the Java Platform, Standard Edition
      * 16.
      *
+     * Additions in this release include records and pattern matching
+     * for {@code instanceof}.
+     *
      * @since 16
+     *
+     * @see <a
+     * href="https://docs.oracle.com/javase/specs/jls/se16/html/index.html">
+     * <cite>The Java Language Specification, Java SE 16 Edition</cite></a>
+     * @see <a href="https://openjdk.org/jeps/394">
+     * JEP 394: Pattern Matching for instanceof</a>
+     * @see <a href="https://openjdk.org/jeps/395">
+     * JEP 395: Records</a>
      */
-    RELEASE_16;
+    RELEASE_16,
+
+    /**
+     * The version introduced by the Java Platform, Standard Edition
+     * 17.
+     *
+     * Additions in this release include sealed classes and
+     * restoration of always-strict floating-point semantics.
+     *
+     * @since 17
+     *
+     * @see <a
+     * href="https://docs.oracle.com/javase/specs/jls/se17/html/index.html">
+     * <cite>The Java Language Specification, Java SE 17 Edition</cite></a>
+     * @see <a href="https://openjdk.org/jeps/306">
+     * JEP 306: Restore Always-Strict Floating-Point Semantics</a>
+     * @see <a href="https://openjdk.org/jeps/409">
+     * JEP 409: Sealed Classes</a>
+     */
+    RELEASE_17,
+
+    /**
+     * The version introduced by the Java Platform, Standard Edition
+     * 18.
+     *
+     * No major changes from the prior release.
+     *
+     * @since 18
+     *
+     * @see <a
+     * href="https://docs.oracle.com/javase/specs/jls/se18/html/index.html">
+     * <cite>The Java Language Specification, Java SE 18 Edition</cite></a>
+     */
+    RELEASE_18,
+
+    /**
+     * The version introduced by the Java Platform, Standard Edition
+     * 19.
+     *
+     * No major changes from the prior release.
+     *
+     * @since 19
+     *
+     * @see <a
+     * href="https://docs.oracle.com/javase/specs/jls/se19/html/index.html">
+     * <cite>The Java Language Specification, Java SE 19 Edition</cite></a>
+     */
+    RELEASE_19,
+
+    /**
+     * The version introduced by the Java Platform, Standard Edition
+     * 20.
+     *
+     * No major changes from the prior release.
+     *
+     * @since 20
+     *
+     * @see <a
+     * href="https://docs.oracle.com/javase/specs/jls/se20/html/index.html">
+     * <cite>The Java Language Specification, Java SE 20 Edition</cite></a>
+     */
+    RELEASE_20,
+
+    /**
+     * The version introduced by the Java Platform, Standard Edition
+     * 21.
+     *
+     * Additions in this release include record patterns and pattern
+     * matching for {@code switch}.
+     *
+     * @since 21
+     *
+     * @see <a
+     * href="https://docs.oracle.com/javase/specs/jls/se21/html/index.html">
+     * <cite>The Java Language Specification, Java SE 21 Edition</cite></a>
+     * @see <a href="https://openjdk.org/jeps/440">
+     * Record Patterns</a>
+     * @see <a href="https://openjdk.org/jeps/441">
+     * Pattern Matching for switch</a>
+     */
+    RELEASE_21;
 
     // Note that when adding constants for newer releases, the
     // behavior of latest() and latestSupported() must be updated too.
@@ -232,7 +404,7 @@ public enum SourceVersion {
      * {@return the latest source version that can be modeled}
      */
     public static SourceVersion latest() {
-        return RELEASE_16;
+        return RELEASE_21;
     }
 
     private static final SourceVersion latestSupported = getLatestSupported();
@@ -247,7 +419,7 @@ public enum SourceVersion {
     private static SourceVersion getLatestSupported() {
         int intVersion = Runtime.version().feature();
         return (intVersion >= 11) ?
-            valueOf("RELEASE_" + Math.min(16, intVersion)):
+            valueOf("RELEASE_" + Math.min(21, intVersion)):
             RELEASE_10;
     }
 
@@ -283,8 +455,8 @@ public enum SourceVersion {
      * Character#isJavaIdentifierStart(int)} returns {@code true},
      * followed only by characters for which {@link
      * Character#isJavaIdentifierPart(int)} returns {@code true}.
-     * This pattern matches regular identifiers, keywords, restricted
-     * keywords, restricted identifiers and the literals {@code "true"},
+     * This pattern matches regular identifiers, keywords, contextual
+     * keywords, and the literals {@code "true"},
      * {@code "false"}, {@code "null"}.
      *
      * The method returns {@code false} for all other strings.
@@ -330,8 +502,8 @@ public enum SourceVersion {
      * {@code false} for keywords, boolean literals, and the null
      * literal in any segment.
      *
-     * This method returns {@code true} for <i>restricted
-     * keywords</i> and <i>restricted identifiers</i>.
+     * This method returns {@code true} for <i>contextual
+     * keywords</i>.
      *
      * @param name the string to check
      * @return {@code true} if this string is a
@@ -356,8 +528,8 @@ public enum SourceVersion {
      * {@code false} for keywords, boolean literals, and the null
      * literal in any segment.
      *
-     * This method returns {@code true} for <i>restricted
-     * keywords</i> and <i>restricted identifiers</i>.
+     * This method returns {@code true} for <i>contextual
+     * keywords</i>.
      *
      * @param name the string to check
      * @param version the version to use
@@ -380,8 +552,8 @@ public enum SourceVersion {
     /**
      * Returns whether or not {@code s} is a keyword, boolean literal,
      * or null literal in the latest source version.
-     * This method returns {@code false} for <i>restricted
-     * keywords</i> and <i>restricted identifiers</i>.
+     * This method returns {@code false} for <i>contextual
+     * keywords</i>.
      *
      * @param s the string to check
      * @return {@code true} if {@code s} is a keyword, or boolean
@@ -397,8 +569,8 @@ public enum SourceVersion {
     /**
      * Returns whether or not {@code s} is a keyword, boolean literal,
      * or null literal in the given source version.
-     * This method returns {@code false} for <i>restricted
-     * keywords</i> and <i>restricted identifiers</i>.
+     * This method returns {@code false} for <i>contextual
+     * keywords</i>.
      *
      * @param s the string to check
      * @param version the version to use
@@ -470,4 +642,59 @@ public enum SourceVersion {
             return false;
         }
     }
+
+    /**
+     * {@return the latest source version that is usable under the
+     * runtime version argument} If the runtime version's {@linkplain
+     * Runtime.Version#feature() feature} is greater than the feature
+     * of the {@linkplain #runtimeVersion() runtime version} of the
+     * {@linkplain #latest() latest source version}, an {@code
+     * IllegalArgumentException} is thrown.
+     *
+     * <p>Because the source versions of the Java programming language
+     * have so far followed a linear progression, only the feature
+     * component of a runtime version is queried to determine the
+     * mapping to a source version. If that linearity changes in the
+     * future, other components of the runtime version may influence
+     * the result.
+     *
+     * @apiNote
+     * An expression to convert from a string value, for example
+     * {@code "17"}, to the corresponding source version, {@code
+     * RELEASE_17}, is:
+     *
+     * {@snippet lang="java" :
+     * SourceVersion.valueOf(Runtime.Version.parse("17"))}
+     *
+     * @param rv runtime version to map to a source version
+     * @throws IllegalArgumentException if the feature of version
+     * argument is greater than the feature of the platform version.
+     * @since 18
+     */
+    public static SourceVersion valueOf(Runtime.Version rv) {
+        // Could also implement this as a switch where a case was
+        // added with each new release.
+        return valueOf("RELEASE_" + rv.feature());
+    }
+
+    /**
+     * {@return the least runtime version that supports this source
+     * version; otherwise {@code null}} The returned runtime version
+     * has a {@linkplain Runtime.Version#feature() feature} large
+     * enough to support this source version and has no other elements
+     * set.
+     *
+     * Source versions greater than or equal to {@link RELEASE_6}
+     * have non-{@code null} results.
+     * @since 18
+     */
+    public Runtime.Version runtimeVersion() {
+        // The javax.lang.model API was added in JDK 6; for now,
+        // limiting supported range to 6 and up.
+        if (this.compareTo(RELEASE_6) >= 0) {
+            return Runtime.Version.parse(Integer.toString(ordinal()));
+        } else {
+            return null;
+        }
+    }
 }
