@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2005, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -42,20 +42,36 @@ import javax.lang.model.element.*;
  * <p><b>Compatibility Note:</b> Methods may be added to this interface
  * in future releases of the platform.
  *
- * @author Joseph D. Darcy
- * @author Scott Seligman
- * @author Peter von der Ah&eacute;
  * @see javax.annotation.processing.ProcessingEnvironment#getElementUtils
  * @since 1.6
  */
 public interface Elements {
 
     /**
-     * Returns a package given its fully qualified name if the package is unique in the environment.
-     * If running with modules, all modules in the modules graph are searched for matching packages.
+     * Returns a package given its fully qualified name if the package is uniquely
+     * determinable in the environment.
      *
-     * @param name  fully qualified package name, or an empty string for an unnamed package
-     * @return the specified package, or {@code null} if it cannot be uniquely found
+     * If running with modules, packages of the given name are searched in a
+     * two-stage process:
+     * <ul>
+     *     <li>find non-empty packages with the given name returned by
+     *         {@link #getPackageElement(ModuleElement, CharSequence)},
+     *         where the provided ModuleSymbol is any
+     *         {@linkplain java.lang.module##root-modules root module},
+     *     </li>
+     *     <li>if the above yields an empty list, search
+     *         {@link #getAllModuleElements() all modules} for observable
+     *         packages with the given name
+     *     </li>
+     * </ul>
+     *
+     * If this process leads to a list with a single element, the
+     * single element is returned, otherwise {@code null} is returned.
+     *
+     * @param name fully qualified package name,
+     *             or an empty string for an unnamed package
+     * @return the specified package,
+     *         or {@code null} if no package can be uniquely determined.
      */
     PackageElement getPackageElement(CharSequence name);
 
@@ -119,12 +135,29 @@ public interface Elements {
     }
 
     /**
-     * Returns a type element given its canonical name if the type element is unique in the environment.
-     * If running with modules, all modules in the modules graph are searched for matching
-     * type elements.
+     * Returns a type element given its canonical name if the type element is uniquely
+     * determinable in the environment.
      *
-     * @param name  the canonical name
-     * @return the named type element, or {@code null} if it cannot be uniquely found
+     * If running with modules, type elements of the given name are
+     * searched in a two-stage process:
+     * <ul>
+     *     <li>find type elements with the given name returned by
+     *         {@link #getTypeElement(ModuleElement, CharSequence)},
+     *         where the provided ModuleSymbol is any
+     *         {@linkplain java.lang.module##root-modules root module},
+     *     </li>
+     *     <li>if the above yields an empty list, search
+     *         {@link #getAllModuleElements() all modules} for observable
+     *         type elements with the given name
+     *     </li>
+     * </ul>
+     *
+     * If this process leads to a list with a single element, the
+     * single element is returned, otherwise {@code null} is returned.
+     *
+     * @param name the canonical name
+     * @return the named type element,
+     *         or {@code null} if no type element can be uniquely determined.
      */
     TypeElement getTypeElement(CharSequence name);
 
@@ -203,7 +236,6 @@ public interface Elements {
      * @return the named module element, or {@code null} if it cannot be found
      * @see #getAllModuleElements
      * @since 9
-     * @spec JPMS
      */
     default ModuleElement getModuleElement(CharSequence name) {
         return null;
@@ -222,6 +254,11 @@ public interface Elements {
      * @implSpec The default implementation of this method returns
      * an empty set.
      *
+     * @apiNote
+     * When an environment includes modules, both named modules and
+     * {@linkplain ModuleElement#isUnnamed() unnamed modules} may be
+     * returned.
+     *
      * @return the known module elements, or an empty set if there are no modules
      * @see #getModuleElement(CharSequence)
      * @since 9
@@ -231,11 +268,10 @@ public interface Elements {
     }
 
     /**
-     * Returns the values of an annotation's elements, including defaults.
+     * {@return the values of an annotation's elements, including defaults}
      *
      * @see AnnotationMirror#getElementValues()
      * @param a  annotation to examine
-     * @return the values of the annotation's elements, including defaults
      */
     Map<? extends ExecutableElement, ? extends AnnotationValue>
             getElementValuesWithDefaults(AnnotationMirror a);
@@ -267,15 +303,14 @@ public interface Elements {
     String getDocComment(Element e);
 
     /**
-     * Returns {@code true} if the element is deprecated, {@code false} otherwise.
+     * {@return {@code true} if the element is deprecated, {@code false} otherwise}
      *
      * @param e  the element being examined
-     * @return {@code true} if the element is deprecated, {@code false} otherwise
      */
     boolean isDeprecated(Element e);
 
     /**
-     * Returns the <em>origin</em> of the given element.
+     * {@return the <em>origin</em> of the given element}
      *
      * <p>Note that if this method returns {@link Origin#EXPLICIT
      * EXPLICIT} and the element was created from a class file, then
@@ -291,7 +326,6 @@ public interface Elements {
      * {@link Origin#EXPLICIT EXPLICIT}.
      *
      * @param e  the element being examined
-     * @return the origin of the given element
      * @since 9
      */
     default Origin getOrigin(Element e) {
@@ -299,12 +333,12 @@ public interface Elements {
     }
 
     /**
-     * Returns the <em>origin</em> of the given annotation mirror.
+     * {@return the <em>origin</em> of the given annotation mirror}
      *
      * An annotation mirror is {@linkplain Origin#MANDATED mandated}
      * if it is an implicitly declared <em>container annotation</em>
      * used to hold repeated annotations of a repeatable annotation
-     * type.
+     * interface.
      *
      * <p>Note that if this method returns {@link Origin#EXPLICIT
      * EXPLICIT} and the annotation mirror was created from a class
@@ -321,9 +355,8 @@ public interface Elements {
      *
      * @param c the construct the annotation mirror modifies
      * @param a the annotation mirror being examined
-     * @return the origin of the given annotation mirror
-     * @jls 9.6.3 Repeatable Annotation Types
-     * @jls 9.7.5 Multiple Annotations of the Same Type
+     * @jls 9.6.3 Repeatable Annotation Interfaces
+     * @jls 9.7.5 Multiple Annotations of the Same Interface
      * @since 9
      */
     default Origin getOrigin(AnnotatedConstruct c,
@@ -332,7 +365,7 @@ public interface Elements {
     }
 
     /**
-     * Returns the <em>origin</em> of the given module directive.
+     * {@return the <em>origin</em> of the given module directive}
      *
      * <p>Note that if this method returns {@link Origin#EXPLICIT
      * EXPLICIT} and the module directive was created from a class
@@ -355,7 +388,6 @@ public interface Elements {
      *
      * @param m the module of the directive
      * @param directive  the module directive being examined
-     * @return the origin of the given directive
      * @since 9
      */
     default Origin getOrigin(ModuleElement m,
@@ -393,12 +425,13 @@ public interface Elements {
          *
          * Another example of a mandated construct is an implicitly
          * declared <em>container annotation</em> used to hold
-         * multiple annotations of a repeatable annotation type.
+         * multiple annotations of a repeatable annotation interface.
          *
          * @jls 8.8.9 Default Constructor
          * @jls 8.9.3 Enum Members
-         * @jls 9.6.3 Repeatable Annotation Types
-         * @jls 9.7.5 Multiple Annotations of the Same Type
+         * @jls 8.10.3 Record Members
+         * @jls 9.6.3 Repeatable Annotation Interfaces
+         * @jls 9.7.5 Multiple Annotations of the Same Interface
          */
         MANDATED,
 
@@ -422,14 +455,12 @@ public interface Elements {
     }
 
     /**
-     * Returns {@code true} if the executable element is a bridge
-     * method, {@code false} otherwise.
+     * {@return {@code true} if the executable element is a bridge
+     * method, {@code false} otherwise}
      *
      * @implSpec The default implementation of this method returns {@code false}.
      *
      * @param e  the executable being examined
-     * @return {@code true} if the executable element is a bridge
-     * method, {@code false} otherwise
      * @since 9
      */
     default boolean isBridge(ExecutableElement e) {
@@ -437,10 +468,9 @@ public interface Elements {
     }
 
     /**
-     * Returns the <i>binary name</i> of a type element.
+     * {@return the <i>binary name</i> of a type element}
      *
      * @param type  the type element being examined
-     * @return the binary name
      *
      * @see TypeElement#getQualifiedName
      * @jls 13.1 The Form of a Binary
@@ -449,22 +479,21 @@ public interface Elements {
 
 
     /**
-     * Returns the package of an element.  The package of a package is
+     * {@return the package of an element}  The package of a package is
      * itself.
      * The package of a module is {@code null}.
      *
-     * The package of a top-level type is its {@linkplain
+     * The package of a top-level class or interface is its {@linkplain
      * TypeElement#getEnclosingElement enclosing package}. Otherwise,
      * the package of an element is equal to the package of the
      * {@linkplain Element#getEnclosingElement enclosing element}.
      *
      * @param e the element being examined
-     * @return the package of an element
      */
     PackageElement getPackageOf(Element e);
 
     /**
-     * Returns the module of an element.  The module of a module is
+     * {@return the module of an element}  The module of a module is
      * itself.
      *
      * If a package has a module as its {@linkplain
@@ -487,9 +516,7 @@ public interface Elements {
      * {@code null}.
      *
      * @param e the element being examined
-     * @return the module of an element
      * @since 9
-     * @spec JPMS
      */
     default ModuleElement getModuleOf(Element e) {
         return null;
@@ -509,6 +536,67 @@ public interface Elements {
      */
     List<? extends Element> getAllMembers(TypeElement type);
 
+    /**
+     * {@return the outermost type element an element is contained in
+     * if such a containing element exists; otherwise returns {@code
+     * null}}
+     *
+     * {@linkplain ModuleElement Modules} and {@linkplain
+     * PackageElement packages} do <em>not</em> have a containing type
+     * element and therefore {@code null} is returned for those kinds
+     * of elements.
+     *
+     * A {@linkplain NestingKind#TOP_LEVEL top-level} class or
+     * interface is its own outermost type element.
+     *
+     * @implSpec
+     * The default implementation of this method first checks the kind
+     * of the argument. For elements of kind {@code PACKAGE}, {@code
+     * MODULE}, and {@code OTHER}, {@code null} is returned. For
+     * elements of other kinds, the element is examined to see if it
+     * is a top-level class or interface. If so, that element is
+     * returned; otherwise, the {@linkplain
+     * Element#getEnclosingElement enclosing element} chain is
+     * followed until a top-level class or interface is found. The
+     * element for the eventual top-level class or interface is
+     * returned.
+     *
+     * @param e the element being examined
+     * @see Element#getEnclosingElement
+     * @since 18
+     */
+    default TypeElement getOutermostTypeElement(Element e) {
+        return switch (e.getKind()) {
+        case PACKAGE,
+             MODULE  -> null; // Per the general spec above.
+        case OTHER   -> null; // Outside of base model of the javax.lang.model API
+
+        // Elements of all remaining kinds should be enclosed in some
+        // sort of class or interface. Check to see if the element is
+        // a top-level type; if so, return it. Otherwise, keep going
+        // up the enclosing element chain until a top-level type is
+        // found.
+        default -> {
+            Element enclosing = e;
+            // This implementation is susceptible to infinite loops
+            // for misbehaving element implementations.
+            while (true) {
+                // Conceptual instanceof TypeElement check. If the
+                // argument is a type element, put it into a
+                // one-element list, otherwise an empty list.
+                List<TypeElement> possibleTypeElement = ElementFilter.typesIn(List.of(enclosing));
+                if (!possibleTypeElement.isEmpty()) {
+                    TypeElement typeElement = possibleTypeElement.get(0);
+                    if (typeElement.getNestingKind() == NestingKind.TOP_LEVEL) {
+                        yield typeElement;
+                    }
+                }
+                enclosing = enclosing.getEnclosingElement();
+            }
+        }
+        };
+    }
+
     /**
      * Returns all annotations <i>present</i> on an element, whether
      * directly present or present via inheritance.
@@ -535,10 +623,12 @@ public interface Elements {
     boolean hides(Element hider, Element hidden);
 
     /**
-     * Tests whether one method, as a member of a given type,
+     * Tests whether one method, as a member of a given class or interface,
      * overrides another method.
      * When a non-abstract method overrides an abstract one, the
      * former is also said to <i>implement</i> the latter.
+     * As implied by JLS {@jls 8.4.8.1}, a method does <em>not</em>
+     * override itself. The overrides relation is <i>irreflexive</i>.
      *
      * <p> In the simplest and most typical usage, the value of the
      * {@code type} parameter will simply be the class or interface
@@ -554,8 +644,8 @@ public interface Elements {
      * </blockquote>
      *
      * A more interesting case can be illustrated by the following example
-     * in which a method in type {@code A} does not override a
-     * like-named method in type {@code B}:
+     * in which a method in class {@code A} does not override a
+     * like-named method in interface {@code B}:
      *
      * <blockquote>
      * {@code class A { public void m() {} } }<br>
@@ -567,7 +657,7 @@ public interface Elements {
      *          elements.getTypeElement("A")); }
      * </blockquote>
      *
-     * When viewed as a member of a third type {@code C}, however,
+     * When viewed as a member of a third class {@code C}, however,
      * the method in {@code A} does override the one in {@code B}:
      *
      * <blockquote>
@@ -579,7 +669,7 @@ public interface Elements {
      *
      * @param overrider  the first method, possible overrider
      * @param overridden  the second method, possibly being overridden
-     * @param type   the type of which the first method is a member
+     * @param type   the class or interface of which the first method is a member
      * @return {@code true} if and only if the first method overrides
      *          the second
      * @jls 8.4.8 Inheritance, Overriding, and Hiding
@@ -615,34 +705,43 @@ public interface Elements {
     void printElements(java.io.Writer w, Element... elements);
 
     /**
-     * Return a name with the same sequence of characters as the
-     * argument.
+     * {@return a name with the same sequence of characters as the
+     * argument}
      *
      * @param cs the character sequence to return as a name
-     * @return a name with the same sequence of characters as the argument
      */
     Name getName(CharSequence cs);
 
     /**
-     * Returns {@code true} if the type element is a functional interface, {@code false} otherwise.
+     * {@return {@code true} if the type element is a functional
+     * interface, {@code false} otherwise}
      *
      * @param type the type element being examined
-     * @return {@code true} if the element is a functional interface, {@code false} otherwise
      * @jls 9.8 Functional Interfaces
      * @since 1.8
      */
     boolean isFunctionalInterface(TypeElement type);
 
     /**
-     * {@preview Associated with records, a preview feature of the Java language.
+     * {@return {@code true} if the module element is an automatic
+     * module, {@code false} otherwise}
      *
-     *           This method is associated with <i>records</i>, a preview
-     *           feature of the Java language. Preview features
-     *           may be removed in a future release, or upgraded to permanent
-     *           features of the Java language.}
+     * @implSpec
+     * The default implementation of this method returns {@code
+     * false}.
      *
-     * Returns the record component for the given accessor. Returns null if the
-     * given method is not a record component accessor.
+     * @param module the module element being examined
+     * @jls 7.7.1 Dependences
+     * @since 17
+     */
+    default boolean isAutomaticModule(ModuleElement module) {
+        return false;
+    }
+
+    /**
+     * Returns the record component for the given accessor. Returns
+     * {@code null} if the given method is not a record component
+     * accessor.
      *
      * @implSpec The default implementation of this method checks if the element
      * enclosing the accessor has kind {@link ElementKind#RECORD RECORD} if that is
@@ -653,13 +752,10 @@ public interface Elements {
      * record component is returned, in any other case {@code null} is returned.
      *
      * @param accessor the method for which the record component should be found.
-     * @return the record component, or null if the given method is not an record
-     * component accessor
-     * @since 14
+     * @return the record component, or {@code null} if the given
+     * method is not a record component accessor
+     * @since 16
      */
-    @jdk.internal.PreviewFeature(feature=jdk.internal.PreviewFeature.Feature.RECORDS,
-                                 essentialAPI=false)
-    @SuppressWarnings("preview")
     default RecordComponentElement recordComponentFor(ExecutableElement accessor) {
         if (accessor.getEnclosingElement().getKind() == ElementKind.RECORD) {
             for (RecordComponentElement rec : ElementFilter.recordComponentsIn(accessor.getEnclosingElement().getEnclosedElements())) {
@@ -670,4 +766,145 @@ public interface Elements {
         }
         return null;
     }
+
+    /**
+     * {@return {@code true} if the executable element can be
+     * determined to be a canonical constructor of a record, {@code
+     * false} otherwise}
+     * Note that in some cases there may be insufficient information
+     * to determine if a constructor is a canonical constructor, such
+     * as if the executable element is built backed by a class
+     * file. In such cases, {@code false} is returned.
+     *
+     * @implSpec
+     * The default implementation of this method unconditionally
+     * returns {@code false}.
+     *
+     * @param e  the executable being examined
+     * @jls 8.10.4.1 Normal Canonical Constructors
+     * @since 20
+     */
+    default boolean isCanonicalConstructor(ExecutableElement e) {
+        return false;
+    }
+
+    /**
+     * {@return {@code true} if the executable element can be
+     * determined to be a compact constructor of a record, {@code
+     * false} otherwise}
+     * By definition, a compact constructor is also a {@linkplain
+     * #isCanonicalConstructor(ExecutableElement) canonical
+     * constructor}.
+     * Note that in some cases there may be insufficient information
+     * to determine if a constructor is a compact constructor, such as
+     * if the executable element is built backed by a class file. In
+     * such cases, {@code false} is returned.
+     *
+     * @implSpec
+     * The default implementation of this method unconditionally
+     * returns {@code false}.
+     *
+     * @param e  the executable being examined
+     * @jls 8.10.4.2 Compact Canonical Constructors
+     * @since 20
+     */
+    default boolean isCompactConstructor(ExecutableElement e) {
+        return false;
+    }
+
+    /**
+     * {@return the file object for this element or {@code null} if
+     * there is no such file object}
+     *
+     * <p>The returned file object is for the {@linkplain
+     * javax.lang.model.element##accurate_model reference
+     * representation} of the information used to construct the
+     * element. For example, if during compilation or annotation
+     * processing, a source file for class {@code Foo} is compiled
+     * into a class file, the file object returned for the element
+     * representing {@code Foo} would be for the source file and
+     * <em>not</em> for the class file.
+     *
+     * <p>An implementation may choose to not support the
+     * functionality of this method, in which case {@link
+     * UnsupportedOperationException} is thrown.
+     *
+     * <p>In the context of annotation processing, a non-{@code null}
+     * value is returned if the element was included as part of the
+     * initial inputs or the containing file was created during the
+     * run of the annotation processing tool. Otherwise, a {@code
+     * null} may be returned. In annotation processing, if a
+     * {@linkplain javax.annotation.processing.Filer#createClassFile
+     * class file is created}, that class file can serve as the
+     * reference representation for elements.
+     *
+     * <p>If it has a file object, the file object for a package will
+     * be a {@code package-info} file. A package may exist and not
+     * have any {@code package-info} file even if the package is
+     * (implicitly) created during an annotation processing run from
+     * the creation of source or class files in that package.  An
+     * {@linkplain PackageElement#isUnnamed unnamed package} will have
+     * a {@code null} file since it cannot be declared in a
+     * compilation unit.
+     *
+     * <p>If it has a file object, the file object for a module will
+     * be a {@code module-info} file.  An {@linkplain
+     * ModuleElement#isUnnamed unnamed module} will have a {@code
+     * null} file since it cannot be declared in a compilation unit.
+     * An {@linkplain #isAutomaticModule automatic module} will have a
+     * {@code null} file since it is implicitly declared.
+     *
+     * <p>If it has a file object, the file object for a top-level
+     * {@code public} class or interface will be a source or class
+     * file corresponding to that class or interface. In this case,
+     * typically the leading portion of the name of the file will
+     * match the name of the class or interface. A single compilation
+     * unit can define multiple top-level classes and interfaces, such
+     * as a primary {@code public} class or interfaces whose name
+     * corresponds to the file name and one or more <em>auxiliary</em>
+     * classes or interfaces whose names do not correspond to the file
+     * name. If a source file is providing the reference
+     * representation of an auxiliary class or interface, the file for
+     * the primary class is returned. (An auxiliary class or interface
+     * can also be defined in a {@code package-info} source file, in
+     * which case the file for the {@code package-info} file is
+     * returned.)  If a class file is providing the reference
+     * representation of an auxiliary class or interface, the separate
+     * class file for the auxiliary class is returned.
+     *
+     * <p>For a nested class or interface, if it has a file object:
+     *
+     * <ul>
+     *
+     * <li>if a source file is providing the reference representation,
+     * the file object will be that of the {@linkplain
+     * #getOutermostTypeElement(Element) outermost enclosing} class or
+     * interface
+     *
+     * <li>if a class file is providing the reference representation,
+     * the file object will be that of the nested class or interface
+     * itself
+     *
+     * </ul>
+     *
+     * <p>For other lexically enclosed elements, such as {@linkplain
+     * VariableElement#getEnclosingElement() variables}, {@linkplain
+     * ExecutableElement#getEnclosingElement() methods, and
+     * constructors}, if they have a file object, the file object will
+     * be the object associated with the {@linkplain
+     * Element#getEnclosingElement() enclosing element} of the
+     * lexically enclosed element.
+     *
+     * @implSpec The default implementation unconditionally throws
+     * {@link UnsupportedOperationException}.
+     *
+     * @throws UnsupportedOperationException if this functionality is
+     * not supported
+     *
+     * @param e the element to find a file object for
+     * @since 18
+     */
+    default javax.tools.JavaFileObject getFileObjectOf(Element e) {
+        throw new UnsupportedOperationException();
+    }
 }
