@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1999, 2024, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -33,7 +33,6 @@ import java.util.Arrays;
 
 import jdk.internal.ref.CleanerFactory;
 
-
 /**
  * <p> Underlying security services instantiate and pass a
  * {@code PasswordCallback} to the {@code handle}
@@ -159,9 +158,7 @@ public class PasswordCallback implements Callback, java.io.Serializable {
     }
 
     private static Runnable cleanerFor(char[] password) {
-        return () -> {
-            Arrays.fill(password, ' ');
-        };
+        return () -> Arrays.fill(password, ' ');
     }
 
     /**
@@ -181,7 +178,11 @@ public class PasswordCallback implements Callback, java.io.Serializable {
         }
 
         if (inputPassword != null) {
-            inputPassword = inputPassword.clone();
+            char[] temp = inputPassword;
+            inputPassword = temp.clone();
+            Arrays.fill(temp, '0');
+            cleanable = CleanerFactory.cleaner().register(
+                    this, cleanerFor(inputPassword));
         }
     }
 }
