@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1999, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -45,6 +45,11 @@ import static javax.sound.sampled.AudioFileFormat.Type;
  */
 public abstract class AudioFileWriter {
 
+    /**
+     * Constructor for subclasses to call.
+     */
+    protected AudioFileWriter() {}
+
     /**
      * Obtains the file types for which file writing support is provided by this
      * audio file writer.
