@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -22,6 +22,7 @@
  *
  *
  */
+
 package javax.swing;
 
 import java.awt.Component;
@@ -31,17 +32,22 @@ import java.awt.Graphics;
 import java.awt.Insets;
 import java.awt.LayoutManager;
 import java.awt.LayoutManager2;
-import java.beans.JavaBean;
 import java.beans.BeanProperty;
+import java.beans.JavaBean;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
-
-import javax.swing.plaf.*;
-import javax.accessibility.*;
-
-import java.io.Serializable;
-import java.io.ObjectOutputStream;
 import java.io.IOException;
+import java.io.ObjectOutputStream;
+import java.io.Serial;
+import java.io.Serializable;
+
+import javax.accessibility.Accessible;
+import javax.accessibility.AccessibleContext;
+import javax.accessibility.AccessibleRole;
+import javax.accessibility.AccessibleState;
+import javax.accessibility.AccessibleStateSet;
+import javax.swing.plaf.ToolBarUI;
+import javax.swing.plaf.UIResource;
 
 /**
  * <code>JToolBar</code> provides a component that is useful for
@@ -136,7 +142,7 @@ public class JToolBar extends JComponent implements SwingConstants, Accessible
      * @param name  the name of the tool bar
      * @param orientation  the initial orientation -- it must be
      *          either <code>HORIZONTAL</code> or <code>VERTICAL</code>
-     * @exception IllegalArgumentException if orientation is neither
+     * @throws IllegalArgumentException if orientation is neither
      *          <code>HORIZONTAL</code> nor <code>VERTICAL</code>
      * @since 1.3
      */
@@ -400,7 +406,7 @@ public class JToolBar extends JComponent implements SwingConstants, Accessible
      *
      * @param o  the new orientation -- either <code>HORIZONTAL</code> or
      *                  <code>VERTICAL</code>
-     * @exception IllegalArgumentException if orientation is neither
+     * @throws IllegalArgumentException if orientation is neither
      *          <code>HORIZONTAL</code> nor <code>VERTICAL</code>
      * @see #getOrientation
      */
@@ -687,6 +693,7 @@ public class JToolBar extends JComponent implements SwingConstants, Accessible
      * <code>JComponent</code> for more
      * information about serialization in Swing.
      */
+    @Serial
     private void writeObject(ObjectOutputStream s) throws IOException {
         s.defaultWriteObject();
         if (getUIClassID().equals(uiClassID)) {
@@ -831,6 +838,11 @@ public class JToolBar extends JComponent implements SwingConstants, Accessible
      */
     protected class AccessibleJToolBar extends AccessibleJComponent {
 
+        /**
+         * Constructs an {@code AccessibleJToolBar}.
+         */
+        protected AccessibleJToolBar() {}
+
         /**
          * Get the state of this object.
          *
