@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -23,26 +23,25 @@
  *
  */
 
-
-
 package javax.swing;
 
-
-
-import java.util.*;
-import java.util.concurrent.atomic.AtomicBoolean;
-import java.util.concurrent.locks.*;
-import java.awt.*;
-import java.awt.event.*;
+import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
+import java.io.IOException;
+import java.io.InvalidObjectException;
+import java.io.ObjectInputStream;
+import java.io.Serial;
 import java.io.Serializable;
-import java.io.*;
 import java.security.AccessControlContext;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
+import java.util.EventListener;
+import java.util.concurrent.atomic.AtomicBoolean;
+import java.util.concurrent.locks.Lock;
+import java.util.concurrent.locks.ReentrantLock;
+
 import javax.swing.event.EventListenerList;
 
-
-
 /**
  * Fires one or more {@code ActionEvent}s at specified
  * intervals. An example use is an animation object that uses a
@@ -124,20 +123,13 @@ import javax.swing.event.EventListenerList;
  * <a href="https://docs.oracle.com/javase/tutorial/uiswing/misc/timer.html"
  * target = "_top">How to Use Timers</a>,
  * a section in <em>The Java Tutorial.</em>
- * For more examples and help in choosing between
- * this <code>Timer</code> class and
- * <code>java.util.Timer</code>,
- * see
- * <a href="http://java.sun.com/products/jfc/tsc/articles/timer/"
- * target="_top">Using Timers in Swing Applications</a>,
- * an article in <em>The Swing Connection.</em>
  * <p>
  * <strong>Warning:</strong>
  * Serialized objects of this class will not be compatible with
  * future Swing releases. The current serialization support is
  * appropriate for short term storage or RMI between applications running
  * the same version of Swing.  As of 1.4, support for long term storage
- * of all JavaBeans&trade;
+ * of all JavaBeans
  * has been added to the <code>java.beans</code> package.
  * Please see {@link java.beans.XMLEncoder}.
  *
@@ -164,7 +156,7 @@ public class Timer implements Serializable
     //    EventQueue and be pending (ie in the process of notifying the
     //    ActionListener). If we didn't do this it would allow for a
     //    situation where the app is taking too long to process the
-    //    actionPerformed, and thus we'ld end up queing a bunch of Runnables
+    //    actionPerformed, and thus we'd end up queueing a bunch of Runnables
     //    and the app would never return: not good. This of course implies
     //    you can get dropped events, but such is life.
     // notify is used to indicate if the ActionListener can be notified, when
@@ -220,12 +212,14 @@ public class Timer implements Serializable
     /*
      * The timer's AccessControlContext.
      */
+     @SuppressWarnings("removal")
      private transient volatile AccessControlContext acc =
             AccessController.getContext();
 
     /**
       * Returns the acc this timer was constructed with.
       */
+     @SuppressWarnings("removal")
      final AccessControlContext getAccessControlContext() {
        if (acc == null) {
            throw new SecurityException(
@@ -347,7 +341,7 @@ public class Timer implements Serializable
      *          on this timer,
      *          or an empty array if no such
      *          listeners have been added
-     * @exception ClassCastException if <code>listenerType</code> doesn't
+     * @throws ClassCastException if <code>listenerType</code> doesn't
      *          specify a class or interface that implements
      *          <code>java.util.EventListener</code>
      *
@@ -444,7 +438,7 @@ public class Timer implements Serializable
     /**
      * Returns the {@code Timer}'s initial delay.
      *
-     * @return the {@code Timer}'s intial delay, in milliseconds
+     * @return the {@code Timer}'s initial delay, in milliseconds
      * @see #setInitialDelay
      * @see #setDelay
      */
@@ -615,6 +609,7 @@ public class Timer implements Serializable
     }
 
 
+    @SuppressWarnings("removal")
     void post() {
          if (notify.compareAndSet(false, true) || !coalesce) {
              AccessController.doPrivileged(new PrivilegedAction<Void>() {
@@ -630,6 +625,8 @@ public class Timer implements Serializable
         return lock;
     }
 
+    @SuppressWarnings("removal")
+    @Serial
     private void readObject(ObjectInputStream in)
         throws ClassNotFoundException, IOException
     {
@@ -660,6 +657,7 @@ public class Timer implements Serializable
      * We have to use readResolve because we can not initialize final
      * fields for deserialized object otherwise
      */
+    @Serial
     private Object readResolve() {
         Timer timer = new Timer(getDelay(), null);
         timer.listenerList = listenerList;
