@@ -80,6 +80,11 @@ public class MetalTabbedPaneUI extends BasicTabbedPaneUI {
     // Selected border color for ocean.
     private Color oceanSelectedBorderColor;
 
+    /**
+     * Constructs a {@code MetalTabbedPaneUI}.
+     */
+    public MetalTabbedPaneUI() {}
+
     /**
      * Constructs {@code MetalTabbedPaneUI}.
      *
