@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2021, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -56,6 +56,12 @@ public class SynthTextFieldUI extends BasicTextFieldUI implements SynthUI {
     private SynthStyle style;
     private boolean updateKBAction = true;
 
+    /**
+     *
+     * Constructs a {@code SynthTextFieldUI}.
+     */
+    public SynthTextFieldUI() {}
+
     /**
      * Creates a UI for a JTextField.
      *
@@ -141,8 +147,7 @@ public class SynthTextFieldUI extends BasicTextFieldUI implements SynthUI {
         Caret caret = comp.getCaret();
         if (caret instanceof UIResource) {
             Object o = style.get(context, prefix + ".caretBlinkRate");
-            if (o != null && o instanceof Integer) {
-                Integer rate = (Integer)o;
+            if (o instanceof Integer rate) {
                 caret.setBlinkRate(rate.intValue());
             }
         }
@@ -217,7 +222,7 @@ public class SynthTextFieldUI extends BasicTextFieldUI implements SynthUI {
      */
     @Override
     protected void paintBackground(Graphics g) {
-        // Overriden to do nothing, all our painting is done from update/paint.
+        // Overridden to do nothing, all our painting is done from update/paint.
     }
 
     /**
