@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2004, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -29,8 +29,9 @@ import java.io.File;
 import java.io.IOException;
 
 /*
- * Solaris/Linux implementation of jdk.internal.agent.FileSystem
+ * Linux implementation of jdk.internal.agent.FileSystem
  */
+@SuppressWarnings("removal")
 public class FileSystemImpl extends FileSystem {
 
     public boolean supportsFileSecurity(File f) throws IOException {
@@ -38,6 +39,10 @@ public class FileSystemImpl extends FileSystem {
     }
 
     public boolean isAccessUserOnly(File f) throws IOException {
+        String path = f.getPath();
+        if (path.indexOf(0) >= 0) {
+            throw new IOException("illegal filename");
+        }
         return isAccessUserOnly0(f.getPath());
     }
 
