@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -170,20 +170,16 @@ final class JrtPath implements Path {
 
     @Override
     public final URI toUri() {
-        try {
-            String p = toAbsolutePath().path;
-            if (!p.startsWith("/modules") || p.contains("..")) {
-                throw new IOError(new RuntimeException(p + " cannot be represented as URI"));
-            }
-
-            p = p.substring("/modules".length());
-            if (p.isEmpty()) {
-                p = "/";
-            }
-            return new URI("jrt", p, null);
-        } catch (URISyntaxException ex) {
-            throw new AssertionError(ex);
+        String p = toAbsolutePath().path;
+        if (!p.startsWith("/modules") || p.contains("..")) {
+            throw new IOError(new RuntimeException(p + " cannot be represented as URI"));
         }
+
+        p = p.substring("/modules".length());
+        if (p.isEmpty()) {
+            p = "/";
+        }
+        return toUri(p);
     }
 
     private boolean equalsNameAt(JrtPath other, int index) {
@@ -435,7 +431,7 @@ final class JrtPath implements Path {
         return r;
     }
 
-    // removes redundant slashs, replace "\" to separator "/"
+    // removes redundant slashes, replace "\" to separator "/"
     // and check for invalid characters
     private static String normalize(String path) {
         int len = path.length();
@@ -489,7 +485,7 @@ final class JrtPath implements Path {
     // Remove DotSlash(./) and resolve DotDot (..) components
     private String getResolved() {
         int length = path.length();
-        if (length == 0 || (path.indexOf("./") == -1 && path.charAt(length - 1) != '.')) {
+        if (length == 0 || (!path.contains("./") && path.charAt(length - 1) != '.')) {
             return path;
         } else {
             return resolvePath();
@@ -631,11 +627,9 @@ final class JrtPath implements Path {
     }
 
     final InputStream newInputStream(OpenOption... options) throws IOException {
-        if (options.length > 0) {
-            for (OpenOption opt : options) {
-                if (opt != READ) {
-                    throw new UnsupportedOperationException("'" + opt + "' not allowed");
-                }
+        for (OpenOption opt : options) {
+            if (opt != READ) {
+                throw new UnsupportedOperationException("'" + opt + "' not allowed");
             }
         }
         return jrtfs.newInputStream(this);
@@ -655,11 +649,7 @@ final class JrtPath implements Path {
     }
 
     final JrtFileAttributes getAttributes(LinkOption... options) throws IOException {
-        JrtFileAttributes zfas = jrtfs.getFileAttributes(this, options);
-        if (zfas == null) {
-            throw new NoSuchFileException(toString());
-        }
-        return zfas;
+        return jrtfs.getFileAttributes(this, options);
     }
 
     final void setAttribute(String attribute, Object value, LinkOption... options)
@@ -825,4 +815,135 @@ final class JrtPath implements Path {
             }
         }
     }
+
+    // adopted from sun.nio.fs.UnixUriUtils
+    private static URI toUri(String str) {
+        char[] path = str.toCharArray();
+        assert path[0] == '/';
+        StringBuilder sb = new StringBuilder();
+        sb.append(path[0]);
+        for (int i = 1; i < path.length; i++) {
+            char c = (char)(path[i] & 0xff);
+            if (match(c, L_PATH, H_PATH)) {
+                sb.append(c);
+            } else {
+                sb.append('%');
+                sb.append(hexDigits[(c >> 4) & 0x0f]);
+                sb.append(hexDigits[(c) & 0x0f]);
+            }
+        }
+
+        try {
+            return new URI("jrt:" + sb.toString());
+        } catch (URISyntaxException x) {
+            throw new AssertionError(x);  // should not happen
+        }
+    }
+
+    // The following is copied from java.net.URI
+
+    // Compute the low-order mask for the characters in the given string
+    private static long lowMask(String chars) {
+        int n = chars.length();
+        long m = 0;
+        for (int i = 0; i < n; i++) {
+            char c = chars.charAt(i);
+            if (c < 64)
+                m |= (1L << c);
+        }
+        return m;
+    }
+
+    // Compute the high-order mask for the characters in the given string
+    private static long highMask(String chars) {
+        int n = chars.length();
+        long m = 0;
+        for (int i = 0; i < n; i++) {
+            char c = chars.charAt(i);
+            if ((c >= 64) && (c < 128))
+                m |= (1L << (c - 64));
+        }
+        return m;
+    }
+
+    // Compute a low-order mask for the characters
+    // between first and last, inclusive
+    private static long lowMask(char first, char last) {
+        long m = 0;
+        int f = Math.max(Math.min(first, 63), 0);
+        int l = Math.max(Math.min(last, 63), 0);
+        for (int i = f; i <= l; i++)
+            m |= 1L << i;
+        return m;
+    }
+
+    // Compute a high-order mask for the characters
+    // between first and last, inclusive
+    private static long highMask(char first, char last) {
+        long m = 0;
+        int f = Math.max(Math.min(first, 127), 64) - 64;
+        int l = Math.max(Math.min(last, 127), 64) - 64;
+        for (int i = f; i <= l; i++)
+            m |= 1L << i;
+        return m;
+    }
+
+    // Tell whether the given character is permitted by the given mask pair
+    private static boolean match(char c, long lowMask, long highMask) {
+        if (c < 64)
+            return ((1L << c) & lowMask) != 0;
+        if (c < 128)
+            return ((1L << (c - 64)) & highMask) != 0;
+        return false;
+    }
+
+    // digit    = "0" | "1" | "2" | "3" | "4" | "5" | "6" | "7" |
+    //            "8" | "9"
+    private static final long L_DIGIT = lowMask('0', '9');
+    private static final long H_DIGIT = 0L;
+
+    // upalpha  = "A" | "B" | "C" | "D" | "E" | "F" | "G" | "H" | "I" |
+    //            "J" | "K" | "L" | "M" | "N" | "O" | "P" | "Q" | "R" |
+    //            "S" | "T" | "U" | "V" | "W" | "X" | "Y" | "Z"
+    private static final long L_UPALPHA = 0L;
+    private static final long H_UPALPHA = highMask('A', 'Z');
+
+    // lowalpha = "a" | "b" | "c" | "d" | "e" | "f" | "g" | "h" | "i" |
+    //            "j" | "k" | "l" | "m" | "n" | "o" | "p" | "q" | "r" |
+    //            "s" | "t" | "u" | "v" | "w" | "x" | "y" | "z"
+    private static final long L_LOWALPHA = 0L;
+    private static final long H_LOWALPHA = highMask('a', 'z');
+
+    // alpha         = lowalpha | upalpha
+    private static final long L_ALPHA = L_LOWALPHA | L_UPALPHA;
+    private static final long H_ALPHA = H_LOWALPHA | H_UPALPHA;
+
+    // alphanum      = alpha | digit
+    private static final long L_ALPHANUM = L_DIGIT | L_ALPHA;
+    private static final long H_ALPHANUM = H_DIGIT | H_ALPHA;
+
+    // mark          = "-" | "_" | "." | "!" | "~" | "*" | "'" |
+    //                 "(" | ")"
+    private static final long L_MARK = lowMask("-_.!~*'()");
+    private static final long H_MARK = highMask("-_.!~*'()");
+
+    // unreserved    = alphanum | mark
+    private static final long L_UNRESERVED = L_ALPHANUM | L_MARK;
+    private static final long H_UNRESERVED = H_ALPHANUM | H_MARK;
+
+    // pchar         = unreserved | escaped |
+    //                 ":" | "@" | "&" | "=" | "+" | "$" | ","
+    private static final long L_PCHAR
+        = L_UNRESERVED | lowMask(":@&=+$,");
+    private static final long H_PCHAR
+        = H_UNRESERVED | highMask(":@&=+$,");
+
+   // All valid path characters
+   private static final long L_PATH = L_PCHAR | lowMask(";/");
+   private static final long H_PATH = H_PCHAR | highMask(";/");
+
+   private static final char[] hexDigits = {
+        '0', '1', '2', '3', '4', '5', '6', '7',
+        '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
+    };
 }
