@@ -38,6 +38,7 @@ import java.util.function.BooleanSupplier;
 
 import jdk.internal.vm.annotation.Stable;
 import sun.security.util.SecurityConstants;
+import sun.security.action.GetBooleanAction;
 import sun.security.action.GetPropertyAction;
 
 /**
@@ -121,8 +122,8 @@ public final class LoggerFinderLoader {
     // Whether multiple provider should be considered as an error.
     // This is further submitted to the configuration error policy.
     private static boolean ensureSingletonProvider() {
-        return Boolean.parseBoolean(
-                GetPropertyAction.privilegedGetProperty("jdk.logger.finder.singleton"));
+        return GetBooleanAction.privilegedGetProperty
+            ("jdk.logger.finder.singleton");
     }
 
     @SuppressWarnings("removal")
@@ -183,7 +184,7 @@ public final class LoggerFinderLoader {
                 result = iterator.next();
                 if (iterator.hasNext() && ensureSingletonProvider()) {
                     throw new ServiceConfigurationError(
-                            "More than on LoggerFinder implementation");
+                            "More than one LoggerFinder implementation");
                 }
             } else {
                 result = loadDefaultImplementation();
