@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2024, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -37,6 +37,7 @@ import java.nio.file.Paths;
 import java.security.AccessControlContext;
 import java.security.AccessController;
 import java.util.List;
+import java.util.Objects;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ConcurrentMap;
 import java.util.function.Function;
@@ -45,6 +46,7 @@ import java.net.http.HttpResponse;
 import java.net.http.HttpResponse.BodyHandler;
 import java.net.http.HttpResponse.ResponseInfo;
 import java.net.http.HttpResponse.BodySubscriber;
+import java.util.Set;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import jdk.internal.net.http.ResponseSubscribers.PathSubscriber;
@@ -64,6 +66,7 @@ public final class ResponseBodyHandlers {
     public static class PathBodyHandler implements BodyHandler<Path>{
         private final Path file;
         private final List<OpenOption> openOptions;  // immutable list
+        @SuppressWarnings("removal")
         private final AccessControlContext acc;
         private final FilePermission filePermission;
 
@@ -77,6 +80,7 @@ public final class ResponseBodyHandlers {
         public static PathBodyHandler create(Path file,
                                              List<OpenOption> openOptions) {
             FilePermission filePermission = null;
+            @SuppressWarnings("removal")
             SecurityManager sm = System.getSecurityManager();
             if (sm != null) {
                 try {
@@ -90,13 +94,14 @@ public final class ResponseBodyHandlers {
             }
 
             assert filePermission == null || filePermission.getActions().equals("write");
+            @SuppressWarnings("removal")
             var acc = sm != null ? AccessController.getContext() : null;
             return new PathBodyHandler(file, openOptions, acc, filePermission);
         }
 
         private PathBodyHandler(Path file,
                                 List<OpenOption> openOptions,
-                                AccessControlContext acc,
+                                @SuppressWarnings("removal") AccessControlContext acc,
                                 FilePermission filePermission) {
             this.file = file;
             this.openOptions = openOptions;
@@ -133,16 +138,21 @@ public final class ResponseBodyHandlers {
             if (!initiatingURI.getHost().equalsIgnoreCase(pushRequestURI.getHost()))
                 return;
 
+            String initiatingScheme = initiatingURI.getScheme();
+            String pushRequestScheme = pushRequestURI.getScheme();
+
+            if (!initiatingScheme.equalsIgnoreCase(pushRequestScheme)) return;
+
             int initiatingPort = initiatingURI.getPort();
             if (initiatingPort == -1 ) {
-                if ("https".equalsIgnoreCase(initiatingURI.getScheme()))
+                if ("https".equalsIgnoreCase(initiatingScheme))
                     initiatingPort = 443;
                 else
                     initiatingPort = 80;
             }
             int pushPort = pushRequestURI.getPort();
             if (pushPort == -1 ) {
-                if ("https".equalsIgnoreCase(pushRequestURI.getScheme()))
+                if ("https".equalsIgnoreCase(pushRequestScheme))
                     pushPort = 443;
                 else
                     pushPort = 80;
@@ -160,6 +170,7 @@ public final class ResponseBodyHandlers {
     public static class FileDownloadBodyHandler implements BodyHandler<Path> {
         private final Path directory;
         private final List<OpenOption> openOptions;
+        @SuppressWarnings("removal")
         private final AccessControlContext acc;
         private final FilePermission[] filePermissions;  // may be null
 
@@ -181,6 +192,7 @@ public final class ResponseBodyHandlers {
             }
 
             FilePermission filePermissions[] = null;
+            @SuppressWarnings("removal")
             SecurityManager sm = System.getSecurityManager();
             if (sm != null) {
                 FilePermission writePermission = new FilePermission(fn, "write");
@@ -205,13 +217,14 @@ public final class ResponseBodyHandlers {
 
             assert filePermissions == null || (filePermissions[0].getActions().equals("write")
                     && filePermissions[1].getActions().equals("write"));
+            @SuppressWarnings("removal")
             var acc = sm != null ? AccessController.getContext() : null;
             return new FileDownloadBodyHandler(directory, openOptions, acc, filePermissions);
         }
 
         private FileDownloadBodyHandler(Path directory,
                                        List<OpenOption> openOptions,
-                                       AccessControlContext acc,
+                                       @SuppressWarnings("removal") AccessControlContext acc,
                                        FilePermission... filePermissions) {
             this.directory = directory;
             this.openOptions = openOptions;
@@ -223,16 +236,120 @@ public final class ResponseBodyHandlers {
         static final String DISPOSITION_TYPE = "attachment;";
 
         /** The "filename" parameter. */
-        static final Pattern FILENAME = Pattern.compile("filename\\s*=", CASE_INSENSITIVE);
+        static final Pattern FILENAME = Pattern.compile("filename\\s*=\\s*", CASE_INSENSITIVE);
 
         static final List<String> PROHIBITED = List.of(".", "..", "", "~" , "|");
 
+        // Characters disallowed in token values
+
+        static final Set<Character> NOT_ALLOWED_IN_TOKEN = Set.of(
+            '(', ')', '<', '>', '@',
+            ',', ';', ':', '\\', '"',
+            '/', '[', ']', '?', '=',
+            '{', '}', ' ', '\t');
+
+        static boolean allowedInToken(char c) {
+            if (NOT_ALLOWED_IN_TOKEN.contains(c))
+                return false;
+            // exclude CTL chars <= 31, == 127, or anything >= 128
+            return isTokenText(c);
+        }
+
         static final UncheckedIOException unchecked(ResponseInfo rinfo,
                                                     String msg) {
             String s = String.format("%s in response [%d, %s]", msg, rinfo.statusCode(), rinfo.headers());
             return new UncheckedIOException(new IOException(s));
         }
 
+        static final UncheckedIOException unchecked(String msg) {
+            return new UncheckedIOException(new IOException(msg));
+        }
+
+        // Process a "filename=" parameter, which is either a "token"
+        // or a "quoted string". If a token, it is terminated by a
+        // semicolon or the end of the string.
+        // If a quoted string (surrounded by "" chars then the closing "
+        // terminates the name.
+        // quoted strings may contain quoted-pairs (eg embedded " chars)
+
+        static String processFilename(String src) throws UncheckedIOException {
+            if ("".equals(src))
+                return src;
+            if (src.charAt(0) == '\"') {
+                return processQuotedString(src.substring(1));
+            } else {
+                return processToken(src);
+            }
+        }
+
+        static boolean isTokenText(char c) throws UncheckedIOException {
+            return c > 31 && c < 127;
+        }
+
+        static boolean isQuotedStringText(char c) throws UncheckedIOException {
+            return c > 31;
+        }
+
+        static String processQuotedString(String src) throws UncheckedIOException {
+            boolean inqpair = false;
+            int len = src.length();
+            StringBuilder sb = new StringBuilder();
+
+            for (int i=0; i<len; i++) {
+                char c = src.charAt(i);
+                if (!isQuotedStringText(c)) {
+                    throw unchecked("Illegal character");
+                }
+                if (c == '\"') {
+                    if (!inqpair) {
+                        return sb.toString();
+                    } else {
+                        sb.append(c);
+                    }
+                } else if (c == '\\') {
+                    if (!inqpair) {
+                        inqpair = true;
+                        continue;
+                    } else {
+                        // the quoted char is '\'
+                        sb.append(c);
+                    }
+                } else {
+                    sb.append(c);
+                }
+                if (inqpair) {
+                    inqpair = false;
+                }
+            }
+            // not terminated by "
+            throw unchecked("Invalid quoted string");
+        }
+
+        static String processToken(String src) throws UncheckedIOException {
+            int end = 0;
+            int len = src.length();
+            boolean whitespace = false;
+
+            for (int i=0; i<len; i++) {
+                char c = src.charAt(i);
+                if (c == ';') {
+                    break;
+                }
+                if (c == ' ' || c == '\t') {
+                    // WS only until ; or end of string
+                    whitespace = true;
+                    continue;
+                }
+                end++;
+                if (whitespace || !allowedInToken(c)) {
+                    String msg = whitespace ? "whitespace must be followed by a semicolon"
+                                            : c + " is not allowed in a token";
+                    throw unchecked(msg);
+                }
+            }
+            return src.substring(0, end);
+        }
+
         @Override
         public BodySubscriber<Path> apply(ResponseInfo responseInfo) {
             String dispoHeader = responseInfo.headers().firstValue("Content-Disposition")
@@ -250,13 +367,7 @@ public final class ResponseBodyHandlers {
             }
             int n = matcher.end();
 
-            int semi = dispoHeader.substring(n).indexOf(";");
-            String filenameParam;
-            if (semi < 0) {
-                filenameParam = dispoHeader.substring(n);
-            } else {
-                filenameParam = dispoHeader.substring(n, n + semi);
-            }
+            String filenameParam = processFilename(dispoHeader.substring(n));
 
             // strip all but the last path segment
             int x = filenameParam.lastIndexOf("/");
@@ -270,19 +381,6 @@ public final class ResponseBodyHandlers {
 
             filenameParam = filenameParam.trim();
 
-            if (filenameParam.startsWith("\"")) {  // quoted-string
-                if (!filenameParam.endsWith("\"") || filenameParam.length() == 1) {
-                    throw unchecked(responseInfo,
-                            "Badly quoted Content-Disposition filename parameter");
-                }
-                filenameParam = filenameParam.substring(1, filenameParam.length() -1 );
-            } else {  // token,
-                if (filenameParam.contains(" ")) {  // space disallowed
-                    throw unchecked(responseInfo,
-                            "unquoted space in Content-Disposition filename parameter");
-                }
-            }
-
             if (PROHIBITED.contains(filenameParam)) {
                 throw unchecked(responseInfo,
                         "Prohibited Content-Disposition filename parameter:"
