@@ -28,6 +28,7 @@ package jdk.internal.platform.cgroupv1;
 public class CgroupV1MemorySubSystemController extends CgroupV1SubsystemController {
 
     private boolean hierarchical;
+    private boolean swapenabled;
 
     public CgroupV1MemorySubSystemController(String root, String mountPoint) {
         super(root, mountPoint);
@@ -41,4 +42,11 @@ public class CgroupV1MemorySubSystemController extends CgroupV1SubsystemControll
         this.hierarchical = hierarchical;
     }
 
-}
\ No newline at end of file
+    boolean isSwapEnabled() {
+        return swapenabled;
+    }
+
+    void setSwapEnabled(boolean swapenabled) {
+        this.swapenabled = swapenabled;
+    }
+}
