@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004, 2005, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2004, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -77,7 +77,7 @@ class UnsafeQualifiedStaticObjectFieldAccessorImpl
             throwFinalFieldIllegalAccessException(value);
         }
         if (value != null) {
-            if (!field.getType().isAssignableFrom(value.getClass())) {
+            if (!field.getType().isInstance(value)) {
                 throwSetIllegalArgumentException(value);
             }
         }
