@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2020, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2020, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -210,6 +210,23 @@ public class VectorSupport {
         return defaultImpl.fromBits(bits, s);
     }
 
+    /* ============================================================================ */
+    public interface IndexPartiallyInUpperRangeOperation<E,
+                                                         M extends VectorMask<E>> {
+        M apply(long offset, long limit);
+    }
+
+    @IntrinsicCandidate
+    public static
+    <E,
+     M extends VectorMask<E>>
+    M indexPartiallyInUpperRange(Class<? extends M> mClass, Class<E> eClass,
+                                 int length, long offset, long limit,
+                                 IndexPartiallyInUpperRangeOperation<E, M> defaultImpl) {
+        assert isNonCapturingLambda(defaultImpl) : defaultImpl;
+        return defaultImpl.apply(offset, limit);
+    }
+
     /* ============================================================================ */
     public interface ShuffleIotaOperation<S extends VectorSpecies<?>,
                                           SH extends VectorShuffle<?>> {
