@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -28,7 +28,6 @@ package jdk.javadoc.internal.doclets.formats.html;
 import java.util.List;
 import java.util.ListIterator;
 import java.util.SortedSet;
-import java.util.stream.Collectors;
 import java.util.stream.Stream;
 
 import javax.lang.model.element.Element;
@@ -59,11 +58,6 @@ import jdk.javadoc.internal.doclets.toolkit.util.IndexItem;
  * documented elements, terms defined in some documentation comments,
  * and summary pages.
  *
- *  <p><b>This is NOT part of any supported API.
- *  If you write code that depends on this, you do so at your own risk.
- *  This code and its internal interfaces are subject to change or
- *  deletion without notice.</b>
- *
  * @see IndexBuilder
  */
 public class IndexWriter extends HtmlDocletWriter {
@@ -144,31 +138,31 @@ public class IndexWriter extends HtmlDocletWriter {
     /**
      * Adds a set of items to the page.
      *
-     * @param ch          the first character of the names of the items
-     * @param items       the items
-     * @param contentTree the tree to which to add the items
+     * @param ch      the first character of the names of the items
+     * @param items   the items
+     * @param content the content to which to add the items
      */
-    protected void addContents(char ch, SortedSet<IndexItem> items, Content contentTree) {
-        addHeading(ch, contentTree);
+    protected void addContents(char ch, SortedSet<IndexItem> items, Content content) {
+        addHeading(ch, content);
 
-        HtmlTree dl = HtmlTree.DL(HtmlStyle.index);
+        var dl = HtmlTree.DL(HtmlStyle.index);
         for (IndexItem item : items) {
             addDescription(item, dl);
         }
-        contentTree.add(dl);
+        content.add(dl);
     }
 
     /**
      * Adds a heading containing the first character for a set of items.
      *
-     * @param ch          the first character of the names of the items
-     * @param contentTree the tree to which to add the items
+     * @param ch      the first character of the names of the items
+     * @param content the content to which to add the items
      */
-    protected void addHeading(char ch, Content contentTree) {
+    protected void addHeading(char ch, Content content) {
         Content headContent = Text.of(String.valueOf(ch));
-        HtmlTree heading = HtmlTree.HEADING(Headings.CONTENT_HEADING, HtmlStyle.title, headContent)
+        var heading = HtmlTree.HEADING(Headings.CONTENT_HEADING, HtmlStyle.title, headContent)
                 .setId(HtmlIds.forIndexChar(ch));
-        contentTree.add(heading);
+        content.add(heading);
     }
 
     /**
@@ -189,9 +183,9 @@ public class IndexWriter extends HtmlDocletWriter {
      * Add one line summary comment for the item.
      *
      * @param item the item to be documented
-     * @param dlTree the content tree to which the description will be added
+     * @param target the content to which the description will be added
      */
-    protected void addElementDescription(IndexItem item, Content dlTree) {
+    protected void addElementDescription(IndexItem item, Content target) {
         Content dt;
         Element element = item.getElement();
         String label = item.getLabel();
@@ -215,7 +209,7 @@ public class IndexWriter extends HtmlDocletWriter {
             case ANNOTATION_TYPE:
             case INTERFACE:
                 dt = HtmlTree.DT(getLink(new HtmlLinkInfo(configuration,
-                        HtmlLinkInfo.Kind.INDEX, (TypeElement) element).style(HtmlStyle.typeNameLink)));
+                        HtmlLinkInfo.Kind.SHOW_TYPE_PARAMS_IN_LABEL, (TypeElement) element).style(HtmlStyle.typeNameLink)));
                 dt.add(" - ");
                 addClassInfo((TypeElement) element, dt);
                 break;
@@ -225,7 +219,7 @@ public class IndexWriter extends HtmlDocletWriter {
             case FIELD:
             case ENUM_CONSTANT:
                 TypeElement containingType = item.getContainingTypeElement();
-                dt = HtmlTree.DT(getDocLink(HtmlLinkInfo.Kind.INDEX, containingType, element,
+                dt = HtmlTree.DT(getDocLink(HtmlLinkInfo.Kind.PLAIN, containingType, element,
                                 label, HtmlStyle.memberNameLink));
                 dt.add(" - ");
                 addMemberDesc(element, containingType, dt);
@@ -234,24 +228,24 @@ public class IndexWriter extends HtmlDocletWriter {
             default:
                 throw new Error();
         }
-        dlTree.add(dt);
-        Content dd = new HtmlTree(TagName.DD);
+        target.add(dt);
+        var dd = new HtmlTree(TagName.DD);
         if (element.getKind() == ElementKind.MODULE || element.getKind() == ElementKind.PACKAGE) {
             addSummaryComment(element, dd);
         } else {
             addComment(element, dd);
         }
-        dlTree.add(dd);
+        target.add(dd);
     }
 
     /**
      * Adds information for the given type element.
      *
-     * @param te          the element
-     * @param contentTree the content tree to which the class info will be added
+     * @param te      the element
+     * @param content the content to which the class info will be added
      */
-    protected void addClassInfo(TypeElement te, Content contentTree) {
-        contentTree.add(contents.getContent("doclet.in",
+    protected void addClassInfo(TypeElement te, Content content) {
+        content.add(contents.getContent("doclet.in",
                 utils.getTypeElementKindName(te, false),
                 getPackageLink(utils.containingPackage(te),
                     getLocalizedPackageName(utils.containingPackage(te)))
@@ -262,23 +256,23 @@ public class IndexWriter extends HtmlDocletWriter {
      * Adds a description for an item found in a documentation comment.
      *
      * @param item   the item
-     * @param dlTree the list to which to add the description
+     * @param target the list to which to add the description
      */
-    protected void addTagDescription(IndexItem item, Content dlTree) {
+    protected void addTagDescription(IndexItem item, Content target) {
         String itemPath = pathToRoot.isEmpty() ? "" : pathToRoot.getPath() + "/";
         itemPath += item.getUrl();
-        HtmlTree labelLink = HtmlTree.A(itemPath, Text.of(item.getLabel()));
-        Content dt = HtmlTree.DT(labelLink.setStyle(HtmlStyle.searchTagLink));
+        var labelLink = HtmlTree.A(itemPath, Text.of(item.getLabel()));
+        var dt = HtmlTree.DT(labelLink.setStyle(HtmlStyle.searchTagLink));
         dt.add(" - ");
         dt.add(contents.getContent("doclet.Search_tag_in", item.getHolder()));
-        dlTree.add(dt);
-        Content dd = new HtmlTree(TagName.DD);
+        target.add(dt);
+        var dd = new HtmlTree(TagName.DD);
         if (item.getDescription().isEmpty()) {
             dd.add(Entity.NO_BREAK_SPACE);
         } else {
             dd.add(item.getDescription());
         }
-        dlTree.add(dd);
+        target.add(dd);
     }
 
     /**
@@ -288,40 +282,39 @@ public class IndexWriter extends HtmlDocletWriter {
      * the start and then print the normal comment.
      *
      * @param element     the element
-     * @param contentTree the content tree to which the comment will be added
+     * @param content the content to which the comment will be added
      */
-    protected void addComment(Element element, Content contentTree) {
-        Content span = HtmlTree.SPAN(HtmlStyle.deprecatedLabel, getDeprecatedPhrase(element));
-        HtmlTree div = new HtmlTree(TagName.DIV);
-        div.setStyle(HtmlStyle.deprecationBlock);
+    protected void addComment(Element element, Content content) {
+        var span = HtmlTree.SPAN(HtmlStyle.deprecatedLabel, getDeprecatedPhrase(element));
+        var div = HtmlTree.DIV(HtmlStyle.deprecationBlock);
         if (utils.isDeprecated(element)) {
             div.add(span);
             List<? extends DeprecatedTree> tags = utils.getDeprecatedTrees(element);
             if (!tags.isEmpty())
                 addInlineDeprecatedComment(element, tags.get(0), div);
-            contentTree.add(div);
+            content.add(div);
         } else {
             TypeElement encl = utils.getEnclosingTypeElement(element);
             while (encl != null) {
                 if (utils.isDeprecated(encl)) {
                     div.add(span);
-                    contentTree.add(div);
+                    content.add(div);
                     break;
                 }
                 encl = utils.getEnclosingTypeElement(encl);
             }
-            addSummaryComment(element, contentTree);
+            addSummaryComment(element, content);
         }
     }
 
     /**
      * Adds a description for a member element.
      *
-     * @param member      the element
-     * @param enclosing   the enclosing type element
-     * @param contentTree the content tree to which the member description will be added
+     * @param member    the element
+     * @param enclosing the enclosing type element
+     * @param content   the content to which the member description will be added
      */
-    protected void addMemberDesc(Element member, TypeElement enclosing, Content contentTree) {
+    protected void addMemberDesc(Element member, TypeElement enclosing, Content content) {
         String kindName = utils.getTypeElementKindName(enclosing, true);
         String resource = switch (member.getKind()) {
             case ENUM_CONSTANT ->
@@ -331,24 +324,24 @@ public class IndexWriter extends HtmlDocletWriter {
             case CONSTRUCTOR ->
                     "doclet.Constructor_for";
             case METHOD ->
-                    utils.isAnnotationType(enclosing) ? "doclet.Element_in"
+                    utils.isAnnotationInterface(enclosing) ? "doclet.Element_in"
                             : utils.isStatic(member) ? "doclet.Static_method_in" : "doclet.Method_in";
             case RECORD_COMPONENT ->
                     "doclet.Record_component_in";
             default -> throw new IllegalArgumentException(member.getKind().toString());
         };
-        contentTree.add(contents.getContent(resource, kindName)).add(" ");
-        addPreQualifiedClassLink(HtmlLinkInfo.Kind.INDEX, enclosing,
-                null, contentTree);
+        content.add(contents.getContent(resource, kindName)).add(" ");
+        addPreQualifiedClassLink(HtmlLinkInfo.Kind.PLAIN, enclosing,
+                null, content);
     }
 
     /**
      * Add links for all the index files, based on the first character of the names of the items.
      *
      * @param allFirstCharacters the list of all first characters to be linked
-     * @param contentTree        the content tree to which the links for indexes will be added
+     * @param content            the content to which the links for indexes will be added
      */
-    protected void addLinksForIndexes(List<Character> allFirstCharacters, Content contentTree) {
+    protected void addLinksForIndexes(List<Character> allFirstCharacters, Content content) {
         ListIterator<Character> iter = allFirstCharacters.listIterator();
         while (iter.hasNext()) {
             char ch = iter.next();
@@ -356,11 +349,11 @@ public class IndexWriter extends HtmlDocletWriter {
             Content link = splitIndex
                     ? links.createLink(DocPaths.indexN(iter.nextIndex()), label)
                     : links.createLink(HtmlIds.forIndexChar(ch), label);
-            contentTree.add(link);
-            contentTree.add(Entity.NO_BREAK_SPACE);
+            content.add(link);
+            content.add(Entity.NO_BREAK_SPACE);
         }
 
-        contentTree.add(new HtmlTree(TagName.BR));
+        content.add(new HtmlTree(TagName.BR));
         List<Content> pageLinks = Stream.of(IndexItem.Category.values())
                 .flatMap(c -> mainIndex.getItems(c).stream())
                 .filter(i -> !(i.isElementItem() || i.isTagItem()))
@@ -368,7 +361,7 @@ public class IndexWriter extends HtmlDocletWriter {
                 .map(i -> links.createLink(pathToRoot.resolve(i.getUrl()),
                         contents.getNonBreakString(i.getLabel())))
                 .toList();
-        contentTree.add(contents.join(getVerticalSeparator(), pageLinks));
+        content.add(contents.join(getVerticalSeparator(), pageLinks));
     }
 
     private Content getVerticalSeparator() {
