@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2001, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -27,8 +27,9 @@ package jdk.javadoc.internal.doclets.formats.html;
 
 import jdk.javadoc.internal.doclets.formats.html.markup.Head;
 
-import java.io.*;
-import java.util.List;
+import java.io.IOException;
+import java.io.LineNumberReader;
+import java.io.Reader;
 
 import javax.lang.model.element.Element;
 import javax.lang.model.element.ModuleElement;
@@ -36,13 +37,12 @@ import javax.lang.model.element.PackageElement;
 import javax.lang.model.element.TypeElement;
 import javax.tools.FileObject;
 
-import jdk.javadoc.doclet.DocletEnvironment;
-import jdk.javadoc.internal.doclets.formats.html.markup.HtmlAttr;
 import jdk.javadoc.internal.doclets.formats.html.markup.HtmlDocument;
+import jdk.javadoc.internal.doclets.formats.html.markup.HtmlId;
 import jdk.javadoc.internal.doclets.formats.html.markup.HtmlStyle;
-import jdk.javadoc.internal.doclets.formats.html.markup.HtmlTag;
+import jdk.javadoc.internal.doclets.formats.html.markup.TagName;
 import jdk.javadoc.internal.doclets.formats.html.markup.HtmlTree;
-import jdk.javadoc.internal.doclets.formats.html.markup.StringContent;
+import jdk.javadoc.internal.doclets.formats.html.markup.Text;
 import jdk.javadoc.internal.doclets.toolkit.Content;
 import jdk.javadoc.internal.doclets.toolkit.Messages;
 import jdk.javadoc.internal.doclets.toolkit.Resources;
@@ -50,20 +50,11 @@ import jdk.javadoc.internal.doclets.toolkit.util.DocFile;
 import jdk.javadoc.internal.doclets.toolkit.util.DocFileIOException;
 import jdk.javadoc.internal.doclets.toolkit.util.DocPath;
 import jdk.javadoc.internal.doclets.toolkit.util.DocPaths;
-import jdk.javadoc.internal.doclets.toolkit.util.DocletConstants;
 import jdk.javadoc.internal.doclets.toolkit.util.SimpleDocletException;
 import jdk.javadoc.internal.doclets.toolkit.util.Utils;
 
 /**
  * Converts Java Source Code to HTML.
- *
- *  <p><b>This is NOT part of any supported API.
- *  If you write code that depends on this, you do so at your own risk.
- *  This code and its internal interfaces are subject to change or
- *  deletion without notice.</b>
- *
- * @author Jamie Ho
- * @author Bhavesh Patel (Modified)
  */
 public class SourceToHTMLConverter {
 
@@ -77,15 +68,14 @@ public class SourceToHTMLConverter {
     /**
      * New line to be added to the documentation.
      */
-    private static final String NEW_LINE = DocletConstants.NL;
+    private static final String NEW_LINE = Text.NL;
 
     private final HtmlConfiguration configuration;
+    private final HtmlOptions options;
     private final Messages messages;
     private final Resources resources;
     private final Utils utils;
 
-    private final DocletEnvironment docEnv;
-
     private final DocPath outputdir;
 
     /**
@@ -94,13 +84,12 @@ public class SourceToHTMLConverter {
      */
     private DocPath relativePath = DocPath.empty;
 
-    private SourceToHTMLConverter(HtmlConfiguration configuration, DocletEnvironment rd,
-                                  DocPath outputdir) {
+    private SourceToHTMLConverter(HtmlConfiguration configuration, DocPath outputdir) {
         this.configuration  = configuration;
+        this.options = configuration.getOptions();
         this.messages = configuration.getMessages();
-        this.resources = configuration.resources;
+        this.resources = configuration.docResources;
         this.utils = configuration.utils;
-        this.docEnv = rd;
         this.outputdir = outputdir;
     }
 
@@ -108,37 +97,36 @@ public class SourceToHTMLConverter {
      * Translate the TypeElements in the given DocletEnvironment to HTML representation.
      *
      * @param configuration the configuration.
-     * @param docEnv the DocletEnvironment to convert.
      * @param outputdir the name of the directory to output to.
      * @throws DocFileIOException if there is a problem generating an output file
      * @throws SimpleDocletException if there is a problem reading a source file
      */
-    public static void convertRoot(HtmlConfiguration configuration, DocletEnvironment docEnv,
-                                   DocPath outputdir) throws DocFileIOException, SimpleDocletException {
-        new SourceToHTMLConverter(configuration, docEnv, outputdir).generate();
+    public static void convertRoot(HtmlConfiguration configuration, DocPath outputdir)
+            throws DocFileIOException, SimpleDocletException {
+        new SourceToHTMLConverter(configuration, outputdir).generate();
     }
 
     void generate() throws DocFileIOException, SimpleDocletException {
-        if (docEnv == null || outputdir == null) {
+        if (outputdir == null) {
             return;
         }
         for (ModuleElement mdl : configuration.getSpecifiedModuleElements()) {
             // If -nodeprecated option is set and the module is marked as deprecated,
             // do not convert the module files to HTML.
-            if (!(configuration.nodeprecated && utils.isDeprecated(mdl)))
+            if (!(options.noDeprecated() && utils.isDeprecated(mdl)))
                 convertModule(mdl, outputdir);
         }
         for (PackageElement pkg : configuration.getSpecifiedPackageElements()) {
             // If -nodeprecated option is set and the package is marked as deprecated,
             // do not convert the package files to HTML.
-            if (!(configuration.nodeprecated && utils.isDeprecated(pkg)))
+            if (!(options.noDeprecated() && utils.isDeprecated(pkg)))
                 convertPackage(pkg, outputdir);
         }
         for (TypeElement te : configuration.getSpecifiedTypeElements()) {
             // If -nodeprecated option is set and the class is marked as deprecated
             // or the containing package is deprecated, do not convert the
             // package files to HTML.
-            if (!(configuration.nodeprecated &&
+            if (!(options.noDeprecated() &&
                   (utils.isDeprecated(te) || utils.isDeprecated(utils.containingPackage(te)))))
                 convertClass(te, outputdir);
         }
@@ -157,13 +145,13 @@ public class SourceToHTMLConverter {
         if (pkg == null) {
             return;
         }
-        for (Element te : utils.getAllClasses(pkg)) {
+        for (TypeElement te : utils.getAllClasses(pkg)) {
             // If -nodeprecated option is set and the class is marked as deprecated,
             // do not convert the package files to HTML. We do not check for
             // containing package deprecation since it is already check in
             // the calling method above.
-            if (!(configuration.nodeprecated && utils.isDeprecated(te)))
-                convertClass((TypeElement)te, outputdir);
+            if (!(options.noDeprecated() && utils.isDeprecated(te)))
+                convertClass(te, outputdir);
         }
     }
 
@@ -181,9 +169,9 @@ public class SourceToHTMLConverter {
             return;
         }
         for (Element elem : mdl.getEnclosedElements()) {
-            if (elem instanceof PackageElement && configuration.docEnv.isIncluded(elem)
-                    && !(configuration.nodeprecated && utils.isDeprecated(elem))) {
-                convertPackage((PackageElement) elem, outputdir);
+            if (elem instanceof PackageElement pkg && configuration.docEnv.isIncluded(elem)
+                    && !(options.noDeprecated() && utils.isDeprecated(elem))) {
+                convertPackage(pkg, outputdir);
             }
         }
     }
@@ -213,8 +201,8 @@ public class SourceToHTMLConverter {
                     .resolve(configuration.docPaths.forPackage(te))
                     .invert();
             Content body = getHeader();
-            Content pre = new HtmlTree(HtmlTag.PRE);
-            try (LineNumberReader reader = new LineNumberReader(r)) {
+            var pre = new HtmlTree(TagName.PRE);
+            try (var reader = new LineNumberReader(r)) {
                 while ((line = reader.readLine()) != null) {
                     addLineNo(pre, lineno);
                     addLine(pre, line, lineno);
@@ -222,7 +210,7 @@ public class SourceToHTMLConverter {
                 }
             }
             addBlankLines(pre);
-            Content div = HtmlTree.DIV(HtmlStyle.sourceContainer, pre);
+            var div = HtmlTree.DIV(HtmlStyle.sourceContainer, pre);
             body.add(HtmlTree.MAIN(div));
             writeToFile(body, outputdir.resolve(configuration.docPaths.forClass(te)), te);
         } catch (IOException e) {
@@ -238,28 +226,27 @@ public class SourceToHTMLConverter {
      * @param path the path for the file.
      */
     private void writeToFile(Content body, DocPath path, TypeElement te) throws DocFileIOException {
-        Head head = new Head(path, configuration.docletVersion)
-//                .setTimestamp(!configuration.notimestamp) // temporary: compatibility!
+        Head head = new Head(path, configuration.getDocletVersion(), configuration.getBuildDate())
+//                .setTimestamp(!options.notimestamp) // temporary: compatibility!
                 .setTitle(resources.getText("doclet.Window_Source_title"))
-//                .setCharset(configuration.charset) // temporary: compatibility!
+//                .setCharset(options.charset) // temporary: compatibility!
                 .setDescription(HtmlDocletWriter.getDescription("source", te))
                 .setGenerator(HtmlDocletWriter.getGenerator(getClass()))
                 .addDefaultScript(false)
                 .setStylesheets(configuration.getMainStylesheet(), configuration.getAdditionalStylesheets());
-        Content htmlTree = HtmlTree.HTML(configuration.getLocale().getLanguage(),
-                head.toContent(), body);
-        HtmlDocument htmlDocument = new HtmlDocument(htmlTree);
+        var html = HtmlTree.HTML(configuration.getLocale().getLanguage(), head, body);
+        HtmlDocument document = new HtmlDocument(html);
         messages.notice("doclet.Generating_0", path.getPath());
-        htmlDocument.write(DocFile.createFileForOutput(configuration, path));
+        document.write(DocFile.createFileForOutput(configuration, path));
     }
 
     /**
      * Returns a link to the stylesheet file.
      *
-     * @param head an HtmlTree to which the stylesheet links will be added
+     * @param head the content to which the stylesheet links will be added
      */
     public void addStyleSheetProperties(Content head) {
-        String filename = configuration.stylesheetfile;
+        String filename = options.stylesheetFile();
         DocPath stylesheet;
         if (filename.length() > 0) {
             DocFile file = DocFile.createFileForInput(configuration, filename);
@@ -268,22 +255,19 @@ public class SourceToHTMLConverter {
             stylesheet = DocPaths.STYLESHEET;
         }
         DocPath p = relativePath.resolve(stylesheet);
-        HtmlTree link = HtmlTree.LINK("stylesheet", "text/css", p.getPath(), "Style");
+        var link = HtmlTree.LINK("stylesheet", "text/css", p.getPath(), "Style");
         head.add(link);
         addStylesheets(head);
     }
 
-    protected void addStylesheets(Content tree) {
-        List<String> stylesheets = configuration.additionalStylesheets;
-        if (!stylesheets.isEmpty()) {
-            stylesheets.forEach((ssheet) -> {
-                DocFile file = DocFile.createFileForInput(configuration, ssheet);
-                DocPath ssheetPath = DocPath.create(file.getName());
-                HtmlTree slink = HtmlTree.LINK("stylesheet", "text/css", relativePath.resolve(ssheetPath).getPath(),
-                        "Style");
-                tree.add(slink);
-            });
-        }
+    protected void addStylesheets(Content head) {
+        options.additionalStylesheets().forEach(css -> {
+            DocFile file = DocFile.createFileForInput(configuration, css);
+            DocPath cssPath = DocPath.create(file.getName());
+            var slink = HtmlTree.LINK("stylesheet", "text/css", relativePath.resolve(cssPath).getPath(),
+                                      "Style");
+            head.add(slink);
+        });
     }
 
     /**
@@ -292,22 +276,21 @@ public class SourceToHTMLConverter {
      * @return the header content for the HTML file
      */
     private static Content getHeader() {
-        return new HtmlTree(HtmlTag.BODY).put(HtmlAttr.CLASS, "source");
+        return new HtmlTree(TagName.BODY).setStyle(HtmlStyle.sourcePage);
     }
 
     /**
      * Add the line numbers for the source code.
      *
-     * @param pre the content tree to which the line number will be added
+     * @param pre the content to which the line number will be added
      * @param lineno The line number
      */
     private static void addLineNo(Content pre, int lineno) {
-        HtmlTree span = new HtmlTree(HtmlTag.SPAN);
-        span.setStyle(HtmlStyle.sourceLineNo);
+        var span = HtmlTree.SPAN(HtmlStyle.sourceLineNo);
         if (lineno < 10) {
-            span.add("00" + Integer.toString(lineno));
+            span.add("00" + lineno);
         } else if (lineno < 100) {
-            span.add("0" + Integer.toString(lineno));
+            span.add("0" + lineno);
         } else {
             span.add(Integer.toString(lineno));
         }
@@ -317,16 +300,16 @@ public class SourceToHTMLConverter {
     /**
      * Add a line from source to the HTML file that is generated.
      *
-     * @param pre the content tree to which the line will be added.
+     * @param pre the content to which the line will be added.
      * @param line the string to format.
      * @param currentLineNo the current number.
      */
-    private void addLine(Content pre, String line, int currentLineNo) {
+    private void addLine(HtmlTree pre, String line, int currentLineNo) {
         if (line != null) {
-            Content anchor = HtmlTree.A_ID(
-                    "line." + Integer.toString(currentLineNo),
-                    new StringContent(utils.replaceTabs(line)));
-            pre.add(anchor);
+            var anchor = HtmlTree.SPAN_ID(
+                    HtmlIds.forLine(currentLineNo),
+                    Text.of(utils.replaceTabs(line)));
+            pre.addUnchecked(anchor);
             pre.add(NEW_LINE);
         }
     }
@@ -334,7 +317,7 @@ public class SourceToHTMLConverter {
     /**
      * Add trailing blank lines at the end of the page.
      *
-     * @param pre the content tree to which the blank lines will be added.
+     * @param pre the content to which the blank lines will be added.
      */
     private static void addBlankLines(Content pre) {
         for (int i = 0; i < NUM_BLANK_LINES; i++) {
@@ -349,7 +332,7 @@ public class SourceToHTMLConverter {
      * @param e the element to check.
      * @return the name of the anchor.
      */
-    public static String getAnchorName(Utils utils, Element e) {
-        return "line." + utils.getLineNumber(e);
+    public static HtmlId getAnchorName(Utils utils, Element e) {
+        return HtmlIds.forLine((int) utils.getLineNumber(e));
     }
 }
