@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,18 +25,41 @@
 
 package jdk.javadoc.internal.doclets.toolkit;
 
-import java.io.*;
-import java.util.*;
 
+import java.io.File;
+import java.io.IOException;
+import java.nio.file.InvalidPathException;
+import java.nio.file.Path;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.LinkedHashSet;
+import java.util.List;
+import java.util.Locale;
+import java.util.Map;
+import java.util.Set;
+import java.util.SortedMap;
+import java.util.SortedSet;
+import java.util.TreeMap;
+import java.util.TreeSet;
+import java.util.function.Function;
+
+import javax.lang.model.SourceVersion;
 import javax.lang.model.element.Element;
 import javax.lang.model.element.ModuleElement;
 import javax.lang.model.element.PackageElement;
 import javax.lang.model.element.TypeElement;
+import javax.lang.model.util.Elements;
 import javax.lang.model.util.SimpleElementVisitor14;
+import javax.tools.DocumentationTool;
 import javax.tools.JavaFileManager;
 import javax.tools.JavaFileObject;
+import javax.tools.StandardJavaFileManager;
 
-import com.sun.source.util.DocTreePath;
+import com.sun.source.tree.CompilationUnitTree;
+import com.sun.source.util.TreePath;
 import com.sun.tools.javac.util.DefinedBy;
 import com.sun.tools.javac.util.DefinedBy.Api;
 import jdk.javadoc.doclet.Doclet;
@@ -44,13 +67,11 @@ import jdk.javadoc.doclet.DocletEnvironment;
 import jdk.javadoc.doclet.Reporter;
 import jdk.javadoc.doclet.StandardDoclet;
 import jdk.javadoc.doclet.Taglet;
-import jdk.javadoc.internal.doclets.formats.html.HtmlDoclet;
 import jdk.javadoc.internal.doclets.toolkit.builders.BuilderFactory;
 import jdk.javadoc.internal.doclets.toolkit.taglets.TagletManager;
 import jdk.javadoc.internal.doclets.toolkit.util.Comparators;
 import jdk.javadoc.internal.doclets.toolkit.util.DocFile;
 import jdk.javadoc.internal.doclets.toolkit.util.DocFileFactory;
-import jdk.javadoc.internal.doclets.toolkit.util.DocFileIOException;
 import jdk.javadoc.internal.doclets.toolkit.util.Extern;
 import jdk.javadoc.internal.doclets.toolkit.util.Group;
 import jdk.javadoc.internal.doclets.toolkit.util.MetaKeywords;
@@ -60,16 +81,13 @@ import jdk.javadoc.internal.doclets.toolkit.util.Utils;
 import jdk.javadoc.internal.doclets.toolkit.util.Utils.Pair;
 import jdk.javadoc.internal.doclets.toolkit.util.VisibleMemberCache;
 import jdk.javadoc.internal.doclets.toolkit.util.VisibleMemberTable;
+import jdk.javadoc.internal.doclint.DocLint;
+import jdk.javadoc.internal.doclint.Env;
 
 /**
- * Configure the output based on the options. Doclets should sub-class
+ * Configure the output based on the options. Doclets should subclass
  * BaseConfiguration, to configure and add their own options. This class contains
  * all user options which are supported by the standard doclet.
- * <p>
- * <p><b>This is NOT part of any supported API.
- * If you write code that depends on this, you do so at your own risk.
- * This code and its internal interfaces are subject to change or
- * deletion without notice.</b>
  */
 public abstract class BaseConfiguration {
     /**
@@ -87,16 +105,6 @@ public abstract class BaseConfiguration {
      */
     public TagletManager tagletManager;
 
-    /**
-     * The path to the builder XML input file.
-     */
-    public String builderXMLPath;
-
-    /**
-     * The default path to the builder XML.
-     */
-    public static final String DEFAULT_BUILDER_XML = "resources/doclet.xml";
-
     /**
      * The meta tag keywords instance.
      */
@@ -191,7 +199,7 @@ public abstract class BaseConfiguration {
     protected static final String sharedResourceBundleName =
             "jdk.javadoc.internal.doclets.toolkit.resources.doclets";
 
-    VisibleMemberCache visibleMemberCache = null;
+    private VisibleMemberCache visibleMemberCache;
 
     public PropertyUtils propertyUtils = null;
 
@@ -201,7 +209,7 @@ public abstract class BaseConfiguration {
      * @apiNote The {@code doclet} parameter is used when
      * {@link Taglet#init(DocletEnvironment, Doclet) initializing tags}.
      * Some doclets (such as the {@link StandardDoclet}), may delegate to another
-     * (such as the {@link HtmlDoclet}).  In such cases, the primary doclet (i.e
+     * (such as the {@code HtmlDoclet}).  In such cases, the primary doclet (i.e
      * {@code StandardDoclet}) should be provided here, and not any internal
      * class like {@code HtmlDoclet}.
      *
@@ -219,7 +227,8 @@ public abstract class BaseConfiguration {
 
     private boolean initialized = false;
 
-    protected void initConfiguration(DocletEnvironment docEnv) {
+    protected void initConfiguration(DocletEnvironment docEnv,
+                                     Function<String, String> resourceKeyMapper) {
         if (initialized) {
             throw new IllegalStateException("configuration previously initialized");
         }
@@ -233,6 +242,8 @@ public abstract class BaseConfiguration {
             options.setJavaFX(isJavaFXMode());
         }
 
+        getDocResources().setKeyMapper(resourceKeyMapper);
+
         // Once docEnv and Utils have been initialized, others should be safe.
         metakeywords = new MetaKeywords(this);
         cmtUtils = new CommentUtils(this);
@@ -329,7 +340,7 @@ public abstract class BaseConfiguration {
         }
 
         // add entries for modules which may not have exported packages
-        modules.forEach(mdle -> modulePackages.computeIfAbsent(mdle, m -> Collections.emptySet()));
+        modules.forEach(mdle -> modulePackages.computeIfAbsent(mdle, m -> Set.of()));
 
         modules.addAll(modulePackages.keySet());
         showModules = !modules.isEmpty();
@@ -358,7 +369,32 @@ public abstract class BaseConfiguration {
         for (Pair<String, String> linkOfflinePair : options.linkOfflineList()) {
             extern.link(linkOfflinePair.first, linkOfflinePair.second, reporter);
         }
+        if (!options.noPlatformLinks()) {
+            extern.checkPlatformLinks(options.linkPlatformProperties(), reporter);
+        }
         typeElementCatalog = new TypeElementCatalog(includedTypeElements, this);
+
+        String snippetPath = options.snippetPath();
+        if (snippetPath != null) {
+            Messages messages = getMessages();
+            JavaFileManager fm = getFileManager();
+            if (fm instanceof StandardJavaFileManager) {
+                try {
+                    List<Path> sp = Arrays.stream(snippetPath.split(File.pathSeparator))
+                            .map(Path::of)
+                            .toList();
+                    StandardJavaFileManager sfm = (StandardJavaFileManager) fm;
+                    sfm.setLocationFromPaths(DocumentationTool.Location.SNIPPET_PATH, sp);
+                } catch (IOException | InvalidPathException e) {
+                    throw new SimpleDocletException(messages.getResources().getText(
+                            "doclet.error_setting_snippet_path", snippetPath, e), e);
+                }
+            } else {
+                throw new SimpleDocletException(messages.getResources().getText(
+                        "doclet.cannot_use_snippet_path", snippetPath));
+            }
+        }
+
         initTagletManager(options.customTagStrs());
         options.groupPairs().forEach(grp -> {
             if (showModules) {
@@ -367,7 +403,16 @@ public abstract class BaseConfiguration {
                 group.checkPackageGroups(grp.first, grp.second);
             }
         });
-        overviewElement = new OverviewElement(workArounds.getUnnamedPackage(), getOverviewPath());
+
+        PackageElement unnamedPackage;
+        Elements elementUtils = utils.elementUtils;
+        if (docEnv.getSourceVersion().compareTo(SourceVersion.RELEASE_9) >= 0) {
+            ModuleElement unnamedModule = elementUtils.getModuleElement("");
+            unnamedPackage = elementUtils.getPackageElement(unnamedModule, "");
+        } else {
+            unnamedPackage = elementUtils.getPackageElement("");
+        }
+        overviewElement = new OverviewElement(unnamedPackage, getOverviewPath());
         return true;
     }
 
@@ -380,10 +425,8 @@ public abstract class BaseConfiguration {
     public boolean setOptions() throws DocletException {
         initPackages();
         initModules();
-        if (!finishOptionSettings0() || !finishOptionSettings())
-            return false;
-
-        return true;
+        return finishOptionSettings0()
+                && finishOptionSettings();
     }
 
     private void initDestDirectory() throws DocletException {
@@ -428,9 +471,13 @@ public abstract class BaseConfiguration {
                     tagletManager.addCustomTag(args.get(1), fileManager);
                     continue;
                 }
-                List<String> tokens = tokenize(args.get(1), TagletManager.SIMPLE_TAGLET_OPT_SEPARATOR, 3);
+                /* Since there are few constraints on the characters in a tag name,
+                 * and real world examples with ':' in the tag name, we cannot simply use
+                 * String.split(regex);  instead, we tokenize the string, allowing
+                 * special characters to be escaped with '\'. */
+                List<String> tokens = tokenize(args.get(1), 3);
                 switch (tokens.size()) {
-                    case 1:
+                    case 1 -> {
                         String tagName = args.get(1);
                         if (tagletManager.isKnownCustomTag(tagName)) {
                             //reorder a standard tag
@@ -441,18 +488,16 @@ public abstract class BaseConfiguration {
                             heading.setCharAt(0, Character.toUpperCase(tagName.charAt(0)));
                             tagletManager.addNewSimpleCustomTag(tagName, heading.toString(), "a");
                         }
-                        break;
+                    }
 
-                    case 2:
+                    case 2 ->
                         //Add simple taglet without heading, probably to excluding it in the output.
                         tagletManager.addNewSimpleCustomTag(tokens.get(0), tokens.get(1), "");
-                        break;
 
-                    case 3:
+                    case 3 ->
                         tagletManager.addNewSimpleCustomTag(tokens.get(0), tokens.get(2), tokens.get(1));
-                        break;
 
-                    default:
+                    default ->
                         messages.error("doclet.Error_invalid_custom_tag_argument", args.get(1));
                 }
             }
@@ -462,18 +507,17 @@ public abstract class BaseConfiguration {
     }
 
     /**
-     * Given a string, return an array of tokens.  The separator can be escaped
-     * with the '\' character.  The '\' character may also be escaped by the
-     * '\' character.
+     * Given a string, return an array of tokens, separated by ':'.
+     * The separator character can be escaped with the '\' character.
+     * The '\' character may also be escaped with the '\' character.
      *
-     * @param s         the string to tokenize.
-     * @param separator the separator char.
+     * @param s         the string to tokenize
      * @param maxTokens the maximum number of tokens returned.  If the
      *                  max is reached, the remaining part of s is appended
      *                  to the end of the last token.
-     * @return an array of tokens.
+     * @return an array of tokens
      */
-    private List<String> tokenize(String s, char separator, int maxTokens) {
+    private List<String> tokenize(String s, int maxTokens) {
         List<String> tokens = new ArrayList<>();
         StringBuilder token = new StringBuilder();
         boolean prevIsEscapeChar = false;
@@ -483,7 +527,7 @@ public abstract class BaseConfiguration {
                 // Case 1:  escaped character
                 token.appendCodePoint(currentChar);
                 prevIsEscapeChar = false;
-            } else if (currentChar == separator && tokens.size() < maxTokens - 1) {
+            } else if (currentChar == ':' && tokens.size() < maxTokens - 1) {
                 // Case 2:  separator
                 tokens.add(token.toString());
                 token = new StringBuilder();
@@ -574,18 +618,6 @@ public abstract class BaseConfiguration {
      */
     public abstract WriterFactory getWriterFactory();
 
-    /**
-     * Return the input stream to the builder XML.
-     *
-     * @return the input steam to the builder XML.
-     * @throws DocFileIOException when the given XML file cannot be found or opened.
-     */
-    public InputStream getBuilderXML() throws DocFileIOException {
-        return builderXMLPath == null ?
-                BaseConfiguration.class.getResourceAsStream(DEFAULT_BUILDER_XML) :
-                DocFile.createFileForInput(this, builderXMLPath).openInputStream();
-    }
-
     /**
      * Return the Locale for this document.
      *
@@ -607,15 +639,10 @@ public abstract class BaseConfiguration {
      */
     public abstract JavaFileManager getFileManager();
 
-    public abstract boolean showMessage(DocTreePath path, String key);
-
-    public abstract boolean showMessage(Element e, String key);
-
     /*
      * Splits the elements in a collection to its individual
      * collection.
      */
-    @SuppressWarnings("preview")
     private static class Splitter {
 
         final Set<ModuleElement> mset = new LinkedHashSet<>();
@@ -672,7 +699,7 @@ public abstract class BaseConfiguration {
         return getOptions().allowScriptInComments();
     }
 
-    public synchronized VisibleMemberTable getVisibleMemberTable(TypeElement te) {
+    public VisibleMemberTable getVisibleMemberTable(TypeElement te) {
         return visibleMemberCache.getVisibleMemberTable(te);
     }
 
@@ -690,4 +717,101 @@ public abstract class BaseConfiguration {
                 || javafxModule.isUnnamed()
                 || javafxModule.getQualifiedName().contentEquals("javafx.base");
     }
+
+
+    //<editor-fold desc="DocLint support">
+
+    private DocLint doclint;
+
+    Map<CompilationUnitTree, Boolean> shouldCheck = new HashMap<>();
+
+    public void runDocLint(TreePath path) {
+        CompilationUnitTree unit = path.getCompilationUnit();
+        if (doclint != null && shouldCheck.computeIfAbsent(unit, doclint::shouldCheck)) {
+            doclint.scan(path);
+        }
+    }
+
+    /**
+     * Initializes DocLint, if appropriate, depending on options derived
+     * from the doclet command-line options, and the set of custom tags
+     * that should be ignored by DocLint.
+     *
+     * DocLint is not enabled if the option {@code -Xmsgs:none} is given,
+     * and it is not followed by any options to enable any groups.
+     * Note that arguments for {@code -Xmsgs:} can be given individually
+     * in separate {@code -Xmsgs:} options, or in a comma-separated list
+     * for a single option. For example, the following are equivalent:
+     * <ul>
+     *     <li>{@code -Xmsgs:all} {@code -Xmsgs:-html}
+     *     <li>{@code -Xmsgs:all,-html}
+     * </ul>
+     *
+     * @param opts  options for DocLint, derived from the corresponding doclet
+     *              command-line options
+     * @param customTagNames the names of custom tags, to be ignored by doclint
+     */
+    public void initDocLint(List<String> opts, Set<String> customTagNames) {
+        List<String> doclintOpts = new ArrayList<>();
+
+        // basic analysis of -Xmsgs and -Xmsgs: options to see if doclint is enabled
+        Set<String> groups = new HashSet<>();
+        boolean seenXmsgs = false;
+        for (String opt : opts) {
+            if (opt.equals(DocLint.XMSGS_OPTION)) {
+                groups.add("all");
+                seenXmsgs = true;
+            } else if (opt.startsWith(DocLint.XMSGS_CUSTOM_PREFIX)) {
+                String[] args = opt.substring(DocLint.XMSGS_CUSTOM_PREFIX.length())
+                        .split(DocLint.SEPARATOR);
+                for (String a : args) {
+                    if (a.equals("none")) {
+                        groups.clear();
+                    } else if (a.startsWith("-")) {
+                        groups.remove(a.substring(1));
+                    } else {
+                        groups.add(a);
+                    }
+                }
+                seenXmsgs = true;
+            }
+            doclintOpts.add(opt);
+        }
+
+        if (seenXmsgs) {
+            if (groups.isEmpty()) {
+                // no groups enabled; do not init doclint
+                return;
+            }
+        } else {
+            // no -Xmsgs options of any kind, use default
+            doclintOpts.add(DocLint.XMSGS_OPTION);
+        }
+
+        if (!customTagNames.isEmpty()) {
+            String customTags = String.join(DocLint.SEPARATOR, customTagNames);
+            doclintOpts.add(DocLint.XCUSTOM_TAGS_PREFIX + customTags);
+        }
+
+        doclint = new DocLint();
+        doclint.init(docEnv.getDocTrees(), docEnv.getElementUtils(), docEnv.getTypeUtils(),
+                doclintOpts.toArray(new String[0]));
+    }
+
+    public boolean isDocLintReferenceGroupEnabled() {
+        return isDocLintGroupEnabled(jdk.javadoc.internal.doclint.Messages.Group.REFERENCE);
+    }
+
+    public boolean isDocLintSyntaxGroupEnabled() {
+        return isDocLintGroupEnabled(jdk.javadoc.internal.doclint.Messages.Group.SYNTAX);
+    }
+
+    private boolean isDocLintGroupEnabled(jdk.javadoc.internal.doclint.Messages.Group group) {
+        // Use AccessKind.PUBLIC as a stand-in, since it is not common to
+        // set DocLint options per access kind (as is common with javac.)
+        // A more sophisticated solution might be to derive the access kind from the
+        // element owning the comment, and its enclosing elements.
+        return doclint != null && doclint.isGroupEnabled(group, Env.AccessKind.PUBLIC);
+    }
+    //</editor-fold>
 }
