@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,18 +25,12 @@
 
 package jdk.javadoc.internal.doclets.toolkit;
 
-import javax.lang.model.element.Element;
 import javax.lang.model.element.PackageElement;
 import javax.tools.FileObject;
 
 /**
  * This is a pseudo element wrapper for the overview element, essentially to
  * associate overview documentation's DocCommentTree to this element.
- *
- *  <p><b>This is NOT part of any supported API.
- *  If you write code that depends on this, you do so at your own risk.
- *  This code and its internal interfaces are subject to change or
- *  deletion without notice.</b>
  */
 public class OverviewElement implements DocletElement {
 
@@ -53,6 +47,7 @@ public class OverviewElement implements DocletElement {
         return pkg;
     }
 
+    @Override
     public FileObject getFileObject() {
         return fo;
     }
