@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -36,13 +36,8 @@ import jdk.javadoc.internal.doclets.toolkit.util.VisibleMemberTable.Kind;
 
 /**
  * The superclass for all member builders.  Member builders are only executed
- * within Class Builders.  They essentially build sub-components.  For example,
- * method documentation is a sub-component of class documentation.
- *
- *  <p><b>This is NOT part of any supported API.
- *  If you write code that depends on this, you do so at your own risk.
- *  This code and its internal interfaces are subject to change or
- *  deletion without notice.</b>
+ * within Class Builders.  They essentially build subcomponents.  For example,
+ * method documentation is a subcomponent of class documentation.
  */
 public abstract class AbstractMemberBuilder extends AbstractBuilder {
 
@@ -62,7 +57,7 @@ public abstract class AbstractMemberBuilder extends AbstractBuilder {
     }
 
     /**
-     * This method is not supported by sub-builders.
+     * This method is not supported by subbuilders.
      *
      * @throws AssertionError always
      */
@@ -73,13 +68,12 @@ public abstract class AbstractMemberBuilder extends AbstractBuilder {
     }
 
     /**
-     *
      * Build the documentation.
      *
-     * @param contentTree The content tree into which to add the documentation
-     * @throws DocletException  if there is a problem building the documentation
+     * @param target the content into which to add the documentation
+     * @throws DocletException if there is a problem building the documentation
      */
-    public abstract void build(Content contentTree) throws DocletException;
+    public abstract void build(Content target) throws DocletException;
 
     /**
      * Returns true if this subbuilder has anything to document.
@@ -94,7 +88,7 @@ public abstract class AbstractMemberBuilder extends AbstractBuilder {
      * @param kind of members
      * @return a list of members
      */
-    protected List<? extends Element> getVisibleMembers(Kind kind) {
+    protected List<Element> getVisibleMembers(Kind kind) {
         return visibleMemberTable.getVisibleMembers(kind);
     }
 }
