@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,27 +25,13 @@
 
 package jdk.javadoc.internal.doclets.toolkit.util;
 
-import java.util.ArrayList;
-import java.util.Collections;
-import java.util.List;
-import java.util.stream.Collectors;
-
-import javax.lang.model.element.Element;
-import javax.lang.model.element.ModuleElement;
-import javax.lang.model.element.PackageElement;
-import javax.lang.model.element.TypeElement;
-import javax.lang.model.type.TypeMirror;
-
-import com.sun.source.doctree.AttributeTree;
-import com.sun.source.doctree.AttributeTree.ValueKind;
 import com.sun.source.doctree.AuthorTree;
 import com.sun.source.doctree.BlockTagTree;
 import com.sun.source.doctree.CommentTree;
 import com.sun.source.doctree.DeprecatedTree;
 import com.sun.source.doctree.DocCommentTree;
 import com.sun.source.doctree.DocTree;
-import com.sun.source.doctree.EndElementTree;
-import com.sun.source.doctree.EntityTree;
+import com.sun.source.doctree.EscapeTree;
 import com.sun.source.doctree.IdentifierTree;
 import com.sun.source.doctree.InlineTagTree;
 import com.sun.source.doctree.LinkTree;
@@ -59,7 +45,7 @@ import com.sun.source.doctree.SerialDataTree;
 import com.sun.source.doctree.SerialFieldTree;
 import com.sun.source.doctree.SerialTree;
 import com.sun.source.doctree.SinceTree;
-import com.sun.source.doctree.StartElementTree;
+import com.sun.source.doctree.SpecTree;
 import com.sun.source.doctree.TextTree;
 import com.sun.source.doctree.ThrowsTree;
 import com.sun.source.doctree.UnknownBlockTagTree;
@@ -71,25 +57,30 @@ import com.sun.source.util.DocTrees;
 import com.sun.source.util.SimpleDocTreeVisitor;
 import com.sun.source.util.TreePath;
 import jdk.javadoc.internal.doclets.toolkit.BaseConfiguration;
+import jdk.javadoc.internal.doclets.toolkit.util.DocFinder.Result;
 
-import static com.sun.source.doctree.DocTree.Kind.*;
+import javax.lang.model.element.Element;
+import javax.lang.model.element.ElementKind;
+import javax.lang.model.element.ExecutableElement;
+import javax.lang.model.element.ModuleElement;
+import javax.lang.model.element.PackageElement;
+import javax.lang.model.element.TypeElement;
+import javax.lang.model.type.TypeKind;
+import javax.lang.model.type.TypeMirror;
+import java.util.List;
+import java.util.Optional;
+
+import static com.sun.source.doctree.DocTree.Kind.SEE;
+import static com.sun.source.doctree.DocTree.Kind.SERIAL_FIELD;
 
 /**
- *  A utility class.
- *
- *  <p><b>This is NOT part of any supported API.
- *  If you write code that depends on this, you do so at your own risk.
- *  This code and its internal interfaces are subject to change or
- *  deletion without notice.</b>
+ * A utility class.
  */
 public class CommentHelper {
     private final BaseConfiguration configuration;
     public final TreePath path;
     public final DocCommentTree dcTree;
     public final Element element;
-    private Element overriddenElement;
-
-    public static final String SPACER = " ";
 
     /**
      * Creates a utility class to encapsulate the contextual information for a doc comment tree.
@@ -106,53 +97,26 @@ public class CommentHelper {
         this.dcTree = dcTree;
     }
 
-    public void setOverrideElement(Element ove) {
-        if (this.element == ove) {
-            throw new AssertionError("cannot set given element as overridden element");
-        }
-        overriddenElement = ove;
-    }
-
     public String getTagName(DocTree dtree) {
-        switch (dtree.getKind()) {
-            case AUTHOR:
-            case DEPRECATED:
-            case PARAM:
-            case PROVIDES:
-            case RETURN:
-            case SEE:
-            case SERIAL_DATA:
-            case SERIAL_FIELD:
-            case THROWS:
-            case UNKNOWN_BLOCK_TAG:
-            case USES:
-            case VERSION:
-                return ((BlockTagTree) dtree).getTagName();
-            case UNKNOWN_INLINE_TAG:
-                return ((InlineTagTree) dtree).getTagName();
-            case ERRONEOUS:
-                return "erroneous";
-            default:
-                return dtree.getKind().tagName;
-        }
+        return switch (dtree.getKind()) {
+            case AUTHOR, DEPRECATED, PARAM, PROVIDES, RETURN, SEE, SERIAL_DATA, SERIAL_FIELD,
+                    THROWS, UNKNOWN_BLOCK_TAG, USES, VERSION ->
+                    ((BlockTagTree) dtree).getTagName();
+            case UNKNOWN_INLINE_TAG ->
+                    ((InlineTagTree) dtree).getTagName();
+            case ERRONEOUS ->
+                    "erroneous";
+            default ->
+                    dtree.getKind().tagName;
+        };
     }
 
-    public boolean isTypeParameter(DocTree dtree) {
-        if (dtree.getKind() == PARAM) {
-            return ((ParamTree)dtree).isTypeParameter();
-        }
-        return false;
-    }
-
-    public String getParameterName(DocTree dtree) {
-        if (dtree.getKind() == PARAM) {
-            return ((ParamTree) dtree).getName().toString();
-        } else {
-            return null;
-        }
+    public String getParameterName(ParamTree p) {
+        return p.getName().getName().toString();
     }
 
     Element getElement(ReferenceTree rtree) {
+        // We need to lookup type variables and other types
         Utils utils = configuration.utils;
         // likely a synthesized tree
         if (path == null) {
@@ -165,37 +129,22 @@ public class CommentHelper {
             }
             return configuration.docEnv.getTypeUtils().asElement(symbol);
         }
-        // case A: the element contains no comments associated and
-        // the comments need to be copied from ancestor
-        // case B: the element has @inheritDoc, then the ancestral comment
-        // as appropriate has to be copied over.
-
-        // Case A.
-        if (dcTree == null && overriddenElement != null) {
-            CommentHelper ovch = utils.getCommentHelper(overriddenElement);
-            return ovch.getElement(rtree);
-        }
-        if (dcTree == null) {
-            return null;
-        }
-        DocTreePath docTreePath = DocTreePath.getPath(path, dcTree, rtree);
+        DocTreePath docTreePath = getDocTreePath(rtree);
         if (docTreePath == null) {
-            // Case B.
-            if (overriddenElement != null) {
-                CommentHelper ovch = utils.getCommentHelper(overriddenElement);
-                return ovch.getElement(rtree);
-            }
             return null;
         }
         DocTrees doctrees = configuration.docEnv.getDocTrees();
-        return doctrees.getElement(docTreePath);
+        // Workaround for JDK-8284193
+        // DocTrees.getElement(DocTreePath) returns javac-internal Symbols
+        var e = doctrees.getElement(docTreePath);
+        return e == null || e.getKind() == ElementKind.CLASS && e.asType().getKind() != TypeKind.DECLARED ? null : e;
     }
 
     public TypeMirror getType(ReferenceTree rtree) {
-        DocTreePath docTreePath = DocTreePath.getPath(path, dcTree, rtree);
+        DocTreePath docTreePath = getDocTreePath(rtree);
         if (docTreePath != null) {
-            DocTrees doctrees = configuration.docEnv.getDocTrees();
-            return doctrees.getType(docTreePath);
+            DocTrees docTrees = configuration.docEnv.getDocTrees();
+            return docTrees.getType(docTreePath);
         }
         return null;
     }
@@ -208,165 +157,8 @@ public class CommentHelper {
         return getTags(dtree);
     }
 
-    public String getText(List<? extends DocTree> list) {
-        StringBuilder sb = new StringBuilder();
-        for (DocTree dt : list) {
-            sb.append(getText0(dt));
-        }
-        return sb.toString();
-    }
-
-    public String getText(DocTree dt) {
-        return getText0(dt).toString();
-    }
-
-    private StringBuilder getText0(DocTree dt) {
-        final StringBuilder sb = new StringBuilder();
-        new SimpleDocTreeVisitor<Void, Void>() {
-            @Override
-            public Void visitAttribute(AttributeTree node, Void p) {
-                sb.append(SPACER).append(node.getName());
-                if (node.getValueKind() == ValueKind.EMPTY) {
-                    return null;
-                }
-
-                sb.append("=");
-                String quote;
-                switch (node.getValueKind()) {
-                    case DOUBLE:
-                        quote = "\"";
-                        break;
-                    case SINGLE:
-                        quote = "'";
-                        break;
-                    default:
-                        quote = "";
-                        break;
-                }
-                sb.append(quote);
-                node.getValue().forEach(dt -> dt.accept(this, null));
-                sb.append(quote);
-                return null;
-            }
-
-            @Override
-            public Void visitEndElement(EndElementTree node, Void p) {
-                sb.append("</")
-                        .append(node.getName())
-                        .append(">");
-                return null;
-            }
-
-            @Override
-            public Void visitEntity(EntityTree node, Void p) {
-                sb.append(node.toString());
-                return null;
-            }
-
-            @Override
-            public Void visitLink(LinkTree node, Void p) {
-                if (node.getReference() == null) {
-                    return null;
-                }
-
-                node.getReference().accept(this, null);
-                node.getLabel().forEach(dt -> dt.accept(this, null));
-                return null;
-            }
-
-            @Override
-            public Void visitLiteral(LiteralTree node, Void p) {
-                if (node.getKind() == CODE) {
-                    sb.append("<").append(node.getKind().tagName).append(">");
-                }
-                sb.append(node.getBody().toString());
-                if (node.getKind() == CODE) {
-                    sb.append("</").append(node.getKind().tagName).append(">");
-                }
-                return null;
-            }
-
-            @Override
-            public Void visitReference(ReferenceTree node, Void p) {
-                sb.append(node.getSignature());
-                return null;
-            }
-
-            @Override
-            public Void visitSee(SeeTree node, Void p) {
-                node.getReference().forEach(dt -> dt.accept(this, null));
-                return null;
-            }
-
-            @Override
-            public Void visitSerial(SerialTree node, Void p) {
-                node.getDescription().forEach(dt -> dt.accept(this, null));
-                return null;
-            }
-
-            @Override
-            public Void visitStartElement(StartElementTree node, Void p) {
-                sb.append("<");
-                sb.append(node.getName());
-                node.getAttributes().forEach(dt -> dt.accept(this, null));
-                sb.append(node.isSelfClosing() ? "/>" : ">");
-                return null;
-            }
-
-            @Override
-            public Void visitText(TextTree node, Void p) {
-                sb.append(node.getBody());
-                return null;
-            }
-
-            @Override
-            public Void visitUnknownBlockTag(UnknownBlockTagTree node, Void p) {
-                node.getContent().forEach(dt -> dt.accept(this, null));
-                return null;
-            }
-
-            @Override
-            public Void visitValue(ValueTree node, Void p) {
-                return node.getReference().accept(this, null);
-            }
-
-            @Override
-            protected Void defaultAction(DocTree node, Void p) {
-                sb.append(node.toString());
-                return null;
-            }
-        }.visit(dt, null);
-        return sb;
-    }
-
-    public String getLabel(DocTree dtree) {
-        return new SimpleDocTreeVisitor<String, Void>() {
-            @Override
-            public String visitLink(LinkTree node, Void p) {
-                return node.getLabel().stream()
-                        .map(dt -> getText(dt))
-                        .collect(Collectors.joining());
-            }
-
-            @Override
-            public String visitSee(SeeTree node, Void p) {
-                Utils utils = configuration.utils;
-                return node.getReference().stream()
-                        .filter(utils::isText)
-                        .map(dt -> ((TextTree) dt).getBody())
-                        .collect(Collectors.joining());
-            }
-
-            @Override
-            protected String defaultAction(DocTree node, Void p) {
-                return "";
-            }
-        }.visit(dtree, null);
-    }
-
-    public TypeElement getReferencedClass(DocTree dtree) {
+    public TypeElement getReferencedClass(Element e) {
         Utils utils = configuration.utils;
-        Element e = getReferencedElement(dtree);
         if (e == null) {
             return null;
         } else if (utils.isTypeElement(e)) {
@@ -377,35 +169,36 @@ public class CommentHelper {
         return null;
     }
 
-    public String getReferencedModuleName(DocTree dtree) {
-        String s = getReferencedSignature(dtree);
-        if (s == null || s.contains("#") || s.contains("(")) {
+    public String getReferencedModuleName(String signature) {
+        if (signature == null || signature.contains("#") || signature.contains("(")) {
             return null;
         }
-        int n = s.indexOf("/");
-        return (n == -1) ? s : s.substring(0, n);
+        int n = signature.indexOf("/");
+        return (n == -1) ? signature : signature.substring(0, n);
     }
 
     public Element getReferencedMember(DocTree dtree) {
-        Utils utils = configuration.utils;
         Element e = getReferencedElement(dtree);
+        return getReferencedMember(e);
+    }
+
+    public Element getReferencedMember(Element e) {
+        Utils utils = configuration.utils;
         if (e == null) {
             return null;
         }
         return (utils.isExecutableElement(e) || utils.isVariableElement(e)) ? e : null;
     }
 
-    public String getReferencedMemberName(DocTree dtree) {
-        String s = getReferencedSignature(dtree);
-        if (s == null) {
+    public String getReferencedFragment(String signature) {
+        if (signature == null) {
             return null;
         }
-        int n = s.indexOf("#");
-        return (n == -1) ? null : s.substring(n + 1);
+        int n = signature.indexOf("#");
+        return (n == -1) ? null : signature.substring(n + 1);
     }
 
-    public PackageElement getReferencedPackage(DocTree dtree) {
-        Element e = getReferencedElement(dtree);
+    public PackageElement getReferencedPackage(Element e) {
         if (e != null) {
             Utils utils = configuration.utils;
             return utils.containingPackage(e);
@@ -413,24 +206,18 @@ public class CommentHelper {
         return null;
     }
 
-    public ModuleElement getReferencedModule(DocTree dtree) {
-        Element e = getReferencedElement(dtree);
+    public ModuleElement getReferencedModule(Element e) {
         if (e != null && configuration.utils.isModule(e)) {
             return (ModuleElement) e;
         }
         return null;
     }
 
-
     public List<? extends DocTree> getFirstSentenceTrees(List<? extends DocTree> body) {
         return configuration.docEnv.getDocTrees().getFirstSentence(body);
     }
 
-    public List<? extends DocTree> getFirstSentenceTrees(DocTree dtree) {
-        return getFirstSentenceTrees(getBody(dtree));
-    }
-
-    private Element getReferencedElement(DocTree dtree) {
+    public Element getReferencedElement(DocTree dtree) {
         return new ReferenceDocTreeVisitor<Element>() {
             @Override
             public Element visitReference(ReferenceTree node, Void p) {
@@ -457,15 +244,70 @@ public class CommentHelper {
         return null;
     }
 
-    public  String getReferencedSignature(DocTree dtree) {
+    /**
+     * {@return the normalized signature from a {@code ReferenceTree}}
+     */
+    public String getReferencedSignature(DocTree dtree) {
         return new ReferenceDocTreeVisitor<String>() {
             @Override
             public String visitReference(ReferenceTree node, Void p) {
-                return node.getSignature();
+                return normalizeSignature(node.getSignature());
             }
         }.visit(dtree, null);
     }
 
+    @SuppressWarnings("fallthrough")
+    private static String normalizeSignature(String sig) {
+        if (sig == null
+                || (!sig.contains(" ") && !sig.contains("\n")
+                 && !sig.contains("\r") && !sig.endsWith("/"))) {
+            return sig;
+        }
+        StringBuilder sb = new StringBuilder();
+        char lastChar = 0;
+        for (int i = 0; i < sig.length(); i++) {
+            char ch = sig.charAt(i);
+            switch (ch) {
+                case '\n':
+                case '\r':
+                case '\f':
+                case '\t':
+                case ' ':
+                    // Add at most one space char, or none if it isn't needed
+                    switch (lastChar) {
+                        case 0:
+                        case'(':
+                        case'<':
+                        case ' ':
+                        case '.':
+                            break;
+                        default:
+                            sb.append(' ');
+                            lastChar = ' ';
+                            break;
+                    }
+                    break;
+                case ',':
+                case '>':
+                case ')':
+                case '.':
+                    // Remove preceding space character
+                    if (lastChar == ' ') {
+                        sb.setLength(sb.length() - 1);
+                    }
+                    // fallthrough
+                default:
+                    sb.append(ch);
+                    lastChar = ch;
+            }
+        }
+        // Delete trailing slash
+        if (lastChar == '/') {
+            sb.setLength(sb.length() - 1);
+        }
+        return sb.toString();
+    }
+
     private static class ReferenceDocTreeVisitor<R> extends SimpleDocTreeVisitor<R, Void> {
         @Override
         public R visitSee(SeeTree node, Void p) {
@@ -510,29 +352,19 @@ public class CommentHelper {
         return dtree.getKind() == SEE ? ((SeeTree)dtree).getReference() : null;
     }
 
-    public ReferenceTree getExceptionName(DocTree dtree) {
-        return (dtree.getKind() == THROWS || dtree.getKind() == EXCEPTION)
-                ? ((ThrowsTree)dtree).getExceptionName()
-                : null;
-    }
-
     public IdentifierTree getName(DocTree dtree) {
-        switch (dtree.getKind()) {
-            case PARAM:
-                return ((ParamTree)dtree).getName();
-            case SERIAL_FIELD:
-                return ((SerialFieldTree)dtree).getName();
-            default:
-                return null;
-            }
+        return switch (dtree.getKind()) {
+            case PARAM -> ((ParamTree) dtree).getName();
+            case SERIAL_FIELD -> ((SerialFieldTree) dtree).getName();
+            default -> null;
+        };
     }
 
     public List<? extends DocTree> getTags(DocTree dtree) {
         return new SimpleDocTreeVisitor<List<? extends DocTree>, Void>() {
-            List<? extends DocTree> asList(String content) {
-                List<DocTree> out = new ArrayList<>();
-                out.add(configuration.cmtUtils.makeTextTree(content));
-                return out;
+
+            private List<DocTree> asList(String content) {
+                return List.of(configuration.cmtUtils.makeTextTree(content));
             }
 
             @Override
@@ -555,6 +387,11 @@ public class CommentHelper {
                 return node.getBody();
             }
 
+            @Override
+            public List<? extends DocTree> visitEscape(EscapeTree node, Void p) {
+                return asList(node.getBody());
+            }
+
             @Override
             public List<? extends DocTree> visitLiteral(LiteralTree node, Void p) {
                 return asList(node.getBody().getBody());
@@ -562,7 +399,7 @@ public class CommentHelper {
 
             @Override
             public List<? extends DocTree> visitProvides(ProvidesTree node, Void p) {
-                 return node.getDescription();
+                return node.getDescription();
             }
 
             @Override
@@ -582,7 +419,7 @@ public class CommentHelper {
 
             @Override
             public List<? extends DocTree> visitParam(ParamTree node, Void p) {
-               return node.getDescription();
+                return node.getDescription();
             }
 
             @Override
@@ -610,9 +447,14 @@ public class CommentHelper {
                 return node.getDescription();
             }
 
+            @Override
+            public List<? extends DocTree> visitSpec(SpecTree node, Void p) {
+                return node.getTitle();
+            }
+
             @Override
             public List<? extends DocTree> visitThrows(ThrowsTree node, Void p) {
-                 return node.getDescription();
+                return node.getDescription();
             }
 
             @Override
@@ -622,12 +464,12 @@ public class CommentHelper {
 
             @Override
             public List<? extends DocTree> visitUses(UsesTree node, Void p) {
-                 return node.getDescription();
+                return node.getDescription();
             }
 
             @Override
             protected List<? extends DocTree> defaultAction(DocTree node, Void p) {
-               return Collections.emptyList();
+                return List.of();
             }
         }.visit(dtree, null);
     }
@@ -645,13 +487,31 @@ public class CommentHelper {
     }
 
     public DocTreePath getDocTreePath(DocTree dtree) {
-        if (path == null || dcTree == null || dtree == null)
+        if (dcTree == null && element instanceof ExecutableElement ee) {
+            return getInheritedDocTreePath(dtree, ee);
+        }
+        if (path == null || dcTree == null || dtree == null) {
             return null;
-        return DocTreePath.getPath(path, dcTree, dtree);
+        }
+        DocTreePath dtPath = DocTreePath.getPath(path, dcTree, dtree);
+        if (dtPath == null && element instanceof ExecutableElement ee) {
+            // The overriding element has a doc tree, but it doesn't contain what we're looking for.
+            return getInheritedDocTreePath(dtree, ee);
+        }
+        return dtPath;
     }
 
-    public Element getOverriddenElement() {
-        return overriddenElement;
+    private DocTreePath getInheritedDocTreePath(DocTree dtree, ExecutableElement ee) {
+        Utils utils = configuration.utils;
+        var docFinder = utils.docFinder();
+        Optional<ExecutableElement> inheritedDoc = docFinder.search(ee,
+                (m -> {
+                    Optional<ExecutableElement> optional = utils.getFullBody(m).isEmpty() ? Optional.empty() : Optional.of(m);
+                    return Result.fromOptional(optional);
+                })).toOptional();
+        return inheritedDoc.isEmpty() || inheritedDoc.get().equals(ee)
+                ? null
+                : utils.getCommentHelper(inheritedDoc.get()).getDocTreePath(dtree);
     }
 
     /**
@@ -660,20 +520,10 @@ public class CommentHelper {
      */
     @Override
     public String toString() {
-        StringBuilder sb = new StringBuilder("CommentHelper{" + "path=" + path + ", dcTree=" + dcTree);
-        sb.append(", element=");
-        sb.append(element.getEnclosingElement());
-        sb.append("::");
-        sb.append(element);
-        sb.append(", overriddenElement=");
-        if (overriddenElement != null) {
-            sb.append(overriddenElement.getEnclosingElement());
-            sb.append("::");
-            sb.append(overriddenElement);
-        } else {
-            sb.append("<none>");
-        }
-        sb.append('}');
-        return sb.toString();
+        return "CommentHelper{"
+                + "path=" + path
+                + ", dcTree=" + dcTree
+                + ", element=" + element.getEnclosingElement() + "::" + element
+                + '}';
     }
 }
