@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -30,7 +30,7 @@ import java.util.List;
 import java.util.Objects;
 
 import jdk.jfr.SettingDescriptor;
-
+import jdk.jfr.internal.periodic.PeriodicEvents;
 /**
  * Implementation of event type.
  *
@@ -65,7 +65,6 @@ public final class PlatformEventType extends Type {
     private boolean registered = true;
     private boolean committable = enabled && registered;
 
-
     // package private
     PlatformEventType(String name, long id, boolean isJDK, boolean dynamicSettings) {
         super(name, Type.SUPER_TYPE_EVENT, id);
@@ -85,7 +84,7 @@ public final class PlatformEventType extends Type {
         return false;
     }
 
-    private static boolean isUsingHandler(String name) {
+    private static boolean isUsingConfiguration(String name) {
         switch (name) {
             case Type.EVENT_NAME_PREFIX + "SocketRead"  :
             case Type.EVENT_NAME_PREFIX + "SocketWrite" :
@@ -102,11 +101,11 @@ public final class PlatformEventType extends Type {
             if (isExceptionEvent(name)) {
                 return 4;
             }
-            if (isUsingHandler(name)) {
+            if (isUsingConfiguration(name)) {
                 return 3;
             }
         }
-        return 4;
+        return 3;
     }
 
     public void add(SettingDescriptor settingDescriptor) {
@@ -201,26 +200,34 @@ public final class PlatformEventType extends Type {
     }
 
     public void setEnabled(boolean enabled) {
+        boolean changed = enabled != this.enabled;
         this.enabled = enabled;
         updateCommittable();
         if (isJVM) {
             if (isMethodSampling) {
                 long p = enabled ? period : 0;
-                JVM.getJVM().setMethodSamplingInterval(getId(), p);
+                JVM.getJVM().setMethodSamplingPeriod(getId(), p);
             } else {
                 JVM.getJVM().setEnabled(getId(), enabled);
             }
         }
+        if (changed) {
+            PeriodicEvents.setChanged();
+        }
     }
 
     public void setPeriod(long periodMillis, boolean beginChunk, boolean endChunk) {
         if (isMethodSampling) {
             long p = enabled ? periodMillis : 0;
-            JVM.getJVM().setMethodSamplingInterval(getId(), p);
+            JVM.getJVM().setMethodSamplingPeriod(getId(), p);
         }
         this.beginChunk = beginChunk;
         this.endChunk = endChunk;
+        boolean changed = period != periodMillis;
         this.period = periodMillis;
+        if (changed) {
+            PeriodicEvents.setChanged();
+        }
     }
 
     public void setStackTraceEnabled(boolean stackTraceEnabled) {
@@ -237,7 +244,11 @@ public final class PlatformEventType extends Type {
         }
     }
 
-    public boolean isEveryChunk() {
+    /**
+     * Returns true if "beginChunk", "endChunk" or "everyChunk" have
+     * been set.
+     */
+    public boolean isChunkTime() {
         return period == 0;
     }
 
@@ -259,6 +270,7 @@ public final class PlatformEventType extends Type {
 
     public void setEventHook(boolean hasHook) {
         this.hasHook = hasHook;
+        PeriodicEvents.setChanged();
     }
 
     public boolean isBeginChunk() {
