@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -29,7 +29,9 @@ import java.io.IOException;
 import java.net.InetAddress;
 import java.net.Socket;
 
+import jdk.jfr.events.EventConfigurations;
 import jdk.jfr.events.SocketReadEvent;
+import jdk.jfr.internal.event.EventConfiguration;
 
 /**
  * See {@link JITracer} for an explanation of this code.
@@ -40,33 +42,30 @@ final class SocketInputStreamInstrumentor {
     private SocketInputStreamInstrumentor() {
     }
 
-    @SuppressWarnings("deprecation")
     @JIInstrumentationMethod
     public int read(byte b[], int off, int length) throws IOException {
-        SocketReadEvent event = SocketReadEvent.EVENT.get();
-        if (!event.isEnabled()) {
+        EventConfiguration eventConfiguration = EventConfigurations.SOCKET_READ;
+        if (!eventConfiguration.isEnabled()) {
             return read(b, off, length);
         }
         int bytesRead = 0;
+        long start = 0;
         try {
-            event.begin();
+            start = EventConfiguration.timestamp();
             bytesRead = read(b, off, length);
         } finally {
-            event.end();
-            if (event.shouldCommit()) {
+            long duration = EventConfiguration.timestamp() - start;
+            if (eventConfiguration.shouldCommit(duration)) {
                 InetAddress remote = parent.getInetAddress();
-                event.host = remote.getHostName();
-                event.address = remote.getHostAddress();
-                event.port = parent.getPort();
+                String host = remote.getHostName();
+                String address = remote.getHostAddress();
+                int port = parent.getPort();
+                int timeout = parent.getSoTimeout();
                 if (bytesRead < 0) {
-                    event.endOfStream = true;
+                    SocketReadEvent.commit(start, duration, host, address, port, timeout, 0L, true);
                 } else {
-                    event.bytesRead = bytesRead;
+                    SocketReadEvent.commit(start, duration, host, address, port, timeout, bytesRead, false);
                 }
-                event.timeout = parent.getSoTimeout();
-
-                event.commit();
-                event.reset();
             }
         }
         return bytesRead;
