@@ -50,7 +50,6 @@ import java.awt.image.Raster;
 import java.awt.image.WritableRaster;
 import java.io.BufferedReader;
 import java.io.IOException;
-import java.io.InputStreamReader;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.util.Arrays;
@@ -261,11 +260,7 @@ public abstract class UNIXToolkit extends SunToolkit
                 result = shouldDisableSystemTray;
                 if (result == null) {
                     if ("gnome".equals(getDesktop())) {
-                        @SuppressWarnings("removal")
-                        Integer gnomeShellMajorVersion =
-                                AccessController
-                                        .doPrivileged((PrivilegedAction<Integer>)
-                                                this::getGnomeShellMajorVersion);
+                        Integer gnomeShellMajorVersion = getGnomeShellMajorVersion();
 
                         if (gnomeShellMajorVersion == null
                                 || gnomeShellMajorVersion < 45) {
@@ -280,34 +275,36 @@ public abstract class UNIXToolkit extends SunToolkit
         return result;
     }
 
-    private Integer getGnomeShellMajorVersion() {
-        try {
-            Process process =
-                new ProcessBuilder("/usr/bin/gnome-shell", "--version")
-                        .start();
-            try (InputStreamReader isr = new InputStreamReader(process.getInputStream());
-                 BufferedReader reader = new BufferedReader(isr)) {
+    @SuppressWarnings("removal")
+    public Integer getGnomeShellMajorVersion() {
+        return AccessController.doPrivileged(
+                            (PrivilegedAction<Integer>) () -> {
+            try {
+                Process process =
+                    new ProcessBuilder("/usr/bin/gnome-shell", "--version")
+                            .start();
+                try (BufferedReader reader = process.inputReader()) {
+                    if (process.waitFor(2, SECONDS) &&  process.exitValue() == 0) {
+                        String line = reader.readLine();
+                        if (line != null) {
+                            String[] versionComponents = line
+                                    .replaceAll("[^\\d.]", "")
+                                    .split("\\.");
 
-                if (process.waitFor(2, SECONDS) &&  process.exitValue() == 0) {
-                    String line = reader.readLine();
-                    if (line != null) {
-                        String[] versionComponents = line
-                                .replaceAll("[^\\d.]", "")
-                                .split("\\.");
-
-                        if (versionComponents.length >= 1) {
-                            return Integer.parseInt(versionComponents[0]);
+                            if (versionComponents.length >= 1) {
+                                return Integer.parseInt(versionComponents[0]);
+                            }
                         }
                     }
                 }
+            } catch (IOException
+                    | InterruptedException
+                    | IllegalThreadStateException
+                    | NumberFormatException ignored) {
             }
-        } catch (IOException
-                 | InterruptedException
-                 | IllegalThreadStateException
-                 | NumberFormatException ignored) {
-        }
 
-        return null;
+            return null;
+        });
     }
 
     /**
