@@ -45,7 +45,7 @@ public class  XMSelection {
      * number and the new owning window when onwership is established, or
      * None if the owner is gone.
      *
-     * Events in extra_mask are selected for on owning windows (exsiting
+     * Events in extra_mask are selected for on owning windows (existing
      * ones and on new owners when established) and otherEvent of the
      * XMWSelectionListener will be called with the screen number and an event.
      *
@@ -206,7 +206,7 @@ public class  XMSelection {
         XClientMessageEvent xce = xev.get_xclient();
         if (xce.get_message_type() == XA_MANAGER.getAtom()) {
             if (log.isLoggable(PlatformLogger.Level.FINE)) {
-                log.fine("client messags = " + xce);
+                log.fine("client messages = " + xce);
             }
             long timestamp = xce.get_data(0) & 0xFFFFFFFFL;
             long atom = xce.get_data(1);
