@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1999, 2018, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -23,21 +23,21 @@
  *
  */
 
-/**
- * Basic .wav audio handler.
- * @author  Jeff Nisewanger
- */
 package sun.awt.www.content.audio;
 
-import java.net.*;
 import java.io.IOException;
-import sun.applet.AppletAudioClip;
+import java.net.ContentHandler;
+import java.net.URLConnection;
+
+import com.sun.media.sound.JavaSoundAudioClip;
 
 /**
- * Returns an AppletAudioClip object.
+ * Basic .wav audio handler returns an JavaSoundAudioClip object.
+ *
+ * @author Jeff Nisewanger
  */
 public class wav extends ContentHandler {
     public Object getContent(URLConnection uc) throws IOException {
-        return new AppletAudioClip(uc);
+        return JavaSoundAudioClip.create(uc);
     }
 }
