@@ -71,6 +71,8 @@ class ExtendedTextSourceLabel extends ExtendedTextLabel implements Decoration.La
   StandardGlyphVector gv;
   float[] charinfo;
 
+  float advTracking;
+
   /**
    * Create from a TextSource.
    */
@@ -110,6 +112,8 @@ class ExtendedTextSourceLabel extends ExtendedTextLabel implements Decoration.La
           source.getStart() + source.getLength(), source.getFRC());
       cm = CoreMetrics.get(lm);
     }
+
+    advTracking = font.getSize() * AttributeValues.getTracking(atts);
   }
 
 
@@ -378,10 +382,10 @@ class ExtendedTextSourceLabel extends ExtendedTextLabel implements Decoration.La
     validate(index);
     float[] charinfo = getCharinfo();
     int idx = l2v(index) * numvals + advx;
-    if (charinfo == null || idx >= charinfo.length) {
+    if (charinfo == null || idx >= charinfo.length || charinfo[idx] == 0) {
         return 0f;
     } else {
-        return charinfo[idx];
+        return charinfo[idx] + advTracking;
     }
   }
 
@@ -477,16 +481,25 @@ class ExtendedTextSourceLabel extends ExtendedTextLabel implements Decoration.La
   }
 
   public int getLineBreakIndex(int start, float width) {
+    final float epsilon = 0.005f;
+
     float[] charinfo = getCharinfo();
     int length = source.getLength();
+
+    if (advTracking > 0) {
+      width += advTracking;
+    }
+
     --start;
-    while (width >= 0 && ++start < length) {
+    while (width >= -epsilon && ++start < length) {
       int cidx = l2v(start) * numvals + advx;
       if (cidx >= charinfo.length) {
           break; // layout bailed for some reason
       }
       float adv = charinfo[cidx];
-      width -= adv;
+      if (adv != 0) {
+          width -= adv + advTracking;
+      }
     }
 
     return start;
@@ -502,7 +515,10 @@ class ExtendedTextSourceLabel extends ExtendedTextLabel implements Decoration.La
       if (cidx >= charinfo.length) {
           break; // layout bailed for some reason
       }
-      a += charinfo[cidx];
+      float adv = charinfo[cidx];
+      if (adv != 0) {
+          a += adv + advTracking;
+      }
     }
 
     return a;
