@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -38,6 +38,7 @@ import sun.jvm.hotspot.gc.parallel.*;
 import sun.jvm.hotspot.gc.shared.*;
 import sun.jvm.hotspot.gc.shenandoah.*;
 import sun.jvm.hotspot.gc.g1.*;
+import sun.jvm.hotspot.gc.x.*;
 import sun.jvm.hotspot.gc.z.*;
 import sun.jvm.hotspot.interpreter.*;
 import sun.jvm.hotspot.oops.*;
@@ -1104,9 +1105,6 @@ public class HSDB implements ObjectHistogramPanel.Listener, SAListener {
                           } else if (region.isHumongous()) {
                             anno = "Humongous ";
                             bad = false;
-                          } else if (region.isPinned()) {
-                            anno = "Pinned ";
-                            bad = false;
                           } else if (region.isOld()) {
                             anno = "Old ";
                             bad = false;
@@ -1127,6 +1125,10 @@ public class HSDB implements ObjectHistogramPanel.Listener, SAListener {
                           ShenandoahHeap heap = (ShenandoahHeap) collHeap;
                           anno = "ShenandoahHeap ";
                           bad = false;
+                        } else if (collHeap instanceof XCollectedHeap) {
+                          XCollectedHeap heap = (XCollectedHeap) collHeap;
+                          anno = "ZHeap ";
+                          bad = false;
                         } else if (collHeap instanceof ZCollectedHeap) {
                           ZCollectedHeap heap = (ZCollectedHeap) collHeap;
                           anno = "ZHeap ";
