@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -41,9 +41,7 @@ import sun.jvm.hotspot.types.TypeDataBase;
 
 public class HeapRegionManager extends VMObject {
     // G1HeapRegionTable _regions
-    static private long regionsFieldOffset;
-    // uint _committed_length
-    static private CIntegerField numCommittedField;
+    private static long regionsFieldOffset;
 
     static {
         VM.registerVMInitializedObserver(new Observer() {
@@ -53,17 +51,15 @@ public class HeapRegionManager extends VMObject {
             });
     }
 
-    static private synchronized void initialize(TypeDataBase db) {
+    private static synchronized void initialize(TypeDataBase db) {
         Type type = db.lookupType("HeapRegionManager");
 
         regionsFieldOffset = type.getField("_regions").getOffset();
-        numCommittedField = type.getCIntegerField("_num_committed");
     }
 
     private G1HeapRegionTable regions() {
         Address regionsAddr = addr.addOffsetTo(regionsFieldOffset);
-        return (G1HeapRegionTable) VMObjectFactory.newObject(G1HeapRegionTable.class,
-                                                             regionsAddr);
+        return VMObjectFactory.newObject(G1HeapRegionTable.class, regionsAddr);
     }
 
     public long capacity() {
@@ -74,10 +70,6 @@ public class HeapRegionManager extends VMObject {
         return regions().length();
     }
 
-    public long committedLength() {
-        return numCommittedField.getValue(addr);
-    }
-
     public Iterator<HeapRegion> heapRegionIterator() {
         return regions().heapRegionIterator(length());
     }
