@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2021, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -81,9 +81,8 @@ public class DataLayout {
     return data.getJShortAt(offset + at) & 0xffff;
   }
 
-  int cellAt(int index) {
-    // Cells are intptr_t sized but only contain ints as raw values
-    return (int)data.getCIntegerAt(offset + cellOffset(index), MethodData.cellSize, false);
+  long cellAt(int index) {
+    return data.getCIntegerAt(offset + cellOffset(index), MethodData.cellSize, false);
   }
 
   public Address addressAt(int index) {
@@ -121,7 +120,7 @@ public class DataLayout {
       return VM.getVM().isLP64() ? 1 : 2;
   }
 
-  static public int computeSizeInBytes(int cellCount) {
+  public static int computeSizeInBytes(int cellCount) {
     return headerSizeInBytes() + cellCount * MethodData.cellSize;
   }
 
@@ -172,7 +171,7 @@ public class DataLayout {
     return 2;
   }
   public static int cellOffset(int index) {
-    return MethodData.cellSize + index * MethodData.cellSize;
+    return (headerSizeInCells() + index) * MethodData.cellSize;
   }
   // // Return a value which, when or-ed as a byte into _flags, sets the flag.
   // static int flagNumberToByteConstant(int flagNumber) {
