@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -38,8 +38,8 @@ public class ThreadInfoAction extends StateChangeAction {
     public static final String VALUE_NAME = "Show Thread Information...";
     public static final String VALUE_SMALL_ICON = "general/Information16.gif";
     public static final String VALUE_LARGE_ICON = "general/Information24.gif";
-    public static final Integer VALUE_MNEMONIC = new Integer('I');
-    public static final String VALUE_SHORT_DESCRIPTION = "Show Thread Informaion";
+    public static final Integer VALUE_MNEMONIC = (int) 'I';
+    public static final String VALUE_SHORT_DESCRIPTION = "Show Thread Information";
     public static final String VALUE_LONG_DESCRIPTION = "Show information about the current thread";
 
     public ThreadInfoAction() {
