@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -382,7 +382,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
       Formatter buf = new Formatter(genHTML);
       buf.append(genMethodLink(m));
       buf.append(" of ");
-      buf.append(genKlassLink((InstanceKlass) m.getMethodHolder()));
+      buf.append(genKlassLink(m.getMethodHolder()));
       return buf.toString();
    }
 
@@ -453,13 +453,13 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
       buf.headerCell("Constant Value");
       buf.endTag("tr");
 
-      final int length = (int) cpool.getLength();
+      final int length = cpool.getLength();
       // zero'th pool entry is always invalid. ignore it.
       for (int index = 1; index < length; index++) {
          buf.beginTag("tr");
          buf.cell(Integer.toString(index));
 
-         int ctag = (int) cpool.getTags().at((int) index);
+         int ctag = cpool.getTags().at(index);
          switch (ctag) {
             case JVM_CONSTANT_Integer:
                buf.cell("JVM_CONSTANT_Integer");
@@ -497,7 +497,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
 
             case JVM_CONSTANT_Class:
                buf.cell("JVM_CONSTANT_Class");
-               Klass klass = (Klass) cpool.getKlassAt(index);
+               Klass klass = cpool.getKlassAt(index);
                if (klass instanceof InstanceKlass) {
                   buf.cell(genKlassLink((InstanceKlass) klass));
                } else {
@@ -586,7 +586,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
          Formatter buf = new Formatter(genHTML);
          buf.genHTMLPrologue(genConstantPoolTitle(cpool));
          buf.h3("Holder Class");
-         buf.append(genKlassLink((InstanceKlass) cpool.getPoolHolder()));
+         buf.append(genKlassLink(cpool.getPoolHolder()));
          buf.h3("Constants");
          buf.append(genHTMLTableForConstantPool(cpool));
          buf.genHTMLEpilogue();
@@ -603,7 +603,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
    protected String genConstantPoolTitle(ConstantPool cpool) {
       Formatter buf = new Formatter(genHTML);
       buf.append("Constant Pool of [");
-      buf.append(genKlassTitle((InstanceKlass) cpool.getPoolHolder()));
+      buf.append(genKlassTitle(cpool.getPoolHolder()));
       buf.append("] @");
       buf.append(cpool.getAddress().toString());
       return buf.toString();
@@ -621,7 +621,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
          buf.genHTMLPrologue(genMethodTitle(method));
 
          buf.h3("Holder Class");
-         buf.append(genKlassLink((InstanceKlass) method.getMethodHolder()));
+         buf.append(genKlassLink(method.getMethodHolder()));
 
          NMethod nmethod = method.getNativeMethod();
          if (nmethod != null) {
@@ -630,7 +630,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
          }
 
          boolean hasThrows = method.hasCheckedExceptions();
-         ConstantPool cpool = ((InstanceKlass) method.getMethodHolder()).getConstants();
+         ConstantPool cpool = method.getMethodHolder().getConstants();
          if (hasThrows) {
             buf.h3("Checked Exception(s)");
             CheckedExceptionElement[] exceptions = method.getCheckedExceptions();
@@ -675,9 +675,9 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
                              buf.beginTag("tr");
                              if (hasLineNumbers) {
                                 int lineNumber = method.getLineNumberFromBCI(curBci);
-                                buf.cell(Integer.toString(lineNumber) + spaces);
+                                buf.cell(lineNumber + spaces);
                              }
-                             buf.cell(Integer.toString(curBci) + spaces);
+                             buf.cell(curBci + spaces);
 
                              buf.beginTag("td");
                              String instrStr = null;
@@ -708,7 +708,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
                                   if (m != null) {
                                     buf.link(genMethodHref(m), instrStr);
                                     buf.append(" of ");
-                                    InstanceKlass klass = (InstanceKlass) m.getMethodHolder();
+                                    InstanceKlass klass = m.getMethodHolder();
                                     buf.link(genKlassHref(klass), genKlassTitle(klass));
                                   } else {
                                     buf.append(instrStr);
@@ -815,7 +815,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
       CodeBlob blob = null;
 
       try {
-         blob = (CodeBlob)VM.getVM().getCodeCache().findBlobUnsafe(pc);
+         blob = VM.getVM().getCodeCache().findBlobUnsafe(pc);
       } catch (Exception exp) {
          // ignore
       }
@@ -883,10 +883,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
    }
 
    protected String genMultPCHref(String pcs) {
-      StringBuffer buf = new StringBuffer(genBaseHref());
-      buf.append("pc_multiple=");
-      buf.append(pcs);
-      return buf.toString();
+      return genBaseHref() + "pc_multiple=" + pcs;
    }
 
    protected String genPCHref(Address addr) {
@@ -952,7 +949,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
       }
 
       public void endInstruction(long endPc) {
-         instrSize += endPc - pc;
+         instrSize += (int) (endPc - pc);
          if (genHTML) buf.br();
 
          if (nmethod != null) {
@@ -986,7 +983,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
 
       public void epilogue() {
       }
-   };
+   }
 
    protected String genHTMLForRawDisassembly(sun.jvm.hotspot.debugger.Address addr,
                                              int size,
@@ -1010,7 +1007,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
          Formatter tmpBuf = new Formatter(genHTML);
          long startPc = addressToLong(addr);
          tmpBuf.append("0x");
-         tmpBuf.append(Long.toHexString(startPc + visitor.getInstructionSize()).toString());
+         tmpBuf.append(Long.toHexString(startPc + visitor.getInstructionSize()));
          tmpBuf.append(",0x");
          tmpBuf.append(Long.toHexString(startPc));
          if (prevPCs != null) {
@@ -1123,18 +1120,14 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
                 buf.append(" " + kls.getName().asString() + "={");
                 int flen = ov.fieldsSize();
 
-                U2Array klfields = kls.getFields();
-                int klen = (int) klfields.length();
+                sun.jvm.hotspot.oops.Field[] fields = sun.jvm.hotspot.oops.Field.getFields(kls);
                 int findex = 0;
-                for (int index = 0; index < klen; index++) {
-                    int accsFlags = kls.getFieldAccessFlags(index);
-                    Symbol f_name = kls.getFieldName(index);
-                    AccessFlags access = new AccessFlags(accsFlags);
-                    if (!access.isStatic()) {
-                        ScopeValue svf = ov.getFieldAt(findex++);
-                        String    fstr = scopeValueAsString(sd, svf);
-                        buf.append(" [" + f_name.asString() + " :"+ index + "]=(#" + fstr + ")");
-                    }
+                for (int index = 0; index < fields.length; index++) {
+                  if (!fields[index].getAccessFlagsObj().isStatic()) {
+                     ScopeValue svf = ov.getFieldAt(findex++);
+                     String    fstr = scopeValueAsString(sd, svf);
+                     buf.append(" [" + fields[index].getName().asString() + " :"+ index + "]=(#" + fstr + ")");
+                  }
                 }
                 buf.append(" }");
             } else {
@@ -1183,7 +1176,8 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
       Formatter buf = new Formatter(genHTML);
 
       final class OopMapValueIterator {
-         final Formatter iterate(OopMapStream oms, String type, boolean printContentReg) {
+         final Formatter iterate(OopMapStream oms, String type, boolean printContentReg,
+                                 OopMapValue.OopTypes filter) {
             Formatter tmpBuf = new Formatter(genHTML);
             boolean found = false;
             tmpBuf.beginTag("tr");
@@ -1191,7 +1185,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
             tmpBuf.append(type);
             for (; ! oms.isDone(); oms.next()) {
                OopMapValue omv = oms.getCurrent();
-               if (omv == null) {
+               if (omv == null || omv.getType() != filter) {
                   continue;
                }
                found = true;
@@ -1227,17 +1221,21 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
       buf.beginTable(0);
 
       OopMapValueIterator omvIterator = new OopMapValueIterator();
-      OopMapStream oms = new OopMapStream(map, OopMapValue.OopTypes.OOP_VALUE);
-      buf.append(omvIterator.iterate(oms, "Oops:", false));
+      OopMapStream oms = new OopMapStream(map);
+      buf.append(omvIterator.iterate(oms, "Oops:", false,
+                                     OopMapValue.OopTypes.OOP_VALUE));
 
-      oms = new OopMapStream(map, OopMapValue.OopTypes.NARROWOOP_VALUE);
-      buf.append(omvIterator.iterate(oms, "NarrowOops:", false));
+      oms = new OopMapStream(map);
+      buf.append(omvIterator.iterate(oms, "NarrowOops:", false,
+                                     OopMapValue.OopTypes.NARROWOOP_VALUE));
 
-      oms = new OopMapStream(map, OopMapValue.OopTypes.CALLEE_SAVED_VALUE);
-      buf.append(omvIterator.iterate(oms, "Callee saved:",  true));
+      oms = new OopMapStream(map);
+      buf.append(omvIterator.iterate(oms, "Callee saved:", true,
+                                     OopMapValue.OopTypes.CALLEE_SAVED_VALUE));
 
-      oms = new OopMapStream(map, OopMapValue.OopTypes.DERIVED_OOP_VALUE);
-      buf.append(omvIterator.iterate(oms, "Derived oops:", true));
+      oms = new OopMapStream(map);
+      buf.append(omvIterator.iterate(oms, "Derived oops:", true,
+                                     OopMapValue.OopTypes.DERIVED_OOP_VALUE));
 
       buf.endTag("table");
       return buf.toString();
@@ -1279,8 +1277,6 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
          if (w == Location.Where.ON_STACK) {
             buf.append("stack[" + loc.getStackOffset() + "]");
          } else if (w == Location.Where.IN_REGISTER) {
-            boolean isFloat = (type == Location.Type.FLOAT_IN_DBL ||
-                               type == Location.Type.DBL);
             int regNum = loc.getRegisterNumber();
             VMReg vmReg = new VMReg(regNum);
             buf.append(VMRegImpl.getRegisterName(vmReg.getValue()));
@@ -1542,7 +1538,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
    }
 
    protected String genDumpKlassesHref(InstanceKlass[] klasses) {
-      StringBuffer buf = new StringBuffer(genBaseHref());
+      StringBuilder buf = new StringBuilder(genBaseHref());
       buf.append("jcore_multiple=");
       for (int k = 0; k < klasses.length; k++) {
          buf.append(klasses[k].getAddress().toString());
@@ -1680,15 +1676,15 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
 
    protected String genHTMLListForFields(InstanceKlass klass) {
       Formatter buf = new Formatter(genHTML);
-      U2Array fields = klass.getFields();
-      int numFields = klass.getAllFieldsCount();
+      sun.jvm.hotspot.oops.Field[] fields = sun.jvm.hotspot.oops.Field.getFields(klass);
+      int numFields = fields.length;
       if (numFields != 0) {
          buf.h3("Fields");
          buf.beginList();
          for (int f = 0; f < numFields; f++) {
-           sun.jvm.hotspot.oops.Field field = klass.getFieldByIndex(f);
-           String f_name = ((NamedFieldIdentifier)field.getID()).getName();
-           Symbol f_sig  = field.getSignature();
+           sun.jvm.hotspot.oops.Field field = fields[f];
+           String f_name = field.getName().asString();
+           Symbol f_sig = field.getSignature();
            Symbol f_genSig = field.getGenericSignature();
            AccessFlags acc = field.getAccessFlagsObj();
 
@@ -1702,7 +1698,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
            buf.append(f_name);
            buf.append(';');
            // is it generic?
-           if (f_genSig != null) {
+           if (field.isGeneric()) {
               buf.append(" [signature ");
               buf.append(escapeHTMLSpecialChars(f_genSig.asString()));
               buf.append("] ");
