@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -28,7 +28,10 @@ package sun.management;
 import java.lang.management.ManagementFactory;
 import java.lang.management.ThreadInfo;
 import java.lang.management.ThreadMXBean;
+import java.util.stream.Stream;
 import javax.management.ObjectName;
+import java.util.Arrays;
+import java.util.Objects;
 
 /**
  * Implementation for java.lang.management.ThreadMXBean as well as providing the
@@ -112,42 +115,34 @@ public class ThreadImpl implements ThreadMXBean {
         return cpuTimeEnabled;
     }
 
-    protected boolean isThreadAllocatedMemoryEnabled() {
+    private void ensureThreadAllocatedMemorySupported() {
         if (!isThreadAllocatedMemorySupported()) {
             throw new UnsupportedOperationException(
-                "Thread allocated memory measurement is not supported");
+                "Thread allocated memory measurement is not supported.");
         }
+    }
+
+    protected boolean isThreadAllocatedMemoryEnabled() {
+        ensureThreadAllocatedMemorySupported();
         return allocatedMemoryEnabled;
     }
 
     @Override
     public long[] getAllThreadIds() {
         Util.checkMonitorAccess();
-
         Thread[] threads = getThreads();
-        int length = threads.length;
-        long[] ids = new long[length];
-        for (int i = 0; i < length; i++) {
-            Thread t = threads[i];
-            ids[i] = t.getId();
-        }
-        return ids;
+        return threadIds(threads);
     }
 
     @Override
     public ThreadInfo getThreadInfo(long id) {
-        long[] ids = new long[1];
-        ids[0] = id;
-        final ThreadInfo[] infos = getThreadInfo(ids, 0);
-        return infos[0];
+        return getThreadInfo(id, 0);
     }
 
     @Override
     public ThreadInfo getThreadInfo(long id, int maxDepth) {
-        long[] ids = new long[1];
-        ids[0] = id;
-        final ThreadInfo[] infos = getThreadInfo(ids, maxDepth);
-        return infos[0];
+        long[] ids = new long[] { id };
+        return getThreadInfo(ids, maxDepth)[0];
     }
 
     @Override
@@ -155,16 +150,18 @@ public class ThreadImpl implements ThreadMXBean {
         return getThreadInfo(ids, 0);
     }
 
-    private void verifyThreadIds(long[] ids) {
-        if (ids == null) {
-            throw new NullPointerException("Null ids parameter.");
+    private void verifyThreadId(long id) {
+        if (id <= 0) {
+            throw new IllegalArgumentException(
+                "Invalid thread ID parameter: " + id);
         }
+    }
+
+    private void verifyThreadIds(long[] ids) {
+        Objects.requireNonNull(ids);
 
         for (int i = 0; i < ids.length; i++) {
-            if (ids[i] <= 0) {
-                throw new IllegalArgumentException(
-                    "Invalid thread ID parameter: " + ids[i]);
-            }
+            verifyThreadId(ids[i]);
         }
     }
 
@@ -218,7 +215,6 @@ public class ThreadImpl implements ThreadMXBean {
     }
 
     private boolean verifyCurrentThreadCpuTime() {
-        // check if Thread CPU time measurement is supported.
         if (!isCurrentThreadCpuTimeSupported()) {
             throw new UnsupportedOperationException(
                 "Current thread CPU time measurement is not supported.");
@@ -228,7 +224,7 @@ public class ThreadImpl implements ThreadMXBean {
 
     @Override
     public long getCurrentThreadCpuTime() {
-        if (verifyCurrentThreadCpuTime()) {
+        if (verifyCurrentThreadCpuTime() && !Thread.currentThread().isVirtual()) {
             return getThreadTotalCpuTime0(0);
         }
         return -1;
@@ -255,7 +251,7 @@ public class ThreadImpl implements ThreadMXBean {
         if (!isThreadCpuTimeSupported()) {
             // support current thread only
             for (int i = 0; i < ids.length; i++) {
-                if (ids[i] != Thread.currentThread().getId()) {
+                if (ids[i] != Thread.currentThread().threadId()) {
                     throw new UnsupportedOperationException(
                         "Thread CPU time measurement is only supported" +
                         " for the current thread.");
@@ -276,10 +272,12 @@ public class ThreadImpl implements ThreadMXBean {
         if (verified) {
             if (length == 1) {
                 long id = ids[0];
-                if (id == Thread.currentThread().getId()) {
-                    id = 0;
+                Thread thread = Thread.currentThread();
+                if (id == thread.threadId()) {
+                    times[0] = thread.isVirtual() ? -1L : getThreadTotalCpuTime0(0);
+                } else {
+                    times[0] = getThreadTotalCpuTime0(id);
                 }
-                times[0] = getThreadTotalCpuTime0(id);
             } else {
                 getThreadTotalCpuTime1(ids, times);
             }
@@ -289,7 +287,7 @@ public class ThreadImpl implements ThreadMXBean {
 
     @Override
     public long getCurrentThreadUserTime() {
-        if (verifyCurrentThreadCpuTime()) {
+        if (verifyCurrentThreadCpuTime() && !Thread.currentThread().isVirtual()) {
             return getThreadUserCpuTime0(0);
         }
         return -1;
@@ -313,10 +311,12 @@ public class ThreadImpl implements ThreadMXBean {
         if (verified) {
             if (length == 1) {
                 long id = ids[0];
-                if (id == Thread.currentThread().getId()) {
-                    id = 0;
+                Thread thread = Thread.currentThread();
+                if (id == thread.threadId()) {
+                    times[0] = thread.isVirtual() ? -1L : getThreadTotalCpuTime0(0);
+                } else {
+                    times[0] = getThreadUserCpuTime0(id);
                 }
-                times[0] = getThreadUserCpuTime0(id);
             } else {
                 getThreadUserCpuTime1(ids, times);
             }
@@ -342,26 +342,51 @@ public class ThreadImpl implements ThreadMXBean {
         }
     }
 
+    protected long getTotalThreadAllocatedBytes() {
+        if (isThreadAllocatedMemoryEnabled()) {
+            return getTotalThreadAllocatedMemory();
+        }
+        return -1;
+    }
+
+    protected long getCurrentThreadAllocatedBytes() {
+        if (isThreadAllocatedMemoryEnabled() && !Thread.currentThread().isVirtual()) {
+            return getThreadAllocatedMemory0(0);
+        }
+        return -1;
+    }
+
+    private boolean verifyThreadAllocatedMemory(long id) {
+        verifyThreadId(id);
+        return isThreadAllocatedMemoryEnabled();
+    }
+
     protected long getThreadAllocatedBytes(long id) {
-        long[] ids = new long[1];
-        ids[0] = id;
-        final long[] sizes = getThreadAllocatedBytes(ids);
-        return sizes[0];
+        boolean verified = verifyThreadAllocatedMemory(id);
+        if (verified) {
+            Thread thread = Thread.currentThread();
+            if (id == thread.threadId()) {
+                return thread.isVirtual() ? -1L : getThreadAllocatedMemory0(0);
+            } else {
+                return getThreadAllocatedMemory0(id);
+            }
+        }
+        return -1;
     }
 
     private boolean verifyThreadAllocatedMemory(long[] ids) {
         verifyThreadIds(ids);
-
-        // check if Thread allocated memory measurement is supported.
-        if (!isThreadAllocatedMemorySupported()) {
-            throw new UnsupportedOperationException(
-                "Thread allocated memory measurement is not supported.");
-        }
-
         return isThreadAllocatedMemoryEnabled();
     }
 
     protected long[] getThreadAllocatedBytes(long[] ids) {
+        Objects.requireNonNull(ids);
+
+        if (ids.length == 1) {
+            long size = getThreadAllocatedBytes(ids[0]);
+            return new long[] { size };
+        }
+
         boolean verified = verifyThreadAllocatedMemory(ids);
 
         long[] sizes = new long[ids.length];
@@ -374,10 +399,7 @@ public class ThreadImpl implements ThreadMXBean {
     }
 
     protected void setThreadAllocatedMemoryEnabled(boolean enable) {
-        if (!isThreadAllocatedMemorySupported()) {
-            throw new UnsupportedOperationException(
-                "Thread allocated memory measurement is not supported.");
-        }
+        ensureThreadAllocatedMemorySupported();
 
         Util.checkControlAccess();
         synchronized (this) {
@@ -389,21 +411,24 @@ public class ThreadImpl implements ThreadMXBean {
         }
     }
 
+    /**
+     * Returns an array of thread identifiers for the threads in the given
+     * array. Returns {@code null} if {@code threads} is null or the array
+     * of threads is empty.
+     */
+    private long[] threadsToIds(Thread[] threads) {
+        if (threads != null && threads.length > 0) {
+            return threadIds(threads);
+        } else {
+            return null;
+        }
+    }
+
     @Override
     public long[] findMonitorDeadlockedThreads() {
         Util.checkMonitorAccess();
-
         Thread[] threads = findMonitorDeadlockedThreads0();
-        if (threads == null) {
-            return null;
-        }
-
-        long[] ids = new long[threads.length];
-        for (int i = 0; i < threads.length; i++) {
-            Thread t = threads[i];
-            ids[i] = t.getId();
-        }
-        return ids;
+        return threadsToIds(threads);
     }
 
     @Override
@@ -416,16 +441,7 @@ public class ThreadImpl implements ThreadMXBean {
         Util.checkMonitorAccess();
 
         Thread[] threads = findDeadlockedThreads0();
-        if (threads == null) {
-            return null;
-        }
-
-        long[] ids = new long[threads.length];
-        for (int i = 0; i < threads.length; i++) {
-            Thread t = threads[i];
-            ids[i] = t.getId();
-        }
-        return ids;
+        return threadsToIds(threads);
     }
 
     @Override
@@ -463,10 +479,10 @@ public class ThreadImpl implements ThreadMXBean {
     public ThreadInfo[] getThreadInfo(long[] ids,
                                       boolean lockedMonitors,
                                       boolean lockedSynchronizers) {
-        return dumpThreads0(ids, lockedMonitors, lockedSynchronizers,
-                            Integer.MAX_VALUE);
+        return dumpThreads0(ids, lockedMonitors, lockedSynchronizers, Integer.MAX_VALUE);
     }
 
+    @Override
     public ThreadInfo[] getThreadInfo(long[] ids,
                                       boolean lockedMonitors,
                                       boolean lockedSynchronizers,
@@ -487,10 +503,10 @@ public class ThreadImpl implements ThreadMXBean {
     @Override
     public ThreadInfo[] dumpAllThreads(boolean lockedMonitors,
                                        boolean lockedSynchronizers) {
-        return dumpAllThreads(lockedMonitors, lockedSynchronizers,
-                              Integer.MAX_VALUE);
+        return dumpAllThreads(lockedMonitors, lockedSynchronizers, Integer.MAX_VALUE);
     }
 
+    @Override
     public ThreadInfo[] dumpAllThreads(boolean lockedMonitors,
                                        boolean lockedSynchronizers,
                                        int maxDepth) {
@@ -499,7 +515,10 @@ public class ThreadImpl implements ThreadMXBean {
                     "Invalid maxDepth parameter: " + maxDepth);
         }
         verifyDumpThreads(lockedMonitors, lockedSynchronizers);
-        return dumpThreads0(null, lockedMonitors, lockedSynchronizers, maxDepth);
+        ThreadInfo[] infos = dumpThreads0(null, lockedMonitors, lockedSynchronizers, maxDepth);
+        return Arrays.stream(infos)
+                .filter(ti -> ti != null)
+                .toArray(ThreadInfo[]::new);
     }
 
     // VM support where maxDepth == -1 to request entire stack dump
@@ -511,7 +530,9 @@ public class ThreadImpl implements ThreadMXBean {
     private static native void getThreadTotalCpuTime1(long[] ids, long[] result);
     private static native long getThreadUserCpuTime0(long id);
     private static native void getThreadUserCpuTime1(long[] ids, long[] result);
+    private static native long getThreadAllocatedMemory0(long id);
     private static native void getThreadAllocatedMemory1(long[] ids, long[] result);
+    private static native long getTotalThreadAllocatedMemory();
     private static native void setThreadCpuTimeEnabled0(boolean enable);
     private static native void setThreadAllocatedMemoryEnabled0(boolean enable);
     private static native void setThreadContentionMonitoringEnabled0(boolean enable);
@@ -531,4 +552,12 @@ public class ThreadImpl implements ThreadMXBean {
         return Util.newObjectName(ManagementFactory.THREAD_MXBEAN_NAME);
     }
 
+    /**
+     * Returns the thread identifiers of the threads in the given array.
+     */
+    private static long[] threadIds(Thread[] threads) {
+        return Stream.of(threads)
+                .mapToLong(Thread::threadId)
+                .toArray();
+    }
 }
