@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2025, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -24,8 +24,6 @@
  */
 package sun.net.ftp.impl;
 
-
-
 import java.io.BufferedInputStream;
 import java.io.BufferedOutputStream;
 import java.io.BufferedReader;
@@ -64,6 +62,7 @@ import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import javax.net.ssl.SSLSocket;
 import javax.net.ssl.SSLSocketFactory;
+
 import sun.net.ftp.FtpDirEntry;
 import sun.net.ftp.FtpDirParser;
 import sun.net.ftp.FtpProtocolException;
@@ -71,6 +70,7 @@ import sun.net.ftp.FtpReplyCode;
 import sun.net.util.IPAddressUtil;
 import sun.util.logging.PlatformLogger;
 
+import static sun.net.util.ProxyUtil.copyProxy;
 
 public class FtpClient extends sun.net.ftp.FtpClient {
 
@@ -134,17 +134,15 @@ public class FtpClient extends sun.net.ftp.FtpClient {
     private static final boolean acceptPasvAddressVal;
     static {
         final int vals[] = {0, 0};
-        final String encs[] = {null};
         final String acceptPasvAddress[] = {null};
-        AccessController.doPrivileged(
-                new PrivilegedAction<Object>() {
-
-                    public Object run() {
+        @SuppressWarnings("removal")
+        final String enc = AccessController.doPrivileged(
+                new PrivilegedAction<String>() {
+                    public String run() {
                         acceptPasvAddress[0] = System.getProperty("jdk.net.ftp.trustPasvAddress", "false");
                         vals[0] = Integer.getInteger("sun.net.client.defaultReadTimeout", 300_000).intValue();
                         vals[1] = Integer.getInteger("sun.net.client.defaultConnectTimeout", 300_000).intValue();
-                        encs[0] = System.getProperty("file.encoding", "ISO8859_1");
-                        return null;
+                        return System.getProperty("file.encoding", "ISO8859_1");
                     }
                 });
         if (vals[0] == 0) {
@@ -159,7 +157,7 @@ public class FtpClient extends sun.net.ftp.FtpClient {
             defaultConnectTimeout = vals[1];
         }
 
-        encoding = encs[0];
+        encoding = enc;
         try {
             if (!isASCIISuperset(encoding)) {
                 encoding = "ISO8859_1";
@@ -316,7 +314,7 @@ public class FtpClient extends sun.net.ftp.FtpClient {
         }
     }
 
-    private class MLSxParser implements FtpDirParser {
+    private static class MLSxParser implements FtpDirParser {
         public FtpDirEntry parseLine(String line) {
             String name = null;
             int i = line.lastIndexOf(';');
@@ -602,7 +600,7 @@ public class FtpClient extends sun.net.ftp.FtpClient {
             // the format will be :
             //  229 Entering Extended PASSIVE Mode (|||58210|)
             //
-            // So we'll use the regular expresions package to parse the output.
+            // So we'll use the regular expressions package to parse the output.
 
             if (epsvPat == null) {
                 epsvPat = Pattern.compile("^229 .* \\(\\|\\|\\|(\\d+)\\|\\)");
@@ -664,13 +662,10 @@ public class FtpClient extends sun.net.ftp.FtpClient {
         Socket s;
         if (proxy != null) {
             if (proxy.type() == Proxy.Type.SOCKS) {
-                s = AccessController.doPrivileged(
-                        new PrivilegedAction<Socket>() {
-
-                            public Socket run() {
-                                return new Socket(proxy);
-                            }
-                        });
+                PrivilegedAction<Socket> pa = () -> new Socket(proxy);
+                @SuppressWarnings("removal")
+                var tmp = AccessController.doPrivileged(pa);
+                s = tmp;
             } else {
                 s = new Socket(Proxy.NO_PROXY);
             }
@@ -678,13 +673,9 @@ public class FtpClient extends sun.net.ftp.FtpClient {
             s = new Socket();
         }
 
-        InetAddress serverAddress = AccessController.doPrivileged(
-                new PrivilegedAction<InetAddress>() {
-                    @Override
-                    public InetAddress run() {
-                        return server.getLocalAddress();
-                    }
-                });
+        PrivilegedAction<InetAddress> pa = () -> server.getLocalAddress();
+        @SuppressWarnings("removal")
+        InetAddress serverAddress = AccessController.doPrivileged(pa);
 
         // Bind the socket to the same address as the control channel. This
         // is needed in case of multi-homed systems.
@@ -772,7 +763,9 @@ public class FtpClient extends sun.net.ftp.FtpClient {
     private static InetAddress privilegedLocalHost() throws FtpProtocolException {
         PrivilegedExceptionAction<InetAddress> action = InetAddress::getLocalHost;
         try {
-            return AccessController.doPrivileged(action);
+            @SuppressWarnings("removal")
+            var tmp = AccessController.doPrivileged(action);
+            return tmp;
         } catch (Exception e) {
             var ftpEx = new FtpProtocolException(ERROR_MSG);
             ftpEx.initCause(e);
@@ -783,7 +776,9 @@ public class FtpClient extends sun.net.ftp.FtpClient {
     private static InetAddress[] privilegedGetAllByName(String hostName) throws FtpProtocolException {
         PrivilegedExceptionAction<InetAddress[]> pAction = () -> InetAddress.getAllByName(hostName);
         try {
-            return AccessController.doPrivileged(pAction);
+            @SuppressWarnings("removal")
+            var tmp =AccessController.doPrivileged(pAction);
+            return tmp;
         } catch (Exception e) {
             var ftpEx = new FtpProtocolException(ERROR_MSG);
             ftpEx.initCause(e);
@@ -987,7 +982,7 @@ public class FtpClient extends sun.net.ftp.FtpClient {
     }
 
     public sun.net.ftp.FtpClient setProxy(Proxy p) {
-        proxy = p;
+        proxy = copyProxy(p);
         return this;
     }
 
@@ -1026,13 +1021,10 @@ public class FtpClient extends sun.net.ftp.FtpClient {
         Socket s;
         if (proxy != null) {
             if (proxy.type() == Proxy.Type.SOCKS) {
-                s = AccessController.doPrivileged(
-                        new PrivilegedAction<Socket>() {
-
-                            public Socket run() {
-                                return new Socket(proxy);
-                            }
-                        });
+                PrivilegedAction<Socket> pa = () -> new Socket(proxy);
+                @SuppressWarnings("removal")
+                var tmp = AccessController.doPrivileged(pa);
+                s = tmp;
             } else {
                 s = new Socket(Proxy.NO_PROXY);
             }
@@ -1308,18 +1300,17 @@ public class FtpClient extends sun.net.ftp.FtpClient {
      * Retrieves a file from the ftp server and writes it to the specified
      * <code>OutputStream</code>.
      * If the restart offset was set, then a <code>REST</code> command will be
-     * sent before the RETR in order to restart the tranfer from the specified
+     * sent before the RETR in order to restart the transfer from the specified
      * offset.
      * The <code>OutputStream</code> is not closed by this method at the end
      * of the transfer.
      *
      * @param name a {@code String} containing the name of the file to
-     *        retreive from the server.
+     *        retrieve from the server.
      * @param local the <code>OutputStream</code> the file should be written to.
      * @throws IOException if the transfer fails.
      */
     public sun.net.ftp.FtpClient getFile(String name, OutputStream local) throws sun.net.ftp.FtpProtocolException, IOException {
-        int mtu = 1500;
         if (restartOffset > 0) {
             Socket s;
             try {
@@ -1329,27 +1320,15 @@ public class FtpClient extends sun.net.ftp.FtpClient {
             }
             issueCommandCheck("RETR " + name);
             getTransferSize();
-            InputStream remote = createInputStream(s.getInputStream());
-            byte[] buf = new byte[mtu * 10];
-            int l;
-            while ((l = remote.read(buf)) >= 0) {
-                if (l > 0) {
-                    local.write(buf, 0, l);
-                }
+            try (InputStream remote = createInputStream(s.getInputStream())) {
+                remote.transferTo(local);
             }
-            remote.close();
         } else {
             Socket s = openDataConnection("RETR " + name);
             getTransferSize();
-            InputStream remote = createInputStream(s.getInputStream());
-            byte[] buf = new byte[mtu * 10];
-            int l;
-            while ((l = remote.read(buf)) >= 0) {
-                if (l > 0) {
-                    local.write(buf, 0, l);
-                }
+            try (InputStream remote = createInputStream(s.getInputStream())) {
+                remote.transferTo(local);
             }
-            remote.close();
         }
         return completePending();
     }
@@ -1446,18 +1425,11 @@ public class FtpClient extends sun.net.ftp.FtpClient {
      */
     public sun.net.ftp.FtpClient putFile(String name, InputStream local, boolean unique) throws sun.net.ftp.FtpProtocolException, IOException {
         String cmd = unique ? "STOU " : "STOR ";
-        int mtu = 1500;
         if (type == TransferType.BINARY) {
             Socket s = openDataConnection(cmd + name);
-            OutputStream remote = createOutputStream(s.getOutputStream());
-            byte[] buf = new byte[mtu * 10];
-            int l;
-            while ((l = local.read(buf)) >= 0) {
-                if (l > 0) {
-                    remote.write(buf, 0, l);
-                }
+            try (OutputStream remote = createOutputStream(s.getOutputStream())) {
+                local.transferTo(remote);
             }
-            remote.close();
         }
         return completePending();
     }
@@ -1475,17 +1447,10 @@ public class FtpClient extends sun.net.ftp.FtpClient {
      * @throws IOException if an error occurred during the transmission.
      */
     public sun.net.ftp.FtpClient appendFile(String name, InputStream local) throws sun.net.ftp.FtpProtocolException, IOException {
-        int mtu = 1500;
         Socket s = openDataConnection("APPE " + name);
-        OutputStream remote = createOutputStream(s.getOutputStream());
-        byte[] buf = new byte[mtu * 10];
-        int l;
-        while ((l = local.read(buf)) >= 0) {
-            if (l > 0) {
-                remote.write(buf, 0, l);
-            }
+        try (OutputStream remote = createOutputStream(s.getOutputStream())) {
+            local.transferTo(remote);
         }
-        remote.close();
         return completePending();
     }
 
@@ -1683,7 +1648,7 @@ public class FtpClient extends sun.net.ftp.FtpClient {
      * <p>This method will actually block reading on the command channel for a
      * notification from the server that the command is finished. Such a
      * notification often carries extra information concerning the completion
-     * of the pending action (e.g. number of bytes transfered).</p>
+     * of the pending action (e.g. number of bytes transferred).</p>
      * <p>Note that this will return true immediately if no command or action
      * is pending</p>
      * <p>It should be also noted that most methods issuing commands to the ftp
@@ -1891,7 +1856,7 @@ public class FtpClient extends sun.net.ftp.FtpClient {
         return this;
     }
 
-    private class FtpFileIterator implements Iterator<FtpDirEntry>, Closeable {
+    private static class FtpFileIterator implements Iterator<FtpDirEntry>, Closeable {
 
         private BufferedReader in = null;
         private FtpDirEntry nextFile = null;
@@ -1961,7 +1926,7 @@ public class FtpClient extends sun.net.ftp.FtpClient {
      * is finished iterating through the files.
      *
      * @param path the pathname of the directory to list or <code>null</code>
-     *        for the current working directoty.
+     *        for the current working directory.
      * @return a <code>Iterator</code> of files or <code>null</code> if the
      *         command failed.
      * @throws IOException if an error occurred during the transmission
@@ -2011,7 +1976,7 @@ public class FtpClient extends sun.net.ftp.FtpClient {
      * Attempts to use Kerberos GSSAPI as an authentication mechanism with the
      * ftp server. This will issue an <code>AUTH GSSAPI</code> command, and if
      * it is accepted by the server, will followup with <code>ADAT</code>
-     * command to exchange the various tokens until authentification is
+     * command to exchange the various tokens until authentication is
      * successful. This conforms to Appendix I of RFC 2228.
      *
      * @return <code>true</code> if authentication was successful.
@@ -2093,7 +2058,7 @@ public class FtpClient extends sun.net.ftp.FtpClient {
 
     /**
      * Returns, when available, the size of the latest started transfer.
-     * This is retreived by parsing the response string received as an initial
+     * This is retrieved by parsing the response string received as an initial
      * response to a RETR or similar request.
      *
      * @return the size of the latest transfer or -1 if either there was no
@@ -2104,12 +2069,12 @@ public class FtpClient extends sun.net.ftp.FtpClient {
     }
 
     /**
-     * Returns, when available, the remote name of the last transfered file.
+     * Returns, when available, the remote name of the last transferred file.
      * This is mainly useful for "put" operation when the unique flag was
      * set since it allows to recover the unique file name created on the
      * server which may be different from the one submitted with the command.
      *
-     * @return the name the latest transfered file remote name, or
+     * @return the name the latest transferred file remote name, or
      *         <code>null</code> if that information is unavailable.
      */
     public String getLastFileName() {
@@ -2171,7 +2136,7 @@ public class FtpClient extends sun.net.ftp.FtpClient {
     /**
      * Sends a <code>CCC</code> command followed by a <code>PROT C</code>
      * command to the server terminating an encrypted session and reverting
-     * back to a non crypted transmission.
+     * back to a non encrypted transmission.
      *
      * @return <code>true</code> if the operation was successful.
      * @throws IOException if an error occurred during transmission.
