@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -35,6 +35,7 @@ import java.util.List;
 import java.io.IOException;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
+import java.util.Locale;
 import java.util.StringJoiner;
 import java.util.regex.Pattern;
 import java.util.stream.Stream;
@@ -66,7 +67,7 @@ public class DefaultProxySelector extends ProxySelector {
      *   listed in order of priority.
      * Example:
      * {"ftp", "ftp.proxy", "ftpProxy", "proxy", "socksProxy"},
-     * means for FTP we try in that oder:
+     * means for FTP we try in that order:
      *          + ftp.proxyHost & ftp.proxyPort
      *          + ftpProxyHost & ftpProxyPort
      *          + proxyHost & proxyPort
@@ -92,6 +93,7 @@ public class DefaultProxySelector extends ProxySelector {
 
     static {
         final String key = "java.net.useSystemProxies";
+        @SuppressWarnings("removal")
         Boolean b = AccessController.doPrivileged(
             new PrivilegedAction<Boolean>() {
                 public Boolean run() {
@@ -103,6 +105,7 @@ public class DefaultProxySelector extends ProxySelector {
         }
     }
 
+    @SuppressWarnings("removal")
     public static int socksProxyVersion() {
         return AccessController.doPrivileged(
                 new PrivilegedAction<Integer>() {
@@ -122,7 +125,7 @@ public class DefaultProxySelector extends ProxySelector {
 
     static class NonProxyInfo {
         // Default value for nonProxyHosts, this provides backward compatibility
-        // by excluding localhost and its litteral notations.
+        // by excluding localhost and its literal notations.
         static final String defStringVal = "localhost|127.*|[::1]|0.0.0.0|[::0]";
 
         String hostsSource;
@@ -203,7 +206,7 @@ public class DefaultProxySelector extends ProxySelector {
          */
         final String proto = protocol;
         final NonProxyInfo nprop = pinfo;
-        final String urlhost = host.toLowerCase();
+        final String urlhost = host.toLowerCase(Locale.ROOT);
 
         /**
          * This is one big doPrivileged call, but we're trying to optimize
@@ -211,6 +214,7 @@ public class DefaultProxySelector extends ProxySelector {
          * System properties it does help having only 1 call to doPrivileged.
          * Be mindful what you do in here though!
          */
+        @SuppressWarnings("removal")
         Proxy[] proxyArray = AccessController.doPrivileged(
             new PrivilegedAction<Proxy[]>() {
                 public Proxy[] run() {
