@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1994, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1994, 2025, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -46,7 +46,6 @@ import java.net.Proxy;
 import java.net.ProxySelector;
 import java.util.List;
 import java.util.StringTokenizer;
-import java.util.Iterator;
 import java.security.Permission;
 import java.util.Properties;
 import sun.net.NetworkClient;
@@ -57,11 +56,11 @@ import sun.net.www.URLConnection;
 import sun.net.www.protocol.http.HttpURLConnection;
 import sun.net.ftp.FtpClient;
 import sun.net.ftp.FtpProtocolException;
-import sun.net.ProgressSource;
-import sun.net.ProgressMonitor;
 import sun.net.www.ParseUtil;
 import sun.security.action.GetPropertyAction;
 
+import static sun.net.util.ProxyUtil.copyProxy;
+
 
 /**
  * This class Opens an FTP input (or output) stream given a URL.
@@ -121,7 +120,7 @@ public class FtpURLConnection extends URLConnection {
      *   - The command socket (FtpClient).
      * Since that's the only class that needs to see that, it is an inner class.
      */
-    protected class FtpInputStream extends FilterInputStream {
+    protected static class FtpInputStream extends FilterInputStream {
         FtpClient ftp;
         FtpInputStream(FtpClient cl, InputStream fd) {
             super(new BufferedInputStream(fd));
@@ -144,7 +143,7 @@ public class FtpURLConnection extends URLConnection {
      *   - The command socket (FtpClient).
      * Since that's the only class that needs to see that, it is an inner class.
      */
-    protected class FtpOutputStream extends FilterOutputStream {
+    protected static class FtpOutputStream extends FilterOutputStream {
         FtpClient ftp;
         FtpOutputStream(FtpClient cl, OutputStream fd) {
             super(fd);
@@ -160,17 +159,15 @@ public class FtpURLConnection extends URLConnection {
         }
     }
 
-    static URL checkURL(URL u) throws IllegalArgumentException {
+    private static URL checkURL(URL u) throws MalformedURLException {
         if (u != null) {
             if (u.toExternalForm().indexOf('\n') > -1) {
-                Exception mfue = new MalformedURLException("Illegal character in URL");
-                throw new IllegalArgumentException(mfue.getMessage(), mfue);
+                throw new MalformedURLException("Illegal character in URL");
             }
         }
-        String s = IPAddressUtil.checkAuthority(u);
-        if (s != null) {
-            Exception mfue = new MalformedURLException(s);
-            throw new IllegalArgumentException(mfue.getMessage(), mfue);
+        String errMsg = IPAddressUtil.checkAuthority(u);
+        if (errMsg != null) {
+            throw new MalformedURLException(errMsg);
         }
         return u;
     }
@@ -180,14 +177,14 @@ public class FtpURLConnection extends URLConnection {
      *
      * @param   url     The {@code URL} to retrieve or store.
      */
-    public FtpURLConnection(URL url) {
+    public FtpURLConnection(URL url) throws MalformedURLException {
         this(url, null);
     }
 
     /**
      * Same as FtpURLconnection(URL) with a per connection proxy specified
      */
-    FtpURLConnection(URL url, Proxy p) {
+    FtpURLConnection(URL url, Proxy p) throws MalformedURLException {
         super(checkURL(url));
         instProxy = p;
         host = url.getHost();
@@ -235,6 +232,7 @@ public class FtpURLConnection extends URLConnection {
             /**
              * Do we have to use a proxy?
              */
+            @SuppressWarnings("removal")
             ProxySelector sel = java.security.AccessController.doPrivileged(
                     new java.security.PrivilegedAction<ProxySelector>() {
                         public ProxySelector run() {
@@ -249,9 +247,8 @@ public class FtpURLConnection extends URLConnection {
                 } catch (IllegalArgumentException iae) {
                     throw new IOException("Failed to select a proxy", iae);
                 }
-                final Iterator<Proxy> it = proxies.iterator();
-                while (it.hasNext()) {
-                    p = it.next();
+                for (Proxy proxy : proxies) {
+                    p = copyProxy(proxy);
                     if (p == null || p == Proxy.NO_PROXY ||
                         p.type() == Proxy.Type.SOCKS) {
                         break;
@@ -411,13 +408,13 @@ public class FtpURLConnection extends URLConnection {
     }
 
     /**
-     * Get the InputStream to retreive the remote file. It will issue the
+     * Get the InputStream to retrieve the remote file. It will issue the
      * "get" (or "dir") command to the ftp server.
      *
      * @return  the {@code InputStream} to the connection.
      *
      * @throws  IOException if already opened for output
-     * @throws  FtpProtocolException if errors occur during the transfert.
+     * @throws  FtpProtocolException if errors occur during the transfer.
      */
     @Override
     public InputStream getInputStream() throws IOException {
@@ -469,17 +466,7 @@ public class FtpURLConnection extends URLConnection {
 
                     // Wrap input stream with MeteredStream to ensure read() will always return -1
                     // at expected length.
-
-                    // Check if URL should be metered
-                    boolean meteredInput = ProgressMonitor.getDefault().shouldMeterInput(url, "GET");
-                    ProgressSource pi = null;
-
-                    if (meteredInput) {
-                        pi = new ProgressSource(url, "GET", l);
-                        pi.beginTracking();
-                    }
-
-                    is = new MeteredStream(is, pi, l);
+                    is = new MeteredStream(is, l);
                 }
             } catch (Exception e) {
                 e.printStackTrace();
@@ -511,17 +498,7 @@ public class FtpURLConnection extends URLConnection {
                 is = new FtpInputStream(ftp, ftp.list(null));
                 msgh.add("content-type", "text/plain");
                 msgh.add("access-type", "directory");
-            } catch (IOException ex) {
-                FileNotFoundException fnfe = new FileNotFoundException(fullpath);
-                if (ftp != null) {
-                    try {
-                        ftp.close();
-                    } catch (IOException ioe) {
-                        fnfe.addSuppressed(ioe);
-                    }
-                }
-                throw fnfe;
-            } catch (FtpProtocolException ex2) {
+            } catch (IOException | FtpProtocolException ex) {
                 FileNotFoundException fnfe = new FileNotFoundException(fullpath);
                 if (ftp != null) {
                     try {
@@ -554,7 +531,7 @@ public class FtpURLConnection extends URLConnection {
      *
      * @throws  IOException if already opened for input or the URL
      *          points to a directory
-     * @throws  FtpProtocolException if errors occur during the transfert.
+     * @throws  FtpProtocolException if errors occur during the transfer.
      */
     @Override
     public OutputStream getOutputStream() throws IOException {
@@ -564,7 +541,7 @@ public class FtpURLConnection extends URLConnection {
 
         if (http != null) {
             OutputStream out = http.getOutputStream();
-            // getInputStream() is neccessary to force a writeRequests()
+            // getInputStream() is necessary to force a writeRequests()
             // on the http client.
             http.getInputStream();
             return out;
