@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -65,6 +65,7 @@ class UnixAsynchronousServerSocketChannelImpl
     private PendingFuture<AsynchronousSocketChannel,Object> acceptFuture;
 
     // context for permission check when security manager set
+    @SuppressWarnings("removal")
     private AccessControlContext acceptAcc;
 
 
@@ -172,7 +173,7 @@ class UnixAsynchronousServerSocketChannelImpl
             }
         }
 
-        // copy field befores accept is re-renabled
+        // copy field before accept is re-renabled
         CompletionHandler<AsynchronousSocketChannel,Object> handler = acceptHandler;
         Object att = acceptAttachment;
         PendingFuture<AsynchronousSocketChannel,Object> future = acceptFuture;
@@ -200,6 +201,7 @@ class UnixAsynchronousServerSocketChannelImpl
      * with an IOException or SecurityException then the channel/file descriptor
      * will be closed.
      */
+    @SuppressWarnings("removal")
     private AsynchronousSocketChannel finishAccept(FileDescriptor newfd,
                                                    final InetSocketAddress remote,
                                                    AccessControlContext acc)
@@ -244,6 +246,7 @@ class UnixAsynchronousServerSocketChannelImpl
         return ch;
     }
 
+    @SuppressWarnings("removal")
     @Override
     Future<AsynchronousSocketChannel> implAccept(Object att,
         CompletionHandler<AsynchronousSocketChannel,Object> handler)
