@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,16 +25,40 @@
 
 package sun.nio.fs;
 
-import java.nio.file.*;
-import java.nio.file.attribute.*;
-import java.nio.file.spi.FileTypeDetector;
-import java.nio.channels.*;
-import java.net.URI;
-import java.util.concurrent.ExecutorService;
 import java.io.IOException;
 import java.io.FilePermission;
-import java.util.*;
+import java.net.URI;
+import java.nio.channels.AsynchronousFileChannel;
+import java.nio.channels.FileChannel;
+import java.nio.channels.SeekableByteChannel;
+import java.nio.file.AccessMode;
+import java.nio.file.CopyOption;
+import java.nio.file.DirectoryNotEmptyException;
+import java.nio.file.DirectoryStream;
+import java.nio.file.FileAlreadyExistsException;
+import java.nio.file.FileStore;
+import java.nio.file.FileSystem;
+import java.nio.file.FileSystemAlreadyExistsException;
+import java.nio.file.LinkOption;
+import java.nio.file.LinkPermission;
+import java.nio.file.NotDirectoryException;
+import java.nio.file.NotLinkException;
+import java.nio.file.OpenOption;
+import java.nio.file.Path;
+import java.nio.file.ProviderMismatchException;
+import java.nio.file.attribute.BasicFileAttributes;
+import java.nio.file.attribute.BasicFileAttributeView;
+import java.nio.file.attribute.FileAttribute;
+import java.nio.file.attribute.FileAttributeView;
+import java.nio.file.attribute.FileOwnerAttributeView;
+import java.nio.file.attribute.PosixFileAttributes;
+import java.nio.file.attribute.PosixFileAttributeView;
+import java.nio.file.spi.FileTypeDetector;
+import java.util.Map;
+import java.util.Set;
+import java.util.concurrent.ExecutorService;
 
+import jdk.internal.util.StaticProperty;
 import sun.nio.ch.ThreadPool;
 import sun.security.util.SecurityConstants;
 import static sun.nio.fs.UnixNativeDispatcher.*;
@@ -47,13 +71,11 @@ import static sun.nio.fs.UnixConstants.*;
 public abstract class UnixFileSystemProvider
     extends AbstractFileSystemProvider
 {
-    private static final String USER_DIR = "user.dir";
     private static final byte[] EMPTY_PATH = new byte[0];
     private final UnixFileSystem theFileSystem;
 
     public UnixFileSystemProvider() {
-        String userDir = System.getProperty(USER_DIR);
-        theFileSystem = newFileSystem(userDir);
+        theFileSystem = newFileSystem(StaticProperty.userDir());
     }
 
     UnixFileSystem theFileSystem() {
@@ -127,7 +149,7 @@ public abstract class UnixFileSystemProvider
             return (V) UnixFileAttributeViews.createOwnerView(file, followLinks);
         if (type == null)
             throw new NullPointerException();
-        return (V) null;
+        return null;
     }
 
     @Override
@@ -149,6 +171,26 @@ public abstract class UnixFileSystemProvider
         return (A) getFileAttributeView(file, view, options).readAttributes();
     }
 
+    @Override
+    public <A extends BasicFileAttributes> A readAttributesIfExists(Path path,
+                                                                    Class<A> type,
+                                                                    LinkOption... options)
+        throws IOException
+    {
+        if (type == BasicFileAttributes.class && Util.followLinks(options)) {
+            UnixPath file = UnixPath.toUnixPath(path);
+            file.checkRead();
+            try {
+                @SuppressWarnings("unchecked")
+                A attrs = (A) UnixFileAttributes.getIfExists(file);
+                return attrs;
+            } catch (UnixException e) {
+                e.rethrowAsIOException(file);
+            }
+        }
+        return super.readAttributesIfExists(path, type, options);
+    }
+
     @Override
     protected DynamicFileAttributeView getFileAttributeView(Path obj,
                                                             String name,
@@ -255,18 +297,18 @@ public abstract class UnixFileSystemProvider
     public void copy(Path source, Path target, CopyOption... options)
         throws IOException
     {
-        UnixCopyFile.copy(UnixPath.toUnixPath(source),
-                          UnixPath.toUnixPath(target),
-                          options);
+        theFileSystem.copy(UnixPath.toUnixPath(source),
+                           UnixPath.toUnixPath(target),
+                           options);
     }
 
     @Override
     public void move(Path source, Path target, CopyOption... options)
         throws IOException
     {
-        UnixCopyFile.move(UnixPath.toUnixPath(source),
-                          UnixPath.toUnixPath(target),
-                          options);
+        theFileSystem.move(UnixPath.toUnixPath(source),
+                           UnixPath.toUnixPath(target),
+                           options);
     }
 
     @Override
@@ -282,10 +324,9 @@ public abstract class UnixFileSystemProvider
         } else {
             for (AccessMode mode: modes) {
                 switch (mode) {
-                    case READ : r = true; break;
-                    case WRITE : w = true; break;
-                    case EXECUTE : x = true; break;
-                    default: throw new AssertionError("Should not get here");
+                    case READ -> r = true;
+                    case WRITE -> w = true;
+                    case EXECUTE -> x = true;
                 }
             }
         }
@@ -300,6 +341,7 @@ public abstract class UnixFileSystemProvider
             mode |= W_OK;
         }
         if (x) {
+            @SuppressWarnings("removal")
             SecurityManager sm = System.getSecurityManager();
             if (sm != null) {
                 // not cached
@@ -321,9 +363,8 @@ public abstract class UnixFileSystemProvider
             return true;
         if (obj2 == null)
             throw new NullPointerException();
-        if (!(obj2 instanceof UnixPath))
+        if (!(obj2 instanceof UnixPath file2))
             return false;
-        UnixPath file2 = (UnixPath)obj2;
 
         // check security manager access to both files
         file1.checkRead();
@@ -372,6 +413,7 @@ public abstract class UnixFileSystemProvider
     @Override
     public FileStore getFileStore(Path obj) throws IOException {
         UnixPath file = UnixPath.toUnixPath(obj);
+        @SuppressWarnings("removal")
         SecurityManager sm = System.getSecurityManager();
         if (sm != null) {
             sm.checkPermission(new RuntimePermission("getFileStoreAttributes"));
@@ -430,13 +472,14 @@ public abstract class UnixFileSystemProvider
             dfd2 = dup(dfd1);
             dp = fdopendir(dfd1);
         } catch (UnixException x) {
+            IOException ioe = x.errno() == UnixConstants.ENOTDIR ?
+                new NotDirectoryException(dir.getPathForExceptionMessage()) :
+                x.asIOException(dir);
             if (dfd1 != -1)
-                UnixNativeDispatcher.close(dfd1);
+                UnixNativeDispatcher.close(dfd1, e -> null);
             if (dfd2 != -1)
-                UnixNativeDispatcher.close(dfd2);
-            if (x.errno() == UnixConstants.ENOTDIR)
-                throw new NotDirectoryException(dir.getPathForExceptionMessage());
-            x.rethrowAsIOException(dir);
+                UnixNativeDispatcher.close(dfd2, e -> null);
+            throw ioe;
         }
         return new UnixSecureDirectoryStream(dir, dp, dfd2, filter);
     }
@@ -456,6 +499,7 @@ public abstract class UnixFileSystemProvider
         }
 
         // permission check
+        @SuppressWarnings("removal")
         SecurityManager sm = System.getSecurityManager();
         if (sm != null) {
             sm.checkPermission(new LinkPermission("symbolic"));
@@ -476,6 +520,7 @@ public abstract class UnixFileSystemProvider
         UnixPath existing = UnixPath.toUnixPath(obj2);
 
         // permission check
+        @SuppressWarnings("removal")
         SecurityManager sm = System.getSecurityManager();
         if (sm != null) {
             sm.checkPermission(new LinkPermission("hard"));
@@ -493,6 +538,7 @@ public abstract class UnixFileSystemProvider
     public Path readSymbolicLink(Path obj1) throws IOException {
         UnixPath link = UnixPath.toUnixPath(obj1);
         // permission check
+        @SuppressWarnings("removal")
         SecurityManager sm = System.getSecurityManager();
         if (sm != null) {
             FilePermission perm = new FilePermission(link.getPathForPermissionCheck(),
@@ -511,28 +557,16 @@ public abstract class UnixFileSystemProvider
     }
 
     @Override
-    public final boolean isDirectory(Path obj) {
-        UnixPath file = UnixPath.toUnixPath(obj);
-        file.checkRead();
-        int mode = UnixNativeDispatcher.stat(file);
-        return ((mode & UnixConstants.S_IFMT) == UnixConstants.S_IFDIR);
-    }
+    public boolean exists(Path path, LinkOption... options) {
+        if (Util.followLinks(options)) {
+            UnixPath file = UnixPath.toUnixPath(path);
+            file.checkRead();
+            return UnixNativeDispatcher.exists(file);
+        } else {
+            return super.exists(path, options);
+        }
 
-    @Override
-    public final boolean isRegularFile(Path obj) {
-        UnixPath file = UnixPath.toUnixPath(obj);
-        file.checkRead();
-        int mode = UnixNativeDispatcher.stat(file);
-        return ((mode & UnixConstants.S_IFMT) == UnixConstants.S_IFREG);
     }
-
-    @Override
-    public final boolean exists(Path obj) {
-        UnixPath file = UnixPath.toUnixPath(obj);
-        file.checkRead();
-        return UnixNativeDispatcher.exists(file);
-    }
-
     /**
      * Returns a {@code FileTypeDetector} for this platform.
      */
