@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -83,14 +83,14 @@ public class AnnotationParser {
      * Like {@link #parseAnnotations(byte[], sun.reflect.ConstantPool, Class)}
      * with an additional parameter {@code selectAnnotationClasses} which selects the
      * annotation types to parse (other than selected are quickly skipped).<p>
-     * This method is only used to parse select meta annotations in the construction
+     * This method is used to parse select meta annotations in the construction
      * phase of {@link AnnotationType} instances to prevent infinite recursion.
      *
      * @param selectAnnotationClasses an array of annotation types to select when parsing
      */
     @SafeVarargs
     @SuppressWarnings("varargs") // selectAnnotationClasses is used safely
-    static Map<Class<? extends Annotation>, Annotation> parseSelectAnnotations(
+    public static Map<Class<? extends Annotation>, Annotation> parseSelectAnnotations(
                 byte[] rawAnnotations,
                 ConstantPool constPool,
                 Class<?> container,
@@ -280,9 +280,8 @@ public class AnnotationParser {
                 skipMemberValue(buf);
             } else {
                 Object value = parseMemberValue(memberType, buf, constPool, container);
-                if (value instanceof AnnotationTypeMismatchExceptionProxy)
-                    ((AnnotationTypeMismatchExceptionProxy) value).
-                        setMember(type.members().get(memberName));
+                if (value instanceof AnnotationTypeMismatchExceptionProxy exceptProxy)
+                    exceptProxy.setMember(type.members().get(memberName));
                 memberValues.put(memberName, value);
             }
         }
@@ -337,6 +336,8 @@ public class AnnotationParser {
                                           ByteBuffer buf,
                                           ConstantPool constPool,
                                           Class<?> container) {
+        // Note that VMSupport.encodeAnnotation (used by JVMCI) may need to
+        // be updated if new annotation member types are added.
         Object result = null;
         int tag = buf.get();
         switch(tag) {
@@ -443,9 +444,8 @@ public class AnnotationParser {
         return toClass(result);
     }
     static Class<?> toClass(Type o) {
-        if (o instanceof GenericArrayType)
-            return Array.newInstance(toClass(((GenericArrayType)o).getGenericComponentType()),
-                                     0)
+        if (o instanceof GenericArrayType gat)
+            return Array.newInstance(toClass(gat.getGenericComponentType()), 0)
                 .getClass();
         return (Class)o;
     }
@@ -744,8 +744,9 @@ public class AnnotationParser {
             int tag = buf.get();
             if (tag == expectedTag) {
                 Object value = parseElement.get();
-                if (value instanceof ExceptionProxy) {
-                    if (exceptionProxy == null) exceptionProxy = (ExceptionProxy) value;
+                if (value instanceof ExceptionProxy proxyValue) {
+                    if (exceptionProxy == null)
+                        exceptionProxy = proxyValue;
                 } else {
                     result[i] = value;
                 }
