@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1996, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -80,12 +80,14 @@ public class TCPEndpoint implements Endpoint {
     private static boolean localHostKnown;
 
     // this should be a *private* method since it is privileged
+    @SuppressWarnings("removal")
     private static int getInt(String name, int def) {
         return AccessController.doPrivileged(
                 (PrivilegedAction<Integer>) () -> Integer.getInteger(name, def));
     }
 
     // this should be a *private* method since it is privileged
+    @SuppressWarnings("removal")
     private static boolean getBoolean(String name) {
         return AccessController.doPrivileged(
                 (PrivilegedAction<Boolean>) () -> Boolean.getBoolean(name));
@@ -94,6 +96,7 @@ public class TCPEndpoint implements Endpoint {
     /**
      * Returns the value of the java.rmi.server.hostname property.
      */
+    @SuppressWarnings("removal")
     private static String getHostnameProperty() {
         return AccessController.doPrivileged(
             (PrivilegedAction<String>) () -> System.getProperty("java.rmi.server.hostname"));
@@ -102,7 +105,7 @@ public class TCPEndpoint implements Endpoint {
     /**
      * Find host name of local machine.  Property "java.rmi.server.hostname"
      * is used if set, so server administrator can compensate for the possible
-     * inablility to get fully qualified host name from VM.
+     * inability to get fully qualified host name from VM.
      */
     static {
         localHostKnown = true;
@@ -127,7 +130,7 @@ public class TCPEndpoint implements Endpoint {
                     localHost = FQDN.attemptFQDN(localAddr);
                 } else {
                     /* default to using ip addresses, names will
-                     * work across seperate domains.
+                     * work across separate domains.
                      */
                     localHost = localAddr.getHostAddress();
                 }
@@ -381,7 +384,7 @@ public class TCPEndpoint implements Endpoint {
         Set<TCPTransport> s;
         synchronized (localEndpoints) {
             // presize s to number of localEndpoints
-            s = new HashSet<TCPTransport>(localEndpoints.size());
+            s = HashSet.newHashSet(localEndpoints.size());
             for (LinkedList<TCPEndpoint> epList : localEndpoints.values()) {
                 /*
                  * Each local endpoint has its transport added to s.
@@ -486,8 +489,7 @@ public class TCPEndpoint implements Endpoint {
     }
 
     public boolean equals(Object obj) {
-        if ((obj != null) && (obj instanceof TCPEndpoint)) {
-            TCPEndpoint ep = (TCPEndpoint) obj;
+        if (obj instanceof TCPEndpoint ep) {
             if (port != ep.port || !host.equals(ep.host))
                 return false;
             if (((csf == null) ^ (ep.csf == null)) ||
@@ -760,6 +762,7 @@ public class TCPEndpoint implements Endpoint {
         private void getFQDN() {
 
             /* FQDN finder will run in RMI threadgroup. */
+            @SuppressWarnings("removal")
             Thread t = AccessController.doPrivileged(
                 new NewThreadAction(FQDN.this, "FQDN Finder", true));
             t.start();
