@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2021, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,14 +25,13 @@
 
 package sun.security.ec;
 
-import java.io.IOException;
-import java.math.BigInteger;
 import java.security.*;
 import java.security.spec.AlgorithmParameterSpec;
 import java.security.spec.ECGenParameterSpec;
 import java.security.spec.ECParameterSpec;
 import java.security.spec.ECPoint;
 import java.security.spec.InvalidParameterSpecException;
+import java.util.Arrays;
 import java.util.Optional;
 
 import sun.security.jca.JCAUtil;
@@ -192,18 +191,10 @@ public final class ECKeyPairGenerator extends KeyPairGeneratorSpi {
         int seedSize = (seedBits + 7) / 8;
         byte[] privArr = generatePrivateScalar(random, ops, seedSize);
 
-        ECPoint genPoint = ecParams.getGenerator();
-        ImmutableIntegerModuloP x = field.getElement(genPoint.getAffineX());
-        ImmutableIntegerModuloP y = field.getElement(genPoint.getAffineY());
-        AffinePoint affGen = new AffinePoint(x, y);
-        Point pub = ops.multiply(affGen, privArr);
-        AffinePoint affPub = pub.asAffine();
+        ECPrivateKeyImpl privateKey = new ECPrivateKeyImpl(privArr, ecParams);
+        Arrays.fill(privArr, (byte)0);
 
-        PrivateKey privateKey = new ECPrivateKeyImpl(privArr, ecParams);
-
-        ECPoint w = new ECPoint(affPub.getX().asBigInteger(),
-            affPub.getY().asBigInteger());
-        PublicKey publicKey = new ECPublicKeyImpl(w, ecParams);
+        PublicKey publicKey = privateKey.calculatePublicKey();
 
         return Optional.of(new KeyPair(publicKey, privateKey));
     }
