@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -54,7 +54,7 @@ public final class Krb5MechFactory implements MechanismFactory {
     static final Oid NT_GSS_KRB5_PRINCIPAL =
         createOid("1.2.840.113554.1.2.2.1");
 
-    private static Oid[] nameTypes =
+    private static final Oid[] nameTypes =
         new Oid[] { GSSName.NT_USER_NAME,
                         GSSName.NT_HOSTBASED_SERVICE,
                         GSSName.NT_EXPORT_NAME,
@@ -146,15 +146,14 @@ public final class Krb5MechFactory implements MechanismFactory {
         SecurityManager sm = System.getSecurityManager();
         if (sm != null) {
             String realm = (name.getKrb5PrincipalName()).getRealmAsString();
-            String tgsPrincipal =
-                new String("krbtgt/" + realm + '@' + realm);
+            String tgsPrincipal = "krbtgt/" + realm + '@' + realm;
             ServicePermission perm =
                 new ServicePermission(tgsPrincipal, "initiate");
             try {
                 sm.checkPermission(perm);
             } catch (SecurityException e) {
                 if (DEBUG) {
-                    System.out.println("Permission to initiate" +
+                    System.out.println("Permission to initiate " +
                         "kerberos init credential" + e.getMessage());
                 }
                 throw e;
@@ -215,7 +214,7 @@ public final class Krb5MechFactory implements MechanismFactory {
     }
 
 
-    public final Oid getMechanismOid() {
+    public Oid getMechanismOid() {
         return GSS_KRB5_MECH_OID;
     }
 
