@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -65,7 +65,7 @@ import java.security.MessageDigest;
  * </pre>
  * Where "s" indicates the size of the checksum.
  * <p>
- * As always, this is preceeded by a GSSHeader.
+ * As always, this is preceded by a GSSHeader.
  *
  * @author Mayank Upadhyay
  * @author Ram Marti
@@ -141,7 +141,7 @@ abstract class MessageToken extends Krb5Token {
 
     /**
      * Constructs a MessageToken from a byte array. If there are more bytes
-     * in the array than needed, the extra bytes are simply ignroed.
+     * in the array than needed, the extra bytes are simply ignored.
      *
      * @param tokenId the token id that should be contained in this token as
      * it is read.
@@ -394,11 +394,10 @@ abstract class MessageToken extends Krb5Token {
             if (initiator)
                 directionByte = (byte) 0xff; // Received token from acceptor
 
-            if ((seqNumberData[4] == directionByte) &&
-                  (seqNumberData[5] == directionByte) &&
-                  (seqNumberData[6] == directionByte) &&
-                  (seqNumberData[7] == directionByte))
-                return true;
+            return (seqNumberData[4] == directionByte) &&
+                    (seqNumberData[5] == directionByte) &&
+                    (seqNumberData[6] == directionByte) &&
+                    (seqNumberData[7] == directionByte);
         }
 
         return false;
@@ -406,7 +405,7 @@ abstract class MessageToken extends Krb5Token {
     }
 
     public final int getSequenceNumber() {
-        int sequenceNum = 0;
+        int sequenceNum;
         if (cipherHelper.isArcFour()) {
             sequenceNum = readBigEndian(seqNumberData, 0, 4);
         } else {
@@ -536,7 +535,7 @@ abstract class MessageToken extends Krb5Token {
     }
 
     /**
-     * Obtains the conext key that is associated with this token.
+     * Obtains the context key associated with this token.
      * @return the context key
      */
     /*
@@ -586,18 +585,18 @@ abstract class MessageToken extends Krb5Token {
      */
     class MessageTokenHeader {
 
-         private int tokenId;
-         private int signAlg;
-         private int sealAlg;
+         private final int tokenId;
+         private final int signAlg;
+         private final int sealAlg;
 
-         private byte[] bytes = new byte[8];
+         private final byte[] bytes = new byte[8];
 
         /**
          * Constructs a MessageTokenHeader for the specified token type with
          * appropriate checksum and encryption algorithms fields.
          *
          * @param tokenId the token id for this message token
-         * @param conf true if confidentiality will be resuested with this
+         * @param conf true if confidentiality will be requested with this
          * message token, false otherwise.
          * @param qop the value of the quality of protection that will be
          * desired.
