@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -30,6 +30,7 @@ import javax.crypto.NoSuchPaddingException;
 import javax.crypto.spec.SecretKeySpec;
 import java.security.*;
 import java.util.Arrays;
+import java.util.HexFormat;
 import java.util.Locale;
 
 public class CtrDrbg extends AbstractDrbg {
@@ -92,8 +93,7 @@ public class CtrDrbg extends AbstractDrbg {
                 }
                 this.securityStrength = tryStrength;
             } else {
-                this.securityStrength = (DEFAULT_STRENGTH > supportedStrength) ?
-                        supportedStrength : DEFAULT_STRENGTH;
+                this.securityStrength = Math.min(DEFAULT_STRENGTH, supportedStrength);
             }
         } else {
             int tryStrength = (requestedInstantiationSecurityStrength < 0) ?
@@ -181,8 +181,8 @@ public class CtrDrbg extends AbstractDrbg {
 
     private void status() {
         if (debug != null) {
-            debug.println(this, "Key = " + hex(k));
-            debug.println(this, "V   = " + hex(v));
+            debug.println(this, "Key = " + HexFormat.of().formatHex(k));
+            debug.println(this, "V   = " + HexFormat.of().formatHex(v));
             debug.println(this, "reseed counter = " + reseedCounter);
         }
     }
@@ -243,7 +243,7 @@ public class CtrDrbg extends AbstractDrbg {
                 more = nonce;
             } else {
                 if (nonce.length + personalizationString.length < 0) {
-                    // Length must be represented as a 32 bit integer in df()
+                    // Length must be represented as a 32-bit integer in df()
                     throw new IllegalArgumentException(
                             "nonce plus personalization string is too long");
                 }
@@ -327,7 +327,7 @@ public class CtrDrbg extends AbstractDrbg {
             try {
                 cipher.init(Cipher.ENCRYPT_MODE, new SecretKeySpec(k, keyAlg));
                 int tailLen = temp.length - blockLen*i;
-                // 14. requested_bits = leftmost(temp, nuumber_of_bits_to_return)
+                // 14. requested_bits = leftmost(temp, number_of_bits_to_return)
                 if (tailLen > blockLen) {
                     tailLen = blockLen;
                 }
@@ -394,7 +394,7 @@ public class CtrDrbg extends AbstractDrbg {
             // Step 1: cat bytes
             if (additionalInput != null) {
                 if (ei.length + additionalInput.length < 0) {
-                    // Length must be represented as a 32 bit integer in df()
+                    // Length must be represented as a 32-bit integer in df()
                     throw new IllegalArgumentException(
                             "entropy plus additional input is too long");
                 }
@@ -494,13 +494,16 @@ public class CtrDrbg extends AbstractDrbg {
             // Step 4.1. Increment
             addOne(v, ctrLen);
             try {
-                // Step 4.2. Encrypt
                 cipher.init(Cipher.ENCRYPT_MODE, new SecretKeySpec(k, keyAlg));
-                byte[] out = cipher.doFinal(v);
-
+                // Step 4.2. Encrypt
                 // Step 4.3 and 5. Cat bytes and leftmost
-                System.arraycopy(out, 0, result, pos,
-                        (len > blockLen) ? blockLen : len);
+                if (len > blockLen) {
+                    cipher.doFinal(v, 0, blockLen, result, pos);
+                } else {
+                    byte[] out = cipher.doFinal(v);
+                    System.arraycopy(out, 0, result, pos, len);
+                    Arrays.fill(out, (byte)0);
+                }
             } catch (GeneralSecurityException e) {
                 throw new InternalError(e);
             }
