@@ -26,7 +26,6 @@
 package sun.security.provider.certpath;
 
 import java.io.IOException;
-import java.security.AccessController;
 import java.security.GeneralSecurityException;
 import java.security.cert.*;
 import java.util.*;
@@ -51,7 +50,7 @@ import sun.security.x509.X509CertImpl;
  * @author      Yassir Elley
  */
 
-public abstract class Builder {
+abstract class Builder {
 
     private static final Debug debug = Debug.getInstance("certpath");
     private Set<String> matchingPolicies;
@@ -63,13 +62,13 @@ public abstract class Builder {
      * Authority Information Access extension shall be enabled. Currently
      * disabled by default for compatibility reasons.
      */
-    static final boolean USE_AIA = AccessController.doPrivileged
-        (new GetBooleanAction("com.sun.security.enableAIAcaIssuers"));
+    static final boolean USE_AIA = GetBooleanAction
+            .privilegedGetProperty("com.sun.security.enableAIAcaIssuers");
 
     /**
      * Initialize the builder with the input parameters.
      *
-     * @param params the parameter set used to build a certification path
+     * @param buildParams the parameter set used to build a certification path
      */
     Builder(BuilderParams buildParams) {
         this.buildParams = buildParams;
@@ -158,7 +157,6 @@ public abstract class Builder {
         case GeneralNameInterface.NAME_MATCH:
             return 0;
         case GeneralNameInterface.NAME_WIDENS:
-            break;
         case GeneralNameInterface.NAME_NARROWS:
             break;
         default: // should never occur
@@ -204,10 +202,9 @@ public abstract class Builder {
             return 0;
         case GeneralNameInterface.NAME_WIDENS:
             /* base is ancestor of test */
-            return (test.subtreeDepth()-base.subtreeDepth());
         case GeneralNameInterface.NAME_NARROWS:
             /* base is descendant of test */
-            return (test.subtreeDepth()-base.subtreeDepth());
+            return test.subtreeDepth() - base.subtreeDepth();
         default: // should never occur
             return incomparable;
         }
@@ -233,7 +230,7 @@ public abstract class Builder {
             int commonDistance = commonName.subtreeDepth();
             int baseDistance = baseName.subtreeDepth();
             int testDistance = testName.subtreeDepth();
-            return (baseDistance + testDistance - (2 * commonDistance));
+            return baseDistance + testDistance - (2 * commonDistance);
         }
     }
 
@@ -303,8 +300,7 @@ public abstract class Builder {
         SubjectAlternativeNameExtension altNameExt =
             certImpl.getSubjectAlternativeNameExtension();
         if (altNameExt != null) {
-            GeneralNames altNames = altNameExt.get(
-                    SubjectAlternativeNameExtension.SUBJECT_NAME);
+            GeneralNames altNames = altNameExt.getNames();
             /* see if any alternative name matches target */
             if (altNames != null) {
                 for (int j = 0, n = altNames.size(); j < n; j++) {
@@ -330,20 +326,18 @@ public abstract class Builder {
             constraints.merge(ncExt);
         } else {
             // Make sure we do a clone here, because we're probably
-            // going to modify this object later and we don't want to
+            // going to modify this object later, and we don't want to
             // be sharing it with a Certificate object!
             constraints = (NameConstraintsExtension) ncExt.clone();
         }
 
         if (debug != null) {
             debug.println("Builder.targetDistance() merged constraints: "
-                + String.valueOf(constraints));
+                + constraints);
         }
         /* reduce permitted by excluded */
-        GeneralSubtrees permitted =
-                constraints.get(NameConstraintsExtension.PERMITTED_SUBTREES);
-        GeneralSubtrees excluded =
-                constraints.get(NameConstraintsExtension.EXCLUDED_SUBTREES);
+        GeneralSubtrees permitted = constraints.getPermittedSubtrees();
+        GeneralSubtrees excluded = constraints.getExcludedSubtrees();
         if (permitted != null) {
             permitted.reduce(excluded);
         }
@@ -365,7 +359,7 @@ public abstract class Builder {
             GeneralNameInterface perName = permitted.get(i).getName().getName();
             int distance = distance(perName, target, -1);
             if (distance >= 0) {
-                return (distance + 1);
+                return distance + 1;
             }
         }
         /* no matching type in permitted; cert holder could certify target */
@@ -404,7 +398,7 @@ public abstract class Builder {
             } else {
                 // we just return an empty set to make sure that there is
                 // at least a certificate policies extension in the cert
-                matchingPolicies = Collections.<String>emptySet();
+                matchingPolicies = Collections.emptySet();
             }
         }
         return matchingPolicies;
@@ -435,8 +429,7 @@ public abstract class Builder {
                 if (debug != null) {
                     debug.println("Builder.addMatchingCerts: " +
                         "adding target cert" +
-                        "\n  SN: " + Debug.toHexString(
-                                            targetCert.getSerialNumber()) +
+                        "\n  SN: " + Debug.toString(targetCert.getSerialNumber()) +
                         "\n  Subject: " + targetCert.getSubjectX500Principal() +
                         "\n  Issuer: " + targetCert.getIssuerX500Principal());
                 }
