@@ -60,12 +60,12 @@ public final class RSAPublicKeyImpl extends X509Key implements RSAPublicKey {
     private BigInteger n;       // modulus
     private BigInteger e;       // public exponent
 
-    private transient KeyType type;
+    private final transient KeyType type;
 
     // optional parameters associated with this RSA key
     // specified in the encoding of its AlgorithmId
     // must be null for "RSA" keys.
-    private transient AlgorithmParameterSpec keyParams;
+    private final transient AlgorithmParameterSpec keyParams;
 
     /**
      * Generate a new RSAPublicKey from the specified type, format, and
@@ -106,7 +106,7 @@ public final class RSAPublicKeyImpl extends X509Key implements RSAPublicKey {
     }
 
     /**
-     * Construct a RSA key from the specified type and components. Used by
+     * Construct an RSA key from the specified type and components. Used by
      * RSAKeyFactory and RSAKeyPairGenerator.
      */
     RSAPublicKeyImpl(KeyType type, AlgorithmParameterSpec keyParams,
@@ -128,19 +128,14 @@ public final class RSAPublicKeyImpl extends X509Key implements RSAPublicKey {
         this.type = type;
         this.keyParams = keyParams;
 
-        try {
-            // generate the key encoding
-            DerOutputStream out = new DerOutputStream();
-            out.putInteger(n);
-            out.putInteger(e);
-            byte[] keyArray =
+        // generate the key encoding
+        DerOutputStream out = new DerOutputStream();
+        out.putInteger(n);
+        out.putInteger(e);
+        byte[] keyArray =
                 new DerValue(DerValue.tag_Sequence,
-                             out.toByteArray()).toByteArray();
-            setKey(new BitArray(keyArray.length*8, keyArray));
-        } catch (IOException exc) {
-            // should never occur
-            throw new InvalidKeyException(exc);
-        }
+                        out.toByteArray()).toByteArray();
+        setKey(new BitArray(keyArray.length * 8, keyArray));
     }
 
     /**
