@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2020, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -35,21 +35,27 @@ package sun.security.util;
 public final class Event {
     private Event() {}
 
+    public enum ReporterCategory {
+        CRLCHECK(),
+        ZIPFILEATTRS();
+
+        private Reporter reporter;
+    }
+
     public interface Reporter {
-        public void handle(String type, Object... args);
+        void handle(String type, Object... args);
     }
 
-    private static Reporter reporter;
-    public static void setReportListener(Reporter re) {
-        reporter = re;
+    public static void setReportListener(ReporterCategory cat, Reporter re) {
+        cat.reporter = re;
     }
 
-    public static void clearReportListener() {
-        reporter = null;
+    public static void clearReportListener(ReporterCategory cat) {
+        cat.reporter = null;
     }
 
-    public static void report(String type, Object... args) {
-        Reporter currentReporter = reporter;
+    public static void report(ReporterCategory cat, String type, Object... args) {
+        Reporter currentReporter = cat.reporter;
 
         if (currentReporter != null) {
             currentReporter.handle(type, args);
