@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -26,11 +26,10 @@
 package sun.security.util;
 
 import java.lang.reflect.ReflectPermission;
-import java.net.SocketPermission;
 import java.net.NetPermission;
-import java.security.Permission;
-import java.security.SecurityPermission;
+import java.net.SocketPermission;
 import java.security.AllPermission;
+import java.security.SecurityPermission;
 import sun.security.action.GetPropertyAction;
 
 /**
@@ -97,7 +96,7 @@ public final class SecurityConstants {
     public static final NetPermission GET_RESPONSECACHE_PERMISSION =
        new NetPermission("getResponseCache");
 
-    // java.net.Socket
+    // java.net.ServerSocket, java.net.Socket
     public static final NetPermission SET_SOCKETIMPL_PERMISSION =
         new NetPermission("setSocketImpl");
 
@@ -125,10 +124,6 @@ public final class SecurityConstants {
     public static final RuntimePermission GET_CLASSLOADER_PERMISSION =
         new RuntimePermission("getClassLoader");
 
-    // java.lang.Thread
-    public static final RuntimePermission STOP_THREAD_PERMISSION =
-       new RuntimePermission("stopThread");
-
     // java.lang.Thread
     public static final RuntimePermission GET_STACK_TRACE_PERMISSION =
        new RuntimePermission("getStackTrace");
