@@ -50,7 +50,7 @@ import sun.security.util.*;
  * @author Hemma Prafullchandra
  */
 public class DNSName implements GeneralNameInterface {
-    private String name;
+    private final String name;
 
     private static final String alphaDigits =
         "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
@@ -83,7 +83,7 @@ public class DNSName implements GeneralNameInterface {
          * Name will consist of label components separated by "."
          * startIndex is the index of the first character of a component
          * endIndex is the index of the last character of a component plus 1
-        */
+         */
         for (int endIndex,startIndex = 0; startIndex < name.length(); startIndex = endIndex+1) {
             endIndex = name.indexOf('.', startIndex);
             if (endIndex < 0) {
@@ -100,7 +100,7 @@ public class DNSName implements GeneralNameInterface {
                 if (alphaDigits.indexOf(name.charAt(startIndex)) < 0) {
                     // Checking to make sure the wildcard only appears in the first component,
                     // and it has to be at least 3-char long with the form of *.[alphaDigit]
-                    if ((name.length() < 3) || (name.indexOf('*', 0) != 0) ||
+                    if ((name.length() < 3) || (name.indexOf('*') != 0) ||
                         (name.charAt(startIndex+1) != '.') ||
                         (alphaDigits.indexOf(name.charAt(startIndex+2)) < 0))
                         throw new IOException("DNSName components must begin with a letter, digit, "
@@ -150,9 +150,9 @@ public class DNSName implements GeneralNameInterface {
      * Encode the DNSName into the DerOutputStream.
      *
      * @param out the DER stream to encode the DNSName to.
-     * @exception IOException on encoding errors.
      */
-    public void encode(DerOutputStream out) throws IOException {
+    @Override
+    public void encode(DerOutputStream out) {
         out.putIA5String(name);
     }
 
@@ -173,11 +173,9 @@ public class DNSName implements GeneralNameInterface {
         if (this == obj)
             return true;
 
-        if (!(obj instanceof DNSName))
+        if (!(obj instanceof DNSName other))
             return false;
 
-        DNSName other = (DNSName)obj;
-
         // RFC5280 mandates that these names are
         // not case-sensitive
         return name.equalsIgnoreCase(other.name);
@@ -202,18 +200,12 @@ public class DNSName implements GeneralNameInterface {
      * </ul>.  These results are used in checking NameConstraints during
      * certification path verification.
      * <p>
-     * RFC5280: DNS name restrictions are expressed as host.example.com.
+     * RFC5280: For DNS names, restrictions MUST use the DNSName syntax in Section 4.2.1.6.
      * Any DNS name that can be constructed by simply adding zero or more
      * labels to the left-hand side of the name satisfies the name constraint.
      * For example, www.host.example.com would satisfy the constraint but
      * host1.example.com would not.
      * <p>
-     * draft-ietf-pkix-new-part1-00.txt:  DNSName restrictions are expressed as foo.bar.com.
-     * Any DNSName that
-     * can be constructed by simply adding to the left hand side of the name
-     * satisfies the name constraint. For example, www.foo.bar.com would
-     * satisfy the constraint but foo1.bar.com would not.
-     * <p>
      * RFC1034: By convention, domain names can be stored with arbitrary case, but
      * domain name comparisons for all present domain functions are done in a
      * case-insensitive manner, assuming an ASCII character set, and a high
@@ -238,13 +230,13 @@ public class DNSName implements GeneralNameInterface {
                 constraintType = NAME_MATCH;
             else if (thisName.endsWith(inName)) {
                 int inNdx = thisName.lastIndexOf(inName);
-                if (thisName.charAt(inNdx-1) == '.' )
+                if (thisName.charAt(inNdx-1) == '.' ^ inName.charAt(0) == '.')
                     constraintType = NAME_WIDENS;
                 else
                     constraintType = NAME_SAME_TYPE;
             } else if (inName.endsWith(thisName)) {
                 int ndx = inName.lastIndexOf(thisName);
-                if (inName.charAt(ndx-1) == '.' )
+                if (inName.charAt(ndx-1) == '.' ^ thisName.charAt(0) == '.')
                     constraintType = NAME_NARROWS;
                 else
                     constraintType = NAME_SAME_TYPE;
