@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2005, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -26,9 +26,6 @@
 package sun.security.x509;
 
 import java.io.IOException;
-import java.io.OutputStream;
-
-import java.util.*;
 
 import sun.security.util.DerInputStream;
 import sun.security.util.DerOutputStream;
@@ -46,7 +43,7 @@ import sun.security.util.DerValue;
  *
  * <p>
  * The extension is defined in Section 5.2.5 of
- * <a href="http://tools.ietf.org/html/rfc5280">Internet X.509 PKI
+ * <a href="https://tools.ietf.org/html/rfc5280">Internet X.509 PKI
  * Certificate and Certificate Revocation List (CRL) Profile</a>.
  *
  * <p>
@@ -66,26 +63,9 @@ import sun.security.util.DerValue;
  * @see DistributionPoint
  * @since 1.6
  */
-public class IssuingDistributionPointExtension extends Extension
-        implements CertAttrSet<String> {
+public class IssuingDistributionPointExtension extends Extension {
 
-    /**
-     * Identifier for this attribute, to be used with the
-     * get, set, delete methods of Certificate, x509 type.
-     */
-    public static final String IDENT =
-                                "x509.info.extensions.IssuingDistributionPoint";
-
-    /**
-     * Attribute names.
-     */
     public static final String NAME = "IssuingDistributionPoint";
-    public static final String POINT = "point";
-    public static final String REASONS = "reasons";
-    public static final String ONLY_USER_CERTS = "only_user_certs";
-    public static final String ONLY_CA_CERTS = "only_ca_certs";
-    public static final String ONLY_ATTRIBUTE_CERTS = "only_attribute_certs";
-    public static final String INDIRECT_CRL = "indirect_crl";
 
     /*
      * The distribution point name for the CRL.
@@ -130,15 +110,22 @@ public class IssuingDistributionPointExtension extends Extension
      *        issuer CRL entry extension.
      * @throws IllegalArgumentException if more than one of
      *        <code>hasOnlyUserCerts</code>, <code>hasOnlyCACerts</code>,
-     *        <code>hasOnlyAttributeCerts</code> is set to <code>true</code>.
-     * @throws IOException on encoding error.
+     *        <code>hasOnlyAttributeCerts</code> is set to <code>true</code>,
+     *        or all arguments are either <code>null</code> or <code>false</code>.
      */
     public IssuingDistributionPointExtension(
         DistributionPointName distributionPoint, ReasonFlags revocationReasons,
         boolean hasOnlyUserCerts, boolean hasOnlyCACerts,
-        boolean hasOnlyAttributeCerts, boolean isIndirectCRL)
-            throws IOException {
+        boolean hasOnlyAttributeCerts, boolean isIndirectCRL) {
 
+        if (distributionPoint == null &&
+                revocationReasons == null &&
+                !hasOnlyUserCerts &&
+                !hasOnlyCACerts &&
+                !hasOnlyAttributeCerts &&
+                !isIndirectCRL) {
+            throw new IllegalArgumentException("elements cannot be empty");
+        }
         if ((hasOnlyUserCerts && (hasOnlyCACerts || hasOnlyAttributeCerts)) ||
             (hasOnlyCACerts && (hasOnlyUserCerts || hasOnlyAttributeCerts)) ||
             (hasOnlyAttributeCerts && (hasOnlyUserCerts || hasOnlyCACerts))) {
@@ -187,7 +174,7 @@ public class IssuingDistributionPointExtension extends Extension
         }
 
         DerInputStream in = val.data;
-        while (in != null && in.available() != 0) {
+        while (in.available() != 0) {
             DerValue opt = in.getDerValue();
 
             if (opt.isContextSpecific(TAG_DISTRIBUTION_POINT) &&
@@ -221,8 +208,9 @@ public class IssuingDistributionPointExtension extends Extension
     }
 
     /**
-     * Returns the name of this attribute.
+     * Returns the name of this extension.
      */
+    @Override
     public String getName() {
         return NAME;
     }
@@ -232,149 +220,48 @@ public class IssuingDistributionPointExtension extends Extension
      * DerOutputStream.
      *
      * @param out the output stream.
-     * @exception IOException on encoding error.
      */
-    public void encode(OutputStream out) throws IOException {
-        DerOutputStream tmp = new DerOutputStream();
+    @Override
+    public void encode(DerOutputStream out) {
         if (this.extensionValue == null) {
             this.extensionId = PKIXExtensions.IssuingDistributionPoint_Id;
             this.critical = false;
             encodeThis();
         }
-        super.encode(tmp);
-        out.write(tmp.toByteArray());
+        super.encode(out);
     }
 
-    /**
-     * Sets the attribute value.
-     */
-    public void set(String name, Object obj) throws IOException {
-        if (name.equalsIgnoreCase(POINT)) {
-            if (!(obj instanceof DistributionPointName)) {
-                throw new IOException(
-                    "Attribute value should be of type DistributionPointName.");
-            }
-            distributionPoint = (DistributionPointName)obj;
-
-        } else if (name.equalsIgnoreCase(REASONS)) {
-            if (!(obj instanceof ReasonFlags)) {
-                throw new IOException(
-                    "Attribute value should be of type ReasonFlags.");
-            }
-            revocationReasons = (ReasonFlags)obj;
-
-        } else if (name.equalsIgnoreCase(INDIRECT_CRL)) {
-            if (!(obj instanceof Boolean)) {
-                throw new IOException(
-                    "Attribute value should be of type Boolean.");
-            }
-            isIndirectCRL = ((Boolean)obj).booleanValue();
-
-        } else if (name.equalsIgnoreCase(ONLY_USER_CERTS)) {
-            if (!(obj instanceof Boolean)) {
-                throw new IOException(
-                    "Attribute value should be of type Boolean.");
-            }
-            hasOnlyUserCerts = ((Boolean)obj).booleanValue();
-
-        } else if (name.equalsIgnoreCase(ONLY_CA_CERTS)) {
-            if (!(obj instanceof Boolean)) {
-                throw new IOException(
-                    "Attribute value should be of type Boolean.");
-            }
-            hasOnlyCACerts = ((Boolean)obj).booleanValue();
-
-        } else if (name.equalsIgnoreCase(ONLY_ATTRIBUTE_CERTS)) {
-            if (!(obj instanceof Boolean)) {
-                throw new IOException(
-                    "Attribute value should be of type Boolean.");
-            }
-            hasOnlyAttributeCerts = ((Boolean)obj).booleanValue();
-
-        } else {
-            throw new IOException("Attribute name [" + name +
-                "] not recognized by " +
-                "CertAttrSet:IssuingDistributionPointExtension.");
-        }
+    public void setRevocationReasons(ReasonFlags val) throws IOException {
+        revocationReasons = val;
         encodeThis();
     }
 
-    /**
-     * Gets the attribute value.
-     */
-    public Object get(String name) throws IOException {
-        if (name.equalsIgnoreCase(POINT)) {
-            return distributionPoint;
-
-        } else if (name.equalsIgnoreCase(INDIRECT_CRL)) {
-            return Boolean.valueOf(isIndirectCRL);
-
-        } else if (name.equalsIgnoreCase(REASONS)) {
-            return revocationReasons;
-
-        } else if (name.equalsIgnoreCase(ONLY_USER_CERTS)) {
-            return Boolean.valueOf(hasOnlyUserCerts);
-
-        } else if (name.equalsIgnoreCase(ONLY_CA_CERTS)) {
-            return Boolean.valueOf(hasOnlyCACerts);
-
-        } else if (name.equalsIgnoreCase(ONLY_ATTRIBUTE_CERTS)) {
-            return Boolean.valueOf(hasOnlyAttributeCerts);
-
-        } else {
-            throw new IOException("Attribute name [" + name +
-                "] not recognized by " +
-                "CertAttrSet:IssuingDistributionPointExtension.");
-        }
+    public DistributionPointName getDistributionPoint() {
+        return distributionPoint;
     }
 
-    /**
-     * Deletes the attribute value.
-     */
-    public void delete(String name) throws IOException {
-        if (name.equalsIgnoreCase(POINT)) {
-            distributionPoint = null;
-
-        } else if (name.equalsIgnoreCase(INDIRECT_CRL)) {
-            isIndirectCRL = false;
-
-        } else if (name.equalsIgnoreCase(REASONS)) {
-            revocationReasons = null;
-
-        } else if (name.equalsIgnoreCase(ONLY_USER_CERTS)) {
-            hasOnlyUserCerts = false;
-
-        } else if (name.equalsIgnoreCase(ONLY_CA_CERTS)) {
-            hasOnlyCACerts = false;
-
-        } else if (name.equalsIgnoreCase(ONLY_ATTRIBUTE_CERTS)) {
-            hasOnlyAttributeCerts = false;
-
-        } else {
-            throw new IOException("Attribute name [" + name +
-                "] not recognized by " +
-                "CertAttrSet:IssuingDistributionPointExtension.");
-        }
-        encodeThis();
+    public ReasonFlags getRevocationReasons() {
+        return revocationReasons;
     }
 
-    /**
-     * Returns an enumeration of names of attributes existing within this
-     * attribute.
-     */
-    public Enumeration<String> getElements() {
-        AttributeNameEnumeration elements = new AttributeNameEnumeration();
-        elements.addElement(POINT);
-        elements.addElement(REASONS);
-        elements.addElement(ONLY_USER_CERTS);
-        elements.addElement(ONLY_CA_CERTS);
-        elements.addElement(ONLY_ATTRIBUTE_CERTS);
-        elements.addElement(INDIRECT_CRL);
-        return elements.elements();
+    public boolean hasOnlyUserCerts() {
+        return hasOnlyUserCerts;
+    }
+
+    public boolean hasOnlyCACerts() {
+        return hasOnlyCACerts;
+    }
+
+    public boolean hasOnlyAttributeCerts() {
+        return hasOnlyAttributeCerts;
+    }
+
+    public boolean isIndirectCRL() {
+        return isIndirectCRL;
     }
 
      // Encodes this extension value
-    private void encodeThis() throws IOException {
+    private void encodeThis() {
 
         if (distributionPoint == null &&
             revocationReasons == null &&
