@@ -25,10 +25,13 @@
 
 package sun.swing;
 
-import java.awt.*;
+import java.awt.Component;
+import java.awt.Point;
 import java.lang.invoke.MethodHandles;
-import javax.swing.*;
+import javax.accessibility.Accessible;
+import javax.accessibility.AccessibleContext;
 
+import javax.swing.*;
 import javax.swing.text.JTextComponent;
 
 /**
@@ -47,6 +50,14 @@ public final class SwingAccessor {
     private SwingAccessor() {
     }
 
+    /**
+     * This interface provides access to the renderer's accessibility component.
+     * For example, the renderer of a list element, a table cell, or a tree node
+     */
+    public interface AccessibleComponentAccessor {
+        Accessible getCurrentAccessible(AccessibleContext ac);
+    }
+
     /**
      * An accessor for the JComponent class.
      */
@@ -290,6 +301,21 @@ public final class SwingAccessor {
         SwingAccessor.keyStrokeAccessor = accessor;
     }
 
+    private static AccessibleComponentAccessor accessibleComponentAccessor = null;
+
+    public static AccessibleComponentAccessor getAccessibleComponentAccessor() {
+        var access = accessibleComponentAccessor;
+        if (access == null) {
+            ensureClassInitialized(JTree.class);
+            access = accessibleComponentAccessor;
+        }
+        return access;
+    }
+
+    public static void setAccessibleComponentAccessor(final AccessibleComponentAccessor accessibleAccessor) {
+        accessibleComponentAccessor = accessibleAccessor;
+    }
+
     private static void ensureClassInitialized(Class<?> c) {
         try {
             MethodHandles.lookup().ensureInitialized(c);
