@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2004, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -216,9 +216,7 @@ class ThreadTab extends Tab implements ActionListener, DocumentListener, ListSel
                     }
                     timeStamp = System.currentTimeMillis();
                     return true;
-                } catch (IOException e) {
-                    return false;
-                } catch (UndeclaredThrowableException e) {
+                } catch (IOException | UndeclaredThrowableException e) {
                     return false;
                 }
             }
@@ -426,9 +424,7 @@ class ThreadTab extends Tab implements ActionListener, DocumentListener, ListSel
                                         }
                                     });
                                     sleep(30 * 1000);
-                                } catch (InterruptedException ex) {
-                                    // Ignore
-                                } catch (InvocationTargetException ex) {
+                                } catch (InterruptedException | InvocationTargetException ex) {
                                     // Ignore
                                 }
                                 SwingUtilities.invokeLater(new Runnable() {
@@ -619,7 +615,7 @@ class ThreadTab extends Tab implements ActionListener, DocumentListener, ListSel
         }
     }
 
-    private class PromptingTextField extends JTextField implements FocusListener {
+    private static class PromptingTextField extends JTextField implements FocusListener {
         private String prompt;
         boolean promptRemoved = false;
         Color fg;
@@ -665,7 +661,7 @@ class ThreadTab extends Tab implements ActionListener, DocumentListener, ListSel
         }
 
         public void focusLost(FocusEvent e) {
-            if (promptRemoved && getText().equals("")) {
+            if (promptRemoved && getText().isEmpty()) {
                 setText(prompt);
                 setForeground(Color.gray);
                 promptRemoved = false;
