@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -26,9 +26,8 @@
 /*
  * COPYRIGHT AND PERMISSION NOTICE
  *
- * Copyright (C) 1991-2016 Unicode, Inc. All rights reserved.
- * Distributed under the Terms of Use in 
- * http://www.unicode.org/copyright.html.
+ * Copyright (c) 1991-2022 Unicode, Inc. All rights reserved.
+ * Distributed under the Terms of Use in https://www.unicode.org/copyright.html.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of the Unicode data files and any associated documentation
@@ -37,14 +36,11 @@
  * without restriction, including without limitation the rights to use,
  * copy, modify, merge, publish, distribute, and/or sell copies of
  * the Data Files or Software, and to permit persons to whom the Data Files
- * or Software are furnished to do so, provided that
- * (a) this copyright and permission notice appear with all copies 
- * of the Data Files or Software,
- * (b) this copyright and permission notice appear in associated 
- * documentation, and
- * (c) there is clear notice in each modified Data File or in the Software
- * as well as in the documentation associated with the Data File(s) or
- * Software that the data or software has been modified.
+ * or Software are furnished to do so, provided that either
+ * (a) this copyright and permission notice appear with all copies
+ * of the Data Files or Software, or
+ * (b) this copyright and permission notice appear in associated
+ * Documentation.
  *
  * THE DATA FILES AND SOFTWARE ARE PROVIDED "AS IS", WITHOUT WARRANTY OF
  * ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
@@ -310,14 +306,6 @@ public class TimeZoneNames_ur_IN extends TimeZoneNamesBundle {
                "",
                "",
             };
-        final String[] Macquarie = new String[] {
-               "\u0645\u06a9\u0648\u0622\u0631\u06cc \u0622\u0626\u0644\u06cc\u0646\u0688 \u0679\u0627\u0626\u0645",
-               "",
-               "",
-               "",
-               "",
-               "",
-            };
         final String[] Venezuela = new String[] {
                "\u0648\u06cc\u0646\u0632\u0648\u0626\u06cc\u0644\u0627 \u0679\u0627\u0626\u0645",
                "",
@@ -406,14 +394,6 @@ public class TimeZoneNames_ur_IN extends TimeZoneNamesBundle {
                "\u0648\u0633\u0637\u06cc \u06cc\u0648\u0631\u067e \u06a9\u0627 \u0648\u0642\u062a",
                "",
             };
-        final String[] Argentina_Western = new String[] {
-               "\u0645\u063a\u0631\u0628\u06cc \u0627\u0631\u062c\u0646\u0679\u06cc\u0646\u0627 \u0633\u0679\u06cc\u0646\u0688\u0631\u0688 \u0679\u0627\u0626\u0645",
-               "",
-               "\u0645\u063a\u0631\u0628\u06cc \u0627\u0631\u062c\u0646\u0679\u06cc\u0646\u0627 \u0633\u0645\u0631 \u0679\u0627\u0626\u0645",
-               "",
-               "\u0645\u063a\u0631\u0628\u06cc \u0627\u0631\u062c\u0646\u0679\u06cc\u0646\u0627 \u0679\u0627\u0626\u0645",
-               "",
-            };
         final String[] Kazakhstan_Eastern = new String[] {
                "\u0645\u0634\u0631\u0642\u06cc \u0642\u0632\u0627\u062e\u0633\u062a\u0627\u0646 \u0679\u0627\u0626\u0645",
                "",
@@ -484,6 +464,7 @@ public class TimeZoneNames_ur_IN extends TimeZoneNamesBundle {
             { "Asia/Calcutta", India },
             { "Asia/Dushanbe", Tajikistan },
             { "Asia/Katmandu", Nepal },
+            { "Asia/Qostanay", Kazakhstan_Eastern },
             { "Asia/Tashkent", Uzbekistan },
             { "Europe/Berlin", Europe_Central },
             { "Europe/Dublin", GMT },
@@ -508,7 +489,7 @@ public class TimeZoneNames_ur_IN extends TimeZoneNamesBundle {
             { "America/Maceio", Brasilia },
             { "America/Manaus", Amazon },
             { "America/Recife", Brasilia },
-            { "Asia/Qyzylorda", Kazakhstan_Eastern },
+            { "Asia/Qyzylorda", Kazakhstan_Western },
             { "Asia/Samarkand", Uzbekistan },
             { "Europe/Andorra", Europe_Central },
             { "Europe/Belfast", GMT },
@@ -516,6 +497,7 @@ public class TimeZoneNames_ur_IN extends TimeZoneNamesBundle {
             { "Pacific/Easter", Easter },
             { "Africa/Freetown", GMT },
             { "Africa/Monrovia", GMT },
+            { "Africa/Sao_Tome", GMT },
             { "Africa/Timbuktu", GMT },
             { "America/Caracas", Venezuela },
             { "America/Cayenne", French_Guiana },
@@ -563,9 +545,7 @@ public class TimeZoneNames_ur_IN extends TimeZoneNamesBundle {
             { "Arctic/Longyearbyen", Europe_Central },
             { "America/Campo_Grande", Amazon },
             { "America/Danmarkshavn", GMT },
-            { "Antarctica/Macquarie", Macquarie },
             { "Antarctica/South_Pole", New_Zealand },
-            { "America/Argentina/San_Luis", Argentina_Western },
             { "timezone.excity.Africa/Accra", "\u0627\u06a9\u0631\u0627" },
             { "timezone.excity.America/Thule", "\u062a\u06be\u0648\u0644\u06d2" },
             { "timezone.excity.Europe/Budapest", "\u0628\u0648\u0688\u0627\u067e\u06cc\u0633\u0679" },
